// ---------------------------------------
// Functions provided as examples 
// load_EOP: load IERS data Bulletin B (csv or txt format)
//---------------------------------------

// ======================================
// Public functions
// ======================================

// --------------------------------------
// Loads all IERS data Bulletin B (csv or txt format) from a directory D
// If the 2 formats exist, the "csv" file is loaded. 
// Return: data (struct): date(cjd, TREF), ut1_utc(s), xp(rad), yp(rad)
// all fields: 1xN 
// --------------------------------------

function [data] = load_EOP(D)
  // Filter file names without file extension
  fnames = CL_path("bulletinb*" + [".csv", ".txt"], D, "all", maxdepth=0); 
  [path, filename] = fileparts(fnames);
  names = unique(path + filename);

  M = []; 

  for (k = 1 : size(names, "*"))
    if (isfile(names(k) + ".csv"))  
      M1 = read_eop_csv(names(k) + ".csv"); 
    else
      M1 = read_eop_txt(names(k) + ".txt"); 
    end 
    M = [M, M1]; 
  end

  data = gen_eop_data(M);  
endfunction 


// ======================================
// Private functions
// ======================================

// --------------------------------------
// Read IERS data Bulletin B (csv format)
// Return: M = [date_utc; ut1_utc(s); xp(rad); yp(rad)] 
// size = 4xN 
// --------------------------------------
function [M] = read_eop_csv(fname)
  str = csvRead(fname, ";", [], "string"); 

  desc = str(1,:); 
  str = str(2:$,:); 

  i_mjd = find(desc == "MJD"); 
  i_ut1utc = find(desc == "UT1-UTC"); 
  i_xp = find(desc == "x_pole"); 
  i_yp = find(desc == "y_pole"); 
  i_type = find(desc == "Type");

  I = find(str(:, i_type(1)) == "final"); 
  M = strtod(str(I,[i_mjd, i_ut1utc, i_xp, i_yp]))'; 

  M(1,:) = CL_dat_convert("mjd", "cjd", M(1,:)); 
  M(2,:) = M(2,:) / 1000; 
  M(3:4,:) = CL_unitConvert(M(3:4,:), "mas", "rad");
endfunction


// --------------------------------------
// Read IERS data Bulletin B (text format)
// Return: M = [date_utc; ut1_utc(s); xp(rad); yp(rad)] 
// size = 4xN 
// --------------------------------------
function [M] = read_eop_txt(fname)
  str = stripblanks(mgetl(fname), %t);  
  I= find(stripblanks(str, %t) <> ""); 
  str = str(I); 
  I1 = grep(str, "Mean formal error"); 
  I2 = grep(str, "Preliminary extension"); 
  str = str(I1(1)+1 : I2(1)-1); 

  form = strcat(repmat("%f", 1, 14), " "); 
  M = msscanf(-1, str, form); 
  M = M(:, [4, 7, 5, 6])'; 

  M(1,:) = CL_dat_convert("mjd", "cjd", M(1,:)); 
  M(2,:) = M(2,:) / 1000; 
  M(3:4,:) = CL_unitConvert(M(3:4,:), "mas", "rad");
endfunction


// --------------------------------------
// Generates a structure containing: 
// date_utc, ut1_utc(s), xp(rad), yp(rad)
// all fields: 1xN 
// --------------------------------------
function [data] = gen_eop_data(M)
  M = CL_sortMat(M, M(1,:));

  data = struct(); 
  data.cjd_utc = M(1,:); 
  data.ut1_utc = M(2,:); 
  data.xp = M(3,:); 
  data.yp = M(4,:); 

  // check data exist every day 
  d = diff(data.cjd_utc); 
  if (find(d < 0.5 | d > 1.5) <> [])
    error("Missing or invalid EOP data"); 
  end
endfunction



