// ===============================================
// Utility functions (STELA shapes) : 
// - make_spacecraft
// - plot_spacecraft
// ===============================================

// -----------------------------------------------
// Creation of a simple spacecraft (body + 2 solar arrays) 
// dim_body: size of body edges along (x, y, z) (3x1) 
// dim_array : size of solar array (x,y) (2x1)
// +z considered the side with cells (if rot_angle = 0)
// gap: space between body and solar array
// rot_angle: solar array rotation angle around +x axis 
// -----------------------------------------------
function [obj] = make_spacecraft(dim_body, dim_array, gap, rot_angle)

  function  [obj] = add_part(obj, part); 
    for elem = ["id", "shap", "dim", "z_axis", "x_axis", "orig"]
      obj(elem) = [obj(elem), part(elem)];
    end 
  endfunction 
  
  function [part] = make_part(id, shap, dim, z_axis, x_axis, orig) 
    part = struct(); 
    part.id = id; 
    part.shap = shap; 
    part.dim = dim;  
    part.z_axis = z_axis; 
    part.x_axis = x_axis; 
    part.orig = orig; 
  endfunction 

  // body: cube centered on origin 
  obj = make_part("body", "Cuboid", dim_body([2, 3, 1]), ... 
                  [0; 0; 1], [1; 0; 0],[0; 0; 0]); 
  
  // compute z_axis and x_axis 
  M = CL_rot_angles2matrix(1, rot_angle); 
  z_axis = M' * [0; 0; 1]; 
  x_axis = M' * [1; 0; 0]; // = [1; 0; 0]
  
  // array1: l_array along x, w_array along y 
  // rotation axis = +x axis, normal = +z_axis  
  part = make_part("array1", "Rectangle", [dim_array([2, 1]); %nan], ...
                   z_axis, x_axis, ... 
                   [dim_body(1) / 2 + gap + dim_array(1) / 2; 0; 0]); 
  obj = add_part(obj, part); 
  
  // array2: l_array along x, w_array along y 
  // rotation axis = +x axis, normal = +z_axis  
  part = make_part("array2", "Rectangle", [dim_array([2, 1]); %nan], ...
                   z_axis, x_axis, ... 
                   [-(dim_body(1) / 2 + gap + dim_array(1) / 2); 0; 0]); 
  obj = add_part(obj, part); 
  
endfunction 

// -----------------------------------------------
// Plot a cuboid 
// dim: subpart dimensions (Stela convention: y, z, x) 
// C: center of subpart in object frame (3x1)
// M: transf matrix from object frame to subpart frame (3x3)
// bounds: 2x3 : [xmin, ymin, zmin; xmax, ymax, zmax]
// -----------------------------------------------
function [bounds] = _plot_cuboid(dim, C, M); 
  // coordinates of vertices in subpart frame 
  S = [ (dim(3) / 2) * [1, -1,  1, -1,  1, -1,  1, -1]; 
        (dim(1) / 2) * [1,  1, -1, -1,  1,  1, -1, -1]; 
        (dim(2) / 2) * [1,  1,  1,  1, -1, -1, -1, -1]]; 

  // Check that the directions are not aligned with X, Y, z axes (bug Scilab) 
  // => small rotation if it is the case.  

  eps = 1.e-6; 
  if (find(abs(M) > 1 - eps) <> [])
    M_eps = CL_rot_quat2matrix(CL_rot_axAng2quat([1; 1; 1], eps)); 
    M = M_eps * M; 
  end 
  
  // coordinates of vertices in object frame 
  S = M' * S + repmat(C, 1, size(S, 2)); 

  // sides (numbers of the 4 vertices) 
  numf = [ [1; 5; 7; 3], [2; 4; 8; 6], [1; 2; 6; 5], [7; 8; 4; 3], [1; 3; 4; 2], [6; 8; 7; 5] ]; 

  x = matrix(S(1,numf), 4, -1); 
  y = matrix(S(2,numf), 4, -1); 
  z = matrix(S(3,numf), 4, -1); 

  cols = repmat(color("grey70"), 1, 6); 

  plot3d(x, y, list(z, cols)); 
  e = gce(); 
  e.hiddencolor = color("grey30"); 
  
  bounds = [min(x), min(y), min(z); max(x), max(y), max(z)]; 
endfunction 

// -----------------------------------------------
// Plot a rectangle 
// dim: subpart dimensions (Stela convention: y, x) 
// C: center of subpart in object frame (3x1)
// M: transf matrix from object frame to subpart frame (3x3)
// bounds: 2x3 : [xmin, ymin, zmin; xmax, ymax, zmax]
// -----------------------------------------------
function [bounds] = _plot_rectangle(dim, C, M); 
  // coordinates of vertices in subpart frame 
  S = [ (dim(2) / 2) * [1,  1, -1, -1]; 
        (dim(1) / 2) * [1, -1, -1,  1]; 
                       [0,  0,  0,  0]]; 

  // Check that the directions are not aligned with X, Y, z axes (bug Scilab) 
  // => small rotation if it is the case.  

  eps = 1.e-6; 
  if (find(abs(M) > 1 - eps) <> [])
    M_eps = CL_rot_quat2matrix(CL_rot_axAng2quat([1; 1; 1], eps)); 
    M = M_eps * M; 
  end 

  // coordinates of vertices in object frame 
  S = M' * S + repmat(C, 1, size(S, 2)); 

  // sides (numbers of the 4 vertices) 
  numf = [ [1; 2; 3; 4] ]; 

  x = matrix(S(1,numf), 4, -1); 
  y = matrix(S(2,numf), 4, -1); 
  z = matrix(S(3,numf), 4, -1); 

  cols = color("grey70"); 

  plot3d(x, y, list(z, cols)); 
  e = gce(); 
  e.hiddencolor = color("grey30"); 

  bounds = [min(x), min(y), min(z); max(x), max(y), max(z)]; 

endfunction 

// -----------------------------------------------
// Plot an object (made of cuboids and rectangles) 
// obj: object to plot  
// M: transf matrix from plot frame to object frame (3x3). 
// Default = Identity (plot frame = object frame) 
// -----------------------------------------------
function plot_spacecraft(obj, M)
  if (argn(2) == 1)
    M = eye(3,3); 
  end 
  
  // For each object: 
  // Msub : transformation matrix from object frame to subpart frame 
  // C = center of subpart in object frame. 
  // Msub and C transform so that the plot is in the specified frame
  //              M                Msub
  // (plot frame) -> object frame  ->  subpart frame  

  bounds = [%inf, %inf, %inf; -%inf, -%inf, -%inf];   
  for k = 1 : size(obj.id, "*")
    Msub = CL_rot_defFrameVec(obj.z_axis(:,k), obj.x_axis(:,k), 3, 1); 
    C = obj.orig(:,k); 
    
    if (obj.shap(k) == "Cuboid")
      bounds_sub = _plot_cuboid(obj.dim(:,k), M' * C, Msub * M);
      
    elseif (obj.shap(k) == "Rectangle")
      bounds_sub = _plot_rectangle(obj.dim(:,k), M' * C, Msub * M);
      
    else
      error("Unhandled shape: " + obj.shap(k)); 
    end
    
    bounds = [min(bounds(1,:), bounds_sub(1,:)); max(bounds(2,:), bounds_sub(2,:))];     
  end 

  a = gca(); 
  a.isoview = "on"; 
  
  center = mean(bounds, "r"); 
  L = max(diff(bounds, 1, "r")); 
  a.data_bounds = [center - 1.05 * L/2; center + 1.05 * L/2];  
  
endfunction 



