// ==========================================================
// Definition of solar system data :
// Equatorial radius, oblateness and mu for solar system bodies
//
// eqRad: Equatorial radius (m)
// obla: Oblateness
// mu: Gravitational constant (m^3/s^2)
//
// Note:
// The gravitational constants for Mars, Jupiter, Saturn, Uranus, Neptune and Pluto
// are the gravitational constant of the body and its natural satellites.
//
// Bibliography
// 1) The shape of bodies can be found in the following document:
// Report of the IAU Working Group on Cartographic Coordinates and Rotational Elements: 2009
// (Chapter 7, Table 4 and 5)
//
// 2) The gravitational constants can be found in the following document:
// The IAU 2009 system of astronomical constants: the report of the IAU working
// group on numerical standards for Fundamental Astronomy
// (Chapter 2, Table 1)
//
// ==========================================================


// Sun - IAU 2009
data.body.Sun.eqRad = 696000e3
polRad = 696000e3
data.body.Sun.obla = 1 - polRad / data.body.Sun.eqRad
data.body.Sun.mu = 1.32712440041e20    // TDB compatible
// Other values for info:
// data.body.Sun.mu = 1.32712442099e20 // TCB compatible

// Mercury - IAU 2009
data.body.Mercury.eqRad = 2439.7e3
polRad = 2439.7e3
data.body.Mercury.obla = 1 - polRad / data.body.Mercury.eqRad
data.body.Mercury.mu = (1/6.0236e6) * data.body.Sun.mu

// Venus - IAU 2009
data.body.Venus.eqRad = 6051.8e3
polRad = 6051.8e3
data.body.Venus.obla = 1 - polRad / data.body.Venus.eqRad
data.body.Venus.mu = (1/4.08523719e5) * data.body.Sun.mu

// Earth - Same values as in physical_data.scd for consistency
data.body.Earth.eqRad = 6378.1363e3
data.body.Earth.obla = 1/298.257223563
data.body.Earth.mu = 3.986004415e14  // same value as IAU 2009 - TT compatible
// Other values for info:
// data.body.Earth.mu = 3.986004418e14  // IAU 2009 - TCB compatible
// data.body.Earth.mu = 3.986004356e14  // IAU 2009 - TDB compatible

// Moon - IAU 2009
data.body.Moon.eqRad = 1737.4e3
data.body.Moon.obla = 0
data.body.Moon.mu = 1.23000371e-2 * data.body.Earth.mu

// Mars - IAU 2009
data.body.Mars.eqRad = 3396.19e3
polRad = 3376.2e3 // Average polar radius (N 3373.19e3 - S 3379.21e3)
data.body.Mars.obla = 1 - polRad / data.body.Mars.eqRad
data.body.Mars.mu = (1/3.09870359e6) * data.body.Sun.mu

// Phobos - Source = https://ssd.jpl.nasa.gov/?sat_phys_par
// Note: Represented here as "average" sphere
// Actual size ~ 26.8 × 22.4 × 18.4 km 
data.body.Phobos.eqRad = 11.1e3 // mean radius 
data.body.Phobos.obla = 0 
data.body.Phobos.mu = 0.0007112e9 // ±0.0000010e9 

// Deimos - Source = https://ssd.jpl.nasa.gov/?sat_phys_par
// Note: Represented here as "average" sphere
// Actual size ~ 15.0 × 12 × 10.4 km 
data.body.Deimos.eqRad = 6.2e3; 
data.body.Deimos.obla = 0 
data.body.Deimos.mu = 0.0000985e9 // ±0.0000024e9 

// Jupiter - IAU 2009
data.body.Jupiter.eqRad = 71492e3
polRad = 66854e3
data.body.Jupiter.obla = 1 - polRad / data.body.Jupiter.eqRad
data.body.Jupiter.mu = (1/1.047348644e3) * data.body.Sun.mu

// Saturn - IAU 2009
data.body.Saturn.eqRad = 60268e3
polRad = 54364e3
data.body.Saturn.obla = 1 - polRad / data.body.Saturn.eqRad
data.body.Saturn.mu = (1/3.4979018e3) * data.body.Sun.mu

// Uranus - IAU 2009
data.body.Uranus.eqRad = 25559e3
polRad = 24973e3
data.body.Uranus.obla = 1 - polRad / data.body.Uranus.eqRad
data.body.Uranus.mu = (1/2.290298e4) * data.body.Sun.mu

// Neptune - IAU 2009
data.body.Neptune.eqRad = 24764e3
polRad = 24341e3
data.body.Neptune.obla = 1 - polRad / data.body.Neptune.eqRad
data.body.Neptune.mu = (1/1.941226e4) * data.body.Sun.mu

// Pluto - IAU 2009
data.body.Pluto.eqRad = 1195e3
polRad = 1195e3
data.body.Pluto.obla = 1 - polRad / data.body.Pluto.eqRad
data.body.Pluto.mu = (1/1.36566e8) * data.body.Sun.mu


// ==========================================================
// Orientation of celestial bodies
// Data from IAU 2009
// Reference : Report of the IAU/IAG working group on cartographic coordinates 
// and rotational elements: 2009
//
// Syntax:
// data.body.<body_name>.orient = <structure>
//
// Each structure has the following fields :
// - coefPol : characteristic angles polynomial coefficients if any (PxN) 
//   each row defines the polynomial coefficients of one particular angle: degree0, degree1, ...
//   N is the degree of the polynomials + 1
//   P is the number of angles alpha, delta and wrot depend on (see below). 
//
// For X = alpha, delta, wrot:
// - X.coefPol : polynomial part (1xM) : degree 0, 1, ... 
// - X.coefCos : coefficients of cos(angles) if any (1xP) 
// - X.coefSin : coefficients of sin(angles) if any (1xP) 
//
// alpha and delta are the right ascension and declination of the pole axis in ICRF. 
// wrot is the rotation angle.
//
// The arrays: "coefPol", "X.coefPol", "X.coefCos", "X.coefSin"  may be empty or absent. 
//
// Units are radians, radians/days, radians/days^2, ...
//
// Example (Mercury):
// d = struct();
// d.coefPol =  [ 174.791086,  4.092335;
//                349.582171,  8.184670;
//                164.373257, 12.277005;
//                339.164343, 16.369340;
//                153.955429, 20.461675] * DEG2RAD;
// ...
//
// d.wrot.coefPol = [329.5469, 6.1385025] * DEG2RAD;
// d.wrot.coefSin = [0.00993822, -0.00104581, -0.00010280, -0.00002364, -0.00000532] * DEG2RAD;
//
// wrot is computed at time t by:
// wrot = 329.5469 * DEG2RAD + 6.1385025 * DEG2RAD * t
//     + 0.00993822 * DEG2RAD * sin(174.791086 * DEG2RAD + 4.092335 * DEG2RAD * t)
//     - 0.00104581 * DEG2RAD * sin(349.582171 * DEG2RAD + 8.184670 * DEG2RAD * t)
//     + ...
//
// where t in the number of days since J2000 epoch.
//
// ==========================================================
// Local constants : 
// - CEN: Number of Julian days in a Julian century 
// - DEG2RAD: Degrees to radians conversion coefficient
CEN = 36525;
DEG2RAD = %pi / 180;

// ---------------------------
// Sun
// ---------------------------
d = struct();

d.alpha.coefPol = [286.13] * DEG2RAD;
d.delta.coefPol = [63.87] * DEG2RAD;
d.wrot.coefPol = [84.176, 14.1844000] * DEG2RAD;

data.body.Sun.orient = d;

// ---------------------------
// Mercury
// ---------------------------
d = struct();

d.coefPol = [ 174.791086,  4.092335;
              349.582171,  8.184670;
              164.373257, 12.277005;
              339.164343, 16.369340;
              153.955429, 20.461675 ] * DEG2RAD;

d.alpha.coefPol = [281.0097, -0.0328 / CEN] * DEG2RAD;
d.delta.coefPol = [61.4143, -0.0049 / CEN] * DEG2RAD;
d.wrot.coefPol = [329.5469, 6.1385025] * DEG2RAD;
d.wrot.coefSin = [0.00993822, -0.00104581, -0.00010280, -0.00002364, -0.00000532] * DEG2RAD;

data.body.Mercury.orient = d;

// ---------------------------
// Venus
// ---------------------------
d = struct();

d.alpha.coefPol = [272.76] * DEG2RAD;
d.delta.coefPol = [67.16] * DEG2RAD;
d.wrot.coefPol = [160.20, -1.4813688] * DEG2RAD;

data.body.Venus.orient = d;

// ---------------------------
// Earth
// ---------------------------
d = struct();

d.alpha.coefPol = [0, -0.641 / CEN] * DEG2RAD;
d.delta.coefPol = [90.00, -0.557 / CEN] * DEG2RAD;
d.wrot.coefPol = [190.147, 360.9856235] * DEG2RAD;

data.body.Earth.orient = d;

// ---------------------------
// Moon
// ---------------------------
d = struct();

d.coefPol = [ 125.045,  -0.0529921;
              250.089,  -0.1059842;
              260.008,  13.0120009;
              176.625,  13.3407154;
              357.529,   0.9856003;
              311.589,  26.4057084;
              134.963,  13.0649930;
              276.617,   0.3287146;
               34.226,   1.7484877;
               15.134,  -0.1589763;
              119.743,   0.0036096;
              239.961,   0.1643573;
               25.053,  12.9590088 ] * DEG2RAD;

d.alpha.coefPol = [269.9949,  0.0031 / CEN] * DEG2RAD;
d.alpha.coefSin = [-3.8787, -0.1204, 0.0700, -0.0172, 0, 0.0072, 0, 0, 0, -0.0052, 0, 0, 0.0043] * DEG2RAD;
d.delta.coefPol = [66.5392, 0.0130 / CEN] * DEG2RAD;
d.delta.coefCos = [1.5419, 0.0239, -0.0278, 0.0068, 0, -0.0029, 0.0009, 0, 0, 0.0008, 0, 0, -0.0009] * DEG2RAD;
d.wrot.coefPol = [38.3213, 13.17635815, -1.4e-12] * DEG2RAD;
d.wrot.coefSin = [3.5610, 0.1208, -0.0642, 0.0158, 0.0252, -0.0066, -0.0047, -0.0046, ...
                0.0028, 0.0052, 0.0040, 0.0019, -0.0044] * DEG2RAD;

data.body.Moon.orient = d;

// ---------------------------
// Mars
// ---------------------------
d = struct();

d.alpha.coefPol = [317.68143, -0.1061 / CEN] * DEG2RAD;
d.delta.coefPol = [52.88650, -0.0609 / CEN] * DEG2RAD;
d.wrot.coefPol = [176.630, 350.89198226] * DEG2RAD;

data.body.Mars.orient = d;

// ---------------------------
// Phobos
// ---------------------------
d = struct();

d.coefPol = [ 169.51, -0.4357640, 0;
              192.93, 1128.4096700, 8.864 / CEN^2] * DEG2RAD;

d.alpha.coefPol = [317.68, -0.108 / CEN] * DEG2RAD;
d.alpha.coefSin = [1.79, 0] * DEG2RAD;
d.delta.coefPol = [52.90, -0.061 / CEN] * DEG2RAD;
d.delta.coefCos = [-1.08, 0] * DEG2RAD;
d.wrot.coefPol = [35.06, 1128.8445850, 8.864 / CEN^2] * DEG2RAD;
d.wrot.coefSin = [-1.42, -0.78] * DEG2RAD;

data.body.Phobos.orient = d;

// ---------------------------
// Deimos
// ---------------------------
d = struct();

d.coefPol =  [53.47, -0.0181510] * DEG2RAD;

d.alpha.coefPol = [316.65, -0.108 / CEN] * DEG2RAD;
d.alpha.coefSin = [2.98] * DEG2RAD;
d.delta.coefPol = [53.52, -0.061 / CEN] * DEG2RAD;
d.delta.coefCos = [-1.78] * DEG2RAD;
d.wrot.coefPol = [79.41, 285.1618970, -0.520 / CEN^2] * DEG2RAD;
d.wrot.coefCos = [0.19] * DEG2RAD;
d.wrot.coefSin = [-2.58] * DEG2RAD;

data.body.Deimos.orient = d;

// ---------------------------
// Jupiter
// ---------------------------
d = struct();

d.coefPol = [  99.360714, 4850.4046 / CEN;
              175.895369, 1191.9605 / CEN;
              300.323162,  262.5475 / CEN;
              114.012305, 6070.2476 / CEN;
               49.511251,   64.3000 / CEN ] * DEG2RAD;

d.alpha.coefPol = [268.056595, -0.006499 / CEN] * DEG2RAD;
d.alpha.coefSin = [0.000117, 0.000938, 0.001432, 0.000030, 0.002150] * DEG2RAD;
d.delta.coefPol = [64.495303, 0.002413 / CEN] * DEG2RAD;
d.delta.coefCos = [0.000050, 0.000404, 0.000617, -0.000013, 0.000926] * DEG2RAD;
d.wrot.coefPol = [284.95, 870.536] * DEG2RAD;

data.body.Jupiter.orient = d;

// ---------------------------
// Saturn
// ---------------------------
d = struct();

d.alpha.coefPol = [40.589, -0.036 / CEN] * DEG2RAD;
d.delta.coefPol = [83.537,  -0.004 / CEN] * DEG2RAD;
d.wrot.coefPol = [38.90, 810.7939024] * DEG2RAD;

data.body.Saturn.orient = d;

// ---------------------------
// Uranus
// ---------------------------
d = struct();

d.alpha.coefPol = [257.311] * DEG2RAD;
d.delta.coefPol = [-15.175] * DEG2RAD;
d.wrot.coefPol = [203.81, -501.1600928] * DEG2RAD;

data.body.Uranus.orient = d;

// ---------------------------
// Neptune
// ---------------------------
d = struct();

d.coefPol = [357.85, 52.316 / CEN] * DEG2RAD;

d.alpha.coefPol = [299.36] * DEG2RAD;
d.alpha.coefSin = [0.70] * DEG2RAD;
d.delta.coefPol = [43.46] * DEG2RAD;
d.delta.coefCos = [-0.51] * DEG2RAD;
d.wrot.coefPol = [253.18, 536.3128492] * DEG2RAD;
d.wrot.coefSin = [-0.48] * DEG2RAD;

data.body.Neptune.orient = d;



// ==========================================================
// Definition of constants for the 3-body problem (see CL_3b_environment):
// mrat: mass ratio (lightest body / heaviest),
// dist: distance between the 2 bodies (m)
// omega: rotation rate around the center of mass od the 2 bodies (rad/s),
//
// Reference for values : unknown
// ==========================================================

// Sun-Earth/Moon system
data.threebody.Sun_EarthMoon.mrat = 0.000003040357143;
data.threebody.Sun_EarthMoon.dist = 149597871410;  // 1 AU
data.threebody.Sun_EarthMoon.omega = 0.0000001990986687;

// Earth-Moon system
data.threebody.Earth_Moon.mrat = 0.012150668;
data.threebody.Earth_Moon.dist = 384399060;
data.threebody.Earth_Moon.omega = 0.0000026617;

// Sun-Jupiter system
data.threebody.Sun_Jupiter.mrat = 0.00095368390550742;
data.threebody.Sun_Jupiter.dist = 149597871410 * 5.2;  // 5.2 AU
data.threebody.Sun_Jupiter.omega = 0.00145029924 / 86400;




