//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> N-body choreographies.  
//> 
//> The term "choreography" was coined by Carles Simo (http://www.maia.ub.es/dsg/nbody.html) and 
//> describes the fact that N bodies move on the same trajectory. 
//> 
//> In the simulation, the gravitational constant G is equal to 1, and all masses are equal to 1/3. 
//> The trajectories are computed over 1 period only. 
//> 
//> Note: 
//> The data (initial conditions) come from: 
//> http://analyticphysics.com/Three-Body%20Problem/Sim%C3%B3's%20Three-Body%20Choreographies%20in%20Action.htm
//
// Author: A. Lamy
// -----------------------------------------------------------

numchor = 1; 
nmaxper = 1; 

desc = list(..
  CL_defParam("Choreography number (1 or 2)", numchor, accv=[1, 2]), ..
  CL_defParam("Number of periods: (1..5)", nmaxper, accv=[1, 2, 3, 4, 5]) ..
);

[numchor, nmaxper] = CL_inputParam(desc);

// -----------------------------------------------------
// Utilities
// -----------------------------------------------------
// State evolution: dy/dt = f(t, y). 
// y = [pos; vel] (state vector)
// t in seconds
// model = central force
function [ydot] = fct(t, y)
  X1 = y(1:2); 
  X2 = y(3:4); 
  X3 = y(5:6); 
  
  X12 = X2 - X1; 
  X13 = X3 - X1; 
  X23 = X3 - X2; 
  
  X12n3 = sqrt(X12(1)^2 + X12(2)^2)^3; 
  X13n3 = sqrt(X13(1)^2 + X13(2)^2)^3; 
  X23n3 = sqrt(X23(1)^2 + X23(2)^2)^3; 
  
  a1 = X12 / X12n3 + X13 / X13n3; 
  a2 = -X12 / X12n3 + X23 / X23n3; 
  a3 = -X13 / X13n3 - X23 / X23n3; 
  
  ydot = zeros(y); 
  ydot(1:6) = y(7:12); 
  ydot(7:12) = [a1; a2; a3] / 3; 
  
endfunction

// -----------------------------------------------------
// Initializations, integration
// -----------------------------------------------------
// initial state  

function [t, y] = gen_trajectories(numchor)
  c = [
    0.1546025015193900 -0.0987561640960160 -1.1843704912618038  0.2521819944674475  0.5587063079197599;
    0.1540289328762600 -0.0932474525425240  0.9635026027520550  0.5076890556787989  0.8985961737399401
  ]; 

  c1 = c(numchor,1);
  c2 = c(numchor,2); 
  c3 = c(numchor,3);   
  c4 = c(numchor,4); 
  c5 = c(numchor,5);  

  y0 = [-2 * c1; 0; c1; c2; c1; -c2; 0; -2 * c4; c3; c4; -c3; c4]; 

  // period
  T = c5 * 3; 

  t0 = 0;  
  t = (0 : 0.01 : T);

  // integration
  rtol = 1.e-12 * ones(12, 1);
  atol = 1.e-12 * ones(12, 1); 

  y = ode(y0, t0, t, rtol, atol, fct);
endfunction 

// -----------------------------------------------------
// Computation
// -----------------------------------------------------

[t, y] = gen_trajectories(numchor); 

// -----------------------------------------------------
// Plot (animation)
// -----------------------------------------------------
function [db] = calcul_db(y, k)
  allX = y(1:2:6,1:k); 
  allY = y(2:2:6,1:k); 
  dim = max(max(allX) - min(allX), max(allY) - min(allY)) + 0.05;  
  cenX = (max(allX) + min(allX))/2;
  cenY = (max(allY) + min(allY))/2;
  db = [cenX - dim/2, cenY - dim/2; cenX + dim/2, cenY + dim/2]; 
endfunction


f = scf(); 
num = f.figure_id; 

f.immediate_drawing = "off"; 
plot(y(1,:), y(2,:), "k"); 
h = CL_g_select(gce(), "Polyline");
h.foreground = color("grey40"); 
plot(y(3,:), y(4,:), "k"); g1 = CL_g_select(gce(), "Polyline");
h = CL_g_select(gce(), "Polyline");
h.foreground = color("grey40"); 
plot(y(5,:), y(6,:), "k"); g1 = CL_g_select(gce(), "Polyline");
h = CL_g_select(gce(), "Polyline");
h.foreground = color("grey40"); 

a = gca(); 

a.data_bounds = calcul_db(y, length(t));  
a.tight_limits = "on"; 
a.isoview = "on"; 
a.title.text = msprintf("Choreography #%d", numchor); 

CL_g_stdaxes(); 
f.visible = "on"; 
f.immediate_drawing = "off"; 

k = 1; 
nper = 0; 

while(is_handle_valid(f))    
  h = []; 
  try 
  plot(y(1,k), y(2,k), "bo"); h = [h, gce()]; h1 = CL_g_select(h($), "Polyline"); 
  plot(y(3,k), y(4,k), "ro"); h = [h, gce()]; h2 = CL_g_select(h($), "Polyline"); 
  plot(y(5,k), y(6,k), "go"); h = [h, gce()]; h3 = CL_g_select(h($), "Polyline");

  h1.mark_background = color("blue"); 
  h2.mark_background = color("red"); 
  h3.mark_background = color("green"); 
  h1.mark_size = 10;  
  h2.mark_size = 10; 
  h3.mark_size = 10;  
  
  f.immediate_drawing = "on"; 
  sleep(20); 
  f.immediate_drawing = "off"; 
    
  delete(h); 
  end
  k = k + 1; 
  if (k > length(t)); k = 1; nper = nper + 1; end
  if (nper >= nmaxper); break; end; 
end 








