//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Two dimensional N-body simulation.  
//> 
//> 3 bodies are placed in a particular configuration, and their
//> trajectories are propagated numerically. 
//> The configuration appears to be very unstable. 
//> 
//> In the simulation the gravitational constant G is 1, and all masses are equal to 1/3. 
//> Some random noise is added to the initial state vectors so that the outcome 
//> is never the same. 
//>
//> Note that it may happen that messages from the propagator (ode) appears. It is due to 
//> inter-body distances that are too small. 
//
// Author: A. Lamy
// -----------------------------------------------------------

duration = 15; 
noise = 1.e-7; 

desc = list(..
  CL_defParam("Simulation duration (between 5 and 100)", duration, valid = "$x >= 5 & $x <= 100"), ..
  CL_defParam("Max amplitude of noise (between 0 and 0.1)", noise, valid = "$x >= 0 & $x <= 0.1") ..
);

[duration, noise] = CL_inputParam(desc);


// -----------------------------------------------------
// Utilities
// -----------------------------------------------------
// State evolution: dy/dt = f(t, y). 
// y = [x1; y1; x2; y2; x3; y3; vx1; vy1; vx2; vy2; vx3; vy3] 
// G = 1; masses = 1/3 
function [ydot] = fct(t, y)
  X1 = y(1:2); 
  X2 = y(3:4); 
  X3 = y(5:6); 
  
  X12 = X2 - X1; 
  X13 = X3 - X1; 
  X23 = X3 - X2; 
  
  X12n3 = sqrt(X12(1)^2 + X12(2)^2)^3; 
  X13n3 = sqrt(X13(1)^2 + X13(2)^2)^3; 
  X23n3 = sqrt(X23(1)^2 + X23(2)^2)^3; 
  
  a1 = X12 / X12n3 + X13 / X13n3; 
  a2 = -X12 / X12n3 + X23 / X23n3; 
  a3 = -X13 / X13n3 - X23 / X23n3; 
  
  ydot = zeros(y); 
  ydot(1:6) = y(7:12); 
  ydot(7:12) = [a1; a2; a3] / 3; 
  
endfunction

// -----------------------------------------------------
// Trajectories 
// -----------------------------------------------------
function [t, y] = gen_trajectories(duration, noise)
  r = 0.4; 
  v = 0.4; 

  y0 = zeros(12, 1); 

  for k = 1 : 3
    y0(2*(k-1)+1) = r * cos((2*%pi/3) * k); 
    y0(2*(k-1)+2) = r * sin((2*%pi/3) * k); 
    y0(2*(k-1)+1+6) = v * cos((2*%pi/3) * k + %pi/2); 
    y0(2*(k-1)+2+6) = v * sin((2*%pi/3) * k + %pi/2); 
  end 

  t0 = 0;  

  OK = %f; 

  // ode options: default values except index 1: computation at mesh points
  // (ode output contains [t, y] 
  %ODEOPTIONS = [1,0,0,%inf,0,2,500,12,5,0,-1,-1]; 
  %ODEOPTIONS(1) = 2; 

  while (~OK)
   
    dy0 = noise * grand(12, 1, "unf", -1, 1); 
    y0 = y0 + dy0;  
   
    // integration
    rtol = 1.e-12 * ones(12, 1);
    atol = 1.e-10 * ones(12, 1); 

    try
      [z] = ode(y0, t0, duration, rtol, atol, fct);
      OK = %t; 
    catch
      mprintf("*** Propagation failed => retry\n"); 
      OK = %f;
    end

  end
  
  t = z(1,:); 
  y = z(2:$,:); 

endfunction

// -----------------------------------------------------
// Add times    
// K: indices of added times in ordered vectors 
// -----------------------------------------------------
function [t, y, K] = add_times(t, y, tadd) 
  yadd = CL_interpLagrange(t, y, tadd); 

  T = [t; 0 * ones(t)]; 
  Tadd = [tadd; 1 * ones(tadd)]; 

  T = [T, Tadd]; 
  y = [y, yadd]; 

  [T, I] = CL_sortMat(T, T(1,:));  

  t = T(1,:); 
  y = y(:,I); 
  K = find(T(2,:) == 1); 
endfunction 


// -----------------------------------------------------
// Computation
// -----------------------------------------------------
[t, y] = gen_trajectories(duration, noise); 
tadd = 0 : 0.05 : t($); 
[t, y, Kstep] = add_times(t, y, tadd); 


// -----------------------------------------------------
// Plot (animation)
// -----------------------------------------------------
// adjust data_bounds 
function [db] = calcul_db(y, K)
  allX = y(1:2:6,K); 
  allY = y(2:2:6,K); 
  dimx = (max(allX) - min(allX)) * 1.1;
  dimy = (max(allY) - min(allY)) * 1.1; 
  cenX = (max(allX) + min(allX))/2;
  cenY = (max(allY) + min(allY))/2;
  dimx = max(dimx, dimy / 0.75); 
  dimy = max(dimy, dimx * 0.75); 
  db = [cenX - dimx/2, cenY - dimy/2; cenX + dimx/2, cenY + dimy/2]; 
endfunction


f = scf(); 
num = f.figure_id; 

a = gca(); 

// new axes for the time string (h_strtime)
b = newaxes(); 

// axes are not filled so that they can be superimposed
a.filled = "off"; 
b.filled = "off"; 
b.margins = [0, 0, 0, 0]; 

db = calcul_db(y, 1:1500); 
a.data_bounds = db;  
a.tight_limits = "on"; 
a.isoview = "on"; 
a.title.text = "3-body simulation"; 

CL_g_stdaxes(a); 
a.sub_ticks = [0,0]; 

sca(b); 
xstring(0.42, 0.005, ""); 
h_strtime = gce(); 
h_strtime.font_size = 3; 
h_strtime.font_foreground = color("brown"); 


f.visible = "on"; 
f.immediate_drawing = "off"; 
sca(a); 

i = 1; 
for (k = Kstep)
  if (~is_handle_valid(f)); break; end    
  K = find(t <= t(k) & t >= t(k) - 2);  
  
  h_strtime.text = msprintf("Time: %.1f", t(k)); 

  try 
  plot(y(1,K), y(2,K), "k"); 
  h = CL_g_select(gce(), "Polyline");
  h.foreground = color("blue"); 
  plot(y(3,K), y(4,K), "k"); 
  h = CL_g_select(gce(), "Polyline");
  h.foreground = color("red"); 
  plot(y(5,K), y(6,K), "k"); 
  h = CL_g_select(gce(), "Polyline");
  h.foreground = color("green"); 

  plot(y(1,k), y(2,k), "bo"); 
  h1 = CL_g_select(gce(), "Polyline"); 
  plot(y(3,k), y(4,k), "ro"); 
  h2 = CL_g_select(gce(), "Polyline"); 
  plot(y(5,k), y(6,k), "go"); 
  h3 = CL_g_select(gce(), "Polyline");

  h1.mark_background = color("blue"); 
  h2.mark_background = color("red"); 
  h3.mark_background = color("green"); 
  h1.mark_size = 10;  
  h2.mark_size = 10; 
  h3.mark_size = 10;  
 
  db = calcul_db(y, K); 
  a.data_bounds = db;  
  a.sub_ticks = [0,0]; 
 
  f.immediate_drawing = "on"; 
  sleep(30); 
  f.immediate_drawing = "off"; 
  
  delete(a.children); 

  end
end 








