//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Simple simulation of yaw steering. 
//> 
//> The nominal spacecraft frame (without yaw steering) is supposed to be the "qsw" local 
//> orbital frame. 
//> In order to keep the Z axis of the spacecraft frame (axis which is also 
//> the rotation axis of the solar array) perpendicular to the Sun direction, a rotation 
//> around X is performed. 
//> The attitude of the spacecraft is then obtained from "qsw" by a single rotation around X. 
//> 
//> Over one orbit period, the Sun direction describes a cone of revolution whose axis is the 
//> orbit's angular momentum vector (Z axis of "qsw" frame). 
//> The angle of the cone is pi/2 - beta, where beta is the Sun beta angle = angle between
//> the Sun and the orbit plane.  
//> 
//> Two solutions (that is two rotations) exist. 
//> By convention, solution 1 is such that the rotation angle is minimum. 
//> If the selected solution is such that the angle between the Sun direction and the Y axis is more 
//> than 90 deg at each time (Y is then the "cold" side), the rotation angle around X is a 
//> continuous function of time except when beta crosses 0: a "yaw flip" is then necessary. 
//>
//> Note: X-axis in figures = Sun azimuth = angle between the projection of the Sun in the orbit plane 
//> and the X axis of the "qsw" frame. 
//>  
// Author: A. Lamy
// -----------------------------------------------------------

// Inputs  
betaa = 30 * %CL_deg2rad; 

desc_param = list(..
   CL_defParam("Sun beta angle", betaa, units = ["rad", "deg"], valid='$x >= -90 & $x <= 90') ...
);

[betaa] = CL_inputParam(desc_param);

// -----------------------------------------------
// Computations
// -----------------------------------------------
// d = Sun direction 
angsun = linspace(0, 2*%pi, 720);
sph = [angsun; betaa * ones(angsun); ones(angsun)];  
d = CL_co_sph2car(sph); 

// axis to be kept perpendicular to the Sun
u = [0;0;1]; 

// rotation axis
axis = [1;0;0]; 

[qa,anga,qb,angb,Inok] = CL_rot_defRot1Ax(u, axis, d, %pi/2, numsol=2); 

Ya = CL_rot_rotVect(qa, [0;1;0]); 
Yb = CL_rot_rotVect(qb, [0;1;0]); 

angya = CL_vectAngle(Ya, d); 
angyb = CL_vectAngle(Yb, d); 

// Mixed solution: such that angyb >= angya
angc = anga; 
angyc = angya;
I = find(angyb >= angya); 
angc(I) = angb(I); 
angyc(I) = angyb(I); 


// -----------------------------------------------
// plots
// -----------------------------------------------
function [e] = Plot(x, y, style, thickness)
  plot(x, y, style, "thickness", thickness); 
  e = gce(); 
  if (part(style, 2) == ":")
    h = CL_g_select(e, "Polyline"); 
    h.line_style = 8; 
  end
endfunction 

// text for titles 
txt = msprintf("beta = %.1f deg", betaa * %CL_rad2deg); 

// Rotation angles
f = scf(); 
f.visible="on";
f.immediate_drawing="off";

hc = Plot(angsun * %CL_rad2deg, angc * %CL_rad2deg, "g", 4); 
ha = Plot(angsun * %CL_rad2deg, anga * %CL_rad2deg, "b:", 3); 
hb = Plot(angsun * %CL_rad2deg, angb * %CL_rad2deg, "r:", 3); 

a = gca(); 
a.title.text = "Yaw steering: rotation angle around +X - " + txt;
a.x_label.text = "Sun azimuth (deg)";
a.y_label.text = "Angle (deg)";

CL_g_legend([ha, hb, hc], ["Solution 1", "Solution 2", "+Y = cold side"]); 
CL_g_stdaxes(); 

f.immediate_drawing="on";
f.visible="on";


// Sun-Y angles
f = scf(); 
f.visible="on";
f.immediate_drawing="off";

hc = Plot(angsun * %CL_rad2deg, angyc * %CL_rad2deg, "g", 4); 
ha = Plot(angsun * %CL_rad2deg, angya * %CL_rad2deg, "b:", 3); 
hb = Plot(angsun * %CL_rad2deg, angyb * %CL_rad2deg, "r:", 3); 

a = gca(); 
a.title.text = "Yaw steering: angle between Sun and +Y - " + txt;
a.x_label.text = "Sun azimuth (deg)";
a.y_label.text = "Angle (deg)";

CL_g_legend([ha, hb, hc], ["Solution 1", "Solution 2", "+Y = cold side"]); 
CL_g_stdaxes(); 

f.immediate_drawing="on";
f.visible="on";

 