//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Sphere of influence of celestial body
//> 
//> Plots the region of influence of the lightest body in a 2-body system. 
//> 
//> The region of influence of body 1 (lightest body) is defined by: 
//> r = (||acc_pert2/1|| / ||acc_central1||) / (||acc_pert1/2|| / ||acc_central2||) <= 1
//> Where: 
//> acc_central1 is the newtonion acceleration due to body 1 
//> acc_central2 is the newtonion acceleration due to body 2
//> acc_pert2/1 is the perturbing acceleration due to body 2 if body1 is the central body
//> acc_pert1/2 is the perturbing acceleration due to body 1 if body2 is the central body
//> 
//> The black "1" contour defines the outlines of the region of influence.  
//> The white-pink dotted line delimits the spherical region as computed by the 
//> "CL_ip_sphereInfluence" function: Rsoi = D * mass_ratio^(2/5) 
//> 
//> Notes: 
//> - The X axis is the line that goes through the 2 bodies. The positive direction is 
//> from the heaviest body to the lightest body. 
//> - The lightest contours corresponds to levels of: ..., 0.2, 0.3, 0.4..., 2, 3, 4,... 
//
// Author: A. Lamy
// -----------------------------------------------------------

bodies = 2;      
mass_ratio = 0.0;   

desc_param = list(..
   CL_defParam("Bodies: 1=Venus/Sun, 2=Earth/Sun, 3=Mars/Sun, 4=Moon/Earth, 0=user-defined", bodies, id="$bodies", accv=[1,2,3,4,0]),...
   CL_defParam("Mass ratio (if user-defined option selected) - lightest / heaviest ", mass_ratio, valid='$bodies <> 0 | ($x > 0 & $x <= 1)')...
);

[bodies, mass_ratio] = CL_inputParam(desc_param);


// -----------------
// Intermediate computations: 
// * mu_b1, mu_b2: grav constants of body 1 and 2
// * D: distance between bodies
// * UNIT: units on x/y in graph
// * info: information for titles...  
// -----------------

MU_SUN = CL_dataGet("body.Sun.mu"); 
MU_VENUS = CL_dataGet("body.Venus.mu"); 
MU_EARTH = CL_dataGet("body.Earth.mu"); 
MU_MARS = CL_dataGet("body.Mars.mu"); 
MU_MOON = CL_dataGet("body.Moon.mu"); 
AU = CL_dataGet("au"); 
// dates for computation os mean distance - if necessary (1 century, every day) 
cjd = CL_dat_cal2cjd(2000,1,1) : CL_dat_cal2cjd(2100,1,1); 

if (bodies == 1)
  mu_b1 = MU_VENUS; 
  mu_b2 = MU_SUN; 
  // average distance from Venus to Sun
  D = mean(CL_norm(CL_eph_planet("Venus", cjd))); 
  UNIT = 1.e9; 
  info = ["Venus / Sun", "", "10^6 km", "Venus"]; 

elseif (bodies == 2)
  mu_b1 = MU_EARTH; 
  mu_b2 = MU_SUN; 
  // average distance from Earth to Sun
  D = 1 * AU; 
  UNIT = 1.e9; 
  info = ["Earth / Sun", "", "10^6 km", "Earth"]; 

elseif (bodies == 3)
  mu_b1 = MU_MARS; 
  mu_b2 = MU_SUN;
  // average distance from Mars to Sun
  D = mean(CL_norm(CL_eph_planet("Mars", cjd))); 
  UNIT = 1.e9; 
  info = ["Mars / Sun", "", "10^6 km", "Mars"]; 
    
elseif (bodies == 4)
  mu_b1 = MU_MOON; 
  mu_b2 = MU_EARTH;
  // average distance from Moon to Earth
  D = mean(CL_norm(CL_eph_moon(cjd))); 
  UNIT = 1.e6; 
  info = ["Moon / Earth", "", "10^3 km", "Moon"]; 
  
else
  mu_b1 = mass_ratio;
  mu_b2 = 1; 
  D = 1; 
  UNIT = D; 
  info = ["", "Mass ratio = " + msprintf("%.6g", mass_ratio), "unit = distance between bodies", "Lightest body"]; 
end

// -----------------
// Display info
// -----------------

mprintf("\n---------------------------\n"); 
mprintf("Bodies = %s\n", info(1)); 
mprintf("Mass ratio = %.2e\n", mu_b1 / mu_b2); 
mprintf("Average distance (" + info(3) + ") = %.1f\n", D / UNIT); 


// -----------------
// Computation
// -----------------

// radius of sphere of influence computed by approximate formula
rsoi = CL_ip_sphereInfluence(mu_b1, mu_b2, D); 

L = rsoi * 1.3; 
x = linspace(-1, 1, 100) * L;  // even number of values to avoid center
y = linspace(-1, 1, 100) * L;  // even number => avoids y = 0

[X, Y] = ndgrid(x, y); 
sz = size(X); 
N = size(X, "*"); 

// positions of : object, body1 (lightest), body2
pos = [matrix(X, 1, -1); matrix(Y, 1, -1); zeros(1,N)]; 
pos_b1 = zeros(3,N); 
pos_b2 = repmat([-D; 0; 0], 1, N); 

// accelerations if body1 = central body
acc1 = CL_fo_centralAcc(pos, mu_b1); 
accp1 = CL_fo_thirdBodyAcc(pos, pos_b2, mu_b2);

// accelerations if body2 = central body
acc2 = CL_fo_centralAcc(pos - pos_b2, mu_b2); 
accp2 = CL_fo_thirdBodyAcc(pos - pos_b2, pos_b1 - pos_b2, mu_b1);

// ratio of perturbing acceleration / central acceleration
// region of influence around body 1 : r <= 1
r = (CL_norm(accp1) ./ CL_norm(acc1)) ./ (CL_norm(accp2) ./ CL_norm(acc2)); 


// -----------------
// Plot
// -----------------

f = scf(); 
f.axes_size = [650, 550]; 
f.visible = "on"; // necessary: scilab bug (contour2d)
f.immediate_drawing = "off"; 

nbcol = 256; 
f.color_map = CL_colormap("jet", nbcol); 

a = gca(); 

// plotted quantity
Z = matrix(log10(r), sz); 

zmin = -2.3; 
zmax = 2.3; 

// colorbar (change graduations: log -> natural)
colorbar(zmin, zmax, [1, nbcol]); 
// cb: type = Axes in Scilab 6.1, but was a child before 
cb = gce(); 
if (cb.type <> "Axes"); cb = cb.parent; end 
cb.y_ticks = tlist(["ticks", "locations", "labels"], (-3:3), string(10.^(-3:3))); 
cb.font_size = 2; 
cb.title.text = "Acc. ratio"; 

Sgrayplot(x/UNIT, y/UNIT, Z, zminmax = [zmin, zmax], colminmax = [1, nbcol]); 

// circle that delimite the sphere of influence (radius = rsoi)
function plot_circle(r, colornames)
  theta = linspace(0, 2*%pi, 200); 
  plot(r * cos(theta) / UNIT, r * sin(theta) / UNIT, "thickness", 4); 
  h = CL_g_select(gce(), "Polyline"); 
  h.foreground = color(colornames(1)); 
  plot(r * cos(theta) / UNIT, r * sin(theta) / UNIT, "thickness", 4); 
  h = CL_g_select(gce(), "Polyline"); 
  h.foreground = color(colornames(2)); 
  h.line_style = 8; 
endfunction

plot_circle(rsoi, ["hotpink", "white"]); 
// plot_circle(rsoi / (2^(1/5)), ["purple", "white"]); 


// contours 
CL_g_tag(a, 0); 

// sub-levels
levels = log10([(2:9) / 100, (2:9) / 10, (2:9), (2:9) * 10]); 
contour2d(x/UNIT, y/UNIT, Z, levels, style=color("grey60")*ones(levels));
CL_g_tag(a, 2); 

// ratio = 0.1 and 10
levels = [-1, 1]; 
contour2d(x/UNIT, y/UNIT, Z, levels, style=color("grey30")*ones(levels));
CL_g_tag(a, 1); 

// ratio = 1 (NB: 2 values necessary!)
levels = [0, 0]; 
contour2d(x/UNIT, y/UNIT, Z, levels, style=color("black")*ones(levels));
CL_g_tag(a, 3); 

h = CL_g_select(a, "Polyline", 1); 
h.thickness = 2; 

h = CL_g_select(a, "Text", 2); 
CL_g_delete(h); 

h = CL_g_select(a, "Polyline", 3); 
h.thickness = 3; 

h = CL_g_select(a, "Text"); 
h.font_size = 3; 
h.font_style = 8; 
CL_g_set(h, "text", string(10.^(strtod(h.text))));

// name of body at origin
rect = xstringl(0, 0, info(4)); 
xstring(rect(1) - rect(3)/2, 0, info(4)); 
h = CL_g_select(gce(), "Text"); 
h.font_foreground = color("mistyrose"); 
h.font_size = 2; 

a.data_bounds = [min(x), min(y); max(x), max(y)] / UNIT; 
a.tight_limits = "on"; 

a.title.text = "Sphere of influence: Acceleration ratio - " + info(1) + " " + info(2); 
a.x_label.text = "X coordinate (" + info(3) + ")"; 
a.y_label.text = "Y coordinate (" + info(3) + ")"; 

a.isoview = "on"; 

CL_g_stdaxes(a); 

f.immediate_drawing = "on"; 
f.visible = "on"; 

