//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Variation of absolute velocity due to a swing-by
//> 
//> The absolute velocity of a spacecraft approaching a body is defined by:  
//> - its norm (relative to the norm of the absolute velocity of the body), 
//> - its direction defined by an oriented angle: angle between the absolute
//> velocity vector of the body and the absolute velocity vector of the spacecraft. 
//> 
//> The swing-by is such that: 
//> 1) The altitude of the periapsis has a constant value
//> 2) The relative trajectory lies in the plane of the two absolute velocity vectors. 
//> There are then 2 possibilities for the swing-by that correspond to which side the arrival 
//> asymptote is relative to the body. See next condition.  
//> 3) One of the following condition is met: 
//> - [direct]: the turn angle is always positive, 
//> - [retrograde]: the turn angle is always negative, 
//> - [max d|V|]: the sign of the turn angle is chosen so as to maximize the variation of 
//> the norm of the absolute velocity vector, 
//> - [min d|V|]: the sign of the turn angle is chosen so as to minimize the variation of 
//> the norm of the absolute velocity vector. 
//> 
//> The plots show: 
//> - the variation of the norm of the absolute velocity vector, 
//> - the deviation (signed angle) of the direction of the absolute velocity vector. 
//> The norm of the relative velocity (velocity at infinity) is optionally  shown (in km/s). 
//> 
//> Note: 
//> - The velocity of the body is an approximate average value computed such that 
//> its trajectory around the central body is circular. 
//
// Author: A. Lamy
// -----------------------------------------------------------

// -----------------------------------------------------------
// Inputs
// -----------------------------------------------------------

bodynum = 3;      
dvmin = -6.e3; 
dvmax = 6.e3; 
angmin = -15 * %CL_deg2rad; 
angmax = 15 * %CL_deg2rad; 
hp = 1000.e3; 
idir = 1; 
iplot = 1; 
plotvinf = 1; 

desc_param = list(..
   CL_defParam("Body: 2=Venus, 3=Earth, 4=Mars, 5=Jupiter, 10=Moon", bodynum, accv=[2, 3, 4, 5, 10]),...
   CL_defParam("Difference of absolute velocity norm (spacecraft minus body) - min", dvmin, id="$vmin", units=["m/s", "km/s"]), ...
   CL_defParam("Difference of absolute velocity norm (spacecraft minus body) - max", dvmax, units=["m/s", "km/s"], valid="$x > $vmin"), ...
   CL_defParam("Angle between absolute velocity vectors (spacecraft / body) - min", angmin, id="$angmin", units=["rad", "deg"], valid="$x >= -180"), ...
   CL_defParam("Angle between absolute velocity vectors (spacecraft / body) - max", angmax, units=["rad", "deg"], valid="$x > $angmin & $x <= 180"), ...
   CL_defParam("Periapsis altitude", hp, units=["m", "km"], valid="$x > 0"), ...
   CL_defParam("Swing-by direction: 1=direct, 2=retrograde, 3=max d|V|, 4=min d|V|", idir, accv = 1:4), ...
   CL_defParam("Plot: 1=variation of norm, 2=variation of direction", iplot, accv = [1, 2]),...
   CL_defParam("Show vinf levels: 1=yes, 0=no", plotvinf, accv = [0, 1])...
);

[bodynum, dvmin, dvmax, angmin, angmax, hp, idir, iplot, plotvinf] = CL_inputParam(desc_param);


// -----------------------------------------------------------
// Utility functions
// -----------------------------------------------------------
// mean absolute velocity norm of body  
function [v] = compute_mean_vel(body, center)
  cjd0 = CL_dat_cal2cjd(2000, 1, 1); 
  cjdf = CL_dat_cal2cjd(2100, 1, 1);
  cjd = linspace(cjd0, cjdf, 1000); 
  [pos, vel] = CL_ephDE_getPVA(body, cjd, center); 
  mu = CL_dataGet("body." + center + ".mu");
  kep = CL_oe_car2kep(pos, vel, mu=mu); 
  sma = mean(kep(1,:)); 
  v = sqrt(mu / sma); 
endfunction

// Signed angle between 2 vectors in x-y plane (vec1-> vec2)
function [ang] = signed_angle(vec1, vec2)
  ang = CL_vectAngle(vec1, vec2); 
  I = find(CL_dot(CL_cross(vec1, vec2), [0;0;1]) < 0); 
  ang(I) = -ang(I); 
endfunction

// data for celestial bodies
function [data] = prepare_data(body_names, centers)
  // structure of data for all bodies 
  data = struct(); 

  for (k = 1 : size(body_names, "*"))
    name = body_names(k); 
    center = centers(k); 
    data(name).mu = CL_dataGet("body." + name + ".mu"); 
    data(name).er = CL_dataGet("body." + name + ".eqRad"); 
    data(name).vel = compute_mean_vel(name, center); 
  end
endfunction


// -----------------------------------------------------------
// Computations
// -----------------------------------------------------------
body_names = ["Venus", "Earth", "Mars", "Jupiter", "Moon"]; 
body_centers = ["Sun", "Sun", "Sun", "Sun", "Earth"]; 
body_nums = [2, 3, 4, 5, 10]; 

data = prepare_data(body_names, body_centers); 

body = body_names(find(body_nums == bodynum)); 

R = data(body).er; 
MU = data(body).mu; 
V = data(body).vel; 
rp = R + hp; 

vbody = V * [1; 0; 0]; 

// Difference of norm of absolute velocity: spacecraft minus body
dv_values = linspace(dvmin, dvmax, 100); 

// Oriented angle between absolute velocity vectors (spacecraft / body) 
theta_values = linspace(angmin, angmax, 102); 

[Theta, DV] = ndgrid(theta_values, dv_values); 

theta = matrix(Theta, 1, -1); 
dv = matrix(DV, 1, -1); 

N = length(theta); 

vabs = CL_dMult(V + dv, [cos(theta); sin(theta); zeros(theta)]); 
vrel = vabs - repmat(vbody, 1, N); 

vinf = CL_norm(vrel);  
alpha = signed_angle([1;0;0], vrel);  

// turn angle (deviation of vinf) - positive value
// Nan replaced by 0 (case vinf = 0) 
turnang =  CL_ip_flybyParams("vinf", vinf, "rp", rp, "turnang", mu=MU); 
turnang(isnan(turnang)) = 0; 

// abs/rel velocity vector after swing-by case "turn angle positive")
alphad1 = alpha + turnang;
vreld1 = CL_dMult(vinf, [cos(alphad1); sin(alphad1); zeros(alphad1)]); 
vabsd1 = repmat(vbody, 1, N) + vreld1; 

// abs/rel velocity vector after swing-by case "turn angle negative")
alphad2 = alpha - turnang;
vreld2 = CL_dMult(vinf, [cos(alphad2); sin(alphad2); zeros(alphad2)]); 
vabsd2 = repmat(vbody, 1, N) + vreld2; 

// compute absolute velocity after swing-by depending on option selected 
if (idir == 1)
  vabsd = vabsd1; 
elseif (idir == 2)
  vabsd = vabsd2; 
elseif (idir == 3)
  [m,k] = max(CL_norm(vabsd1)-CL_norm(vabs), CL_norm(vabsd2)-CL_norm(vabs)); 
  vabsd = vabsd1;
  I = find(k == 2);   
  vabsd(:,I) = vabsd2(:,I); 
elseif (idir == 4)
  [m,k] = min(CL_norm(vabsd1)-CL_norm(vabs), CL_norm(vabsd2)-CL_norm(vabs)); 
  vabsd = vabsd1; 
  I = find(k == 2);   
  vabsd(:,I) = vabsd2(:,I); 
end

vreld = vabsd - repmat(vbody, 1, N); 
delta_alpha = signed_angle(vrel, vreld); 

// final quantities: variation of norm or direction 
delta_vabs = CL_norm(vabsd) - CL_norm(vabs); 
delta_ang = signed_angle(vabs, vabsd); 


// -----------------------------------------------------------
// Print info
// -----------------------------------------------------------

mprintf("----------------------------------\n");  
mprintf("Body = %s\n", body); 
mprintf("Absolute velocity [km/s] = %.3f\n", V/1000); 
mprintf("Gravitational constant [km^3/s^2] = %.3f\n", MU * 1.e-9); 
mprintf("\n");  


// -----------------------------------------------------------
// Plot
// -----------------------------------------------------------

f = scf();
f.figure_size = [700, 550]; 
f.visible = "on"; 
f.immediate_drawing = "off"; 

nbcol = 200;  
f.color_map = 0.2 + 0.8 * CL_colormap("jet", nbcol); 

x = theta_values * %CL_rad2deg; 
y = dv_values / 1000; 

if (iplot == 1)
  Z = matrix(delta_vabs, size(DV)) / 1000; 
else
  Z = matrix(delta_ang, size(DV)) * %CL_rad2deg; 
end

zmin = min(Z); 
zmax = max(Z); 

colorbar(zmin, zmax, colminmax=[1,nbcol], fmt="%.2f");
// cb: type = Axes in Scilab 6.1, but was a child before 
cb = gce(); 
if (cb.type <> "Axes"); cb = cb.parent; end 
cb.font_size = 2; 
cb.title.font_size = 3
if (iplot == 1)
  cb.title.text = "Delta velocity (km/s)"; 
else
  cb.title.text = "Delta angle (deg)"; 
end

Sgrayplot(x, y, Z, colminmax=[1,nbcol], zminmax=[zmin, zmax], colout=[0,0]);
CL_g_tag(gca(), 0); 

// levels of vinf
if (plotvinf == 1)
Zvinf = matrix(vinf, size(DV)) / 1000;
[levels] = CL_autoLevels(min(Zvinf), max(Zvinf), 6);
contour2d(x, y, Zvinf, levels, style=ones(levels) * color("pink"));
CL_g_tag(gca(), 10); 
end

[levels, sublevels] = CL_autoLevels(zmin, zmax, 6, 3);
// sublevels 
contour2d(x, y, Z, sublevels, style=ones(sublevels) * color("grey50"));
CL_g_tag(gca(), 2); 
// main levels
contour2d(x, y, Z, levels, style=ones(levels) * color("black"));
CL_g_tag(gca(), 1); 
// 0 level 
levels = [0,0]; 
contour2d(x, y, Z, levels, style=ones(levels) * color("black"));
CL_g_tag(gca(), 3); 

  
a = gca(); 

h = CL_g_select(a, "Text", 1); 
h.font_size = 3; 
h = CL_g_select(a, "Text", 2); 
CL_g_delete(h); 
h = CL_g_select(a, "Text", 3); 
CL_g_delete(h); 
h = CL_g_select(a, "Polyline", 3); 
h.thickness = 2; 

if (plotvinf == 1)
h = CL_g_select(a, "Text", 10); 
h.font_size = 3;
h.font_foreground = color("pink");
h = CL_g_select(a, "Polyline", 10); 
h.thickness = 2; 
end

a.data_bounds = [min(x), min(y); max(x), max(y)]; 
a.tight_limits = "on"; 

if (iplot == 1)
  text = "Variation of abs. velocity norm"; 
else
  text = "Variation of abs. velocity direction"; 
end

OPTS = ["direct", "retro", "max d|V|", "min d|V|"]; 

a.title.text = msprintf("%s - body=%s - alt=%.1f km [%s]", text, body, hp/1000, OPTS(idir)); 
a.x_label.text = "Angle between absolute velocity vectors - spacecraft / body (deg)"; 
a.y_label.text = "Difference of absolute velocity norms - spacecraft / body (km/s)"; 

CL_g_stdaxes(a); 

f.immediate_drawing = "on"; 
f.visible = "on"; 




