//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Comparison Hohmann - Bi-elliptic (circular initial and final orbits):
//> The plot shows the regions where DV bi-elliptic is smaller than DV Hohmann. 
//> 
//> The hohmann transfer is from an orbit of radius sma to an orbit of 
//> radius sma * r (r = radius ratio). 
//> The bi-elliptic transfer is from an orbit of radius sma to an orbit of 
//> radius sma * r (r = radius ratio), through an orbit of apoapsis radius 
//> sma * rapo (rapo = apoapsis radius ratio). 
//> 
//> The result is independent of the initial radius sma, and also independent 
//> of the gravitational parameter mu. 
//> 
//> Note: the "apoapsis radius" is badly named if this radius if smaller than the 
//> radius of the final orbit. But the results are also valid in this case. 
//
// Author: A. Lamy
// -----------------------------------------------------------


// Default values
ratmin = 12;
ratmax = 25;
ratapomin = 14;
ratapomax = 40;

desc_param = list(..
   CL_defParam("Final orbit: radius ratio - min", ratmin, id='$ratmin', valid='$ratmin>=1' ),..
   CL_defParam("Final orbit: radius ratio - max", ratmax, id='$ratmax', valid='$ratmax>$ratmin' ),..
   CL_defParam("Intermediate orbit: apoapsis radius ratio - min", ratapomin, id='$ratapomin', valid='$ratapomin>=1'),..
   CL_defParam("Intermediate orbit: apoapsis radius ratio - max", ratapomax, id='$ratapomax', valid='$ratapomax>$ratapomin')..
);

[ratmin, ratmax, ratapomin, ratapomax] = CL_inputParam(desc_param);


nbpts = 300;
tabrat = linspace(ratmin, ratmax, nbpts);
tabratapo = linspace(ratapomin, ratapomax, nbpts);

[Tabrat, Tabratapo] = ndgrid(tabrat, tabratapo); 
all_tabrat = matrix(Tabrat, 1, -1); 
all_tabratapo = matrix(Tabratapo, 1, -1); 

// result is independent of sma_ref and mu => normalized values 
sma_ref = 1; 
mu = 1; 

sma = sma_ref * all_tabrat; 
rapo = sma_ref * all_tabratapo; 

dv_hohm = CL_man_dvHohmann(sma_ref, sma, mu = mu);
dv_biel = CL_man_dvBiElliptic(sma_ref, sma, rapo, mu = mu);

// relative gain 
dv_rat = (dv_biel - dv_hohm) ./ dv_hohm;  


// -----------------------------------------------------------
// plot
// -----------------------------------------------------------

f=scf();
f.visible="on";
f.immediate_drawing="off";

f.color_map = CL_colormap("jet", 64);
Noir = addcolor([0,0,0]);
Gris = addcolor([1,1,1]*0.4);

a=gca();

// result in % (value is positive or 0) 
z = -matrix(min(dv_rat, 0) * 100, size(Tabrat)); 
zmin = min(z); 
zmax = max(z); 
z(z == 0) = -1; 

colorbar(zmin, zmax, colminmax=[1,64],fmt="%.1f");
Sgrayplot(tabrat, tabratapo, z, colminmax=[1,64], zminmax=[zmin,zmax], colout = [0,0]);

levels = (0.5 : 0.5 : 10); 
contour2d(tabrat, tabratapo, z, levels, style=Noir*ones(levels));

// general setting
a.data_bounds = [ratmin, ratapomin; ratmax, ratapomax];
a.tight_limits = "on"; 
a.margins = [0.12,0.12,0.12,0.12];
CL_g_stdaxes();

a.title.text = "Gain of bi-elliptic transfer over Hohmann transfer (%)";
a.x_label.text = "Final orbit - radius ratio";
a.y_label.text = "Intermediate orbit - apoapsis radius ratio";

f.immediate_drawing="on";
f.visible="on";

