//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Cost of changing the AOL value (circular orbits only)
//> A first impulsive maneuver makes the AOL drift. A second impulsive maneuver 
//> stops the drift. The cost is the total DV for the 2 maneuvers. 
//>
//> Only changing the semi major axis is possible, as inclination has almost no effect 
//> on the AOL drift rate. The nominal value of inclination has also little impact. 
//> 
//> Notes: 
//> - Valid for small enough maneuvers (1st order approximation) 
//> - No bounds on sma are taken into account, so it should be checked that the 
//>   intermediate orbit is acceptable. 
//>  
// Author: A. Lamy
// -----------------------------------------------------------

alt = 700.e3;
sso = 1; 
inc = 0 * %CL_deg2rad;

Tmin = 5; // days 
Tmax = 30; // days
delta_aol = (30 : 30 : 180) * %CL_deg2rad; 

desc_param = list(..
   CL_defParam("Altitude (semi-major axis minus eq. radius) - min", alt, units=['m', 'km'], valid='$x>=0'),..
   CL_defParam("Sun-synchronous orbit? (1=yes, 0=no)", sso, id='$sso', accv=[0,1]),..
   CL_defParam("Inclination (if not Sun-synchronous)", inc, units=['rad','deg'], id='$inc', valid='$sso == 1 | ($inc>=0 & $inc <= 180)'),..
   CL_defParam("Transfer duration - min", Tmin, units=['days'], id = "$Tmin", valid='$Tmin > 0'),..
   CL_defParam("Transfer duration - max", Tmax, units=['days'], id = "$Tmax", valid='$Tmax > $Tmin'),..
   CL_defParam("Values of delta AOL", delta_aol, units=['rad', 'deg'], dim = -1, valid='$x > 0 & $x <= 360')..
);

[alt, sso, inc, Tmin, Tmax, delta_aol] = CL_inputParam(desc_param);

// -----------------------------------------------------------
// computation 
// -----------------------------------------------------------

nbpts = 200;
T = linspace(Tmin, Tmax, nbpts) * 86400;
ecc = 0;
sma = alt + %CL_eqRad; 
if (sso == 1)
  inc = CL_op_ssoJ2 ("i", sma, ecc); 
end   

// J2 effect 
[dpomdt, dgomdt, danmdt, dpomdotdaei, dgomdotdaei, danmdotdaei] = CL_op_driftJ2(sma, ecc, inc);
psodot = dpomdt + danmdt; // PSO = AOL = pom + anm
dpsodotdaei = dpomdotdaei + danmdotdaei; 

// dva = Ga * da
Ga = psodot / 2; 
Ca = dpsodotdaei(1,:) / Ga; // d (dpsodt) / d DVa

// aol_drift : PxN (P: number of delta_aol values) 
aol_drift = CL_dMult(delta_aol', 1 ./ T); 

// dv corresponds to 2 maneuvers
dv = 2 * abs(aol_drift / Ca);  


// -----------------------------------------------------------
// plot
// -----------------------------------------------------------

f=scf();
f.visible="on";
f.immediate_drawing="off";

f.color_map = CL_colormap("jet", max(3, length(delta_aol)));

for k = 1 : length(delta_aol)
  plot(T / 86400, dv(k, :), "thickness", 2); 
  e = CL_g_select(gce(), "Polyline"); 
  e.foreground = k; 
end 

a = gca(); 
a.x_label.text = "Transfer duration (days)";
a.y_label.text = "DV (m/s)";

a.title.text = "Cost of AOL change";

txt = msprintf("%.6g\n", delta_aol' * %CL_rad2deg)'; 
CL_g_legend(a, txt, header = "Delta AOL (deg)"); 

CL_g_stdaxes(a); 

f.immediate_drawing="on";
f.visible="on";

