//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Cost of changing the AOL (arg of latitude) drift rate (circular orbits only). 
//> The AOL (relative) drift is obtained by one impulsive maneuver. The cost 
//> is the DV corresponding to this maneuver. 
//> 
//> Only changing the semi major axis is possible, as inclination has almost no effect 
//> on the drift rate. The nominal value of inclination has also little impact. 
//> 
//> Notes: 
//> - Valid for "small" increments (1st order approximation) 
//> - No bounds on sma are taken into account, so it should be checked that the 
//>   intermediate orbit is acceptable. 
// 
// Author: A. Lamy
// -----------------------------------------------------------

altmin = 500.e3;
altmax = 1000.e3;
sso = 1; 
inc = 0 * %CL_deg2rad;

aoldrift = 1 * %CL_deg2rad / 86400;

desc_param = list(..
   CL_defParam("Altitude of circular orbit (sma minus eq. radius) - min", altmin, units=['m', 'km'], id='$hmin', valid='$hmin>=0'),..
   CL_defParam("Altitude of circular orbit (sma minus eq. radius) - max", altmax, units=['m', 'km'], id='$hmax', valid='$hmax>$hmin'),..
  CL_defParam("Sun-synchronous orbit? (1=yes, 0=no)", sso, id='$sso', accv=[0,1]),..
  CL_defParam("Inclination (if not Sun-synchronous)", inc, units=['rad','deg'], id='$inc', valid='$sso == 1 | ($inc>=0 & $inc <= 180)'),..
   CL_defParam("AOL drift rate change", aoldrift, units=['rad/s', 'deg/day'], valid='$x > 0')..
);

[altmin, altmax, sso, inc, aoldrift] = CL_inputParam(desc_param);

// -----------------------------------------------------------
// computation 
// -----------------------------------------------------------

nbpts = 80;
alt = linspace(altmin, 	altmax, nbpts);
sma = alt + %CL_eqRad; 
ecc = 0; 
if (sso == 1)
  inc = CL_op_ssoJ2 ("i", sma, ecc); 
end   


[dpomdt, dgomdt, danmdt, dpomdotdaei, dgomdotdaei, danmdotdaei] = CL_op_driftJ2(sma, ecc, inc);
psodot = dpomdt + danmdt; // PSO = AOL = pom + anm
dpsodotdaei = dpomdotdaei + danmdotdaei; 
  
C = dpsodotdaei(1,:) .* (2 ./ psodot); // d (dpsodt) / d DVa

dv = %nan * ones(C); 
I = find(abs(C) > 0)
if (I <> [])
  dv(I) = abs(aoldrift ./ C(I));
end


// -----------------------------------------------------------
// plot
// -----------------------------------------------------------

f=scf();
f.visible="on";
f.immediate_drawing="off";

plot(alt/1000, dv, "thickness", 2);

a = gca(); 
a.x_label.text = "Altitude (semi-major axis minus eq. radius) (km)";
a.y_label.text = "DV (m/s)";

if (sso)
  txt = "SSO"; 
else
  txt = msprintf("inc = %.6g deg", inc * %CL_rad2deg); 
end 

a.title.text = msprintf("Cost of AOL drift rate change of %.6g deg/day - %s", ... 
   aoldrift*%CL_rad2deg*86400, txt);

CL_g_stdaxes(a); 

// adjustments
f.immediate_drawing="on";
f.visible="on";

