//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Relative cost of Hohmann transfer:
//> 
//> Total cost needed for the transfer as function 
//> of the relative variation of the semi major axis.
//> The DV is signed: positive if the semi-major axis increases, negative otherwise. 
//> 
//> The relative semi-major axis variation is defined as (smaf - smai) / smai, 
//> where smai and smaf are the initial and final (circular) orbit semi-major axes
//> respectively. 
//> 
//> The computed result is the delta-V divided by the initial velocity, that is 
//> Vi = sqrt(mu/smai). It means that if the result is 20%, the actual DV is 20% of the 
//> initial velocity. 
//> 
//> Notes: 
//> The function that is plotted (Hohmann DV / initial velocity) is this one: 
//> y = sqrt(2 * (1+x) / (2+x)) - 1 + sqrt(1 / (1+x)) * (1 - sqrt(2 / (2+x)))
//> x is defined by: smaf = smai * (1 + x).
//> 
//> Third order approximation (limited expansion at x = 0) is: 
//> y = (1/2) * x - (3/8) * x^2 + (9/32) * x.^3
//> 
//> When considering the approximation, the range on the relative sma variation  
//> to have a given relative error on the DV (y - y_approx)/y) is the following:
//> Relative error on DV:        10%           1%         0.1%    
//> Range - order 1:             13%         1.3%        0.13%
//> Range - order 2:             40%          13%           4%
//> Range - order 3:             60%          28%          13%
//> 
//> For an orbit radius ratio r = smai/smaf = 1/(1+x) between 1/3 and 3, 
//> the following empirical model is accurate to within +/- 1%: 
//> DV/Vi = 0.6438 - 0.8056 * r + 0.1823 * r^2 - 0.0205 * r^3
// 
// Author: A. Lamy
// -----------------------------------------------------------

arelmin = -50;       
arelmax = 100;   
dev_order = 1; 

desc_param = list(..
   CL_defParam("Relative semi-major axis variation - min", arelmin, units=['%'], id='$arelmin', valid='$arelmin > -100'),..
   CL_defParam("Relative semi-major axis variation - max", arelmax, units=['%'], id='$arelmax', valid='$arelmax > $arelmin'),..
   CL_defParam("Show approximation at order 1..3 ? (0=no, 1..3: order)", dev_order, accv=(0:3))..
);

[arelmin, arelmax, dev_order] = CL_inputParam(desc_param);

// % -> normalized values 
arelmin = arelmin / 100; 
arelmax = arelmax / 100; 

nbpts = 500;
arel = linspace(arelmin, arelmax, nbpts);

smai = 1; 
mu = 1; 
smaf = smai * (1 + arel); 

dv = CL_man_dvHohmann(smai, smaf, mu=mu);
I = find(arel < 0); 
if (I <> [])
  dv(I) = -dv(I);
end   

// -----------------------------------------------------------
// plot
// -----------------------------------------------------------
// limited development 
function [y] = devlim(x, order)
  y = x / 2; 
  if (order >= 2)
    y = y - 3 * x.^2 / 8; 
  end 
  if (order >= 3)
    y = y + 9 * x.^3 / 32; 
  end 
endfunction  
  

f = scf();
f.visible = "on";
f.immediate_drawing = "off";

if (dev_order > 0)
  plot(arel * 100, devlim(arel, dev_order) * 100, "b--"); 
end 
plot(arel * 100, dv * 100, "b", "thickness", 2); 

// adjustments

a = gca();
a.tight_limits(1) = "on"; 

a.title.text = "Relative DV (with sign) needed for a Hohmann transfer";
a.x_label.text = "Relative variation of semi-major axis (%)"; 
a.y_label.text = "DV relative to initial velocity (%)";

CL_g_stdaxes(a);

f.immediate_drawing = "on";
f.visible = "on";

