//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Cost of Hohmann-like transfer including plane rotation
//> 
//> The transfer is derived from a Hohmann transfer (2 velocity increments). 
//> Out-of-plane (non radial) DV components are added to achieve the desired 
//> plane rotation around the axis joining the 2 maneuver locations. 
//> 
//> The Optimal total cost (DV >=0) is computed as function 
//> of relative variation of semi major axis and plane rotation angle. 
//> 
//> The relative semi-major axis variation is defined as (smaf - smai) / smai, 
//> where smai and smaf are the initial and final (circular) orbit semi-major axes
//> respectively. 
//>  
//> The plotted result is the delta-V divided by the initial velocity, that is 
//> sqrt(mu/smai). It means that if the result is 20%, the actual DV is 20% of the 
//> initial velocity. 
//
// Author: A. Lamy
// -----------------------------------------------------------

arelmin = -50;       
arelmax = 150;   
angmin = -20 * %CL_deg2rad; 
angmax = 20 * %CL_deg2rad; 

desc_param = list(..
   CL_defParam("Relative semi-major axis variation - min", arelmin, units=['%'], id='$arelmin', valid='$arelmin > -100'),..
   CL_defParam("Relative semi-major axis variation - max", arelmax, units=['%'], id='$arelmax', valid='$arelmax > $arelmin'),..
   CL_defParam("Plane rotation angle - min", angmin, units=['rad', 'deg'], id='$angmin', valid='$angmin >= -180'),..
   CL_defParam("Plane rotation angle - max", angmax, units=['rad', 'deg'], id='$angmax', valid='$angmax <= 360 & $angmax > $angmin')..
);

[arelmin, arelmax, angmin, angmax] = CL_inputParam(desc_param);

// % -> normalized values 
arelmin = arelmin / 100; 
arelmax = arelmax / 100; 

// --------------------------------
// Note : the relative results would be the same for any other values of 
// smai and mu

smai = 1;
mu = 1; 
arel = linspace(arelmin, arelmax, 130); 
ang = linspace(angmin, angmax, 130); 

[X, Y] = ndgrid(arel, ang); 
smaf = smai * (1 + matrix(X, 1, -1)); 
rotang = CL_rMod(matrix(Y, 1, -1), -%pi, %pi); 
deltav = CL_man_dvHohmannRotP(smai, smaf, rotang, mu=mu);
Deltav = matrix(deltav, size(X)); 

// --------------------------------
// plot 
// -----------------------------

// plotted quantity (DV relative to initial velocity (%))
Z = Deltav * 100; 

f=scf();
f.visible="on";
f.immediate_drawing="off";

f.color_map = CL_colormap("jet", 32);

a = gca();
CL_g_tag(a,0);

[levels, sublevels] = CL_autoLevels(min(Z), max(Z));

Coul1 = 5;
Coul2 = 10;
Coul3 = 2;

contour2d(arel * 100, ang * %CL_rad2deg, Z, levels, style=Coul1*ones(levels));
CL_g_tag(a,1);
contour2d(arel * 100, ang * %CL_rad2deg, Z, sublevels, style=Coul2*ones(sublevels));
CL_g_tag(a,2);

// various updates
h = CL_g_select(a, "Text",2);
CL_g_delete(h);

h = CL_g_select(a, "Text", 1);
CL_g_set(h, "text", string(abs(strtod(h.text))));

h = CL_g_select(a, "Text");
h.font_foreground=Coul3;
h.font_size=2;
h.font_style=8;

h = CL_g_select(a, "Polyline", 1);
h.thickness=2;

a.data_bounds = [arelmin * 100, angmin * %CL_rad2deg; arelmax * 100, angmax * %CL_rad2deg]; 
a.tight_limits = "on";  

a.title.text = "Optimal DV relative to initial velocity (%)"; 
a.x_label.text = "Relative variation of semi-major axis (%)"; 
a.y_label.text = "Plane rotation angle (deg)"; 
CL_g_stdaxes(a); 

f.immediate_drawing="on";
f.visible="on";