//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Cost of changing the RAAN value (circular orbits only)
//> A first impulsive maneuver makes the RAAN drift. A second impulsive maneuver 
//> stops the drift. The cost is the total DV for the 2 maneuvers. 
//>
//> The results should be considered as rough estimates. 
//> 
//> Four maneuver types are possible: 
//> - semi major axis only
//> - inclination only
//> - (coupled = combined) semi major axis and inclination 
//> - (uncoupled) semi major axis and inclination 
//> 
//> Bounds are taken into account (5% on semi-major axis, 5 deg on inclination). 
//> If the bounds are reached, the results are not shown, although other less optimal 
//> solutions could sometimes exist.   
//> 
//> Notes: 
//> - Valid for "small" enough maneuvers (1st order approximation) 
//> - It should be checked that the intermediate orbit is acceptable (high ennough 
//>   altitude for instance). 
//>  
// Author: A. Lamy
// -----------------------------------------------------------

alt = 700.e3;
sso = 1; 
inc = 0 * %CL_deg2rad;

Tmin = 100; // days 
Tmax = 500; // days
mantype = 3; 
delta_raan = (5 : 5 : 30) * %CL_deg2rad; 

desc_param = list(..
   CL_defParam("Altitude of circular orbit (sma minus eq. radius) - min", alt, units=['m', 'km'], valid='$x>=0'),..
   CL_defParam("Sun-synchronous orbit? (1=yes, 0=no)", sso, id='$sso', accv=[0,1]),..
   CL_defParam("Inclination (if not Sun-synchronous)", inc, units=['rad','deg'], id='$inc', valid='$sso == 1 | ($inc>=0 & $inc <= 180)'),..
   CL_defParam("Transfer duration - min", Tmin, units=['days'], id = "$Tmin", valid='$Tmin > 0'),..
   CL_defParam("Transfer duration - max", Tmax, units=['days'], id = "$Tmax", valid='$Tmax > $Tmin'),..
   CL_defParam("Values of delta RAAN", delta_raan, units=['rad', 'deg'], dim = -1, valid='$x > 0 & $x <= 360'),..
   CL_defParam("Maneuver type (1=sma, 2=inc, 3=sma+inc (coupled), 4=sma+inc (uncoupled))", mantype, accv=[1,2,3,4])..
);

[alt, sso, inc, Tmin, Tmax, delta_raan, mantype] = CL_inputParam(desc_param);

// -----------------------------------------------------------
// computation 
// -----------------------------------------------------------

nbpts = 200;
T = linspace(Tmin, Tmax, nbpts) * 86400;
ecc = 0;

sma = alt + %CL_eqRad; 
if (sso == 1)
  inc = CL_op_ssoJ2 ("i", sma, ecc); 
end   

// J2 effect 
[dpomdt, dgomdt, danmdt, dpomdotdaei, dgomdotdaei, danmdotdaei] = CL_op_driftJ2(sma, ecc, inc);
  
nmoy = dpomdt + danmdt; 
V = nmoy .* sma; 

// dva = Ga * da
// dvi = Gi * di 
Ga = nmoy / 2; 
Gi = V; 

Ca = dgomdotdaei(1,:) / Ga; // d (dgomdt) / d DVa
Ci = dgomdotdaei(3,:) / Gi; // d (dgomdt) / d DVi

// raan_drift : PxN (P: number of delta_raan values) 
raan_drift = CL_dMult(delta_raan', 1 ./ T); 

// dva / dvi: one-way (from nominal orbit to intermediate orbit)
// NB: signed values 
  
if (mantype == 1)
  dva = raan_drift / Ca; 
  dvi = zeros(raan_drift); 
elseif (mantype == 2)
  dvi = raan_drift / Ci; 
  dva = zeros(raan_drift); 
elseif (mantype == 3)
  // point on the line closest to (0,0)
  dva = Ca * raan_drift / (Ca^2 + Ci^2); 
  dvi = Ci * raan_drift / (Ca^2 + Ci^2); 
else
  if (abs(Ca) > abs(Ci))
    dva = raan_drift / Ca; 
    dvi = zeros(raan_drift); 
  else
    dvi = raan_drift / Ci; 
    dva = zeros(raan_drift); 
  end 
end

// bounds on sma/inc (5% of sma, 5 deg)
damax = 0.05 * (alt + %CL_eqRad); 
dimax = 5 * %CL_deg2rad; 

// dv: 2 maneuvers (absolute value) 
// check that da and di are acceptable (within bounds) 
dv = 2 * sqrt(dva.^2 + dvi.^2); 
I = find(abs(dva) / Ga > damax | abs(dvi) / Gi > dimax); 
dv(I) = %nan; 


// -----------------------------------------------------------
// plot
// -----------------------------------------------------------

f=scf();
f.visible="on";
f.immediate_drawing="off";

f.color_map = CL_colormap("jet", max(3, length(delta_raan)));

for k = 1 : length(delta_raan)
  plot(T / 86400, dv(k, :), "thickness", 2); 
  e = CL_g_select(gce(), "Polyline"); 
  e.foreground = k; 
end 

a = gca(); 
a.x_label.text = "Transfer duration (days)";
a.y_label.text = "DV (m/s)";
infos = ["sma", "inc", "sma+inc, coupled", "sma+inc, uncoupled"]; 
a.title.text = msprintf("Cost of RAAN change (%s)", infos(mantype));

txt = msprintf("%.6g\n", delta_raan' * %CL_rad2deg)'; 

CL_g_legend(a, txt, header = "Delta RAAN (deg)"); 

CL_g_stdaxes(a); 

f.immediate_drawing="on";
f.visible="on";

