//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Cost of changing the RAAN drift rate (circular orbits only)
//> The RAAN (relative) drift is obtained by one impulsive maneuver. The cost 
//> is the DV corresponding to this maneuver. 
//>
//> The results should be considered as rough estimates. 
//> 
//> Four maneuver types are possible: 
//> - semi major axis only
//> - inclination only
//> - (coupled = combined) semi major axis and inclination 
//> - (uncoupled) semi major axis and inclination 
//> 
//> Notes: 
//> - Valid for small enough maneuvers (1st order approximation) 
//> - no bounds on sma/inc are taken into account, so it should be checked that the 
//>   intermediate orbits are acceptable.
//>  
// Author: A. Lamy
// -----------------------------------------------------------

altmin = 500.e3;
altmax = 1000.e3;

incmin = 95 * %CL_deg2rad;
incmax = 105 * %CL_deg2rad;

raandrift = 0.1 * %CL_deg2rad / 86400;
mantype = 3; 

desc_param = list(..
   CL_defParam("Altitude (semi-major axis minus eq. radius) - min", altmin, units=['m', 'km'], id='$hmin', valid='$hmin>=0'),..
   CL_defParam("Altitude (semi-major axis minus eq. radius) - max", altmax, units=['m', 'km'], id='$hmax', valid='$hmax>$hmin'),..
   CL_defParam("Inclination - min", incmin, units=['rad', 'deg'], id='$imin', valid='$imin>=0 & $imin<=180'),..
   CL_defParam("Inclination - max", incmax, units=['rad', 'deg'], id='$imax', valid='$imax>$imin & $imax<=180'),..
   CL_defParam("RAAN drift rate change", raandrift, units=['rad/s', 'deg/day'], valid='$x > 0'),..
   CL_defParam("Maneuver type (1=sma, 2=inc, 3=sma+inc (coupled), 4=sma+inc (uncoupled))", mantype, accv=[1,2,3,4])..
);

[altmin,altmax,incmin,incmax,raandrift,mantype] = CL_inputParam(desc_param);

// -----------------------------------------------------------
// computation 
// -----------------------------------------------------------
// first order approximation: 
// delta_gomdot = d(gomdot)/da * da +  d(gomdot)/di * di
// dva = Ga * da; Ga = 0.5 * V/a = n / 2 
// dvi = Gi * di; Gi = V = n * a
// => delta_gomdot = [d(gomdot)/da / Ga] * dva +  [d(gomdot)/di / Gi] * dvi 
// NB: dva and dvi and signed quantities 

// -> For coupled (a/i) manoeuvers: 
// dv = sqrt(dva^2 + dvi^2) should be minimized. 
// 
// The problem can then be written (calling x = dva, y = dvi): 
// Ca * x + Ci * y - delta_gomdot = 0
// min(x^2 + y^2) 
// 
// The optimal solution (dv = sqrt(x^2 + y^2)) is the distance between the 
// straight line and the origin (x0 = 0, y0 = 0): 
// See: Wikipedia ("distance point line") 
// dv = |Ca * x0 + Ci * y0 - delta_gomdot| / sqrt(Ca^2 + Ci^2) 
//    = |delta_gomdot| / sqrt(Ca^2 + Ci^2)
// 
// -> For uncoupled (a/i) manoeuvers: 
// dv = |dva| + |dvi| should be minimized. 
// The minimum is obtained for either |dva| = 0 or |dvi| = 0
// (that is when the derivative is not continuous) 
// => dv = min(|delta_gomdot| / Ca, |delta_gomdot| / Ci)
//       = |delta_gomdot| / max(Ca, Ci)
// 
// For "a only" or "i only" manoeuvers, the resolution is straightforward: 
// "a only": dv = |delta_gomdot| / Ca
// "i only": dv = |delta_gomdot| / Ci  


nbpts = 80;
alt = linspace(altmin, altmax, nbpts);
inc = linspace(incmin, incmax, nbpts);
ecc = 0;

sma = alt + %CL_eqRad; 
[Sma, Inc] = ndgrid(sma, inc); 

all_sma = matrix(Sma, 1, -1); 
all_inc = matrix(Inc, 1, -1); 

// J2 effect 
[dpomdt, dgomdt, danmdt, dpomdotdaei, dgomdotdaei, danmdotdaei] = CL_op_driftJ2(all_sma, ecc, all_inc);
  
nmoy = dpomdt + danmdt; 
V = nmoy .* all_sma; 

Ca = dgomdotdaei(1,:) .* (2 ./ nmoy); // d (dgomdt) / d DVa
Ci = dgomdotdaei(3,:) ./ V;           // d (dgomdt) / d DVi

if (mantype == 1)
  C = abs(Ca); 
elseif (mantype == 2)
  C = abs(Ci); 
elseif (mantype == 3)
  C = sqrt(Ca.^2 + Ci.^2); 
else
  C = max(abs(Ca), abs(Ci)); 
end

dv = %nan * ones(C); 
I = find(C > 0)
if (I <> [])
  dv(I) = abs(raandrift ./ C(I));
end

res = matrix(dv, size(Sma)); 


// -----------------------------------------------------------
// plot
// -----------------------------------------------------------

f=scf();
f.visible="on";
f.immediate_drawing="off";

f.color_map = CL_colormap("jet", 32);
Coul1 = 5;
Coul2 = 10;
Coul3 = 2;
Noir = addcolor([1,1,1]*0);

a=gca();

[levels, sublevels] = CL_autoLevels(min(res), max(res));

contour2d(alt/1000, inc*%CL_rad2deg, res, levels, style=Coul1*ones(levels));
CL_g_tag(a,1);

contour2d(alt/1000, inc*%CL_rad2deg, res, sublevels, style=Coul2*ones(sublevels));
CL_g_tag(a,2);

// general setting
CL_g_stdaxes(a);
a.data_bounds = [altmin/1000,incmin*%CL_rad2deg; altmax/1000,incmax*%CL_rad2deg];
a.tight_limits="on";
a.x_label.text = "Altitude (semi-major axis minus eq. radius) (km)";
a.y_label.text = "Inclination (deg)";

infos = ["sma", "inc", "sma+inc, coupled", "sma+inc, uncoupled"]; 
a.title.text = msprintf("Cost of RAAN drift rate change of %.6g deg/day (%s)", ...
               raandrift*%CL_rad2deg*86400, infos(mantype));

// adjustments
h = CL_g_select(a, "Text", 2);
CL_g_delete(h);

h = CL_g_select(a, "Text");
h.font_foreground=Coul3;
h.font_size=3;
h.font_style=8;

h = CL_g_select(a, "Polyline", 1);
h.thickness=2;

f.immediate_drawing="on";
f.visible="on";

