//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Effect of low thrust maneuvers on semi major axis, inclination and 
//> derived parameters (circular orbits). 
//> 
//> The results are function of the duty cycle, which is the percentage of time
//> the propulsion is "on".
//> 
//> The thrust angle is the angle in the "tangential-normal" plane at the ascending node. 
//> The value depends on the thrust angle mode:  
//> -> mode = 0  => user-defined value is used
//> -> mode = 1  => maximize d(sma)/dt <=> thrust angle = 0
//> -> mode = -1 => minimise d(sma)/dt <=> thrust angle = 180 deg
//> -> mode = 2  => maximize d(inc)/dt <=> thrust angle = 90 deg
//> -> mode = -2 => minimise d(inc)/dt <=> thrust angle = -90 deg
//> -> mode = 3  => maximize d2(RAAN)/dt2 
//> -> mode = -3 => minimise d2(RAAN)/dt2 <=> thrust angle = value for mode 3 + 180 deg
//> -> mode = 4  => d2(RAAN)/dt2 = 0 and da/dt > 0
//> -> mode = -4 => d2(RAAN)/dt2 = 0 and da/dt < 0 
//> 
//> Number of thrusting arcs (1 or 2): 
//> - case of "sma" maneuvers: no impact. (There is an impact on eccentricity that is not shown)
//> - case of "inc" or "sma+inc" maneuvers: 1 or 2 (if 2: more efficient, but implies change 
//> of thrust direction at each node). 
//> If the duty cycle is more than 50%, the number of arcs must be 2, otherwise it can be 
//> 1 or 2.  
//> 
//> The thrust directions and the positions are implicit. 
//> The convention is that d2(RAAN)/dt2 should be positive (or 0). If it is 0 (case "sma+inc"),
//> d2(RAAN)/dt2 due to tangential maneuvers only should be positive. 
//> 
//> The quantities that can be plotted are : 
//> - sma: Variation of semi major axis
//> - inc: Variation of inclination
//> - dgomdt: Variation of drift of RAAN
//> - dpsodt: Variation of drift of argument of latitude
//> - dv: Consumed DV
//> - ang: Thrusting angle at asc. node
// 
// Author: A. Lamy
// -----------------------------------------------------------

data = struct(); 
data.alt = 600.e3;  // altitude (m)
data.sso = 1; 
data.inc = 0; 
data.max_thrust_values = (1 : 5) * 0.1; // mN
data.mass = 20; // kg

// duty cycle (between 0 and 100) 
// 100 means continuous 
data.dcminmax_percent = [0, 100]; 

// max number of thrust arcs per orbit : 1 or 2 
// only useful for inc  
// if sma : no effect 
data.num_thrust_orb = 2; 

// trust angle mode 
// mode = 0 => user-defined value
// mode = 1/-1 => tangential (da/dt >0 or <0)
// mode = 2/-2 => out-of-plane (di/dt >0 or <0)
// mode = 3/-3 => max |d2gomdt2| (>0 or <0)
// mode = 4/-4 => dgomdt = 0 (da/dt >0 or <0)
data.thrust_angle_mode = 3; 
data.thrust_angle = 0; 

// plot indicators
data.plots = [1, 1, 1, 1, 1, 0]; 

// 1 = day, 2 = orbit period
data.time_unit = 1; 

function [ok] = valid_dc_minmax(dcminmax)
  ok = (dcminmax(1) >= 0 & dcminmax(2) <= 100 & ...
        dcminmax(1) < dcminmax(2)); 
endfunction 

function [ok] = valid_num_thrust(num, dcminmax)
  if (dcminmax(2) > 50)
    ok = (num == 2); 
  else 
    ok = (num == 1 | num == 2); 
  end 
endfunction 

desc_param = list(..
  CL_defParam("Altitude of circular orbit (sma minus equatorial radius)", data.alt, units=['m','km'], valid='$x>=0'),..
  CL_defParam("Sun-synchronous orbit? (1=yes, 0=no)", data.sso, id='$sso', accv=[0,1]),..
  CL_defParam("Inclination (if not Sun-synchronous)", data.inc, units=['rad','deg'], id='$inc', valid='$sso == 1 | ($inc>=0 & $inc <= 180)'),..
  CL_defParam("Thrust values", data.max_thrust_values, units=['mN'], dim = -1, valid='$x>0'),..
  CL_defParam("Mass", data.mass, units=['kg'], valid='$x>0'),..
  CL_defParam("Duty cycle - min/max (between 0 and 100)", data.dcminmax_percent, units=['%'], id='$dc_minmax', dim = 2, valid='valid_dc_minmax($x)'), ..
  CL_defParam("Thrust angle mode: 0=user-defined value, +/- (1, 2, 3, 4) -> See help for details", data.thrust_angle_mode, accv=[0, 1, -1, 2, -2, 3, -3, 4, -4]), ..
  CL_defParam("Thrust angle value - if mode = 0", data.thrust_angle, units=['rad','deg'], valid='$x >= -180 & $x <= 360'), ..
  CL_defParam("Number of thrusting arcs / orbit: 1 or 2", data.num_thrust_orb, valid='valid_num_thrust($x, $dc_minmax)'), ..
  CL_defParam("Plot: sma, inc, dgomdt, dpsodt, dv, ang: 1=yes, 0=no -> See help for details", data.plots, dim=6, accv=[0, 1]), ..
  CL_defParam("Time unit: 1=day, 2=orbit period", data.time_unit, accv=[1, 2]) ..
);

[data.alt, data.sso, data.inc, data.max_thrust_values, data.mass, data.dcminmax_percent, data.thrust_angle_mode, data.thrust_angle, data.num_thrust_orb, data.plots, data.time_unit] = CL_inputParam(desc_param);

// -----------------------------------------
// Derived values  
// -----------------------------------------
R = CL_dataGet("eqRad"); 

// orbit 
sma = R + data.alt; 
ecc = 0; 
if (data.sso == 1)
  inc = CL_op_ssoJ2 ("i", sma, ecc); 
else
  inc = data.inc; 
end   

// acceleration 
max_acc = (data.max_thrust_values / 1000) / data.mass; 

// ratio (% time when propulsion is used)
r = linspace(data.dcminmax_percent(1), data.dcminmax_percent(2), 101) / 100;  


// --------------------------------------------------------
// Computation
// (continuous approximation) 
// --------------------------------------------------------

// Drift due to J2 
[pomdot, gomdot, anmdot, dpomdotdaei, dgomdotdaei, danmdotdaei] = CL_op_driftJ2(sma, ecc, inc); 
psodot = pomdot + anmdot; 
dgomdotda = dgomdotdaei(1); 
dgomdotdi = dgomdotdaei(3); 
dpsodotda = dpomdotdaei(1) + danmdotdaei(1); 
dpsodotdi = dpomdotdaei(3) + danmdotdaei(3); 


// Gauss coefficients (impulsive maneuvers)
// positive coefficient (ascending node) 
// da/dt = Ka * gammaT
// di/dt = Ki * gammaN
V = sma * psodot; 
Ka = (2 * sma / V);  
Ki = 1 / V;   


// maneuvers efficiency coefficients (sma, inc)
function [Ceffa, Ceffi] = compute_eff(r)
  Ceffa = 1 * ones(r); 

  // 1 thrust interval per orbit => sin(alpha/2) / (alpha/2)
  // 2 intervals => sin(alpha/4) / (alpha/4)
  // sinc(x) = sin(x) / x

  alpha = r * 2 * %pi; 
  if (data.num_thrust_orb == 1)
    Ceffi = sinc(alpha / 2); 
  else 
    Ceffi = sinc(alpha / 4); 
  end 
endfunction 

[Ceffa, Ceffi] = compute_eff(r); 
  
// angtw = angle in tangential - normal plane 
// d(gomdot)/dt = (Sa * dgomdotda * Ka * Ceffa * cos(angtw) + 
//                 Si * dgomdotdi * Ki * Ceffi * sin(angtw)) * max_accel; 

// optimal angle (maximizing d(gomdot)/dt (positive)
// d(gomdot)/dt = sqrt(Ca^2 + Ci^2) * cos(angtw - angtw0) 
// with: cos(angtw0) = Ca / sqrt(), sin(angtw0) = Ci/sqrt() 
// => optimal value of angtw = angtw0 

if (data.thrust_angle_mode == 1)
  angtw = 0 * ones(r); 

elseif (data.thrust_angle_mode == -1)
  angtw = %pi * ones(r); 

elseif (data.thrust_angle_mode == 2)
  angtw = (%pi/2) * ones(r); 

elseif (data.thrust_angle_mode == -2)
  angtw = (-%pi/2) * ones(r); 

elseif (data.thrust_angle_mode == 3)
  Ca = dgomdotda * Ka * Ceffa; 
  Ci = dgomdotdi * Ki * Ceffi; 
  angtw = atan(Ci, Ca); 

elseif (data.thrust_angle_mode == -3)
  Ca = dgomdotda * Ka * Ceffa; 
  Ci = dgomdotdi * Ki * Ceffi; 
  angtw = %pi + atan(Ci, Ca); 
  
elseif (data.thrust_angle_mode == 4)
  Ca = dgomdotda * Ka * Ceffa; 
  Ci = dgomdotdi * Ki * Ceffi; 
  angtw = atan(-Ca, Ci); 

elseif (data.thrust_angle_mode == -4)
  Ca = dgomdotda * Ka * Ceffa; 
  Ci = dgomdotdi * Ki * Ceffi; 
  angtw = %pi + atan(-Ca, Ci); 
  
else // if (data.thrust_angle_mode == 0)
  angtw = data.thrust_angle * ones(r); 

end 

adot = max_acc' * (Ka * Ceffa .* cos(angtw) .* r); // * sign(gammaT)
idot = max_acc' * (Ki * Ceffi .* sin(angtw) .* r); // * sign(gammaN) * cos(pso)
dgomdotdt = adot .* dgomdotda + idot .* dgomdotdi; 
dpsodotdt = adot .* dpsodotda + idot .* dpsodotdi; 
acc = max_acc' * (r .* Ceffa); 

// compute cumulative effects 

if (data.time_unit == 1)
  DT = 86400; 
  time_unit_str = "day"; 
else 
  DT = 2 * %pi / psodot;
  time_unit_str = "orbit period";   
end 

delta_sma = adot * DT; 
delta_inc = idot * DT; 
delta_gomdot = dgomdotdt * DT; 
delta_psodot = dpsodotdt * DT; 
dv = acc * DT; 

// --------------------------------------------------------
// Plots 
// --------------------------------------------------------

function PLOT(x, y, t, ty, leg)
  N = size(y, 1); 
  f = scf(); 
  f.immediate_drawing = "off"; 
  f.color_map = CL_colormap("jet", max(3, N)); 
  for k = 1 : N
    plot(x * 100, y(k,:), "thickness", 2); 
	e = CL_g_select(gce(), "Polyline"); 
	e.foreground = k; 
  end 
  a = gca(); 

  a.title.text = t;  
  a.x_label.text = "Duty cycle (%)"; 
  a.y_label.text = ty; 
  if (leg)
    thrust_str = msprintf("%.10g\n", data.max_thrust_values')'; 
    CL_g_legend(a, thrust_str, header = "Thrust (mN)"); 
  end
  a.tight_limits = ["on", "off", "off"]; 
  CL_g_stdaxes(a); 
  f.immediate_drawing = "on"; 
  f.visible = "on"; 
endfunction 


if (data.plots(1) == 1 & abs(data.thrust_angle_mode) <> 2)
  PLOT(r, delta_sma, "Variation of semi-major axis per " + time_unit_str, "Variation (m)", %t); 
end 

if (data.plots(2) == 1 & abs(data.thrust_angle_mode) <> 1)
  PLOT(r, delta_inc * %CL_rad2deg, "Variation of inclination per " + time_unit_str, "Variation (deg)", %t);
end 

if (data.plots(3) == 1)
  PLOT(r, delta_gomdot * 86400 * %CL_rad2deg, "Variation of d(RAAN)/dt per " + time_unit_str, "Variation (deg/day)", %t); 
end 

if (data.plots(4) == 1)
  PLOT(r, delta_psodot * 86400 * %CL_rad2deg, "Variation of d(PSO)/dt per " + time_unit_str, "Variation (deg/day)", %t); 
end 

if (data.plots(5) == 1)
  PLOT(r, dv, "Consumption per " + time_unit_str, "DV (m/s)", %t); 
end 

if (data.plots(6) == 1)
  PLOT(r, angtw * %CL_rad2deg, "Thrust angle (ascending node)", "Thrust angle (deg)", %f); 
end 



