//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Maintenance of MLTAN (SSO, frozen orbit)
//> The orbit is circular, frozen and Sun-synchronous, and is defined by:
//> - altitude = semi-major axis minus equatorial radius, and 
//> - MLTAN = mean local time of ascending node. 
//> 
//> MLTAN is kept close to the nominal value by performing inclination maneuvers every T days. 
//> 
//> The orbit is first adjusted so as to be frozen and Sun-synchronous. 
//> Inclination and RAAN are then adjusted at the initial time so that no station keeping maneuver
//> is necessary at this time.  
//> A simple algorithm is then used to compute inclination increments every T days. 
//> Only 3rd body (Sun and Moon), apparent acceleration and solid tides perturbations can be considered. 
//> 
//> Notes: 
//> - Short periods are not simulated
//> - The algorithm may not be optimal. It is just a realistic enough example. 
//> - The orbit is propagated using STELA (mean elements). 
//>  
//
// Author: A. Lamy
// -----------------------------------------------------------

// ===========================================================
// Miscellaneous utility functions 
// ===========================================================

// -----------------------------------------------------------
// reglin - deg 2
// t and x: 1xN
// c2, c1, c0: coef of degree 2, 1, 0
// -----------------------------------------------------------
function [c2, c1, c0] = reglin2(t, x)
  M = [ones(t); t; t.^2]; 
  X = lsq(M', x'); 
  c2 = X(3); 
  c1 = X(2); 
  c0 = X(1); 
endfunction


// -----------------------------------------------------------
// d(hdot)/d(inc) (h: MLTAN = mean local time of ascending node) 
// result in h/day/rad
// cst: STELA constants
// -----------------------------------------------------------
function [dhdotdi] = calc_dhdotdi(mean_cir0, cst)
  [pomdot,gomdot,anmdot ,dpomdotdaei,dgomdotdaei,danmdotdaei] = CL_op_driftJ2(mean_cir0(1,:),CL_norm(mean_cir0(2:3,:)),mean_cir0(4,:), er=cst.er, mu=cst.mu, j2=cst.j1jn(2)); 
  dhdotdi = dgomdotdaei(3,:) * %CL_rad2deg * (24 / 360) * 86400; 
endfunction 

// -----------------------------------------------------------
// SSO inclination (J2)
// cst: STELA constants
// -----------------------------------------------------------
function [inc] = calc_inc_ssoJ2(sma, ex, ey, cst)
  inc = CL_op_ssoJ2("i", sma, CL_norm([ex;ey]), er=cst.er, mu=cst.mu, j2=cst.j1jn(2)); 
endfunction 

// -----------------------------------------------------------
// Propagates with STELA and computes MLTAN
// params: STELA model parameters
// -----------------------------------------------------------
function [mlh, mean_cir] = extrap_hloc(cjd0, mean_cir0, cjd, params)
  mlh0 = CL_unMod(CL_op_locTime(cjd0, "ra", mean_cir0(5), "mlh"), 24); 
  mean_cir = CL_stela_extrap("cir", cjd0, mean_cir0, cjd, params, ["m"]);
  mlh = CL_unMod(CL_op_locTime(cjd, "ra", mean_cir(5,:), "mlh"), 24); 
  mlh = mlh - mlh(1) + mlh0; 
endfunction 

// -----------------------------------------------------------
// Adjust inclination and eccentricity vector (nominal orbit)
// D: propagation duration (days) 
// params: STELA model parameters
// -----------------------------------------------------------
function [mean_cir0] = adjust_sso_frozen(cjd0, mean_cir0, params, D, mlh0, dhdotdi)
  cjd = cjd0 + (0 : D);  
  for iter = 1 : 3
    [mlh, mean_cir] = extrap_hloc(cjd0, mean_cir0, cjd, params); 
	  
    [c1, c0] = reglin(cjd-cjd0, mlh - mlh0); 
    delta_inc = -c1 / dhdotdi; 
    mean_cir0(4) = mean_cir0(4) + delta_inc; 
	mean_cir0(2) = (min(mean_cir(2,:)) + max(mean_cir(2,:))) / 2; 
	mean_cir0(3) = (min(mean_cir(3,:)) + max(mean_cir(3,:))) / 2; 
  end
endfunction

// -----------------------------------------------------------
// Adjust initial state (mean_cir0)
// D: propagation duration (days) 
// T: cycle duration (days) 
// params: STELA model parameters
// -----------------------------------------------------------
function [mean_cir0] = adjust_initcond(cjd0, mean_cir0, D, T, params, mlh0, dhdotdi)
  cjd = cjd0 + (0 : D); 

  for (iter = 1 : 2)
    mlh = extrap_hloc(cjd0, mean_cir0, cjd, params); 
    [c2, c1, c0] = reglin2(cjd-cjd0, mlh - mlh0); 

    f = c2 * T^2 / 8; 
    delta_mlh = f - c0; 
    delta_mlhdot = - c2 * T - c1; 
    delta_inc = delta_mlhdot / dhdotdi; 
    delta_gom = delta_mlh * (360 / 24) * %CL_deg2rad; 
    mean_cir0(4) = mean_cir0(4) + delta_inc; 
    mean_cir0(5) = mean_cir0(5) + delta_gom;
  end 
  
endfunction

// -----------------------------------------------------------
// Adjust inclination (station keeping)
// D: propagation duration (days) 
// T: cycle duration (days) 
// params: STELA model parameters
// -----------------------------------------------------------
function [mean_cir0] = adjust_inc_SK(cjd0, mean_cir0, D, T, params, mlh0, dhdotdi)
  cjd = cjd0 + (0 : D); 

  for (iter = 1 : 2)
    mlh = extrap_hloc(cjd0, mean_cir0, cjd, params); 
    [c2, c1, c0] = reglin2(cjd-cjd0, mlh - mlh0); 

    f = c2 * T^2 / 8; 

    delta_c1 = (f - (c0 + c1 * T + c2 * T^2)) / T; 
    delta_inc = delta_c1 / dhdotdi; 
    mean_cir0(4) = mean_cir0(4) + delta_inc; 
  end 

endfunction

// -----------------------------------------------------------
// Station keeping
// D: propagation duration (days) 
// T: cycle duration (days) 
// res = struct: cjd, mean_cir, mlh, dinc (increments), incvar (nat variation)
// params: STELA model parameters
// -----------------------------------------------------------
function [res] = simulate_SK(cjd0, mean_cir0, D, T, params, mlh0, dhdotdi, nsk)
  cjd1 = cjd0; 
  mean_cir1 = mean_cir0; 
  
  res = struct(); 
  res.cjd = []; 
  res.mean_cir = []; 
  res.mlh = []; 
  res.dinc = []; 
  res.incvar = []; 

  wait_win = waitbar("In progress ...");
  
  for iter = 1 : nsk
    inc = mean_cir1(4); 
    mean_cir1 = adjust_inc_SK(cjd1, mean_cir1, D, T, params, mlh0, dhdotdi); 
    res.dinc = [res.dinc, mean_cir1(4) - inc]; 
	
    cjd = cjd1 + (0 : T); 
    [mlh, mean_cir] = extrap_hloc(cjd1, mean_cir1, cjd, params); 
    res.incvar = [res.incvar, mean_cir(4,$) - mean_cir1(4)]; 
	
    res.cjd = [res.cjd, cjd]; 
	res.mean_cir = [res.mean_cir, mean_cir]; 
	res.mlh = [res.mlh, mlh]; 

    cjd1 = cjd($); 
    mean_cir1 = mean_cir(:,$); 
    cjd = cjd1 + (0 : T); 

    waitbar(iter / nsk, wait_win);
  end
  
  sleep(500); 
  close(wait_win); 

endfunction

// -----------------------------------------------------------
// check that date is valid
// -----------------------------------------------------------
function [ok] = isValidDate(scal)  
  ok = %f; 
  cal = CL_dat_str2cal(scal); 
  if (cal(1) <> %nan)
    ok = ( cal(1) >= 2000 & cal(2) <= 2100);
  end
endfunction


// ===========================================================
// Inputs  
// ===========================================================

cal = [2020; 1; 1];
scal = CL_dat_cal2str(cal, nd=-1);  
alt = 700.e3;  
mlh0 = 10.5; 
T = 365; // days
nsk = 5; 
ipert = 2; 

desc_param = list(..
   CL_defParam("Initial date (calendar format, TREF time scale)", scal, typ='cal', valid="isValidDate($x)"),..
   CL_defParam("Altitude (semi-major axis minus equ. radius)", alt, units=['m', 'km'], valid='$x>=0 & $x < 5.e4'),..
   CL_defParam("MLTAN", mlh0, units=["h"], mlh0, valid="$x >= 0 & $x <= 24"), ..
   CL_defParam("Time between inclination increments", T, units=["day"], valid="$x >= 5 & $x <= 366 * 5"), ..
   CL_defParam("Number of station keeping cycles", nsk, accv=1:50), ..
   CL_defParam("Perturbations: 1=Sun only (3rd body), 2=all available", ipert, accv=[1,2]) ..
);

[scal, alt, mlh0, T, nsk, ipert] = CL_inputParam(desc_param);


// ===========================================================
// Computation  
// ===========================================================
// Eccentricity vector
ex = 0; 
ey = 0; 

// Constants from STELA
cst = CL_stela_getConst(); 

sma = alt + %CL_eqRad; 
inc = calc_inc_ssoJ2(sma, ex, ey, cst); 

cjd0 = CL_dat_cal2cjd(CL_dat_str2cal(scal));
gom = CL_op_locTime(cjd0, "mlh", mlh0, "ra"); 

// Keplerian mean orbital elements (frame: ECI)
mean_cir0 = [sma; ex; ey; inc; gom; 0];
dhdotdi = calc_dhdotdi(mean_cir0, cst); 

params = CL_stela_params("none");
params.central_enabled = %t; 
params.zonal_enabled = %t; 
params.tesseral_enabled = %t; 

// Duration for estimation of polynomial
D = T * 2; 

// Ajustement of nominal orbit (1 year = D0) - No 3rd body perturbation
D0 = 365; 
mean_cir0 = adjust_sso_frozen(cjd0, mean_cir0, params, D0, mlh0, dhdotdi); 
inc_nom = mean_cir0(4); 

// Now enable 3rd body perturbation + apparent acceleration + tides
params.thirdBody_enabled = %t; 
if (ipert == 1)
  params.thirdBody_bodies = ["Sun"]; 
else
  params.thirdBody_bodies = ["Sun", "Moon"]; 
  params.appAcc_enabled = %t; 
  params.solidTides_enabled = %t; 
end

// initial orbit 
mean_cir0 = adjust_initcond(cjd0, mean_cir0, D, T, params, mlh0, dhdotdi); 

// SK simulation 
res = simulate_SK(cjd0, mean_cir0, D, T, params, mlh0, dhdotdi, nsk); 

mprintf("-----------\n"); 
mprintf("Total inclination increment (deg)   = %9.2e\n", sum(res.dinc) * %CL_rad2deg); 
mprintf("Total |inclination increment| (deg) = %9.2e\n", sum(abs(res.dinc)) * %CL_rad2deg); 
mprintf("Total inclination nat. variation (deg)   = %9.2e\n", sum(res.incvar) * %CL_rad2deg); 
mprintf("Total |inclination nat. variation| (deg) = %9.2e\n", sum(abs(res.incvar)) * %CL_rad2deg); 


// ===========================================================
// Plot  
// ===========================================================

// Plot1 : MLTAN
f = scf();
f.visible = "on"; 
f.immediate_drawing = "off"; 
plot(res.cjd - cjd0, zeros(res.cjd), "r", "thickness", 2);  
plot(res.cjd - cjd0, (res.mlh - mlh0) * 60) 

a = gca(); 
a.title.text = "MLTAN maintenance"; 
a.x_label.text = "Time (days)"; 
a.y_label.text = msprintf("MLTAN minus %.1fh (min)", mlh0); 
CL_g_stdaxes(a); 
f.immediate_drawing = "on"; 

// Plot2 : Inclination
f = scf();
f.visible = "on"; 
f.immediate_drawing = "off"; 
plot(res.cjd - cjd0, inc_nom * ones(res.cjd) * %CL_rad2deg, "r", "thickness", 2);  
plot(res.cjd - cjd0, (res.mean_cir(4,:)) * %CL_rad2deg);  

a = gca(); 
a.title.text = "MLTAN maintenance"; 
a.x_label.text = "Time (days)"; 
a.y_label.text = "Inclination (deg)"; 
CL_g_stdaxes(a); 
f.immediate_drawing = "on"; 

