//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Maintenance of RAAN or MLTAN (SSO) - window size
//> The orbit is circular and Sun-synchronous, and is defined by:
//> - altitude = semi-major axis minus equatorial radius, and 
//> - MLTAN = mean local time of ascending node. 
//> 
//> RAAN (or MLTAN) is kept in a window [-w, w] by performing maneuvers at a frequency of 1 / T.
//> The value w is evaluated as a function of T. w is expressed in degrees for RAAN or 
//> minutes for MLTAN (1 degree <=> 4 * minutes).  
//> 
//> Warning: The computation is based on a linear evolution of inclination as a function of time 
//> (the motion of the controlled parameter is then parabolic and centered around 0 between -w and w). 
//> Other (periodic) effects due to Sun and Moon (third body perturbation) are not taken into account. 
//> Thus the window size may be inaccurate if the maneuver period is much smaller than 1 year. 
//> 
//
// Author: A. Lamy
// -----------------------------------------------------------

// -----------------------------------------------------------
// Inputs 
// -----------------------------------------------------------
alt = 650.e3; 
tman = 365 * 86400; // seconds
iparam = 1; // RAAN 

desc_param = list(..
   CL_defParam("Altitude (semi-major axis minus equ. radius)", alt, units=['m', 'km'], valid='$x>=0 & $x < 5.e4'),..
   CL_defParam("Time between maneuvers", tman, units=["s", "day"], valid="$x >= 1 & $x <= 5 * 365.25"), ..
   CL_defParam("Controlled parameter: 1=RAAN, 2=MLTAN", iparam, accv=[1,2]) ..
);

[alt, tman, iparam] = CL_inputParam(desc_param);

// -----------------------------------------------------------
// Computation 
// -----------------------------------------------------------
sma = %CL_eqRad + alt; 

mltan = linspace(0, 24, 4*50 + 1); 
ecc = 0; 

inc = CL_op_ssoJ2 ("i", sma, ecc); 
[pomdot, gomdot, anmdot, dpomdotdaei, dgomdotdaei, danmdotdaei] = CL_op_driftJ2(sma, ecc, inc);

dgomdotdi = dgomdotdaei(3); 
incdot = CL_op_incdotSsoCirc(mltan, sma);

// window size in radians 
w = abs(dgomdotdi * incdot) * tman^2 / 16; 


// -----------------------------------------------------------
// Plot 
// -----------------------------------------------------------

unitf = %CL_rad2deg; 
unitstr = "deg"; 
paramstr = "RAAN"; 

if (iparam == 2)
  unitf = %CL_rad2deg * 4; 
  unitstr = "min"; 
  paramstr = "MLTAN"; 
end

f = scf();
f.visible = "on";
f.immediate_drawing="off";

plot(mltan, w * unitf, "thickness", 2);

a = gca(); 
a.title.text = msprintf("Maintenance of %s - Man. period = %.0f days", paramstr, tman / 86400); 
a.x_label.text = "Mean local time of ascending node (h)"; 
a.y_label.text = msprintf("Window size (%s) - %s kept in [-window, +window]", unitstr, paramstr); 

CL_g_stdaxes(a);

f.immediate_drawing="on";
f.visible = "on";

