//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Actual equator crossings:
//> Computes the effect of the Earth potential (tesseral terms) on the 
//> equator crossing longitudes and times (ascending nodes only).
//> 
//> The orbit is a nearly circular repeat orbit charaterized by the repeat 
//> parameters: (N, P, Q). It can be Sun-synchronous or not. 
//> 
//> The plots show the gaps between actual longitudes / times at the ascending 
//> nodes and the respective corresponding evenly spaced values.  
//> 
//
// Author: A. Lamy
// -----------------------------------------------------------

// ----------------------------
// Inputs
// ----------------------------
// Repeat parameters (N,P,Q)
npq = [12, 7, 10];
// SSO or not (0 = no)
sso = 0; 
// Inclination is not Sun-synchronous
inc = 66 * %CL_deg2rad; 
// reference longitude (where delta longitude/time is 0) 
lon0 = 99.9249 * %CL_deg2rad; 
// Max degree/order of gravitational potential
nmax = 36;
// Plot
plot_what = 1; 


// max degree of potential (NB: %cs1nm must be defined) 
NMAX = max(size(CL_dataGet("cs1nm")));

// Check (N, P, Q) is valid  
function [ok] = valid_npq(NPQ)
  [N,P,Q] = (NPQ(1), NPQ(2), NPQ(3)); 
  ok = ( (N == round(N) & N > 0) & ..
         (P == round(P) & P >= 0 & P < Q) & ..
         (Q == round(Q) & Q > 0 & ((P == 0 & Q == 1) | CL_gcd(P, Q) == 1)) ); 
endfunction  


desc_param = list(..
   CL_defParam("Repeat orbit parameters (N,P,Q)", npq, dim=3, valid='valid_npq($x)' ),..
   CL_defParam("Sun-synchronous orbit? (1=yes, 0=no)", sso, id='$sso', accv=[0,1], valid="$sso==0 | $sso==1"),..
   CL_defParam("Inclination (if not Sun-synchronous)", inc, id="$inc", units=['rad', 'deg'], valid="$sso == 1 | ($x >=0 & $x<=180)"),..
   CL_defParam("Reference longitude", lon0, units=["rad", "deg"]), ..
   CL_defParam("Max degree and order of potential (>=2 and <=" + msprintf("%d",NMAX) + ")", nmax, accv=linspace(2,NMAX,NMAX-1)),..
   CL_defParam("Plot effect on: 1=ground track, 2=time", plot_what, accv=[1,2])..
);

[npq, sso, inc, lon0, nmax, plot_what] = CL_inputParam(desc_param);
[N,P,Q] = (npq(1), npq(2), npq(3)); 
lon0 = CL_rMod(lon0, 0, 2*%pi); 

// ----------------------------
// Internal functions
// ----------------------------

// Integrate Gauss equation (circular elements) 
function [dcir] = integrate_gauss(cir0, t, dcirdt)
  N = size(t, 2); 
  sma = cir0(1); 
  ecc = CL_norm(cir0(2:3)); 
  inc = cir0(3); 
  [pomdot, gomdot, anmdot, dpomdotdaei, dgomdotdaei, danmdotdaei] = ... 
       CL_op_driftJ2(sma, ecc, inc);
  
  dcir = CL_integrate(t, dcirdt, meth="sl4");
  
  // remove average
  dcir = dcir - repmat(CL_mean(dcir, "simp", "c"), 1, N); 
  
  dsma = dcir(1,:); 
  decc = 0; 
  dinc = dcir(4,:);
  
  dpomdt = dpomdotdaei(1,:) .* dsma + dpomdotdaei(2,:) .* decc + dpomdotdaei(3,:) .* dinc;
  dgomdt = dgomdotdaei(1,:) .* dsma + dgomdotdaei(2,:) .* decc + dgomdotdaei(3,:) .* dinc;
  danmdt = danmdotdaei(1,:) .* dsma + danmdotdaei(2,:) .* decc + danmdotdaei(3,:) .* dinc;
  
  // add indirect effects due to J2 (sma and inc only)
  dcir(5,:) = dcir(5,:) + CL_integrate(t, dgomdt, meth="sl4");
  dcir(6,:) = dcir(6,:) + CL_integrate(t, dpomdt + danmdt, meth="sl4");
    
endfunction


// ----------------------------
// Computations
// ----------------------------

// Earth rotation rate
OMEGA = CL_dataGet("rotrBody"); 

// --- Reference orbit/ephemeris computed using J2 only (secular effect)
ecc = 0.; // circular
[sma, inc] = CL_op_repeat2smaInc(N, P, Q, ecc, sso, inc); 
pom = %pi/2; // arg of periapsis

// Frame = inertial frame indentical to terrestrial frame at t = 0
cir0 = [sma; ecc*cos(pom); ecc*sin(pom); inc; lon0; 0]; 

[pomdot, gomdot, anmdot] = CL_op_driftJ2(sma, ecc, inc); 
psodot = pomdot + anmdot; 

// orbit period (s) 
T = 2*%pi / psodot; 
Norbcycle = N * Q + P;
Tcycle = T * Norbcycle; 

// Times (s) during one repeat cycle
t = linspace(0, Tcycle, 100*Norbcycle+1);

cir = CL_ex_propagate("j2sec", "cir", 0, cir0, t/86400, "m"); 
pos_ECI = CL_oe_cir2car(cir); 

// inertial -> terrestrial transformation matrix
R = CL_rot_angles2matrix(3, OMEGA * t); 
pos_ECF = R * pos_ECI;


// --- Effect of potential (tesseral terms only)
Gauss = CL_op_gauss("cir", cir, "inertial"); 

// zonals set to 0
j1jn = CL_dataGet("j1jn"); 
j1jn(:) = 0; 

// Acceleration and effect on orbital elements
acc_ECF = CL_fo_sphHarmAcc(pos_ECF, [nmax,nmax], j1jn = j1jn);
acc_ECI = R' * acc_ECF; 
dcirdt = Gauss * acc_ECI; 

dcir = integrate_gauss(cir0, t, dcirdt);


// Effect of longitude/time at ascending nodes 
// NB: eccentricity: comes from: v-M = 2 * e * sin(M) 
// = 2 * e * sin(pso-pom) = -2 * ey for pso = 2*k*pi
dt = -(dcir(6,:) - 2 * dcir(3,:)) / psodot; 
dlon = dcir(5,:) - dt *  (OMEGA - gomdot); 

t_n = (0 : N * Q + P - 1) * T; 
num_n = 1 : length(t_n); 

lon_n = CL_rMod(lon0 - t_n * (OMEGA - gomdot), 0, 2*%pi);
dlon_n = CL_interpLagrange(t, dlon, t_n); 
dlon_n = dlon_n - dlon_n(1); 

dt_n = CL_interpLagrange(t, dt, t_n);
dt_n = dt_n - dt_n(1); 

[lon_n, I] = gsort(lon_n, "c", "i"); 
dlon_n = dlon_n(I);  
dt_n = dt_n(I);  
num_n = num_n(I); 


// ----------------------------
// Plots
// ----------------------------

f = scf();
f.visible="on";
f.immediate_drawing="off";
a = gca(); 

txt = msprintf("NPQ = (%d,%d,%d)", N, P, Q); 
if (plot_what == 1)
  plot([0, 360], [0, 0], "r", "thickness", 2); 
  plot(lon_n*%CL_rad2deg, dlon_n * %CL_eqRad, "thickness", 1); 
  plot(lon_n*%CL_rad2deg, dlon_n * %CL_eqRad, "o"); 
  h = CL_g_select(gce(), "Polyline"); 
  h.mark_size = 3; 
  h.mark_background = 2; 
  xtitle("Effect on ground track at ascending nodes - " + txt, "Longitude (deg)", "Distance on Earth surface (m)"); 
  
else
  plot([0, 360], [0, 0], "r", "thickness", 2); 
  plot(lon_n*%CL_rad2deg, dt_n, "thickness", 1); 
  plot(lon_n*%CL_rad2deg, dt_n, "o"); 
  h = CL_g_select(gce(), "Polyline"); 
  h.mark_size = 3; 
  h.mark_background = 2; 
  xtitle("Effect on time at ascending nodes - " + txt, "Longitude (deg)", "Time difference (s)"); 

end

a.data_bounds = [0, a.y_ticks.locations(1); 360, a.y_ticks.locations($)]; 
a.tight_limits = "on"; 

CL_g_stdaxes();
f.immediate_drawing="on";
f.visible="on";



