//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Atmospheric density given by MSIS2000 model
//> for given date/time and altitude as function of local time and 
//> latitude. 
//
// Author: A. Lamy
// -----------------------------------------------------------

// ===========================================================
// MAIN
// ===========================================================

// date
scal = CL_dat_cal2str([2000;6;21]); 

// altitude (m)
alt = 600.e3; 

// flux
f107 = 150;

// ap
ap = 15 

desc_param = list(..
  CL_defParam("Date (calendar format, TREF time scale)", scal, typ='cal'), ...
  CL_defParam("Geodetic altitude", alt, units=['m', 'km'], valid="$x >= 0 & $x < 10000"), ...
  CL_defParam("Solar flux (10.7 cm)", f107, units='sfu', valid="$x >= 50 & $x <= 400"), ...
  CL_defParam("Geomagnetic index (Ap) - daily value", ap, valid="$x >= 0 & $x <= 400") ...
);

[scal, alt, f107, ap] = CL_inputParam(desc_param);


// ------------------------------------
// Computations 
// ------------------------------------
// date (cjd)
cjd = CL_dat_cal2cjd(CL_dat_str2cal(scal));

// latitude and local solar time
lst = linspace(0, 24, 51); // x axis
lat = linspace(-%pi/2, %pi/2, 50); // y axis

[Lst, Lat] = ndgrid(lst, lat); 
all_lat = matrix(Lat, 1, -1); 
all_lst = matrix(Lst, 1, -1); 

// longitude from date and (true) local time
all_lon = CL_op_locTime(cjd, "tlh", all_lst, "lon"); 

f107a = f107; 

dens = CL_mod_atmMSIS00(cjd, all_lon, all_lat, alt, f107, f107a, ap, res="dens"); 


// -----------------
// Plot
// -----------------

f = scf(); 
f.visible = "on"; // NB: "on" necessary if contour2d is used (Scilab bug)
f.immediate_drawing = "off"; 

nbcol = 256; 
f.color_map = CL_colormap("jet", nbcol); 

a = gca(); 

// power in common for plot (bug in Scilab 6.1)
power10 = round(log10(min(dens))); 

// plotted quantity
Z = matrix(dens, size(Lst)) / (10^power10); 

zmin = min(Z); 
zmax = max(Z); 

// colorbar (change graduations: log -> natural)
colorbar(zmin, zmax, [1, nbcol]); 
// cb: type = Axes in Scilab 6.1, but was a child before 
cb = gce(); 
if (cb.type <> "Axes"); cb = cb.parent; end 
cb.font_size = 1.5; 
cb.title.text = msprintf("Density (1.e%d kg/m^3)", power10); 
cb.title.font_size = 2;

x = lst; 
y = lat * %CL_rad2deg; 
Sgrayplot(x, y, Z, zminmax = [zmin, zmax], colminmax = [1, nbcol]); 

a.data_bounds = [0, -90; 24, 90]; 
a.tight_limits = "on"; 

text1 = msprintf("alt=%.1f km", alt/1000); 
scal_toks = tokens(scal); 
text2 = msprintf("date=%s", part(scal, 1:16)); 

a.title.text = "MSIS2000 atmospheric density - " + text1 + ", " + text2; 
a.x_label.text = "Local solar time (h)"; 
a.y_label.text = "Latitude (deg)"; 

CL_g_stdaxes(a); 

f.immediate_drawing = "on"; 
f.visible = "on"; 
