//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Atmospheric density given by MSIS2000 model along an orbit path. 
//> 
//> The orbit model used is "lydlp": Lyddane model for which mean orbital elements include 
//> long-term perturbation effects. 
// 
// Author: A. Lamy
// -----------------------------------------------------------

// date
scal = CL_dat_cal2str([2000;6;21]); 

// (mean) perigee altitude (m)
hp = 400.e3; 

// (mean) apogee altitude (m)
ha = 600.e3; 

// Inclination (rad)
inc = 98 * %CL_deg2rad; 

// Mean local time of the ascending node (h)
mltan = 14; 

// flux
f107 = 150;

// ap
ap = 15;

// x_axis
x_axis = 1;


desc_param = list(..
  CL_defParam("Date (calendar format, TREF time scale)", scal, typ='cal'), ...
  CL_defParam("Mean Perigee altitude (hp minus eq. radius)", hp, id="$hp", units=['m', 'km'], valid="$x >= 0"), ...
  CL_defParam("Mean Apogee altitude (ha minus eq. radius)", ha, units=['m', 'km'], valid="$x >= 0 & $x >= $hp"), ...
  CL_defParam("Mean inclination", inc, units=['rad', 'deg'], valid="$x >= -90 & $x <= 180"), ...
  CL_defParam("Mean local time of the ascending node", mltan, units=['h'], valid="$x >= 0 & $x <= 24"), ...
  CL_defParam("Solar flux (10.7 cm)", f107, units='sfu', valid="$x >= 50 & $x <= 400"), ...
  CL_defParam("Geomagnetic index (Ap) - daily value", ap, valid="$x >= 0 & $x <= 400"), ...
  CL_defParam("Plot x-axis: 1=true anomaly, 2=true argument of latitude", x_axis, accv=[1,2]) ...
);

[scal, hp, ha, inc, mltan, f107, ap, x_axis] = CL_inputParam(desc_param);

cjd0 = CL_dat_cal2cjd(CL_dat_str2cal(scal)); 

// ------------------------------------
// Internal function 
// ------------------------------------

// kep: mean elements ("lydlp")
function [dens] = compute_density(cjd, kep, f107, ap)
  
  kep = CL_ex_mean2osc("lydlp", "kep", kep); 
  
  [pos, vel] = CL_oe_kep2car(kep); 
  pos_ecf = CL_fr_convert("ECI", "ECF", cjd, pos); 
    
  ell = CL_co_car2ell(pos_ecf); 

  f107a = f107;

  dens = CL_mod_atmMSIS00(cjd, ell(1,:), ell(2,:), ell(3,:), f107, f107a, ap, res="dens"); 
  
endfunction


// ------------------------------------
// Computations 
// ------------------------------------

[sma, ecc] = CL_kp_apsConvert("ra", ha + %CL_eqRad, "rp", hp + %CL_eqRad, ["sma", "ecc"]); 
gom = CL_op_locTime(cjd0, "mlh", mltan, "ra");

n = sqrt(%CL_mu / sma^3); 
T = 2*%pi / n; 

pom = (0 : 45 : 360-1) * %CL_deg2rad; 
ang = linspace(-%pi, %pi, 201);


[M_pom, M_ang] = ndgrid(pom, ang); 
all_pom = matrix(M_pom, 1, -1); 
all_ang = matrix(M_ang, 1, -1); 

if (x_axis == 1)
  anv = all_ang; 
  anv0 = 0; 
else
  anv = all_ang - all_pom; 
  anv0 = -all_pom; 
end

anm = CL_kp_v2M(ecc, anv); 
anm0 = CL_kp_v2M(ecc, anv0); 
t = (anm - anm0) / n; 

kep0 = [sma; ecc; inc; 0; gom; 0]; 
kep = repmat(kep0, 1, length(t)); 
kep(4,:) = all_pom; 
kep(6,:) = anm; 
  
cjd = cjd0 + t / 86400; 
dens = matrix(compute_density(cjd, kep, f107, ap), size(M_ang)); 

// compute mean values (time average)
Anv = matrix(anv, size(M_ang));
dMdv = (1 - ecc.^2).^(1.5) ./ (1 + ecc .* cos(Anv)).^2;      
dens_mean = CL_mean(dens .* dMdv, meth="simp", "c");


// -----------------
// Display results
// -----------------
mprintf("\n-----------------------\n"); 
mprintf("MSIS2000 - Density (kg/m^3), min/max/average for each perigee argument (deg): \n"); 

for k = 1 : size(dens, 1)
  mprintf("%6.1f: %10.2e %10.2e %10.2e\n", pom(k) * %CL_rad2deg, min(dens(k,:)), max(dens(k,:)), dens_mean(k)); 
end


// -----------------
// Plot
// -----------------

f = scf(); 
f.visible = "on"; 
f.immediate_drawing = "off"; 
f.axes_size = [700, 450]; 

N = size(dens, 1); 

f.color_map = CL_colormap("jet", max(N,3)); 
a = gca(); 

for k = 1 : N
  plot(ang * %CL_rad2deg, dens(k,:), "thickness", 2); 
  h = CL_g_select(gce(), "Polyline"); 
  h.foreground = k; 
end

a.title.text = "MSIS2000 atmospheric density"; 
if (x_axis == 1)
  a.x_label.text = "True anomaly (deg)"; 
else
  a.x_label.text = "True argument of latitude (deg)"; 
end
a.y_label.text = "Atmospheric density (kg/m^3)"; 

CL_g_stdaxes(a); 


dens_min = min(dens); 
dens_max = max(dens); 

a.data_bounds = [-180, a.y_ticks.locations(1); 180, a.y_ticks.locations($)]; 
a.tight_limits = "on"; 

if (dens_max > dens_min * 50)
  a.data_bounds = [-180, dens_min * 0.8; 180, dens_max * 1.2]; 
  a.log_flags = "nln";
end

CL_g_legend(a, string(pom * %CL_rad2deg), header="Arg. of perigee (deg)"); 

f.immediate_drawing = "on"; 
f.visible = "on"; 



