//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Atmospheric density given by MSIS2000 model
//> in a region between delimited by 2 spheres, for any date, time and location 
//> (uniformly distributed). 
//
// Author: A. Lamy
// -----------------------------------------------------------

// ===========================================================
// MAIN
// ===========================================================

// altitude (m)
alt_minmax = [500.e3, 700.e3]; 

// flux
flux = 150;

// ap
ap = 15; 


desc_param = list(..
  CL_defParam("Altitude (relative to sphere with eq. radius) - min/max", alt_minmax, units=['m', 'km'], dim = 2, valid="$x >= 0 & $x < 10000"), ...
  CL_defParam("Solar flux (10.7 cm)", flux, units='sfu', valid="$x >= 50 & $x <= 400"), ...
  CL_defParam("Geomagnetic index (Ap) - daily value", ap, valid="$x >= 0 & $x <= 400") ...
);

[alt_minmax, flux, ap] = CL_inputParam(desc_param);


// ------------------------------------
// Computations 
// ------------------------------------
// number of samples
N = 30000; 

cjd_min = CL_dat_cal2cjd(2000,1,1); 
cjd_max = CL_dat_cal2cjd(2020,1,1); 

// uniformy distributed dates/times
cjd = grand(1, N, "unf", cjd_min, cjd_max); 

// lon/lat: uniformly distributed 
lon = grand(1, N, "unf", 0, 2 * %pi); // radians
lat = asin(grand(1, N, "unf", -1, 1)); // radians
r = %CL_eqRad + grand(1, N, "unf", min(alt_minmax), max(alt_minmax)); 

// Geodetic coordinates
ell = CL_co_car2ell(CL_co_sph2car([lon; lat; r])); 

// flux / ap
f107 = flux;
f107a = flux; 

dens = CL_mod_atmMSIS00(cjd, ell(1,:), ell(2,:), ell(3,:), f107, f107a, ap, res="dens"); 

dens_mean = mean(dens); 
dens_median = CL_quantile(dens, 0.5); 


// -----------------
// Display info
// -----------------

mprintf("\n-----------------------\n"); 
mprintf("MSIS2000 - Density statistics (kg/m^3):\n"); 
mprintf("Min value = %.1e\n", min(dens)); 
mprintf("Max value = %.1e\n", max(dens)); 
mprintf("Mean value = %.1e\n", dens_mean); 
mprintf("Mean value (log) = %.1e\n", 10^mean(log10(dens))); 
mprintf("Median value = %.1e\n", CL_quantile(dens, 0.5)); 
mprintf("Max value (90%%) = %.1e\n", CL_quantile(dens, 0.9)); 
mprintf("Max value (99%%) = %.1e\n", CL_quantile(dens, 0.99)); 
mprintf("Max value (99.75%%) = %.1e\n", CL_quantile(dens, 0.9975)); 
mprintf("-----------------------\n"); 


// -----------------
// Plot
// -----------------

f = scf(); 
f.visible = "on"; 
f.immediate_drawing = "off"; 

a = gca(); 

// recreate histplot (log scale)! 
vals = 10.^(linspace(log10(min(dens)), log10(max(dens)), 21));
[X, nb] = dsearch(dens, vals, "c"); 
ratio = 100 * nb / sum(nb); 

x = [vals(1:$-1); vals(1:$-1); vals(2:$); vals(2:$)]; 
y = [zeros(ratio); ratio; ratio; zeros(ratio)]; 

plot(matrix(x, 1, -1), matrix(y, 1, -1)); 
xpolys(x, y);
 
h = CL_g_select(gce(), "Polyline"); 
h.fill_mode = "on"; 
h.background = color("grey70"); 
h.foreground = color("grey40"); 

db = [a.x_ticks.locations(1), a.y_ticks.locations(1); a.x_ticks.locations($), a.y_ticks.locations($)]; 

plot(dens_mean * [1,1], [0, 100], "r", "thickness", 2); 
plot(dens_median * [1,1], [0, 100], "b", "thickness", 2); 

a.data_bounds = db; 

// log scale
xmin = 10^floor(log10(min(dens))); 
// k = floor(min(dens) / xmin); 
// xmin = k * xmin; 

xmax = 10^ceil(log10(max(dens))); 
// k = ceil(10 * max(dens) / xmax); 
// xmax = k * xmax / 10; 

a.data_bounds(:,1) = [xmin; xmax] .* (1 + [-1; 1] * 0.2); 
a.log_flags = "lnn"; 
a.tight_limits = "on"; 
// a.sub_ticks = [8, 0]; 

CL_g_stdaxes(); 

text = msprintf("sph. alt = [%.1f, %.1f] km", min(alt_minmax)/1000, max(alt_minmax)/1000); 

a.title.text = "MSIS2000 atmospheric density - " + text; 
a.x_label.text = "Atmospheric density (kg/m^3)"; 
a.y_label.text = "Ratio (%)"; 

f.immediate_drawing = "on"; 
f.visible = "on"; 
