//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Atmospheric density given by MSIS2000 model
//> for a given location as function of day of year and hour. 
//
// Author: A. Lamy
// -----------------------------------------------------------

// ===========================================================
// MAIN
// ===========================================================

// Longitude (rad)
lon = 0.e3; 

// Latitude (rad)
lat = 0.e3; 

// altitude (m)
alt = 600.e3; 

// flux
f107 = 150;

// ap
ap = 15 

desc_param = list(..
  CL_defParam("Geodetic longitude", lon, units=['rad', 'deg'], valid="$x >= -360 & $x <= 360"), ...
  CL_defParam("Geodetic latitude", lat, units=['rad', 'deg'], valid="$x >= -90 & $x <= 90"), ...
  CL_defParam("Geodetic altitude", alt, units=['m', 'km'], valid="$x >= 0 & $x < 10000"), ...
  CL_defParam("Solar flux (10.7 cm)", f107, units='sfu', valid="$x >= 50 & $x <= 400"), ...
  CL_defParam("Geomagnetic index (Ap) - daily value", ap, valid="$x >= 0 & $x <= 400") ...
);

[lon, lat, alt, f107, ap] = CL_inputParam(desc_param);


// ------------------------------------
// Computations 
// ------------------------------------
cjd0 = CL_dat_cal2cjd(2000,1,1); 

days = 0 : 10 : 360; // x axis
hours = linspace(0, 24, 50); // y axis

[Days, Hours] = ndgrid(days, hours); 
all_days = matrix(Days, 1, -1);
all_hours = matrix(Hours, 1, -1);

cjd = cjd0 + (all_days + all_hours / 24); 

f107a = f107; 

dens = CL_mod_atmMSIS00(cjd, lon, lat, alt, f107, f107a, ap, res="dens"); 


// -----------------
// Plot
// -----------------

f = scf(); 
f.visible = "on"; // NB: "on" necessary if contour2d is used (Scilab bug)
f.immediate_drawing = "off"; 

nbcol = 256; 
f.color_map = CL_colormap("jet", nbcol); 

a = gca(); 

// power in common for plot (bug in Scilab 6.1) 
power10 = round(log10(min(dens))); 

// plotted quantity
Z = matrix(dens, size(Days)) / (10^power10); 

zmin = min(Z); 
zmax = max(Z); 

// colorbar (change graduations: log -> natural)
colorbar(zmin, zmax, [1, nbcol]); 
// cb: type = Axes in Scilab 6.1, but was a child before 
cb = gce(); 
if (cb.type <> "Axes"); cb = cb.parent; end 
cb.font_size = 1.5; 
cb.title.text = msprintf("Density (1.e%d kg/m^3)", power10);; 
cb.title.font_size = 2;

x = days; 
y = hours; 
Sgrayplot(x, y, Z, zminmax = [zmin, zmax], colminmax = [1, nbcol]); 

a.data_bounds = [0, 0; 360, 24]; 
a.tight_limits = "on"; 

text = msprintf("alt=%.1f km, lon=%.1f deg, lat=%.1f deg", alt/1000, lon*%CL_rad2deg, lat*%CL_rad2deg); 

a.title.text = "MSIS2000 atmospheric density - " + text; 
a.x_label.text = "Day of year"; 
a.y_label.text = "Hours (TREF)"; 

CL_g_stdaxes(a); 

f.immediate_drawing = "on"; 
f.visible = "on"; 
