//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Relative secular drift rate due to J2 of a combination of variables: 
//> f(sma, inc) = n1*dw/dt + n2*dW/dt + n3*dM/dt + n4*wE + n5*wS + n6*n 
//> sma, inc: semi-major axis and inclination
//> 
//> with: 
//> n1..n6: Integers
//> dw/dt: Drift of argument of perigee
//> dW/dt: Drift of right ascension of ascending node
//> dM/dt: Drift of mean anomaly
//> wE:    Rotation rate of the Earth
//> wS:    (apparent) Rotation rate of the Sun
//> n:     Keplerian mean motion
//> 
//> Eccentricity is fixed. 
//> 
//> The results are given by: 
//> res = f(sma, inc) - f(sma_ref, inc_ref)
//> sma_ref, inc_ref: reference (fixed) semi-major axis and inclination 
//> 
// Author: A. Lamy
// -----------------------------------------------------------

smaref = 7000.e3; 
incref = 98 * %CL_deg2rad; 

dsmamin = -100.e3;
dsmamax = 100.e3;

dincmin = -3 * %CL_deg2rad;
dincmax = 3 * %CL_deg2rad;

ecc = 0;  

coef = [0, 1, 0, 0, -1, 0]; 

desc_param = list(..
   CL_defParam("Reference semi-major axis", smaref, units=['m', 'km'], id='$aref', valid='$aref>0'),..
   CL_defParam("Reference inclination", incref, units=['rad', 'deg'], id='$iref', valid='$iref>=0 & $iref<=180'),..
   CL_defParam("Delta semi-major axis - min", dsmamin, units=['m', 'km'], id='$damin', valid='$aref + $damin>0'),..
   CL_defParam("Delta semi-major axis - max", dsmamax, units=['m', 'km'], id='$damax', valid='$damax>$damin'),..
   CL_defParam("Delta inclination - min", dincmin, units=['rad', 'deg'], id='$dimin', valid='$iref + $dimin>=0 & $iref + $dimin<=180'),..
   CL_defParam("Delta inclination - max", dincmax, units=['rad', 'deg'], id='$dimax', valid='$iref + $dimax>=0 & $iref + $dimax<=180 & $dimax > $dimin'),..
   CL_defParam("Eccentricity", ecc, valid='$x>=0 & $x<1'),..
   CL_defParam("Coefficients (dw/dt, dW/dt, dM/dt, wE, wS, n)", coef, accv=linspace(-10,10,21), dim=6, valid='round($x) == $x')..
);

[smaref,incref,dsmamin,dsmamax,dincmin,dincmax,ecc,coef] = CL_inputParam(desc_param);

nbpts = 80;
dsma = linspace(dsmamin, dsmamax, nbpts);
dinc = linspace(dincmin, dincmax, nbpts);

[Dsma, Dinc] = ndgrid(dsma, dinc); 
all_dsma = matrix(Dsma(:), 1, -1); 
all_dinc = matrix(Dinc(:), 1, -1); 

sma = smaref + all_dsma; 
inc = incref + all_dinc; 

mm = CL_kp_params(['mm'], sma); // keplerian mean motion
mm_ref = CL_kp_params(['mm'], smaref); // keplerian mean motion

[drift_pom, drift_gom, drift_anm] = CL_op_driftJ2(sma,ecc,inc);
[drift_pom_ref, drift_gom_ref, drift_anm_ref] = CL_op_driftJ2(smaref,ecc,incref);

val = coef(1)*drift_pom + coef(2)*drift_gom + coef(3)*drift_anm + ..
      coef(4)*%CL_rotrBody + coef(5)*%CL_rotrBodySun + coef(6)*mm;   

val_ref = coef(1)*drift_pom_ref + coef(2)*drift_gom_ref + coef(3)*drift_anm_ref + ..
      coef(4)*%CL_rotrBody + coef(5)*%CL_rotrBodySun + coef(6)*mm_ref;   

res = matrix(val - val_ref, size(Dsma)) * %CL_rad2deg*86400; // deg / dey


// -----------------------------------------------------------
// plot
// -----------------------------------------------------------

// generates the name of the plotted parameter
function [str] = Name_param(coef)
  names = ["dw/dt", "dW/dt", "dM/dt", "wE", "wS", "n"]; 
  signes = [" -",""," +"]; 
  str = ""; 
  Inz = [find(coef <> 0), [7]]; 

  for k=Inz(1):6; 
    if (k > Inz(1) | coef(k)<0); str = str + signes(sign(coef(k))+2); end
    if (coef(k) <> 0)
      if (abs(coef(k)) <> 1); str = str + " " + sprintf("%d", abs(coef(k))) + " *" ; end
      str = str + " " + names(k); 
    end
  end
endfunction


f=scf();
f.visible="on";
f.immediate_drawing="off";

f.color_map = CL_colormap("jet", 32);
Coul1 = 5;
Coul2 = 10;
Coul3 = 2;
Noir = addcolor([1,1,1]*0);

a=gca();

[levels, sublevels] = CL_autoLevels(min(res), max(res));

contour2d(dsma/1000, dinc*%CL_rad2deg, res, levels, style=Coul1*ones(levels));
CL_g_tag(a,1);

contour2d(dsma/1000, dinc*%CL_rad2deg, res, sublevels, style=Coul2*ones(sublevels));
CL_g_tag(a,2);

// general setting
CL_g_stdaxes(a);
a.data_bounds = [dsmamin/1000,dincmin*%CL_rad2deg; dsmamax/1000,dincmax*%CL_rad2deg];
a.tight_limits="on";
a.x_label.text = "Delta semi major axis (km)";
a.y_label.text = "Delta inclination (deg)";

info = msprintf("ecc=%.4f, sma=%.1fkm, inc=%.1fdeg", ecc, smaref/1000, incref * %CL_rad2deg); 

a.title.text = "(Relative) " +  Name_param(coef) + " (deg/day)" + " - " + info;

// adjustments
h = CL_g_select(a, "Text", 2);
CL_g_delete(h);

h = CL_g_select(a, "Text");
if (h <> [])
  h.font_foreground=Coul3;
  h.font_size=3;
  h.font_style=8;

  // round-off to 0 if small value
  I = find(abs(strtod(h.text)) < 1.e-12); 
  h.text(I) = "0"; 
end 

h = CL_g_select(a, "Polyline", 1);
h.thickness=2;

f.immediate_drawing="on";
f.visible="on";

