//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Evolution of inclination and MLTAN for nearly SSO, circular orbits.
//> Simplified equations are used: 
//> - RAAN: influenced by J2 only
//> - Inclination: only affected by third body (Sun) effect using 
//> doubly averaged equations
//> 
//
// Author: A. Lamy
// -----------------------------------------------------------

// -----------------------------------------------------------
// Inputs
// -----------------------------------------------------------

// Mean semi-major axis
sma = 7.e6; 

// Check sma - sma: km
function [ok] = isValidSma(sma_km)  
  sma = sma_km * 1000; 
  ok = (sma >= %CL_eqRad & ~isnan(CL_op_ssoJ2("i", sma, 0))); 
endfunction

desc = list(..
   CL_defParam("Mean semi-major axis", sma, units=['m', 'km'], valid='$x > 0 & $x <= 10000 & isValidSma($x)') ... 
);

[sma] = CL_inputParam(desc);


// -----------------------------------------------------------
// Computation
// -----------------------------------------------------------

// Equations: 
// d(inc)/dt = K1 * sin(2*h) * sin(inc)
// dh/dt = d(raan)/dt - omega_sun = K2 * cos(inc) - omega_sun 
// With: h = (MLTAN - 12) * %pi/12
// 
// NOTE: 
// Equations could be approximated like this: 
// inc0 : such that: d(raan)/dt = omega_sun
// inc = inc0 + di, di "small" 
// => 
// d(di)/dt = K1 * sin(2*h) * [sin(inc0) + cos(inc0) * di)] ~ K1 * sin(2*h) * sin(inc0)
// dh/dt = K2 * (cos(inc0) - sin(inc0) * di) - omega_sun = -K2 * sin(inc0) * di 
// K1 => K1 * sin(inc0); 
// K2 => K2 * sin(inc0); 
// 
// Final approximated equations: 
// d(di)/dt = K1 * sin(2*h) 
// dh/dt = -K2 * di 
// 
// => d2h/dt2 = -K1 * K2 * sin(2*h) 
// (2*h => pendulum)  


ecc = 0; 

// SSO orbit (J2) 
inc0 = CL_op_ssoJ2("i", sma, ecc);

if (isnan(inc0))
  error("SSO orbit does not exist"); 
end

// model constants calculation
// K1 : such that: sin(2*h) * sin(inc) = 1
// => inc = %pi/2 and MLTAN = 15
// K2 : such that cos(inc) = 1 
// => inc = 0
K1 = CL_op_incdotSsoCirc(15, sma, %pi/2); 
[pomdot, gomdot, anmdot] = CL_op_driftJ2(sma, ecc, 0); 
K2 = gomdot; 
OMEGA_SUN = %CL_rotrBodySun; 
INC0 = inc0; 

// State evolution (for ode) 
// y = [dinc; h] 
// ydot : per year
function [ydot] = fct(t, y)
  inc = y(1) + INC0; 
  h = y(2); 
  dincdot = K1 * sin(2*h) * sin(inc);
  hdot = K2 * cos(inc) - OMEGA_SUN; 
  ydot = [dincdot, hdot] * 86400 * 365.25; 
endfunction

// propagation function (from -tmax to tmax)
// tmax, tstep : years  
function [dinc, H, t] = propagate(dinc0, H0, tmax, tstep)

  h0 = (H0 - 12) * %pi/12; 
  y0 = [dinc0; h0]; 

  rtol = 1.e-12; 
  atol = 1.e-8; 
  
  t1 = (0 : tstep : tmax); // years 
  y1 = ode(y0, 0, t1, rtol, atol, fct); 
  
  t2 = -t1; 
  y2 = ode(y0, 0, t2, rtol, atol, fct);
  
  // results from -tmax to tmax (increasing time)
  K = ($:-1:1); 
  t = [t2(K), t1]; 
  dinc = [y2(1,K), y1(1,:)]; 
  H = [y2(2,K), y1(2,:)] * (12 / %pi)  + 12; 
endfunction


list_res = list(); 

tmax = 100; // years 
tstep = 0.1; // years

// compute the "limit" curve
// => used to compute the step in inclination 
H0 = 12; 
dinc0 = 1.e-5;  
[dinc_lim1, H_lim1, t] = propagate(dinc0, H0, tmax, tstep); 
list_res($+1) = struct("data", [dinc_lim1; H_lim1; t], "type", 0); 
[dinc_lim2, H_lim2, t] = propagate(-dinc0, H0, tmax, tstep); 
list_res($+1) = struct("data", [dinc_lim2; H_lim2; t], "type", 0); 


// compute all curves starting at 6/18h with variable dinc 
dinc_l1 = max(dinc_lim1);  // positive
dinc_l2 = min(dinc_lim2);  // negative

dinc_bmax = dinc_l1 * 1.3; 
dinc_bmin = dinc_l2 * 1.3; 

nb_steps = 6; // from 0 to limit curve (even number)
dinc_step1 = dinc_l1 / nb_steps;  // positive 
dinc_step2 = dinc_l2 / nb_steps;  // negative

for H0 = [6, 18]
  // periodic
  for k = 1 : nb_steps - 1
    [dinc, H, t] = propagate(k * dinc_step1, H0, tmax, tstep); 
    list_res($+1) = struct("data", [dinc; H; t], "type", k);  
  end
  // top/bottom: 4 steps supposed enough
  for k = nb_steps + 1 : nb_steps + 4
    [dinc, H, t] = propagate(k * dinc_step1, H0, tmax, tstep); 
    list_res($+1) = struct("data", [dinc; H; t], "type", k);  
    [dinc, H, t] = propagate(k * dinc_step2, H0, tmax, tstep); 
    list_res($+1) = struct("data", [dinc; H; t], "type", k);  
  end
end


// -----------------------------------------------------------
// Plot
// -----------------------------------------------------------

f = scf(); 
f.immediate_drawing = "off"; 

for res = list_res
  dinc = res.data(1,:); 
  H = res.data(2,:); 
  thickness = 2; 
  col = color("slateblue"); 
  if (res.type == 0); col = color("darkviolet"); end
  if (modulo(res.type,2) == 1); thickness = 1; end
  plot(H, dinc * %CL_rad2deg, "thickness", thickness);
  h = CL_g_select(gce(), "Polyline"); 
  h.foreground = col; 
end

a=gca(); 
a.data_bounds = [0, dinc_bmin * %CL_rad2deg; 24, dinc_bmax * %CL_rad2deg];
a.tight_limits = "on";  
a.x_label.text = "MLTAN (h)";
a.y_label.text = "Inclination increment (0 <=> SSO) (deg)"; 

txt = msprintf("sma = %.1f km\n", sma/1000); 
a.title.text = "Approx. evolution of inclination and MLTAN - " + txt; 
CL_g_stdaxes(a); 

f.immediate_drawing = "on"; 
f.visible = "on"; 


 
 