//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Evolution of inclination and MLTAN for nearly SSO, circular orbits.
//> Stela is used for orbit propagation. 
//> The propagation model includes the effects of J2/J3 and third body (Sun). 
//> The initial inclination is computed using J2 only.  
//
// Author: A. Lamy
// -----------------------------------------------------------

// -----------------------------------------------------------
// Inputs
// -----------------------------------------------------------
cal = CL_dat_now("cal");
scal = CL_dat_cal2str([cal(1);1;1], nd=-1);  // january 1st
sma = 7.e6; 
ecc = 1.e-3; 
pom = 90 * %CL_deg2rad; // arg of perigee
tab_delta_inc = (-4 : 1 : 4) * 0.1 * %CL_deg2rad; 
tab_mltan0 = [6]; 
dur_years = 8; // years


function [ok] = isValidDate(scal)  
  ok = %f; 
  cal = CL_dat_str2cal(scal); 
  if (cal(1) <> %nan)
    ok = ( cal(1) >= 2000 & cal(2) <= 2100);
  end
endfunction

// Check sma/ecc - sma: km
function [ok] = isValidSmaEcc(sma, ecc)  
  ok = (sma*(1-ecc)*1000 > %CL_eqRad + 1000 & ~isnan(CL_op_ssoJ2("i", sma*1000, ecc))); 
endfunction

desc = list(..
   CL_defParam("Initial date (calendar format, TREF time scale)", scal, typ='cal', valid="isValidDate($x)"),..
   CL_defParam("Mean semi-major axis", sma, units=['m', 'km'], id='$sma', valid='$sma >= 0 & isValidSmaEcc($sma, $ecc)'),..
   CL_defParam("Initial mean eccentricity", ecc, id='$ecc', valid='$ecc >= 0 & $ecc < 1 & isValidSmaEcc($sma, $ecc)'),..
   CL_defParam("Initial mean argument of perigee", pom, units = ["rad", "deg"], valid='$x >= 0 & $x <= 360'),..
   CL_defParam("Values of inclination increments", tab_delta_inc, units = ["rad", "deg"], dim=-1, valid='$x >= -10 & $x <= 10'),..
   CL_defParam("Values of mean local time of the ascending node", tab_mltan0, units = ["h"], dim=-1, valid='$x >= 0 & $x <= 24'),..
   CL_defParam("Propagation duration", dur_years, units = ["yr"], valid='$x >= 1 & $x <= 50')..
);

[scal, sma, ecc, pom, tab_delta_inc, tab_mltan0, dur_years] = CL_inputParam(desc);

cjd0 = CL_dat_cal2cjd(CL_dat_str2cal(scal));

// -----------------------------------------------------------
// Computation
// -----------------------------------------------------------

// STELA model parameters (default values)
params = CL_stela_params("none");
params.central_enabled = %t; 
params.zonal_enabled = %t; 
params.zonal_maxDeg = 3; 
params.thirdBody_enabled = %t; 
params.thirdBody_bodies = ["Sun"];
params.reentryAltitude = 0; 

list_res = list(); 

// SSO orbit using J2 only 
// using STELA parameters
cst = CL_stela_getConst(); 
inc = CL_op_ssoJ2("i", sma, ecc, er=cst.er, mu=cst.mu, j2=cst.j1jn(2)); 

// Keplerian mean orbital elements (frame: ECI)
mean_kep0 = [sma; ecc; inc; pom; 0; 0];

// propagation dates (cjd) 
cjd = cjd0 + (0 : dur_years * 365.25);

[All_mltan0, All_delta_inc] = ndgrid(tab_mltan0, tab_delta_inc); 

ncas = size(All_mltan0, "*"); 

// Check the existence of STELA 
CL_stela_extrap("kep", cjd0, mean_kep0, cjd0, params, ["m"]);

wait_win = waitbar("In progress ...");

for k = 1 : ncas

  mltan0 = All_mltan0(k); 
  dinc = All_delta_inc(k); 
  
  mean_kep1 = mean_kep0; 
  mean_kep1(5) = CL_op_locTime(cjd0, "mlh", mltan0, "ra"); 
  mean_kep1(3) = mean_kep1(3) + dinc; // in [0, %pi] ? 
  
  mean_kep = CL_stela_extrap("kep", cjd0, mean_kep1, cjd, params, ["m"]);
  
  mltan = CL_unMod(CL_op_locTime(cjd, "ra", mean_kep(5,:), "mlh"), 24); 
  mltan = mltan - mltan(1) + mltan0; 
  
  list_res($+1) = [mltan;  mean_kep(3,:)]; 
  
  waitbar(k / ncas, wait_win);
  
end

close(wait_win); 


// -----------------------------------------------------------
// Plot
// -----------------------------------------------------------

function [num]  = numcolor(mltan0, nbcol)
  if (size(tab_mltan0, "*") <= 1)
    num = 1; 
  else
    p = (mltan0 - min(tab_mltan0)) / (max(tab_mltan0) - min(tab_mltan0)); 
    num = round(p * (nbcol - 1)) + 1; 
  end
endfunction

f = scf(); 
f.immediate_drawing = "off"; 
nbcol = 20; 
f.color_map = CL_colormap("jet", nbcol); 

for res = list_res
  numcol = numcolor(res(1,1), nbcol); 
  plot(res(1,:), res(2,:) * %CL_rad2deg, "b", "thickness", 2);
  h = CL_g_select(gce(), "Polyline"); 
  h.foreground = numcol; 
  plot(res(1,$), res(2,$) * %CL_rad2deg, "bo", "thickness", 2);
  h = CL_g_select(gce(), "Polyline"); 
  h.mark_size = 4; 
  h.mark_foreground = numcol; 
end


txt = msprintf("%.1f years", dur_years); 

CL_g_stdaxes(); 
xtitle("Evolution of MLTAN and inclination over " + txt, "MLTAN (h)", "Inclination (deg)");

a = gca(); 

// adjust bounds
db = a.data_bounds; // [xmin, ymin; xmax, ymax]
delta = db(2,:) - db(1,:); // [xmax-xmin, ymax-ymin]
a.data_bounds = db + [-delta; delta] / 20; 
a.tight_limits = "on"; 

f.immediate_drawing = "on"; 
f.visible = "on"; 
