//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Mean solar array efficiency: 
//> It is defined as the mean value of max(cos(ang),0) over one orbit,
//> where ang is the angle between the normal of the solar array (SA) 
//> and the direction of the Sun. 
//> 
//> The tilt angle is the angle between the orbit plane and the SA
//> normal, positive if the angle between the SA normal and
//> the orbit's angular momentum is less than 90 degrees.  
//> 
//> - Fixed solar array: 
//> The orientation of the SA normal is defined by the tilt angle and
//> by the rotation angle (alpha) around w (origin = q axis). Alpha and
//> tilt are the spherical coordinates of the SA normal in qsw. 
//> 
//> - Rotating solar array: 
//> The solar array is rotating around the normal of the orbit plane
//> and is oriented towards the Sun as much as possible. 
//> 
//> NB: 
//> - The orbit is circular. 
//> - Eclipses are taken into account (spherical Earth). 
//> - The Sun is supposed fixed / orbit plane over one orbit. 
//
// Author: A. Lamy
// -----------------------------------------------------------

alt = 700.e3;  // altitude (m)
sa_type = 1; 
sa_az = 0 * %CL_deg2rad; 
sa_tilts = (-90:15:90) * %CL_deg2rad; 
beta_min = -90 * %CL_deg2rad; 
beta_max = 90 * %CL_deg2rad; 

desc_param = list(..
  CL_defParam("Altitude (sma minus equ. radius)", alt, units=['m','km'], valid='$x>=0'),..
  CL_defParam("Solar array type: 1=fixed 2=rotating", sa_type, accv=[1,2]),..
  CL_defParam("SA rotation angle from q, around w (if fixed)", sa_az, units=['rad','deg']),..
  CL_defParam("Values of solar array tilt", sa_tilts, units=['rad','deg'], valid='$x >= -90 & $x <= 90', dim=[1, 20]),..
  CL_defParam("Sun beta angle - min", beta_min, id="$betamin", units=['rad', 'deg'], valid='$x>=-90 & $x<=90'),..
  CL_defParam("Sun beta angle - max", beta_max, units=['rad', 'deg'], valid='$x>=-90 & $x<=90 & $x>$betamin')..
);

[alt, sa_type, sa_az, sa_tilts, beta_min, beta_max] = CL_inputParam(desc_param);

// -----------------------------------------------------------
// Computation
// -----------------------------------------------------------
function [eff]= efficiency_rotating(betaa, tilt, alpha_ecl)
  // Mean value of max(cos(ang), 0)
  // cos(ang) = cos(beta - tilt) 
  eff = max(cos(betaa - tilt), 0) .* (alpha_ecl/%pi); 
endfunction

// betaa : beta angle (1xN) 
// az: solar array rotation angle (1x1)
// tilt: tilt angle (1x1) 
// alpha_ecl: arg of latitude of start of eclipse (origin: Sun) (1xN) 
// NB: length of eclipse = 2 * (pi - alpha_ecl) 
function [eff]= efficiency_fixed(betaa, az, tilt, alpha_ecl)
  // Mean value of max(cos(ang), 0)
  // cos(ang) = cos(beta)*cos(tilt)*cos(alpha+az) + sin(beta)*sin(tilt)
  // 
  // Demonstration : 
  // Conventions: alpha represents the argument of latitude. Origin: point closest to the Sun. 
  // The chosen reference frame (F) is : Z : angular momentum, X : direction closest to the Sun 
  // Sun coordinates in Frame F = S = [cos(beta); 0; sin(beta)] 
  // Normal direction in qsw frame: [cos(tilt)*cos(angrot); cos(tilt)*sin(angrot); sin(tilt)]; 
  // with: angrot = rotation angle of solar array (=az) 
  // Normal direction in Frame F => multiply by matrix Mqsw2inertial: 
  // Mqsw2inertial = [cos(alpha), -sin(alpha), 0; sin(alpha), cos(alpha), 0; 0, 0, 1] 
  // => Normal direction in Frame F = N = [cos(tilt) * cos(alpha+angrot); cos(tilt)*sin(alpha+angrot); sin(tilt)] 
  // efficiency = <N, S> = cos(beta) * cos(tilt) * cos(alpha+angrot) + sin(beta) * sin(tilt)

  // alpha1, alpha2: such that cos(ang) has same sign in [alpha1, alpha2]
  alpha1 = -az - %pi * ones(betaa); 
  alpha2 = -az + %pi * ones(betaa); 
  
  cond_sign = abs(sin(betaa) * sin(tilt)) < abs(cos(betaa) * cos(tilt)); 
  I = find(cond_sign); 
  if (I <> [])
    x = acos(-tan(betaa(I)) * tan(tilt)); 
    alpha1(I) = -az - x; 
    alpha2(I) = -az + x; 
  end
  
  // check value at middle of interval
  // if value is negative and sign changes => take complementary interval
  // if value is negative and sign does not change => 0 length interval

  I = find(cos(betaa-tilt) < 0 & cond_sign); 
  // does not happen if beta and tilt are in [-%pi/2, %pi/2]
  if (I <> [])
    [alpha1(I), alpha2(I)] = (alpha2(I), alpha1(I)+2*%pi); 
  end
  I = find(cos(betaa-tilt) < 0 & ~cond_sign); 
  if (I <> [])
    alpha1(I) = -az; 
    alpha2(I) = -az; 
  end

  // Don't take into account the part of [alpha1, alpha2] in eclipse 
  // alpha1_ecl, alpha2_ecl: in shadow 
  alpha1_ecl = CL_rMod(alpha_ecl, alpha1, alpha1 + 2*%pi); 
  alpha2_ecl = alpha1_ecl + 2 * (%pi - alpha_ecl); 

  // computation over intervals alpha_start -> alpha_stop
  // equivalent to: [alpha1; alpha2] MINUS [alpha1_ecl, alpha2_ecl] mod 2pi 

  alpha1_ecl_a = min(max(alpha1_ecl - 2 * %pi, alpha1), alpha2); 
  alpha2_ecl_a = min(max(alpha2_ecl - 2 * %pi, alpha1), alpha2); 
  alpha1_ecl_b = min(max(alpha1_ecl, alpha1), alpha2); 
  alpha2_ecl_b = min(max(alpha2_ecl, alpha1), alpha2); 
  
  alpha_start = [alpha1; alpha2_ecl_a; alpha2_ecl_b]; 
  alpha_stop = [alpha1_ecl_a; alpha1_ecl_b; alpha2];
    
  // compute mean effect over the 3 intervals 
  eff = zeros(betaa); 
  for k = 1 : 3
    eff = eff + (cos(betaa) * cos(tilt) .* (sin(alpha_stop(k,:)+az) - sin(alpha_start(k,:)+az)) + ...
          sin(betaa) * sin(tilt) .* (alpha_stop(k,:) - alpha_start(k,:))) / (2*%pi); 
  end 
  
endfunction

// Computation of efficiency for verification purposes
// same interface as efficiency_fixed
// NB function is not nominally used. 
// results : same as "efficiency_fixed" (accuracy ~0.5e-2 with 200 points / orbit

function [eff]= efficiency_fixed_verif(betaa, az, tilt, alpha_ecl)
  // number of positions per orbit used to compute the average value 
  N = 200; 

  // Arguments of latitude (origin : projection of Sun direction into orbit plane) 
  alpha = linspace(-%pi, %pi, N+1); 
  alpha = alpha(1 : $-1); 
 
  // build matrices : rows <=> alpha
  Betaa = repmat(betaa, N, 1); 
  Alpha_ecl = repmat(alpha_ecl, N, 1);
  Alpha = repmat(alpha', 1, length(betaa)); 

  Eff = cos(Betaa)*cos(tilt).*cos(Alpha+az) + sin(Betaa)*sin(tilt);
  I = find(Eff < 0 | abs(Alpha) > Alpha_ecl); 
  if (I <> [])
    Eff(I) = 0; 
  end 
  eff = mean(Eff, "r"); 
  
endfunction 



// -----------------------------------------------------------

sma = alt + %CL_eqRad; // semi major axis
betaa = linspace(beta_min, beta_max, 200); 
alpha_ecl = %pi - CL_gm_betaEclipse(sma, betaa); 

// result = eff(%)  nb_tilts x N 
result = []; 

for tilt = sa_tilts
  if (sa_type == 1)
    // replace by efficiency_fixed_verif for comparison
    eff = efficiency_fixed(betaa, sa_az, tilt, alpha_ecl); 
  else
    eff = efficiency_rotating(betaa, tilt, alpha_ecl); 
  end
  result = [result; 100 * eff]; 
end


// -----------------------------------------------------------
// Plot
// -----------------------------------------------------------

f = scf(); //create figure
f.visible = "on";
f.immediate_drawing="off";

nb = length(sa_tilts);
f.color_map = CL_colormap("jet", min(max(nb,3),100));
Noir = addcolor([0,0,0]);

for k = 1:length(sa_tilts)
  plot(betaa * %CL_rad2deg, result(k,:));
  h = CL_g_select(gce(), "Polyline");
  h.thickness = 2;
  h.foreground = k; 
end

a = gca();

if (sa_type == 1); str = "fixed"; else; str = "rotating"; end
a.title.text = msprintf("Mean (%s) solar array efficiency - Alt = %.1f km", str, alt/1000);
a.x_label.text = "Sun beta angle (deg)"; 
a.y_label.text = "Efficiency (%)";

a.data_bounds = [beta_min*%CL_rad2deg, max(min(result)-1, 0); 
                 beta_max*%CL_rad2deg, min(max(result)+1, 100)]; 
a.tight_limits = "on";

h = CL_g_select(a, "Polyline");
h.thickness = 2;

CL_g_legend(a, header = "SA tilt (deg)", str=string(sa_tilts * %CL_rad2deg));

CL_g_stdaxes(a);

f.immediate_drawing="on";
f.visible = "on";


