//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -----------------------------------------------------------
//> Pointing errors:
//> Impact of errors on tangential axis on pointing at aquisition (elliptical orbits).
//> 
//> Evaluates the minimum aperture angle of an antenna needed to compensate 
//> for a given tangential position error on the spacecraft. 
//> The plot gives the maximum angle deviation from the ground station
//> as function of the minimum visibility length at the beginning of pass 
//> (elevation is then the aquisition (AOS) elevation). 
//> All geometry conditions are evaluated (angle between the station (from 
//> Earth center) and the orbit plane and orientation of the orbit plane
//> (that is argument of perigee). 
//> The statistics make sense if the perigee argument can be considered 
//> as uniformly distributed, as well as the angle between the station (from the 
//> Earth center) and the orbit plane. 
//> 
//> NB: Valid for low enough orbits only. 
//> Hypotheses: spherical and non rotating Earth. 
//
// Author: A. Lamy
// -----------------------------------------------------------

hp = 700.e3; 
ha = 700.e3; 
elev_visi = [5, 5] * %CL_deg2rad; 
err_tan = 15.e3; 

desc_param = list(..
   CL_defParam("Perigee altitude (wrt equatorial radius)", hp, units=['m', 'km'], id='$hp', valid='$hp>=0' ),..
   CL_defParam("Apogee altitude (wrt equatorial radius)",  ha, units=['m', 'km'], id='$ha', valid='$ha > 0 & $ha>=$hp' ),..
   CL_defParam("Elevations for visibility start/end", elev_visi, units=['rad', 'deg'], valid='$x >= 0 & $x <= 90', dim=2), ..
   CL_defParam("Spacecraft position error on tangential axis", err_tan, units=['m', 'km'], valid="$x >= 0") ..
);

[hp, ha, elev_visi, err_tan] = CL_inputParam(desc_param);


// -----------------------------------------------------------
// Internal functions
// -----------------------------------------------------------

// Notes : 
// - the orbit is the X-Y plane. ascending node is on X axis. 
// - the ground station is in the X-Z plane (positive X and Z coordinates)
// - "pom" is the argument of perigee. 
// - "phi" : "latitude" of ground station = angle of station direction with orbit plane. 
// 
// visibility calculations : 
// orbits for all values of phi and pom are put together. 
// the discontinuities at true ag of lat = pi have no effects if there are
// enough positions / orbit.  

// -------------------------------------------
// Compute spacecraft position
// sma, ecc: 1x1
// pom (arg of perigee), anv (true anom): 1xN
// pos, vel: 3xN; kep: 6xN
// -------------------------------------------
function [pos, vel, kep] = compute_orb(sma, ecc, pom, anv)
  kep0 = [sma; ecc; 0; 0; 0; 0]; 
  n = size(pom, 2); 
  kep = repmat(kep0, 1, n); 
  kep(4,:) = pom; 
  kep(6,:) = CL_kp_v2M(ecc, anv);
  [pos, vel] = CL_oe_kep2car(kep); 
endfunction


// -------------------------------------------
// Compute station position (altitude = 0 / spherical Earth)
// phi: "latitude" [rad] 1xN
// -------------------------------------------
function [psta] = compute_sta(phi)
  R = %CL_eqRad; 
  psta = R * [cos(phi); zeros(phi); sin(phi)]; 
endfunction


// -------------------------------------------
// Generate:
// -> pom = perigee argument [rad]
// -> phi = latitude of ground station [rad]
// sma, ecc: 1x1
// elev_visi: 1x2 (AOS, LOS)
// pom, phi: 1xN
// -------------------------------------------
function [all_pom, all_phi] = generate_pomphi(sma, ecc, elev_visi)
  R = %CL_eqRad; 
  ha = sma * (1+ecc) - R; 
  hp = sma * (1-ecc) - R; 

  // max center angle (= max value for "phi") 
  elevmin = min(elev_visi); 
  angcmax = CL_gm_visiParams(ha + R, R, "elev", elevmin, "cen"); 

  // empirical formula (avoids too many values of arg of perigee for nearly circular orbits) 
  Npom = min(100, max(1, round(1 + 100 * (ha-hp)/ha))); 
  Nphi = min(ceil(5000 / Npom), 200); 
  
  phi = linspace(0, angcmax, Nphi); 
  pom = linspace(0, 2*%pi, Npom+1); 
  pom = pom(1:$-1); 
  
  // pom varies first
  [Mpom, Mphi] = ndgrid(pom, phi); 
  
  all_phi = matrix(Mphi, 1, -1); 
  all_pom = matrix(Mpom, 1, -1); 
endfunction


// -------------------------------------------
// compute visibility duration and projection factor
// sma, ecc: 1x1; 
// elev_visi:1x2 (AOS, LOS)
// pom, phi: 1xP
// dvisi: visi duration [s] 1xP - nan if not computed 
// fvisi: projection factor (multiplied by tangential error => angular error) [m^-1]
// -------------------------------------------
function [dvisi, fproj] = compute_visires(sma, ecc, pom, phi, elev_visi)

  // Nv: number of positions (true arg of latitude) / orbit
  Nv = 30; 
  psov = linspace(-%pi, %pi, Nv+1); // true PSO - each orbit
  psov = psov(1:$-1); // remove pi (same as -%pi)
  
  N = size(pom, 2); 
  
  // psov varies first 
  all_psov = matrix(repmat(psov', 1, N), 1, -1); 
  all_pom = matrix(repmat(pom, Nv, 1), 1, -1); 
  [pos, vel, kep] = compute_orb(sma, ecc, all_pom, all_psov - all_pom);
  
  // psov made continuous
  k = floor((0 : N*Nv - 1) / Nv); // 0 -> N-1
  x = all_psov + 2*%pi * k; 
  // bounds for visibility intervals (-pi ->pi) 
  K = 0 : N-1; 
  xbounds = [-%pi + 2 * K * %pi; %pi + 2 * K * %pi]; 
  
  // args for zero calculation
  args = struct(); 
  args.x = x; 
  args.pos = pos; 
  args.psta = compute_sta(phi); 
  args.elevmin = %nan; // initialized later
  
  // function for zero calculation 
  function [y] = fct_elev(x, args)
    pos = CL_interpLagrange(args.x, args.pos, x); 
    k = floor((x + %pi) / (2*%pi)) + 1;
    psta = args.psta(:,k); 
    elev = %pi/2 - CL_vectAngle(pos - psta, psta); 
    y = elev - args.elevmin; 
  endfunction
  
  args.elevmin = elev_visi(1); 
  xaos = CL_detectZero(x, list(fct_elev, args), "incr", ytol=1.e-4, xbounds=xbounds);
  kaos = floor((xaos + %pi) / (2*%pi)) + 1;
  
  args.elevmin = elev_visi(2); 
  xlos = CL_detectZero(x, list(fct_elev, args), "decr", ytol=1.e-4, xbounds=xbounds);
  klos = floor((xlos + %pi) / (2*%pi)) + 1;
  
  // check there is one interval per orbit at most
  if (length(unique(kaos)) <> length(kaos) | length(unique(klos)) <> length(klos))
    error("Invalid visibility intervals"); 
  end

  // visibility intervals
  xinterv = %nan * ones(2,N);    
  xinterv(1,kaos) = xaos; 
  xinterv(2,klos) = xlos; 
  
  Kinterv = find(xinterv(1,:) <= xinterv(2,:)); // NB: Nan not included
  xinterv = xinterv(:,Kinterv); 
    
  // associate intervals with input values 
  dvisi = %nan * ones(1,N); 
  fproj = %nan * ones(1,N);
  
  n = sqrt(%CL_mu / sma^3); // mean motion (rad/s)
  
  K = Kinterv; 
  dvisi(K) = (CL_kp_v2M(ecc, xinterv(2,:)) - CL_kp_v2M(ecc, xinterv(1,:))) / n; 
  
  [pos, vel, kep] = compute_orb(sma, ecc, pom(K), xinterv(1,:) - pom(K));
  psta = compute_sta(phi(K));
  fproj(K) = CL_norm(CL_cross(CL_unitVector(vel), CL_unitVector(pos-psta))) ./ CL_norm(pos-psta); 
  
endfunction


// -----------------------------------------------------------
// Computations
// -----------------------------------------------------------
R = %CL_eqRad; 
[sma, ecc] = CL_kp_apsConvert("ra", ha + R, "rp", hp + R, ["sma", "ecc"]); 

[pom, phi] = generate_pomphi(sma, ecc, elev_visi); 

[dvisi, fproj] = compute_visires(sma, ecc, pom, phi, elev_visi); 

// statistics
dvisimin = 0; 
if (min(elev_visi) <> max(elev_visi)); dvisimin = min(dvisi); end 
durations = linspace(dvisimin, max(dvisi), 12);

vals = %nan * ones(3, length(durations)); 
pvals = [0.95, 0.99, 1]; // proba values

for k = 1 : length(durations)
  I = find(dvisi >= durations(k)); 
  if (I <> [])
    for i = 1  : 3
      vals(i,k) = CL_quantile(fproj(I), pvals(i)); 
    end
  end
end


// -----------------------------------------------------------
// Plot
// -----------------------------------------------------------

f = scf();
f.figure_size = [820,600]; 
f.visible="on";
f.immediate_drawing="off";
a = gca(); 

plot(durations/60, vals(1,:) * err_tan * %CL_rad2deg, "b-o", "thickness", 2); 
plot(durations/60, vals(2,:) * err_tan * %CL_rad2deg, "r-o", "thickness", 2); 
plot(durations/60, vals(3,:) * err_tan * %CL_rad2deg, "k-o", "thickness", 2); 

text = msprintf("%.0fx%.0f km, elev=[%.1f,%.1f] deg, err=%.1f km", hp/1000, ha/1000, ..
                elev_visi(1)*%CL_rad2deg, elev_visi(2)*%CL_rad2deg, err_tan/1000); 

a.title.text = "Pointing error - non rot. Earth" + " - " + text;
a.x_label.text = "Minimum duration of visibility intervals (min)";
a.y_label.text = "Maximum simulated angular error at acquisition (deg)";

CL_g_stdaxes(); 

labels = msprintf("%.0f%%\n", pvals' * 100); 
CL_g_legend(a, labels, header="Max value"); 

f.immediate_drawing="on";
f.visible="on";



