//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [cal] = CL__dat_jd2cal(jd)
  // Julian date to calendar date
  // jd = Two-part Julian date. jd(1,:) preferably an integer and jd(2,:) preferably in [0,1[ (2xN)
  // cal = Calendar date (6xN) [year;month;day;hour;minute;second]

  
// sub-function for the algorithm (MEEUS)
function [cal] = dat_jd2cal(jd)

  if (find(jd < 0) <> [])
    CL__error("Julian date cannot be negative");
  end

  // In the original Meeus algorithm: 
  // jd = jd(1,:) + jd(2,:) + 0.5; 
  // F = int(jd); 
  // F = jd - Z;
  // Special adaptations to handle numerical errors:  
  // jd becomes : jd1i + jd2i + frac (see below) 
  // with: frac < 1 (after adjustment)
  // => garantees consistency with the processing of the decimal part (frac) 
  
  // 0.5 added to 2nd part (supposed to be smaller) 
  jd1i = floor(jd(1,:)); 
  jd2i = floor(jd(2,:) + 0.5); 
  // warning : parentheses. frac here is >= 0 and < 2
  frac = (jd(1,:) - jd1i) + ((jd(2,:) + 0.5) - jd2i); 
   
  I = find(frac >= 1); 
  if (I <> [])
    jd2i(I) = jd2i(I) + 1; 
    frac(I) = frac(I) - 1; 
  end 
  
  // Algorithm (see Meeus) 
  // Note on int: int(1.5)=1, int(-1.5)=-1
  // note 2: int replaced by floor here. Same because jd >= 0 
  Z = jd1i + jd2i 
  F = frac;

  A = Z; 
  ALPHA = int((Z - 1867216.25)/36524.25);
  k = find(Z >= 2299161); 
  if (k <> [])
    A(k) = Z(k) + 1 + ALPHA(k) - int(ALPHA(k)/4); 
  end
  
  B = A + 1524;
  C = int((B - 122.1)/365.25);
  D = int(365.25 * C);
  E = int((B - D)/30.6001);

  RD = B - D - int(30.6001*E) + F; // decimal day

  day = floor(RD);

  month = E - 1; 
  k = find(E == 14 | E == 15); 
  if (k <> [])
    month(k) = E(k) - 13; 
  end
  
  year = C - 4716; 
  k = find(month == 1 | month == 2); 
  if (k <> [])
    year(k) = C(k) - 4715; 
  end
  
  // Retrieve decimal part of the day
  rh = (frac - floor(frac))*24; // <=> F (see above = fractional part from 00:00) 
  hours = floor(rh);
  minutes = floor((rh-hours)*60); 
  seconds = ((rh-hours)*60 - minutes)*60; 

  // output
  cal = [year; month; day; hours; minutes; seconds];

endfunction


// [] => []
if (jd == [])
  cal = []; 
  return; 
end

// %nan is returned if %nan is present in input
N = size(jd, 2); 
cal = %nan * ones(6,N); 

// computation for non %nan inputs
I = find(~isnan(sum(jd,"r"))); 
if (I <> [])
  cal(:,I) = dat_jd2cal(jd(:,I)); 
end

endfunction



