//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [tcb_tdb] = CL__dat_tdb2tcb(cjd_tdb)
// TDB to TCB conversion
//
// Calling Sequence
// tcb_tdb = CL__dat_tdb2tcb(cjd_tdb)
//
// Description
// <itemizedlist><listitem>
// <p>Converts the time of a modified Julian date (from 1950.0) from TCB time scale to 
// TDB time scale.</p>
// <p>The conversion is compliant with the new definition for TDB (IAU 2006).</p>
// <p></p></listitem>
// <listitem>
// <p>See <link linkend="Dates_and_time_scales">Dates and time scales</link> for more details.</p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// cjd_tdb: Modified (1950.0) Julian date in TDB time scale. (2xN)
// tcb_tdb: TCB minus TDB = number of seconds to be added to cjd_tdb to get cjd_tcb. (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// IAU 2006 Resolution 3 : Re-definition of Barycentric Dynamical Time, TDB (XXVIth International Astronomical Union General Assembly)
// U.S NAVAL OBSERVATORY CIRCULAR NO. 179, by G.H. Kaplan, 2005/10/20
//
// See also
// CL__dat_tcb2tdb
//
// Examples
// cjd_tdb = CL_dat_cal2cjd(2006,1,15,21,25,42.683799);
// tcb_tdb = CL__dat_tdb2tcb(cjd_tdb);

// Declarations:

// Code:

// See http://syrte.obspm.fr/iauJD16/IAU2006_Resol3.pdf or 
//     http://aa.usno.navy.mil/publications/docs/Circular_179.pdf for more details

// One iteration for a better consistency with CL__dat_tcb2tdb
// Find: TCB such that F(TCB) = TDB, F <=> CL__dat_tcb2tdb
// 1st guess: TCB0 
// Final result: F(TCB0) - TDB removed from TCB0 


// 1st guess 
tcb_tdb = -CL__dat_tcb2tdb(cjd_tdb); 

// Final result
cjd_tcb = [cjd_tdb(1,:) ; cjd_tdb(2,:) + tcb_tdb / 86400]; 
deltat = CL__dat_tcb2tdb(cjd_tcb) + ((cjd_tcb(1,:) - cjd_tdb(1,:)) + (cjd_tcb(2,:) - cjd_tdb(2,:))) * 86400;
tcb_tdb = tcb_tdb - deltat; 

endfunction
