//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [tt_tdb] = CL__dat_tdb2tt(cjd_tdb)
// TDB to TT conversion
//
// Calling Sequence
// tt_tdb = CL__dat_tdb2tt(cjd_tdb)
//
// Description
// <itemizedlist><listitem>
// <p>Converts the time of a modified Julian date (from 1950.0) from TDB time scale to 
// TT time scale.</p>
// <p>By definition : TT = TDB + CL__dat_tdb2tt(TDB) </p>
// <p>The accuracy is about 40 micro seconds. </p>
// <p>The definition is consistent with SPICE to about 1 micro second. </p>
// <p>The amplitude of TT-TDB is less than 2 milliseconds </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// cjd_tdb: Modified (1950.0) Julian date in TDB time scale. (2xN)
// tt_tdb: TT minus TDB = number of seconds to be added to cjd_tdb to get cjd_tt. (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL__dat_tt2tdb
//
// Examples
// cjd_tdb = CL_dat_cal2cjd(2000,1,1,12,0,0);
// tt_tdb = CL__dat_tdb2tt(cjd_tdb);

// Declarations:

// Code:

// One iteration for a better consistency with CL__dat_tt2tdb
// Find: TT such that F(TT) = TDB, F <=> CL__dat_tt2tdb

// 1st guess (using TT = TDB)
tt_tdb = -CL__dat_tt2tdb(cjd_tdb); 

// Final result (using a better estimate of TT)
cjd_tt = cjd_tdb + [zeros(tt_tdb); tt_tdb / 86400]; 
tt_tdb = -CL__dat_tt2tdb(cjd_tt); 


endfunction
