//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [pos_ell,jacob] = CL_co_car2ell(pos_car, er, obla)
// Cartesian coordinates to elliptical coordinates 
// 
// Calling Sequence
// [pos_ell, jacob] = CL_co_car2ell(pos_car [[, er, obla]])
//
// Description
// <itemizedlist><listitem>
// <p>Converts cartesian coordinates into elliptical ("geodetic") coordinates.</p> 
// <p> Elliptical coordinates are longitude, latitude and altitude with respect to 
// some reference ellipsoid. </p>
// <p> The reference ellipsoid is an ellipsoid of revolution around the Z axis. It is  
// characterized by 2 parameters: </p>
// <p> - radius of the sphere in the equatorial plane, </p>
// <p> - oblateness (or flattening) of ellipse in the X-Z plane. </p>
// <p><inlinemediaobject><imageobject><imagedata fileref="elliptical_coord.gif"/></imageobject></inlinemediaobject></p>
// <p></p></listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p> - The transformation jacobian is computed if the corresponding output argument exists. </p>
// <p> - Be careful that the 3rd elliptical coordinate is an altitude and not the distance to the planet center. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// pos_car : Position vector in cartesian coordinates [X;Y;Z]. [m] (3xN)
// er: (optional) Equatorial radius. Default: %CL_eqRad. [m] (1x1)
// obla: (optional) Oblateness of reference ellipsoid. Default: %CL_obla. [-] (1x1)
// pos_ell : Position vector in elliptical ("geodetic") coordinates [lon; lat; alt]. [rad; rad; m] (3xN)
// jacob : Transformation jacobian. (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1) Mecanique Spatiale, Cnes - Cepadues Editions, Tome I, section 3.2.3 (Les reperes de l'espace et du temps, Relations entre les coordonnees)
// 2) CNES - MSLIB FORTRAN 90, Volume T (mt_car_geod)
// 3) GPS - Localisation et navigation, Serge Botton et al, Hermes - page 65
//
// See also
// CL_co_ell2car
// CL_co_car2sph
//
// Examples
// // Example 1
// pos_car = [3000.e3; 4000.e3; 5000.e3];
// pos_ell = CL_co_car2ell(pos_car);
//
// // Example 2 : consistency test
// pos_car = [3000.e3; 4000.e3; 5000.e3];
// [pos_ell, jacob1] = CL_co_car2ell(pos_car);
// [pos_car2, jacob2] = CL_co_ell2car(pos_ell);
// pos_car2 - pos_car  // => zero 
// jacob2 * jacob1  // => identity matrix

// Declarations:

// ----------------------------
// pos_car: 3xN
// er: equatorial radius
// obla: oblateness (in [0, 1[) 
// ----------------------------
function [pos_ell] = car2ell_meth1(pos_car, er, obla)
  // Solves: 
  // x = (N + h) * cos(lat) * cos(lon)
  // y = (N + h) * cos(lat) * sin(lon)
  // z = (N * (1-e^2) + h) * sin(lat)
  // with:
  // N = er / sqrt(1 - e^2 * sin(phi)^2) 
  // e^2 = obla * (2-obla)
  
  e2 = obla * (2 - obla); 
  
  // eps: tolerance on latitude 
  eps = 100 * %eps; 

  x = pos_car(1, :); 
  y = pos_car(2, :); 
  z = pos_car(3, :);
  p = sqrt(x.^2 + y.^2); 

  // lon = 0 if x and y are 0
  lon = %nan * ones(x); 
  lat = %nan * ones(x);  
  alt = %nan * ones(x); 
  
  // K: indices to be computed
  // computes only if components are not %nan and not [0; 0; 0]
  K = find(~isnan(CL_dot(pos_car)) & CL_dot(pos_car) > 0); 
      
  if (K <> [])
    lon(K) = CL_rMod(atan(y(K), x(K)), 0, 2*%pi); // 0 if x = y = 0
    lat(K) = atan(z(K), p(K)); 
  end
  
  // NB: 
  // r <=> N in formulas
  // w = sqrt(1 - e^2 * sin(phi)^2)
  w = zeros(x);
  r = zeros(x); 
  lat0 = zeros(x); 

  iter = 1; 
  iter_max = 20; 
  
  while (K <> [] & iter <= iter_max)     
    lat0(K) = lat(K); 
    w(K) = sqrt(1 - e2 * sin(lat0(K)).^2); 
    r(K) = er ./ w(K); 
  
    alt(K) = p(K) .* cos(lat0(K)) + z(K) .* sin(lat0(K)) - er * w(K); 
    lat(K) = atan(z(K) .* (r(K) + alt(K)), p(K) .* (r(K) * (1-e2) + alt(K))); 
    lat(K) = max(min(lat(K), %pi/2), -%pi/2); 
    
    K = find(abs(lat - lat0) > eps); 
    iter = iter + 1; 
  end
  
  pos_ell = [lon; lat; alt]; 
  pos_ell(:,K) = %nan; // no convergence 
  
endfunction


// ----------------------------
// method 2: works in more cases
// but not accurate (or does not converge) near the poles
// ----------------------------
function [pos_ell] = car2ell_meth2(pos_car, er, obla)
  eps = 1000 * %eps; 
  epsreq = er * eps; 

  x = pos_car(1,:); 
  y = pos_car(2,:); 
  z = pos_car(3,:); 

  disxy2 = x.^2 + y.^2; 
  disxy = sqrt(disxy2); 
  dist2 = z.^2 + disxy2; 

  // constants
  e2 = obla * (2 - obla); 

  lon = CL_rMod(atan(y, x), 0, 2*%pi); // 0 if x = y = 0
  lat = zeros(x);
  alt = zeros(x);

  // --- Case 1 : latitude ~ +/- pi/2
  // Note: Jacobian not computable in this case
  // cond1 = (disxy < epsreq); 
  cond1 = (disxy < eps * abs(z)); 
  I = find(cond1); 
  if (I <> [])
    lat(I) = sign(z(I)) * (%pi/2); 
    alt(I) = abs(z(I)) - er * (1-obla); 
  end

  // --- Case 2: ~ on the equator plane
  // cond2 = (abs(z) < epsreq); 
  cond2 = (abs(z) < eps * disxy); 
  I = find(cond2); 
  if (I <> [])
    lat(I) = 0 * x(I);  // 0 or %nan 
    alt(I) = disxy(I) - er; 
  end

  // --- Case 3: Other cases
  cond3 = ~(cond1 | cond2) & ~isnan(dist2); 
  I = find(cond3); 

  if (I <> [])
    coralt = (disxy2(I) + (z(I).^2) / (1 - obla)^2) / er^2; // size = I
    alt(I) = sqrt(dist2(I)) .* (1 - 1 ./ sqrt(coralt)); 

    lat0 = zeros(x); 
    corxy = (1 - obla)^2 * disxy(I); // size = I
    lat0(I) = atan(z(I) ./ corxy); 

    // intell = %t => inside the ellipsoid
    intell = (disxy2 + (z.^2) / (1 - obla)^2  <  er^2);  

    // Cond = %t => complex case with 3 roots
    Cond = (disxy < e2 * er); 
    K = find(cond3 & intell & Cond); 
    if (K <> [])
      lat(K) = atan(sqrt((er ./ disxy(K)).^2 - 1) / (1 - obla)) .* sign(z(K)); 
    end
    K = find(cond3 & intell & ~Cond); 
    if (K <> [])
      lat(K) = lat0(K) - (obla / (er * (1 - obla))) .* sin(2 * lat0(K)) .* alt(K); 
    end
    
    Cond = (alt <= er & obla <= 0.5); 
    K = find(cond3 & ~intell & Cond); 
    if (K <> [])
      lat(K) = lat0(K) - (obla / (er * (1 - obla))) .* sin(2 * lat0(K)) .* alt(K); 
    end
    K = find(cond3 & ~intell & ~Cond); 
    if (K <> [])
      lat(K) = lat0(K);
    end
    
    K = find(cond3 & lat > %pi/2); 
    if (K <> [])
      lat(K) = %pi - lat(K); 
    end
    
    K = find(cond3 & lat < -%pi/2); 
    if (K <> [])
      lat(K) = -%pi - lat(K); 
    end
    
    // Newton iterations
    iter = 1; 
    iter_max = 30; 

    // K: indices where no convergence yet
    K = find(cond3); 

    while (K <> [] & iter <= iter_max)
      xlat0 = lat(K); 
      sinl0 = sin(xlat0);
      tanl0 = tan(xlat0);
      denomi = sqrt(1 - e2 * sinl0.^2);
      fonc  = -(sinl0 .* e2 * er) ./ denomi + disxy(K) .* tanl0 - z(K);
      dfonc = -(er ./ (denomi.^3)) .* e2 .* cos(xlat0) + disxy(K) .* (1 + tanl0.^2);

      // update latitude by Newton
      xlat = xlat0 - fonc ./ dfonc;  

      // Check latitude is in [-pi/2, pi/2] or adjust
      k = find(xlat > %pi/2);
      if (k <> [])
        xlat(k) = 0.5 * (%pi/2 + xlat0(k)); 
      end
      k = find(xlat < -%pi/2); 
      if (k <> [])
        xlat(k) = 0.5 * (-%pi/2 + xlat0(k)); 
      end
      
      // update
      lat(K) = xlat; 

      // converged ? 
      k = find(abs(xlat - xlat0) > eps);   
      K = K(k); 

      iter = iter + 1;

    end // end while

    // K indices: not converged => %nan
    lat(K) = %nan;

    // --- results (Case 3)
    w2 = 1 - e2 * (sin(lat(I)).^2); // size = size(I)
    r = er ./ sqrt(w2); 
    alt(I) = disxy(I) ./ cos(lat(I)) - r; 

  end
  
  // --- Final results 
  pos_ell = [lon; lat; alt]

endfunction


// ----------------------------
// Jacobian
// ----------------------------
function [jacob] = car2ell_jacob(pos_ell, er, obla)

  e2 = obla * (2 - obla); 
  eps = 100 * %eps; 

  N = size(pos_ell, 2); 
  jacob = %nan * ones(3, 3, N);
  
  lon = pos_ell(1, :); 
  lat = pos_ell(2, :); 
  alt = pos_ell(3, :); 

  w2 = 1 - e2 * sin(lat).^2; // 1xN 
  r = er ./ sqrt(w2); 

  C = (1 - e2) ./ w2; 
  r_plus_h = r + alt; 
  C_r_plus_h = C .* r + alt; 
  
  
  // Cases where jacobian is computable and input <> nan  
  I = find(r_plus_h > 0 & C_r_plus_h > 0 & abs(cos(lat)) > eps); 
  
  if (I <> [])
    coslon = cos(lon(I)); 
    sinlon = sin(lon(I)); 
    coslat = cos(lat(I)); 
    sinlat = sin(lat(I)); 
    
    jacob(2, 1, I) = -sinlat .* coslon ./ C_r_plus_h(I);
    jacob(2, 2, I) = -sinlat .* sinlon ./ C_r_plus_h(I);
    jacob(2, 3, I) = coslat ./ C_r_plus_h(I);

    jacob(1, 1, I) = -sinlon ./ (r_plus_h(I) .* coslat);
    jacob(1, 2, I) = coslon ./ (r_plus_h(I) .* coslat);
    jacob(1, 3, I) = 0 * coslon;  // 0 or %nan 

    jacob(3, 1, I) = coslat .* coslon; 
    jacob(3, 2, I) = coslat .* sinlon; 
    jacob(3, 3, I) = sinlat; 

  end

endfunction


// Code:

if (~exists("er", "local")); er = CL__dataGetEnv("eqRad"); end
if (~exists("obla", "local")); obla = CL__dataGetEnv("obla"); end

if (obla < 0 | obla >= 1) 
  CL__error("Invalid value for obla: should be in [0, 1["); 
end

// Check size
CL__checkInputs(pos_car, 3); 

// default outputs 
pos_ell = []; 
jacob = []; 

// special case ([])
if (pos_car == [])
  return; // RETURN
end


// check number of output arguments (for Jacobian)
if (argn(1) > 1)
  compute_jacob = %t; 
else
  compute_jacob = %f; 
end

// If one coordinate is %nan => all set to %nan 
I = find(or(isnan(pos_car), "r")); 
pos_car(:, I) = %nan; 

// computation (coordinates - algorithm 1) 
pos_ell = car2ell_meth1(pos_car, er, obla); 

// if no convergence => try algorithm 2
I = find(or(isnan(pos_ell), "r") & ~or(isnan(pos_car), "r")); 
if (I <> [])
  pos_ell(:,I) = car2ell_meth2(pos_car(:, I), er, obla); 
end

// Check result 
// eps = tolerance on position 
// eps = 1000 * %eps * er; 
// pos_car2 = CL_co_ell2car(pos_ell, er=er, obla=obla);
// I = find(CL_dot(pos_car2-pos_car) > eps^2);
// pos_ell(:,I) = %nan; 

// jacobian
if (compute_jacob)
  jacob = car2ell_jacob(pos_ell, er, obla); 
end

endfunction


