//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [pos_car,jacob]=CL_co_ell2car(pos_ell, er, obla)
// Elliptical coordinates to cartesian coordinates
// 
// Calling Sequence
// [pos_car, jacob] = CL_co_ell2car(pos_ell [[, er, obla]])
//
// Description
// <itemizedlist><listitem>
// <p>Converts elliptical ("geodetic") coordinates into cartesian coordinates.</p> 
// <p> Elliptical coordinates are longitude, latitude and altitude with respect to 
// some reference ellipsoid. </p>
// <p> The reference ellipsoid is an ellipsoid of revolution around the Z axis. It is  
// characterized by 2 parameters: </p>
// <p> - radius of the sphere in the equatorial plane, </p>
// <p> - oblateness (or flattening) of ellipse in the X-Z plane. </p>
// <p><inlinemediaobject><imageobject><imagedata fileref="elliptical_coord.gif"/></imageobject></inlinemediaobject></p>
// <p></p></listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p> - The transformation jacobian is computed if the corresponding output argument exists. </p>
// <p> - Be careful that the 3rd elliptical coordinate is an altitude and not the distance to the planet center. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// pos_ell : [lon; lat; alt] Position vector in elliptical ("geodetic") coordinates. [rad,m] (3xN)
// er: (optional) Equatorial radius. Default: %CL_eqRad. [m] (1x1)
// obla: (optional) Oblateness of reference ellipsoid. Default: %CL_obla. [-] (1x1)
// pos_car : [x;y;z] Position vector in cartesian coordinates. [m] (3xN)
// jacob : Transformation jacobian. (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1) Mecanique Spatiale, Cnes - Cepadues Editions, Tome I, section 3.2.3 (Les reperes de l'espace et du temps, Relations entre les coordonnees)
// 2) CNES - MSLIB FORTRAN 90, Volume T (mt_car_geod)
// 3) GPS - Localisation et navigation, Serge Botton et al, Hermes - page 65
//
// See also
// CL_co_car2ell
// CL_co_car2sph
//
// Examples
// // Example 1 :
// pos_ell = [0; %pi/4; 1000.e3];
// pos_car = CL_co_ell2car(pos_ell)
//
// // Example 2 : consistency test
// pos_ell = [0; %pi/4; 1000.e3];
// [pos_car, jacob1] = CL_co_ell2car(pos_ell); 
// [pos_ell2, jacob2] = CL_co_car2ell(pos_car);
// pos_ell2 - pos_ell  // => zero 
// jacob2 * jacob1  // => identity matrix

// Declarations:

// pos_ell: 3xN
// er: equatorial radius
// obla: oblateness (in [0, 1[) 
// jacob = [] if not computed
function [pos_car, jacob] = ell2car(pos_ell, er, obla, compute_jacob)
  // Computes: 
  // x = (N + h) * cos(lat) * cos(lon)
  // y = (N + h) * cos(lat) * sin(lon)
  // z = (N * (1 - e^2) + h) * sin(lat)
  // with:
  // N = er / sqrt(1 - e^2 * sin(phi)^2) 
  // e^2 = obla * (2 - obla)

  pos_car = []; 
  jacob = []; 
  
  // (geodetic) longitude, latitude and altitude
  lon = pos_ell(1, :);
  lat = pos_ell(2, :);
  alt = pos_ell(3, :);

  // precomputes for efficiency
  coslon = cos(lon); 
  sinlon = sin(lon); 
  coslat = cos(lat); 
  sinlat = sin(lat); 

  // NB: r <=> N in formulas
  e2 = obla * (2 - obla); 
  w2 = 1 - e2 .* sinlat.^2;  
  r = er ./ sqrt(w2);  
  r_plus_h = r + alt; 

  // Cartesian coordinates
  x = r_plus_h .* coslat .* coslon;
  y = r_plus_h .* coslat .* sinlon;
  z = (r * (1 - e2) + alt) .* sinlat;

  pos_car = [x; y; z];

  if (compute_jacob)
    N = size(pos_ell, 2);
    jacob = zeros(3, 3, N); 

    C = (1 - e2) ./ w2;  
    C_r_plus_h = C .* r + alt; 

    jacob(1,1,1:N) = -r_plus_h .* coslat .* sinlon; 
    jacob(1,2,1:N) = -C_r_plus_h .* sinlat .* coslon; 
    jacob(1,3,1:N) = coslat .* coslon; 

    jacob(2,1,1:N) = r_plus_h .* coslat .* coslon; 
    jacob(2,2,1:N) = -C_r_plus_h .* sinlat .* sinlon; 
    jacob(2,3,1:N) = coslat .* sinlon; 

    jacob(3,1,1:N) = 0 * coslat;  // 0 or %nan
    jacob(3,2,1:N) = C_r_plus_h .* coslat; 
    jacob(3,3,1:N) = sinlat; 

  end

endfunction

  
// Code:

if (~exists("er", "local")); er = CL__dataGetEnv("eqRad"); end
if (~exists("obla", "local")); obla = CL__dataGetEnv("obla"); end

if (obla < 0 | obla >= 1) 
  CL__error("Invalid value for obla: should be in [0, 1["); 
end

// Check size
CL__checkInputs(pos_ell, 3); 

// default outputs 
pos_car = []; 
jacob = []; 

// special case ([])
if (pos_ell == [])
  return; // RETURN
end

// check number of output arguments (for Jacobian)
if (argn(1) > 1)
  compute_jacob = %t; 
else
  compute_jacob = %f; 
end

// If one coordinate is %nan => all set to %nan 
I = find(or(isnan(pos_ell), "r")); 
pos_ell(:, I) = %nan; 

// computation (coordinates and/or Jacobian) 
[pos_car, jacob] = ell2car(pos_ell, er, obla, compute_jacob); 

endfunction

