//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [str] = CL_dat_cal2str(cal, nd, form)
// Calendar date to string 
//
// Calling Sequence
// str = CL_dat_cal2str(cal [[, nd, form]])
//
// Description
// <itemizedlist><listitem>
// <p>Converts a calendar date into a string.</p>
// <p></p>
// <p>The calendar date is a column vector:  [year;month;day] or
// [year;month;day;hours,minutes;seconds]. </p>
// <p></p>
// <p>The argument <b>nd</b> is the number of digits for the fractional seconds. It 
// must be between 0 and 6, or equal to -1. </p>
// <p>- If nd is positive or 0, the seconds are rounded to the nearest 10^-nd second 
// and only nd digits
// are written. </p>
// <p>- if nd is equal to -1, the calendar date should be an exact day, and only 
// the day appear in the string. </p>   
// <p></p>
// <p>The argument <b>form</b> is the output format. It can be "CL" (CelestLab format) or 
// "ISO" (ISO8601 format). The default format is "CL". </p>
// <p></p>
// <p><b>Note:</b></p>
// <p>The input calendar date is supposed to be a valid calendar date. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// cal: Calendar date (3xN or 6xN)
// nd: (optional) Number of digits for the fractional seconds, or -1. Default is 3 (1x1)
// form: (string, optional) Output format: "CL" or "ISO". Default is "CL" (1x1)
// str: (string) Formatted calendar date (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_dat_str2cal
// 
// Examples
// cal = [2012; 1; 1; 23; 59; 59.999]; 
// CL_dat_cal2str(cal) 
// CL_dat_cal2str(cal, nd = 3, form = "ISO")
// CL_dat_cal2str(cal, nd = 0)


// Declarations:

// round sec and recompute calendar date
// (while trying to keep accuracy)
// cal: 6xN
function [cal] = cal2str_adjust(cal, nd)

  jj = CL_dat_cal2cjd(cal(1:3,:)); // day
  secs = cal(4,:)*3600 + cal(5,:)*60 + cal(6,:); // seconds
  
  n = floor(secs / 86400);   
  jj = jj + n; 
  secs = secs - n * 86400; 

  if (nd >= 0)
    // round off seconds to nearest 10^-nd
    P = 10^nd; 

    xsecs = round(secs * P);

    I = find(xsecs == 86400 * P); 
    if (I <> [])
      jj(I) = jj(I) + 1; 
      xsecs(I) = 0; 
    end

    secs = xsecs / P; 

  else // if nd == -1
    // secs should be 0 (exact day)
    I = find(secs <> 0); 
    if (I <> [])
      CL__error("Invalid calendar date: not an exact day"); 
    end

  end

  // regenerate cal
  // only integer part for the day for accuracy
  // seconds : separately

  cal = CL_dat_cjd2cal(jj); // whole number
  cal(4:6,:) = CL_sec2hms(secs); // seconds

endfunction 

function [str] = cal2str_format(cal, nd, form)
  strcal = emptystr(cal); 

  strcal(1,:) = msprintf("%04d\n", cal(1,:)')';   // year
  strcal(2,:) = msprintf("%02d\n", cal(2,:)')'; // month
  strcal(3,:) = msprintf("%02d\n", cal(3,:)')'; // day
  strcal(4,:) = msprintf("%02d\n", cal(4,:)')'; // hours
  strcal(5,:) = msprintf("%02d\n", cal(5,:)')'; // minutes

  // Separators according to format
  if (form == "CL")
    SEP1 = "/"; 
    SEP2 = " "; 
  else // ISO
    SEP1 = "-"; 
    SEP2 = "T"; 
  end 

  if (nd >= 0)
    P = 10^nd;   
    secs = round(cal(6,:) * P) / P; 
    isec = floor(secs);
    fsec = round((secs - isec) * P);

    if (nd == 0)
      strcal(6,:) = msprintf("%02d\n", isec')'; 
    else
      fmt = msprintf("%%02d.%%0%dd", nd); 
      strcal(6,:) = msprintf(fmt + "\n", isec', fsec')'; 
    end
  else
    strcal(6,:) = "00"; // unused 
  end 
    
  // Replace Nan values (bug with some Scilab versions) 
  I = find(isnan(cal)); 
  strcal(I) = "Nan" 
  
  // Final string 
  if (nd >= 0) 
    str = strcal(1,:) + SEP1 + strcal(2,:) + SEP1 + strcal(3,:) + SEP2 + ...
          strcal(4,:) + ":" + strcal(5,:) + ":" + strcal(6,:); 
  else
    str = strcal(1,:) + SEP1 + strcal(2,:) + SEP1 + strcal(3,:); 

  end

endfunction 


// Code:
if (~exists("nd", "local")); nd = 3; end
if (~exists("form", "local")); form = "CL"; end

// check nd is an integer between -1 and 6
if (typeof(nd) <> "constant" | size(nd, "*") <> 1 | ... 
       nd <> round(nd) | (nd < -1 | nd > 6))
  CL__error("Invalid value for argument nd"); 
end

// check value of form
if (form <> "CL" & form <> "ISO")
  CL__error("Invalid value for argument form"); 
end 

if (cal == []); 
  str = []; 
  return;
end

if (size(cal,1) <> 3 & size(cal,1) <> 6)
  CL__error("Invalid argument"); 
end 

if (typeof(cal) <> "constant")
  CL__error("Invalid calendar date argument"); 
end 
  
if (size(cal,1) == 3)
  cal = [cal; zeros(cal)]; 
end

// adjust value considering number of digits 
cal = cal2str_adjust(cal, nd); 

// formatting
str = cal2str_format(cal, nd, form); 

endfunction


