//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [date_out] = CL_dat_convert(type_in, type_out, date_in, dform)
// Date type conversion 
//
// Calling Sequence
// date_out = CL_dat_convert(type_in, type_out, date_in [[, dform]])
//
// Description
// <itemizedlist><listitem>
// <p>Converts a date from one type to another.</p>
// <p>The available types are:</p>
// <p>- "jd": Julian date </p>
// <p>- "cjd": Modified Julian date from 1950.0</p>
// <p>- "mjd": Modified Julian date: MJD = JD - 2400000.5 </p>
// <p>- "cal": Calendar date. [year;month;day] or [year;month;day;hours;minutes;seconds]</p>
// <p></p>
// <p>For non calendar dates, the value in days can be coded on 1 or 2 real numbers. 
// The argument "dform" specifies the desired output format: </p>
// <p>- <b>"d1"</b> : 1 real number </p>
// <p>- <b>"d2"</b> : 2 real numbers (on 2 rows) </p>
// <p>- <b>""</b> : Default value which means "keep accuracy when possible". </p>
// <p></p>
// <p>See <link linkend="Dates_and_time_scales">Dates and time scales</link> for more details.</p>
// <p></p>
// </listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p>The input  and output dates are relative to the same time scale.</p>
// </listitem>
// </itemizedlist>
//
// Parameters
// type_in: (string) Type of input date: "jd", "cjd", "mjd" or "cal" (1x1)
// type_out: (string) Type of output date: "jd", "cjd", "mjd" or "cal" (1x1)
// date_in: Input date. (1xN or 2xN or 3xN or 6xN). (3xN or 6xN only if type_in is "cal")
// date_out: Output date (1xN or 2xN or 6xN). (6xN only if type_out is "cal")
// dform: Output format for "real" dates: "d1" or "d2" or "". Default ls ""
// Authors
// CNES - DCT/SB
//
// Examples
// cjd = 21956.3;
//
// // Convert from cjd to jd
// jd = CL_dat_convert("cjd", "jd", cjd);
// 
// // Convert from jd to calendar
// cal = CL_dat_convert("jd", "cal", jd);
// 

// Declarations:

// Code:
if (argn(2) <> 3 & argn(2) <> 4)
  CL__error("Wrong number of input arguments");
end

if (~exists("dform", "local")); dform = ""; end 

if (dform <> "" & dform <> "d1" & dform <> "d2")
   CL__error("Invalid value for argument dform"); 
end

if (typeof(type_in) <> "string" | typeof(type_out) <> "string")
   CL__error("String expected for type_in and type_out"); 
end

if (size(type_in,"*") <> 1 | size(type_out,"*") <> 1); CL__error("Wrong dimension(s)"); end;
  
// Validity of strings
type_par_names = ["jd", "cjd", "mjd", "cal"];
if (find(type_in == type_par_names) == [] | find(type_out == type_par_names) == [])
  CL__error("Unrecognized string(s)");
end

// Handle [] case
if (date_in == [])
  date_out = [];
  return; // <-- RETURN
end

// Dimension of input date
if (type_in <> "cal")
  if (size(date_in,1) > 2); CL__error("Wrong dimension for input date"); end;
end

if (type_in == "cal" & size(date_in,1) <> 3 & size(date_in,1) <> 6)
  CL__error("Wrong dimension for input date");
end

// change the default value of "dform":
// - same as input when possible
// - d1 for compatibility reason
if (dform == "")
  if (type_in == "cal" | size(date_in, 1) == 1)
    dform = "d1"; 
  else 
    dform = "d2"; 
  end
end  

if (type_in == type_out)
  // Special case
  date_out = date_in; 
  
else
  // convert in/out to 2-part days if type is not "cal" 
  if (size(date_in, 1) == 1)
    date_in = [floor(date_in); date_in - floor(date_in)]; 
  end
  
  date_out = CL__dat_convert(type_in, type_out, date_in); 

  if (type_out <> "cal" & dform == "d1")
    date_out = date_out(1,:) + date_out(2,:); 
  end
  
end
  
endfunction
