//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [seconds] = CL_dat_offset(cjd, cjd_ref)
// Offset between 2 dates
//
// Calling Sequence
// [seconds] = CL_dat_offset(cjd, cjd_ref)
//
// Description
// <itemizedlist><listitem>
// <p>Computes the offset between 2 dates. </p> 
// <p>By definition: <b>seconds</b> = (<b>cjd</b> - <b>cjd_ref</b>) * 86400. </p>
// <p></p> 
// <p>This function can be used for dates represented as 1 real number (size: 1xN) or 2 real numbers (size: 2xN). </p>
// <p></p> 
// </listitem>
// <listitem>
// <p><b>Note</b>:</p>
// <p>All dates and durations are relative to the same time scale. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// cjd: Modified (1950.0) Julian date in any time scale. (1xN or 2xN or 1x1 or 2x1)
// cjd_ref: Modified (1950.0) Julian date in the same time scale as cjd. (1xN or 2xN or 1x1 or 2x1)
// seconds: Number of seconds between cjd_ref and cjd (same time scale as cjd and cjd_ref). (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_dat_shift
//
// Examples
// // Example 1: 
// cjd = CL_dat_cal2cjd(2000,1,1,12,3,1);
// cjd_ref = CL_dat_cal2cjd(2000,1,1,0,0,0);
// seconds = CL_dat_offset(cjd, cjd_ref)
// 
// // Example 2: 
// cjd = CL_dat_cal2cjd2(2000,1,1,12,3,1);
// cjd_ref = CL_dat_cal2cjd2(2000,1,1,0,0,0);
// seconds = CL_dat_offset(cjd, cjd_ref)

// Declarations:

// Code:


// MAIN

// check arguments / resize
if (size(cjd_ref, 1) == 1)
  cjd_ref = [cjd_ref; zeros(cjd_ref)]; 
end

if (size(cjd, 1) == 1)
  cjd = [cjd; zeros(cjd)]; 
end

if (cjd == [] | cjd_ref == [])
  seconds = []; 
  return; 
end 

seconds = ((cjd(1,:) - cjd_ref(1,:)) + (cjd(2,:) - cjd_ref(2,:))) * 86400; 

endfunction
