//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [cjd_shifted] = CL_dat_shift(cjd, seconds)
// Shift a date by a certain quantity
//
// Calling Sequence
// [cjd_shifted] = CL_dat_shift(cjd, seconds)
//
// Description
// <itemizedlist><listitem>
// <p>Shift a date by a certain quantity given in seconds. </p>
// <p>By definition: <b>cjd_shifted</b> = <b>cjd</b> + <b>seconds</b> / 86400. </p>
// <p></p> 
// <p>This function can be used for dates represented as 1 real number (size: 1xN) or 2 real numbers (size: 2xN). </p> 
// <p></p> 
// </listitem>
// <listitem>
// <p><b>Note</b>:</p>
// <p>All dates and durations are relative to the same time scale. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// cjd: Modified (1950.0) Julian date in any time scale. (1xN or 2xN or 1x1)
// seconds: Number of seconds. Same time scale as cjd. (1xN or 1x1)
// cjd_shifted: Shifted date. (same size as cjd)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_dat_offset
//
// Examples
// // Example 1: 
// cjd = CL_dat_cal2cjd(2000,1,1,12,0,0);
// cjd_shifted = CL_dat_shift(cjd, 0 : 60 : 500)
// 
// // Example 2: 
// cjd = CL_dat_cal2cjd2(2000,1,1,12,0,0);
// cjd_shifted = CL_dat_shift(cjd, 0 : 60 : 500)

// Declarations:

// Code:


// MAIN

// date on 2 rows 
nr = 2; 
if (size(cjd, 1) == 1)
  nr = 1; 
  cjd = [cjd; zeros(cjd)]; 
end

[cjd, seconds] = CL__checkInputs(cjd, 2, seconds, 1); 

if (cjd == [] | seconds == [])
  cjd_shifted = []; 
  return; 
end 

iseconds_days = floor(seconds / 86400); 

cjd_shifted = [cjd(1,:) + iseconds_days; cjd(2,:) + (seconds - iseconds_days * 86400) / 86400 ]; 

if (nr == 1)
  cjd_shifted = cjd_shifted(1,:) + cjd_shifted(2,:); 
end 


endfunction
