//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [cal] = CL_dat_str2cal(str)
// String to calendar date 
//
// Calling Sequence
// cal = CL_dat_str2cal(str)
//
// Description
// <itemizedlist>
// <listitem>
// <p>Decodes a string into a calendar date.</p>
// <p></p>
// <p>The input string must have the form: "year/month/day hours:minutes:seconds", or
// "year-month-day hours:minutes:seconds", or "year-month-day{T}hours:minutes:seconds"
// where {T} stands for the letter "T". </p> 
// <p>Space characters around "/", "-", ":", "T" are not significant. </p>
// <p>The "hours:minutes:seconds" part may be omitted. If so, the {T} separator must also 
// be omitted. </p> 
// <p></p>
// <p>The calendar date is a column vector of 6 elements : 
// [year; month; day; hours; minutes; seconds]. </p>
// <p></p>
// </listitem>
// <listitem>
// <p><b>Note</b>:</p>
// <p>Nan is returned (all 6 elements) if the input string format is not valid. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// str: Formatted calendar date (1xN)
// cal: Calendar date (6xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_dat_cal2str
// 
// Examples
// // Various valid strings 
// cal = CL_dat_str2cal("2012/1/1 12:0:0.1")
// cal = CL_dat_str2cal("2012/01/01")
// cal = CL_dat_str2cal("2012-1-1 12:0:0.1")
// // Valid ISO 8601 strings
// cal = CL_dat_str2cal("2012-01-01 12:00:00.1")
// cal = CL_dat_str2cal("2012-01-01T12:00:00.1")
// 
// // Same but with multiple strings (2nd one is invalid) 
// str = ["2012/1/1 12:0:0.1", "2012/1/1T0:0:0", "2012-01-01T12:00:00.1"]; 
// cal = CL_dat_str2cal(str)
// 

// Declarations:

// Read next value until separator "separ";
// str: string to parse (1xN) 
// separ: expected separator. " " valid at end of string (1x1)
// x = decoded value before separator. %nan if anomaly (e.g. separator found <> separ)
// str_next = string following separator (1xN)
function [x, str_next] = str2cal_nextval(str, separ)
  [x, str_next] = strtod(str);

  // no character read
  I = find(length(str_next) == length(str)); 
  x(I) = %nan; 

  // remove separator and get next part  
  // Note : part may add blanks at the end of the strings 
  // => stripblanks needed 
  if (separ == " ")
    sep = part(str_next, 1);
    I = find(sep <> " " & sep <> ""); 
    x(I) = %nan; 
  else
    str_next = stripblanks(str_next, %t); 
    sep = part(str_next, 1);
    I = find(sep <> separ); 
    x(I) = %nan; 
    str_next = part(str_next, 2:max(length(str_next)));
  end
  str_next = stripblanks(str_next,%t); 
  
  // if separator not blank: something must follow
  I = find(sep <> " " & sep <> "" & str_next == ""); 
  x(I) = %nan; 
  
endfunction

// parse string
// str: 1xN
// sep1: day separator (1x1)
// sep2: separator between day part and hour part (1x1)   
function [cal] = str2cal_parse(str, sep1, sep2)
  str = stripblanks(str, %t);

  N = size(str,2); 
  cal = zeros(6,N); 

  // day part (year/month/day) 
  [cal(1,:), str] = str2cal_nextval(str, sep1); 
  [cal(2,:), str] = str2cal_nextval(str, sep1); 
  [cal(3,:), str] = str2cal_nextval(str, sep2); 

  // hour part (hours:minutes:seconds)
  I = find(length(str) <> 0); 
  if (I <> [])
    [cal(4,I), str(I)] = str2cal_nextval(str(I), ":"); 
    [cal(5,I), str(I)] = str2cal_nextval(str(I), ":"); 
    [cal(6,I), str(I)] = str2cal_nextval(str(I), " "); 
  end
  
  I = find(length(str) <> 0); 
  cal(:,I) = %nan; 
  
  // set all elements to Nan if one is equal to Nan
  I = find(isnan(cal));
  if (I <> [])
    [i, j] = ind2sub(size(cal), I); 
    cal(:,j) = %nan; 
  end 
  
endfunction 

// check some values in calendar date 
// invalid dates are set to %nan 
function [cal] = str2cal_checkCal(cal)
  [i,j] = find(cal < 0 | isnan(cal)); 
  cal(:,j) = %nan; 

  [i,j] = find(cal <> round(cal)); 
  I = find(i <> 6); 
  cal(:,j(I)) = %nan; 

  // check values:
  // month in [1, 12], day in [1, 31], hours in [0, 24], mn and sec in [0,60]
  // ([0,24] and [0,60] to ensure reversivility) 
  [i,j] = find(cal(2,:) < 1  | cal(2,:) > 12 | cal(3,:) < 1  | cal(3,:) > 31 ...
             | cal(4,:) > 24 | cal(5,:) > 60 | cal(6,:) > 60 ); 
  cal(:,j) = %nan; 
  
endfunction

// Code:
if (str == []); 
  cal = []; 
  return;
end

// Error checking 
if (typeof(str) <> "string")
  CL__error("Invalid type for argument str"); 
end

if (size(str,1) <> 1)
  CL__error("Invalid size for argument str"); 
end

// try CelestLab format 
cal = str2cal_parse(str, "/", " "); 

// try CelestLab format but "-" instead of "/" 
I = find(isnan(cal(1,:))); 
if (I <> [])
  cal(:,I) = str2cal_parse(str(I), "-", " "); 
end 

// try ISO format (but spaces not significant)  
I = find(isnan(cal(1,:))); 
if (I <> [])
  cal(:,I) = str2cal_parse(str(I), "-", "T"); 
end 
  
// A few additional checks ...
cal = str2cal_checkCal(cal); 
  
endfunction


