//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [cen, r, normal] = CL_gm_circumCircle(P1, P2, P3)
// Circle passing through 3 points 
//
// Calling Sequence
// [cen, r, normal] = CL_gm_circumCircle(P1, P2, P3)
//
// Description
// <itemizedlist><listitem>
// <p>Computes the circle passing through 3 points (<b>P1</b>, <b>P2</b> and <b>P3</b>). </p>
// <p></p>
// <p>The circle is defined by: </p>
// <p>- <b>cen</b>: center of the circle,  </p>
// <p>- <b>r</b>: radius of the circle,  </p>
// <p>- <b>normal</b>: direction perpendicular to the plane containing the circle. </p>
// <p></p>
// </listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p>- The 3 points should be all different and not aligned. </p>
// <p>- All output quantities are set to [] if one of the input points is []. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// P1, P2, P3 : Three points in cartesian coordinates (3xN or 3x1)
// cen : Center of the circle - cartesian coordinates (3xN)
// r : Radius of the circle (1xN)
// normal : Normal direction (unit vector) (3xN)
//
// Authors
// CNES - DCT/SB
//
// Examples
// P1 = 2 * [-1; 0; 0];
// P2 = 2 * [1; 0; 0];
// P3 = 2 * [0; 1; 0];
// [cen, r, normal] = CL_gm_circumCircle(P1, P2, P3)


// Declarations:

// A, B, C: same size and not empty
function [cen, r, normal] = circle_3pts(A, B, C)
// The center X is looked for as : AX = lambda1 * AB + lambda2 * AC
// Writing ||AX|| = ||BX|| and ||AX|| = ||CX|| 
// which is the same as: (X-A).(X-A) = (X-B).(X-B) and (X-A).(X-A) = (X-C).(X-C)
// leads to 2 equations: 
// (u.u) * lambda1 + (u.v) * lambda2 = (u.u)/2
// (u.v) * lambda1 + (v.v) * lambda2 = (v.v)/2
// where (x.y) is the dot product. 
 
  u = B - A; 
  v = C - A; 
  uv = CL_dot(u, v); 
  u2 = CL_dot(u, u); 
  v2 = CL_dot(v, v); 
  
  D =  u2 .* v2 - uv .* uv; 
  lambda1 = v2 .* (u2 - uv) ./ (2 * D);  
  lambda2 = u2 .* (v2 - uv) ./ (2 * D);  

  cen = A + CL_dMult(lambda1, u) + CL_dMult(lambda2, v); 
  r = CL_norm(cen - A); 
  normal = CL_unitVector(CL_cross(u, v)); 
endfunction 


// Code:

// check inputs 
if (argn(2) <> 3)
  CL__error("Invalid number of inputs arguments: 3 expected"); 
end 

[P1, P2, P3] = CL__checkInputs(P1, 3, P2, 3, P3, 3);  

cen = []; 
r = []; 
normal = []; 
  
if (P1 == [] | P2 == [] | P3 == [])
  return; 
end 

[cen, r, normal] = circle_3pts(P1, P2, P3); 


endfunction
