//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [alpha2,delta2] = CL_gm_orbit2inertialSph(inc,raan,alpha1,delta1)
// Spherical coordinates conversion from an orbit frame to an inertial frame
//
// Calling Sequence
// [alpha2, delta2] = CL_gm_orbit2inertialSph(inc, raan, alpha1, delta1)
//
// Description
// <itemizedlist><listitem>
// <p>Performs a conversion of spherical coordinates from an "orbit" reference frame to an inertial reference frame.</p> 
// <p>The "orbit" reference frame is defined as follows:</p>
// <p>- X-axis: Towards the ascending node</p>
// <p>- Z-axis: Parallel to the angular momentum vector</p>
// <p>- Y-axis: Such that the frame is direct. </p>
// <p><inlinemediaobject><imageobject><imagedata fileref="orbit_inertial_sph.gif"/></imageobject></inlinemediaobject></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// inc: Orbit's inclination [rad] (1xN or 1x1)
// raan: Orbit's right ascension of ascending node [rad] (1xN or 1x1)
// alpha1: Right ascension in orbit frame [rad] (1xN or 1x1)
// delta1: Declination in orbit frame [rad] (1xN or 1x1)
// alpha2: Right ascension in inertial frame [rad] (1xN)
// delta2: Declination in inertial frame [rad] (1xN)
//
// Authors
// CNES - DCT/SB
//
// Examples
// inc = 1.5;
// raan = 1;
// alpha1 = 2.2;
// delta1 = 0.7;
// 
// [alpha2, delta2] = CL_gm_orbit2inertialSph(inc,raan,alpha1,delta1); 
// // Check result
// [alpha1b, delta1b] = CL_gm_inertial2orbitSph(inc,raan,alpha2,delta2)


// Declarations:


// Code:

// [] returned if one argument at least is empty
if (inc ==  [] | raan == [] | alpha1 == [] | delta1 == [])    
  alpha2 = []; 
  delta2 = []; 
  return; // <== RETURN 
end

// Regular case
// check size / resize
[inc, raan, alpha1, delta1] = CL__checkInputs(inc, 1, raan, 1, alpha1, 1, delta1, 1);

// Equations obtained by: 
//
// [ cos(d1)*cos(a1) ]        [ cos(d2)*cos(a2) ]
// [ cos(d1)*sin(a1) ]  = M * [ cos(d2)*sin(a2) ]
// [ sin(d1)         ]        [ sin(d2)         ]
// 
// With: 
// 
//     [ 1  0        0        ]   [  cos(raan) sin(raan) 0 ]
// M = [ 0  cos(inc) sin(inc) ] * [ -sin(raan) cos(raan) 0 ]
//     [ 0 -sin(inc) cos(inc) ]   [  0         0         1 ]
//
// Hence: 
// 
// cos(d1)*cos(a1) =  cos(d2) * cos(a2-raan)                                     [Eq1]
// cos(d1)*sin(a1) =  cos(d2) * cos(inc) * sin(a2-raan) + sin(d2) * sin(inc)     [Eq2]
// sin(d1)         = -cos(d2) * sin(inc) * sin(a2-raan) + sin(d2) * cos(inc)     [Eq3]
//
// Inverting: [Eq1], [Eq2]*cos(inc)-[Eq3]*sin(inc), [Eq2]*sin(inc)+[Eq3]*cos(inc)
//
// cos(d2) * cos(a2-raan) = cos(d1) * cos(a1)
// cos(d2) * sin(a2-raan) = cos(d1) * sin(a1) * cos(inc) - sin(d1) * sin(inc) 
// sin(d2)                = cos(d1) * sin(a1) * sin(inc) + sin(d1) * cos(inc) 

cosi = cos(inc);
sini = sin(inc);
ca1 = cos(alpha1);
sa1 = sin(alpha1);
cd1 = cos(delta1);
sd1 = sin(delta1);

// delta2 = elevation wrt equatorial (X,Y) plane

sindelta2 = sa1 .* cd1 .* sini + cosi .* sd1;
sindelta2(sindelta2 < -1) = -1;
sindelta2(sindelta2 > 1) = 1;
delta2 = asin(sindelta2);

// alpha2: angle in equatorial (X,Y) plane

// cos(delta2) * cos(alpha2-raan)
cd2ca2 = cd1 .* ca1; 
// cos(delta2) * sin(alpha2-raan)
cd2sa2 = sa1 .* cd1 .* cosi - sini .* sd1; 

alpha2 = %nan * ones(delta2);

I = find(cd2ca2 <> 0 | cd2sa2 <> 0);
if (I <> [])
  alpha2(I) = CL_rMod(atan(cd2sa2(I), cd2ca2(I)) + raan(I), -%pi, %pi);
end

endfunction
