//  Copyright (c) CNES  2019
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [posp, alpha] = CL_gm_projLine(pos1, pos2, pos)
// Orthogonal projection onto a straight line
// 
// Calling Sequence
// [posp, alpha] = CL_gm_projLine(pos1, pos2, pos)
//
// Description
// <itemizedlist>
// <listitem>
// <p>This function computes the image of a point by an orthogonal
// projection onto a straight line. </p>
// <p>The straight line is defined by two points: <b>pos1</b> and <b>pos2</b>.</p>
// <p>The point to be projected is <b>pos</b>. </p>
// <p></p>
// <p>The image of pos by the projection is defined by: </p> 
// <p><literal><b>posp</b> = pos1 + <b>alpha</b> * (pos2 - pos1) </literal></p>
// <p></p>
// <p>The vector [pos, posp] is by definition perpendicular to the segment 
// [pos1, pos2]. </p> 
// <p>If alpha belongs to the interval [0,1], then posp then belongs to 
// the segment [pos1, pos2]. </p> 
// <p></p> 
// </listitem>
// <listitem>
// <p><b>Notes</b>:</p> 
// <p>- If pos1 and pos2 are (almost) identical, the line passing through pos1 and pos2 
// is undefined, and therefore the results are also undefined (that is %nan). </p>  
// <p>- Empty results are returned if one of the input arguments is empty. </p>
// <p></p> 
// </listitem>
// <listitem>
// <p><b>Application</b>:</p> 
// <p>This function can be used to compute the distance between a straight line
// (defined by 2 points: pos1 and pos2) and a sphere (of radius R and center pos).  
// The distance (supposed to be positive) is simply: ||posp - pos|| - R, with 
// posp = CL_gm_projLine(pos1, pos2, pos). </p>
// <p></p> 
// </listitem>
// </itemizedlist>
//
// Parameters
//  pos1, pos2: Points defining the straight line. (3xN or 3x1)
//  pos: Point to be projected. (3xN or 3x1)
//  posp: Image of pos by the projection. (3xN)
//  alpha: Coefficient defining the projeted point (see above for details). (1xN)
//
// Authors
// CNES
// 
// See also 
// CL_gm_inters3dLineEllips 
// 
// Examples
// // Projection of [0;0;0] on the line defined by pos1 and pos2 
// pos1 = [1; 1; 0]; 
// pos2 = [1; -2; 0]; 
// // Result is [1; 0; 0] and alpha is 1/3 
// [posp, alpha] = CL_gm_projLine(pos1, pos2, [0;0;0])
//
 
// Check number of arguments 
nbargsin = argn(2);  
if (nbargsin <> 3)
  CL__error("Invalid number of input arguments"); 
end

// check size / resize 
[pos1, pos2, pos] = CL__checkInputs(pos1, 3, pos2, 3, pos, 3); 

if (pos1 == [] | pos2 == [] | pos == []) 
  posp = []; 
  alpha = []; 
  return; // <= RETURN 
end 

// Computes posp = pos1 + alpha * (pos2 - pos1)
// with alpha = -(pos1 - pos) . (pos2 - pos1) / ||pos2 - pos1||^2 
// which comes from: (posp - pos).(pos2 - pos1) = 0 ("." = dot product)  
// ||pos2 - pos1|| is considered as 0 if is less than EPS * max(||pos1||, ||pos2||)
EPS = 1.e-15; 

pos2_minus_pos1 = pos2 - pos1; 
d2 = CL_dot(pos2_minus_pos1); 
I = find(d2 <= EPS * max(CL_dot(pos1), CL_dot(pos2))); 
if (I <> [])
  d2(I) = %nan; 
end 

alpha = -CL_dot(pos1 - pos, pos2_minus_pos1) ./ d2; 
posp = pos1 + CL_dMult(alpha, pos2_minus_pos1); 

endfunction
