//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [varargout] = CL_ip_flybyOrbits(vinf, rp, inc, branch, poldir, output, mu)
// Arrival or departure orbits knowing inclination (hyperbolic orbits)
//
// Calling Sequence
// [result1, result2, ...] = CL_ip_flybyOrbits(vinf, rp, inc, branch [[, poldir, output, mu]])
// 
// Description
// <itemizedlist><listitem>
// <p>Computes the orbital elements of an hyperbolic orbit knowing the velocity vector at infinity (<b>vinf</b>, 
// also called "excess velocity vector"), the periapsis radius (<b>rp</b>) and the desired inclination (<b>inc</b>). </p> 
// <p>The function optionally computes the impact position vectors (intersection of asymptote and B-plane), 
// with coordinates in either the implicit frame (in which vinf is defined) or in the B-plane frame. </p> 
// <p></p>
// <p>The "output" argument defines the quantities that can be computed: </p>
// <p><b>kep1</b>: Keplerian orbital elements at pericenter for solution 1 - relative to implicit frame</p>
// <p><b>kep2</b>: Keplerian orbital elements at pericenter for solution 2 - relative to implicit frame</p>
// <p><b>pv1</b>: Position and velocity at pericenter for solution 1 - coordinates in implicit frame</p>
// <p><b>pv2</b>: Position and velocity at pericenter for solution 2 - coordinates in implicit frame</p>
// <p><b>pinf1</b>: Impact position vector for solution 1 - coordinates in implicit frame</p>
// <p><b>pinf2</b>: Impact position vector for solution 2 - coordinates in implicit frame</p>
// <p><b>pinfb1</b>: Impact position vector for solution 1 - coordinates in B-plane frame</p>
// <p><b>pinfb2</b>: Impact position vector for solution 2 - coordinates in B-plane frame</p>
// <p></p>
// <p> Solution 1 is such that the impact position vector has a positive "y" coordinate in the B-plane frame. </p>
// <p></p>
// <p> The B-plane frame is defined as follows: </p>
// <p> - Z axis: same direction as vinf </p>
// <p> - X axis: same direction as vinf ^ poldir.  </p> 
// <p></p>
// <p>The argument "<b>poldir</b>" gives the coordinates of the polar axis. The default value is [0; 0; 1]. </p>
// <p></p>
// <p> The "<b>branch</b>" argument describes which asymptote of the hyperbola is refered to: </p>
// <p> - "a": arrival asymptote </p>
// <p> - "d": departure asymptote </p>
// <p></p>
// </listitem>
// <listitem>
// <p><b>Notes</b>:</p>
// <p>- The norm of the excess velocity vector should be > 0. </p>
// <p>- The (implicit) frame in which vinf is defined should be an (inertial) equatorial frame 
// for the inclination to be meaningful. </p>
// <p>- The B-plane frame can be computed by: CL_rot_defFrameVec(vinf, -poldir, 3, 2). </p>
// <p>- For solutions to exist, the following condition must be met: |delta| < inc < pi - |delta|, 
// where delta is the declination of the excess velocity vector. If the condition is not met, the 
// results are set to %nan. </p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// vinf: Arrival or departure excess velocity vector [m/s]. (3x1 or 3xN)
// rp: Radius at periapsis of hyperbolic orbit [m]. (1x1 or 1xN)
// inc: Inclination of hyperbolic orbit [rad]. (1x1 or 1xN)
// branch: (string) "a": arrival, "d": departure (1x1)
// poldir: (optional) Direction of polar axis. Default value is [0; 0; 1]. (3x1 or 3xN) 
// output: (string, optional) Names of output arguments, see above for details. Default value is ["kep1", "kep2"]. (1x1) 
// mu: (optional) Gravitational constant. Default: %CL_mu. [m^3/s^2] (1x1). (1x1) 
// result1, result2, ...: Computed quantities [m, m/s, rad]. (6xN or 3xN depending on computed quantity) 
// 
// Authors
// CNES - DCT/SB
//
// See also
// CL_ip_flybyParams
// CL_ip_flybyVectors
// 
// Examples
// // Notes: 
// // - The planet is the Earth
// // - The implicit frame is an equatorial frame 
// vinf = [-3000; 200; 500]; 
// rp = 10000.e3; 
// inc = 60 * %pi / 180; 
// branch = "a"; // arrival
// 
// // Results 
// [kep1, kep2] = CL_ip_flybyOrbits(vinf, rp, inc, branch); 
//
// // Check results
// kep1(1) * (kep1(2) - 1) - rp  // => 0 
// kep2(1) * (kep2(2) - 1) - rp  // => 0 
// kep1(3) - inc  // => 0 
// kep2(3) - inc  // => 0 


// Declarations:

// Code
if (~exists("poldir", "local")); poldir = [0; 0; 1]; end
if (~exists("output", "local")); output = ["kep1", "kep2"]; end
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end

// ------------------
// Argument Checking
// ------------------
// number of input arguments
[nbargsout, nbargsin] = argn();

if (nbargsin < 4 | nbargsin > 7)
  CL__error("Invalid number of input arguments (4 or 5 expected)");
end

if (find(rp < 0) <> [])
  CL__error("Invalid value for argument rp");
end

if (find(inc < 0 | inc > %pi + 2 * %eps) <> [])
  CL__error("Invalid value for argument inc");
end

if (typeof(branch) <> "string" | size(branch, "*") <> 1)
  CL__error("Invalid type or size for argument branch: 1x1 string expected");
end
if (branch <> "a" & branch <> "d")
  CL__error("Invalid value for argument branch");
end

if (typeof(output) <> "string" | size(output, 1) <> 1)
  CL__error("Invalid type or size for argument output: 1xN string expected");
end

OUTPUTS = ["kep1", "kep2", "pv1", "pv2", "pinf1", "pinf2", "pinfb1", "pinfb2"]; 
if (CL_setdiff(output, OUTPUTS) <> [])
  CL__error("Invalid value for argument output");
end
if (nbargsout <> size(output, 2))
  CL__error("Invalid number of output arguments");
end

// Check / resize arguments
[vinf, rp, inc, poldir] = CL__checkInputs(vinf, 3, rp, 1, inc, 1, poldir, 3);


// ------------------
// Computations
// ------------------
// u: origin axis in B-plane: in Equator and perpendicular to vinf
u = CL_unitVector(CL_cross(vinf, poldir)); 

// Direction of angular momentum
// => The angle between C and poldir should be equal to inc 
C = CL_unitVector(CL_cross(u, vinf)); 

// Look for the 2 solutions by changing the impact vector
// (=> performs a rotation around vinf) 
// The 2 solutions have opposite rotation angle signs => look for only one. 
[q, ang, Inok] = CL_rot_defRot1Ax(C, vinf, poldir, inc, numsol = 1);

// If no solution exists: set to %nan
ang(Inok) = %nan; 

// Impact distance knowing radius of periapsis 
dinf = CL_ip_flybyParams("vinf", CL_norm(vinf), "rp", rp, "dinf", mu = mu); 

// Components in B-plane frame (Y > 0) 
X = dinf .* cos(ang); 
Y = dinf .* abs(sin(ang)); 

// u is the x axis in the B-plane, the y axis is -C
pinf1 = CL_dMult(X, u) + CL_dMult(Y, -C); // positive y component 
pinf2 = CL_dMult(X, u) - CL_dMult(Y, -C); // negative y component

// Coordinates in B-plane frame (Z is %nan if ang is %nan) 
pinfb1 = [X;  Y; ang .* zeros(X)];
pinfb2 = [X; -Y; ang .* zeros(X)];

// Position and velocity at periapsis 
if (branch == "a")
  inputpv = "pvinfa"; 
else
  inputpv = "pvinfd"; 
end

[posp1, velp1] = CL_ip_flybyVectors(inputpv, pinf1, vinf, ["posp", "velp"], mu = mu); 
[posp2, velp2] = CL_ip_flybyVectors(inputpv, pinf2, vinf, ["posp", "velp"], mu = mu); 

kep1 = CL_oe_car2kep(posp1, velp1, mu = mu); 
kep2 = CL_oe_car2kep(posp2, velp2, mu = mu); 


// store all possible output results in "results" structure
results = struct();
results.kep1 = kep1; 
results.kep2 = kep2; 
results.pv1 = [posp1; velp1]; 
results.pv2 = [posp2; velp2]; 
results.pinf1 = pinf1; 
results.pinf2 = pinf2; 
results.pinfb1 = pinfb1; 
results.pinfb2 = pinfb2; 

for (k = 1 : size(output, "*")) 
  varargout(k) = results(output(k)); 
end 

endfunction




