//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [X0, info] = CL_adjustState(t0, X0, t, Zref, fun, opts) 
// Adjustment of initial state vector (least squares)
//
// Calling Sequence
// [X0, info] = CL_adjustState(t0, X0, t, Zref, fun [[, opts]])
// [X0, info] = CL_adjustState(t0, X0, t, Zref, list(fun, p1, p2 ... pn) [[, opts]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>Adjust the initial state vector so as to minimize measurements residuals. </p>
// <p></p>
// <p>The initial state vector is <b>X0</b> (at time <b>t0</b>). The function <b>fun</b>
// propagates the state from time t0 to time t and computes the measurements vector Z(t) at time t. 
// The quantity that is minimized in the sum of w_tk^2 * |Z_tk - Zref_tk|^2, where Z_tk and Zref_tk are
// respectively the kth components of Z(t) and Zref(t), and w_tk is the associated weight. 
// Zref(t) represents the "real" measurements vector at time t.  </p>
// <p></p>
// <p>The function <b>fun</b> must have the following interface: </p>
// <p>[Z [, jac]] = fun(t0, X0, t [, p1, p2, ... pn]) </p>
// <p>With: </p>
// <p>- t0: Initial time (1x1) </p>
// <p>- X0: Initial state vector (at time t0) (Px1) </p>
// <p>- t: Computation times (1xN) </p>
// <p>- p1, p2, ... pn: Constant parameters of any type passed to the function  </p>
// <p>- Z: Computed measurements at times t (KxN). Undefined measurements components should be 
// set to %nan, in order not to be considered in the computation. </p>
// <p>- jac: Jacobian = dZ/dX0 (KxPxN). It may or not be computed. If it is not, it is safer 
// to set it to [].  </p>
// <p></p>
// <p>The structure <b>opts</b> contains adjustment options: </p>
// <p>- <b>indSf</b>; (integer, size=1xQ) indices of solved-for parameters in state vector. Default is 1:P </p>
// <p>- <b>useJac</b>: (boolean) Indicator of whether the Jacobian d(Z)/d(X0) should be used. 
// %t => Jacobian is used, %f => Jacobian is not used; it is then evaluated automatically using 
// finite difference. Default is %f </p>
// <p>- <b>svScale</b>: Scale factors on the state vector components for a better matrix conditioning 
// (size = Px1). Default is ones(P,1) </p>
// <p>- <b>eps</b>: coefficient for finite difference calculation. State vector increments 
// are equal to eps * svScale. Default is eps = 1.e-8 </p>
// <p>- <b>derivOrder</b>: Order for the computation of the Jacobian if computed using finite difference. Possible values are 1, 2 or 4. Default value is 1. </p> 
// <p>- <b>wm</b>: measurements weights for each type of measurement, that is each row of the measurement vector (Kx1). 
// In theory, the weight should be equal to 1/sigmak where sigmak is the standard deviation of the error on measurement k 
// (k in 1 .. K). Default value is ones(K,1) </p>
// <p> - <b>tau</b>: time constant for additional exponential weighting. Default is %inf (no exponential weighting)</p>
// <p> - <b>residMargin</b>: Criterion for iterations control. Iterations stop when the relative variation of residuals (RMS) between 2 iterations is less than this value. 
// Default is 1.e-3 (0.1%). </p>
// <p> - <b>verbose</b>: Verbose level. 0 means no information is printed on the screen, 1 means information is printed on the screen. 
// Default is 0.  </p>
// <p> - <b>meth</b>: Method used to compute state vector adjustments at each iteration. Can be "lsq" (Scilab "lsq" function is used) 
// or "pinv" (Scilab function "pinv" is used). Default is "pinv".  </p>
// <p> - <b>nIterMax</b>: Maximum number of iterations. Default value is 30.  </p>
// <p> - <b>nTrials</b>: Number of successive cases for which residuals don't decrease (including margin). Default value is 3.  </p>
// <p></p>
// <p><b>info</b> is a structure that contains additional information: </p> 
// <p>- <b>niter</b>: number of iterations before the algorithm stops</p>
// <p>- <b>residuals_rms</b>: root mean square of residuals at the last iteration </p>
// <p>- <b>res</b> contains detailed information for the last iteration, in particular: </p> 
// <p>- X0: Initial state vector before adjustment </p>
// <p>- resid_rms: Residuals RMS corresponding to X0 </p>
// <p>- X0_adj: Initial state vector after adjustment </p>
// <p></p>
// </listitem>
// <listitem>
// <p>The algorithm stops if the following conditions are met: </p> 
// <p>- the residuals (RMS) are greater than the minimum value obtained 'nTrials' times in a row (margin included), or </p> 
// <p>- the maximum number of iterations has been reached, or </p> 
// <p>- the residuals are exactly 0.  </p> 
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// t0: Initial time (1x1)
// X0: Initial/adjusted state vector (Px1) 
// t: Times at which measurements are given (1xN)
// Zref: Reference measurements at times t (KxN)
// fun: Function that computes the measurements: function or list(function [, p1, ...]). See above for details. 
// opts: (structure, optional) Computation options. See above for details.  
// info: (structure) Additional information on the results. See above for details.
//
// Authors
// CNES - DCT/SB
//
// Examples
// // ------------------------------
// // Case 1: Jacobian is not computed
// // ------------------------------
// 
// // Model 
// function [z] = F1(t0, x0, t, p)
//   z = x0(1) + x0(2)^2 * (t - t0) + p; 
// endfunction
// 
// // Initial and measurements times 
// t0 = 0; 
// t = t0 + linspace(0, 2, 100); 
// p1 = 0; 
// 
// // Exact solution 
// x0_ref = [1; 2]; 
// z_ref = F1(t0, x0_ref, t, p1);
// 
// // Initial state vector estimate
// x0_est = x0_ref + [-0.2; 0.1]; 
// 
// // Options: default values   
// // "useJac" explicitly set although %f is the default
// opts = struct(); 
// opts.useJac = %f; 
// 
// x0 = CL_adjustState(t0, x0_est, t, z_ref, list(F1, p1), opts); 
// x0 - x0_ref
// 
// // ------------------------------
// // Case 2: Jacobian is computed 
// // and used 
// // ------------------------------
// 
// function [z, jac] = F2(t0, x0, t, p)
//   z = F1(t0, x0, t, p); 
//   jac = zeros(1, 2, length(t));  
//   jac(1, 1, :) = 1; 
//   jac(1, 2, :) = 2 * x0(2) * (t - t0); 
// endfunction
// 
// // Options: change value of "useJac" 
// opts.useJac = %t; 
// 
// x0 = CL_adjustState(t0, x0_est, t, z_ref, list(F2, p1), opts); 
// x0 - x0_ref
// 

// Declarations:

// ------------------------------------------------
// Call function "fun" with adequate number of parameters
// => fun(t0, X0, t, param1? param2, ...) 
// param1, param2, ...: in list_args 
// compute_jac: %t if Jacobian should be computed
// ------------------------------------------------
function [z, jac] = adjstate_callFun(t0, X0, t, fun, list_args, compute_jac)
  jac = []; 

  // prepare command for "execstr" 
  cmd = "fun(t0, X0, t"; 
  for k = 1 : size(list_args)
    cmd = cmd + ", " + "list_args(" + msprintf("%d", k) + ")"; 
  end
  cmd = cmd + ")";
  
  if (compute_jac)
    cmd = "[z, jac] = " + cmd; 
  else
    cmd = "[z] = " + cmd; 
  end
  
  err = execstr(cmd, "errcatch"); 
  
  if (err <> 0)
    msg = lasterror(); 
    CL__error("Error in call to measurements function: " + msg);
  end
endfunction 
  
// ------------------------------------------------
// Measurements vector in one column
// fun: function or list(function [, args]) 
// ------------------------------------------------
function [z, jac] = adjstate_measurements(t0, X0, t, fun, compute_jac)
  // retrieve function + possible arguments
  list_args = list(); 
  if (typeof(fun) == "list")
    list_args = fun; 
    fun = fun(1); // actual function 
    list_args(1) = null(); // remove
  end 

  // call to user function
  [z, jac] = adjstate_callFun(t0, X0, t, fun, list_args, compute_jac); 
  
  if (compute_jac & jac == [])
    CL__error("Invalid Jacobian: should not be empty"); 
  end 
  
  // one column vector for state and consistent Jacobian 
  z = matrix(z, -1, 1); 
  dim_state = size(X0, 1); 
  jac = matrix(jac', dim_state, -1)'; 

endfunction

// ------------------------------------------------
// Computes sensitivity matrix using finite difference (method 1). 
// State vector is scaled using svScale for a better 
// matrix conditioning. 
// indSf: indices of state vector components to be solved-for
// ------------------------------------------------
function [H] = adjstate_matrix1(t0, X0, t, z0, fun, eps, svScale, indSf)
  compute_jac = %f; 

  dim = length(indSf); 
  H = zeros(size(z0, 1), dim); 
  
  for i = 1 : dim
    k = indSf(i); 
    X0k = X0; 
    X0k(k) = X0k(k) + eps * svScale(k); 

    z = adjstate_measurements(t0, X0k, t, fun, compute_jac); 
    H(:,i) = (z - z0) / eps;   
  end

endfunction


// ------------------------------------------------
// Computes sensitivity matrix using finite difference (method 2: with "numederivative"). 
// State vector is scaled using svScale for a better 
// matrix conditioning. 
// indSf: indices of state vector components to be solved-for
// ------------------------------------------------
function [H] = adjstate_matrix2(t0, X0, t, fun, eps, svScale, indSf, derivOrder)

  function [z] = F(X, fun, t0, X0, t, indSf)
    compute_jac = %f; 
	X0 = X0; 
	X0(indSf) = X; 
    z = adjstate_measurements(t0, X0, t, fun, compute_jac); 
  endfunction
  
  H = numderivative(list(F, fun, t0, X0, t, indSf), X0(indSf), eps * svScale(indSf), derivOrder); 
  H = CL_dMult(H, svScale(indSf)'); // scaling

endfunction


// ------------------------------------------------
// Adjust initial state (least squares) 
// opts: fitting options (struct)
// coef: multiplicative coefficient for adjustment (X = X + dX * coef) 
// resid_rms: residuals BEFORE adjustment
// ------------------------------------------------
function [X0_adj, resid_rms, info] = adjstate_adjust(t0, X0, t, Zref, fun, opts, coef) 
  indSf = opts.indSf; 
  eps = opts.eps; 
  svScale = opts.svScale;  
  wm = opts.wm; 
  tau = opts.tau; 
  useJac = opts.useJac; 
  meth = opts.meth; 
  derivOrder = opts.derivOrder; 

  // measurements matrix
  [z0, jac] = adjstate_measurements(t0, X0, t, fun, useJac); 
  if (~opts.useJac)
    if (derivOrder == 0)
	  H = adjstate_matrix1(t0, X0, t, z0, fun, eps, svScale, indSf); 
	else
      H = adjstate_matrix2(t0, X0, t, fun, eps, svScale, indSf, derivOrder); 
	end 
  else
    jac = CL_dMult(jac, svScale'); // scaling
    H = jac(:, indSf); // selecting solved-for parameters
  end
    
  zref = matrix(Zref, -1, 1); 
  dz = zref - z0; 
  
  // weights associated with each measurement 
  W = matrix(CL_dMult(exp(-(t - t0)/tau), wm), -1, 1); 

  // remove invalid measurements rows (0 weight or %nan measurement value) 
  Imes = find(W > 0 & ~isnan(dz));
  H = H(Imes,:); 
  dz = dz(Imes); 
  W = W(Imes); 
    
  // weighted least squares 
  // (each row is multiplied by the corresponding weight)
  H1 = CL_dMult(W, H); 
  dz1 = CL_dMult(W, dz); 
  
  if (meth == "pinv")
    dx = pinv(H1) * dz1; 
  else
    dx = lsq(H1, dz1); 
  end
  
  X0_adj = X0;  
  X0_adj(indSf) = X0_adj(indSf) + coef * dx .* svScale(indSf); 
  
  // residuals before adjusment
  resid_rms = sqrt(mean(dz1.^2)); 

  
  // additional info
  info = struct(); 
  info.H = H1; 
  info.dz = dz1; 
  info.dx = dx; 
  info.X0 = X0; // before adjustment
  info.X0_adj = X0_adj; // after adjustment
  info.resid_rms = resid_rms; // before adjustment

endfunction
  
// ------------------------------------------------
// Simple algorithm: loop as long as residuals RMS 
// decreases fast enough   
// ------------------------------------------------
function [X0_res, info] = adjstate_iterate(t0, X0, t, Zref, fun, opts, store_res) 
  info = struct(); 

  X0_res = X0; 
  resid_rms_min = %inf; 
  iter = 0; 
  finished = %f; 
  coef = 1; // coef for adjustment
  
  // number of successive times residuals are more than minimum value (including margin)
  // (maximum value = 2); 
  resid_count = 0;   

  while (~finished) 
    iter = iter + 1;   
    if (opts.verbose > 0)  
      mprintf("\nIteration: %d\n", iter); 
    end
	
	// note: resid_rms computed before adjustment, 
	// so is consistent with X0 and not X0_adj
    [X0_adj, resid_rms, info_iter] = adjstate_adjust(t0, X0, t, Zref, fun, opts, coef);
	
    if (opts.verbose > 0)
      mprintf("=> Residuals RMS before adjustment:\n"); 
      mprintf("%15.6e\n", resid_rms); 
    end
	
	// residuals descrease enough ? 
	if (resid_rms >= resid_rms_min * (1 - opts.residMargin))
	  resid_count = resid_count + 1; 
	else 
	  resid_count = 0; 
	end
	
	// finished ? 
	finished = (resid_count >= opts.nTrials | resid_rms == 0 | iter >= opts.nIterMax); 
	
	// state update if residuals are less than minimum value obtained so far
	// margin not taken into account here
    if (resid_rms < resid_rms_min) 
      if (opts.verbose > 0)
        mprintf("=> State vector increments:\n"); 
        mprintf("%15.6e\n", X0_adj - X0); 
      end
	  
	  // note: X0: consistent with residuals (resid_rms)
	  X0_res = X0; 
      resid_rms_min = resid_rms; 
    
      // store info if needed 
      if (store_res)
	    info.niter = iter; 
        info.residuals_rms = resid_rms_min; 
        info.res = info_iter; 
	  end   
    end

	// state updated even if residuals do not decrease 
    X0 = X0_adj; 
	

  end
  
  if (opts.verbose > 0)
    mprintf("=> Stop\n"); 
  end 
  
endfunction
  
  
// ------------------------------------------------
// Check options 
// ------------------------------------------------
function [opts] = adjstate_checkopts(opts, dim_state, dim_meas)

  OPTS = ["indSf", "useJac", "eps", "svScale", "wm", "tau", "residMargin", "verbose", "meth", "derivOrder", "nIterMax", "nTrials"]; 
  opts = opts; 

  if (typeof(opts) <> "st")
    CL__error("Invalid type for argument opts"); 
  end

  names = matrix(fieldnames(opts), 1, -1); 
  names_diff = CL_setdiff(names, OPTS); 
  if (names_diff <> [])
    CL__error("Invalid field name(s) in opts structure: " + strcat(names_diff, ", ")); 
  end

  // Default values 
  if (~isfield(opts, "indSf")); opts.indSf = 1 : dim_state; end
  if (~isfield(opts, "useJac")); opts.useJac = %f; end
  if (~isfield(opts, "eps")); opts.eps = 1.e-8; end
  if (~isfield(opts, "svScale")); opts.svScale = ones(dim_state, 1); end
  if (~isfield(opts, "wm")); opts.wm =  ones(dim_meas, 1); end
  if (~isfield(opts, "tau")); opts.tau = %inf; end
  if (~isfield(opts, "residMargin")); opts.residMargin = 1.e-3; end
  if (~isfield(opts, "verbose")); opts.verbose = 0; end
  if (~isfield(opts, "meth")); opts.meth = "pinv"; end
  if (~isfield(opts, "derivOrder")); opts.derivOrder = 1; end
  if (~isfield(opts, "nIterMax")); opts.nIterMax = 30; end
  if (~isfield(opts, "nTrials")); opts.nTrials = 3; end

  // Check values 
  if (size(opts.indSf, 1) <> 1)
    CL__error("Invalid size for option indSf"); 
  end
  if (length(opts.indSf) <> length(unique(opts.indSf)))
    CL__error("Invalid value for option indSf"); 
  end
  if (find(opts.indSf <> round(opts.indSf) | opts.indSf < 1 | opts.indSf > dim_state) <> [])
    CL__error("Invalid value for option indSf"); 
  end

  if (typeof(opts.useJac) <> "boolean" | size(opts.useJac, "*") <> 1)
    CL__error("Invalid value for option useJac"); 
  end
  
  if (typeof(opts.eps) <> "constant" | size(opts.eps, "*") <> 1 | opts.eps <= 0)
    CL__error("Invalid value for option eps"); 
  end

  if (typeof(opts.svScale) <> "constant" | size(opts.svScale, 1) <> dim_state | size(opts.svScale, 2) <> 1)
    CL__error("Invalid type or size for option svScale"); 
  end

  if (typeof(opts.wm) <> "constant" | size(opts.wm, 1) <> dim_meas | size(opts.wm, 2) <> 1)
    CL__error("Invalid type or size for option wm"); 
  end
  if (find(opts.wm < 0) <> [])
    CL__error("Invalid value for option wm : values should be positive or zero"); 
  end

  if (typeof(opts.tau) <> "constant" | size(opts.tau, "*") <> 1 | opts.tau <= 0)
    CL__error("Invalid value for option tau"); 
  end

  if (typeof(opts.residMargin) <> "constant" | size(opts.residMargin, "*") <> 1 | opts.residMargin < 0)
    CL__error("Invalid value for option residMargin"); 
  end

  if (opts.verbose <> 0 & opts.verbose <> 1)
    CL__error("Invalid value for option verbose: 0 or 1 expected"); 
  end

  if (opts.meth <> "lsq" & opts.meth <> "pinv")
    CL__error("Invalid value for option meth: ''lsq'' or ''pinv'' expected"); 
  end
  
  if (opts.derivOrder <> 1 & opts.derivOrder <> 2 & opts.derivOrder <> 4)
    CL__error("Invalid value for option derivOrder: 1, 2 or 4 expected"); 
  end
  
  if (opts.nIterMax <= 1 | opts.nIterMax <> round(opts.nIterMax))
    CL__error("Invalid value for option nIterMax: integer value >= 1 expected"); 
  end

  if (opts.nTrials <= 1 | opts.nTrials <> round(opts.nTrials))
    CL__error("Invalid value for option nTrials: integer value >= 1 expected"); 
  end

endfunction

  
// ================================================
// MAIN
// ================================================

// Check arguments
nbargsin = argn(2); 
nbargsout = argn(1); 

if (nbargsin < 5 | nbargsin > 6)
  CL__error("Invalid number of input argments"); 
end

if (size(t0, "*") <> 1)
  CL__error("Invalid size for argument t0: scalar expected"); 
end

if (size(X0, 2) <> 1)
  CL__error("Invalid size for argument X0: column vector expected"); 
end

if (size(t, 1) <> 1)
  CL__error("Invalid size for argument t: row vector expected"); 
end

if (typeof(Zref) <> "constant" | size(Zref, 2) <> size(t, 2))
  CL__error("Invalid type or size for argument t or Zref: same number of columns expected"); 
end

// measurement function 
if (typeof(fun) <> "function" & typeof(fun) <> "list")
  CL__error("Invalid type for argument fun: function or list expected"); 
end
if (typeof(fun) == "list" & (size(fun) < 1 | typeof(fun(1)) <> "function"))
  CL__error("Invalid type for argument fun: list(function, arg1, arg2, ... argn) expected"); 
end

// options 
dim_state = size(X0, 1); 
dim_meas = size(Zref, 1); 
opts = adjstate_checkopts(opts, dim_state, dim_meas); 


// adjust state   
// additional info not stored if not needed
store_res = %t; 
if (nbargsout <= 1)
  store_res = %f; 
end 

[X0, info] = adjstate_iterate(t0, X0, t, Zref, fun, opts, store_res); 
    
endfunction


