//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [interv] = CL_intervFromCond(condition, x, margin)
// Intervals where a condition is met
//
// Calling Sequence
// [interv] = CL_intervFromCond(condition [[, x, margin]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>Computation of intervals where a condition is met.</p>
// <p></p>
// <p>The condition is defined by the argument <b>condition</b> 
// which is an array of booleans. </p>
// <p></p>
// <p>The result <b>interv</b> is the set of intervals of the variable 
// <b>x</b> where "condition" is true. </p> 
// <p>By default, x is defined as "1 : N", where N is the number of elements
// in the argument "condition".  </p>
// <p></p>
// <p>The argument <b>margin</b> enables increasing or reducing the interval lengths: </p>
// <p>- If "margin" is positive, the intervals are enlarged by margin * "one step" 
// on either side (except if the upper or lower bounds are reached). </p>
// <p>- If "margin" is negative, the intervals are shrinked by |margin| * "one step"
// on either side (except if the upper or lower bounds are reached). </p>
// <p></p>
// </listitem>
// <listitem>
// <p><b>Note</b>:</p>
// <p>No check is performed on the abscissa "x", in particular, it is not checked 
// that all values are unique, or given in increasing order. </p>
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// condition: (boolean) Condition to be met. (1xN)
// x: (optional) Corresponding abscissa. Default is 1:N. (1xN)
// margin: (optional) Relative margin (between -1 and 1). Default value is 0. (1x1)
// interv: Intervals where "condition" is true. (2xP)
//
// Authors
// CNES - DCT/SB
//
// Examples
// x = linspace(0, 10, 101);
// condition = (cos(x) > 0.2);
// // Intervals of x 
// interv = CL_intervFromCond(condition, x) 
// // Intervals of indices  
// interv = CL_intervFromCond(condition) 
// // Same result as above
// interv = CL_intervFromCond(condition, 1 : length(x)) 
// // Intervals of x with +50% margin 
// interv = CL_intervFromCond(condition, x, margin = 0.5) 
// 

// Declarations:

// Code:
// -----------------------------------------------------------------
// Main 
// -----------------------------------------------------------------

// Error checking
nbargsin = argn(2); 

if (nbargsin < 1 | nbargsin > 3)
  CL__error("Invalid number of input arguments"); 
end
if (~exists("x", "local")); x = []; end 
if (~exists("margin", "local")); margin = 0; end 

if (condition <> [] & typeof(condition) <> "boolean")
  CL__error("Invalid type for argument condition"); 
end 

if (size(condition, 1) > 1)
  CL__error("Invalid size for argument condition"); 
end 

N = size(condition, "*"); 
if (x == [])
  x = 1 : N; 
end 

if (size(x, "*") <> N)
  CL__error("Invalid size for argument x"); 
end 

if (size(margin, "*") <> 1)
  CL__error("Invalid size for argument margin"); 
end 
if (margin < -1 | margin > 1)
  CL__error("Invalid value for argument margin"); 
end 


// --- Step 1: intervals of indices (no margin) 

ind = find(condition); 
nb = length(ind); 

interv_ind = []; 

if (nb == 1)
  interv_ind = [ind; ind]
  
elseif (nb > 1) 
  I = [0, find(ind(2:$) - ind(1:$-1) > 1.5), nb]; 
  nI = length(I); 
  interv_ind = [ind(I(1:nI-1) + 1); ind(I(2:nI))]; 
end 


// --- Step 2: intervals of x (including margin) 

if (interv_ind == [])
  interv = []; 

elseif (margin == 0)
  // no margin (faster)
  // Note that the result would be the same as the "else" clause
  // (=> comparison with 0 allowed here) 
  interv = matrix(x(interv_ind), 2, -1); 

elseif (margin > 0 & margin < 1)
  // includes margin => enlarge intervals 
  // No risk of overlapping in this case 
  interv_ind_m = [max(interv_ind(1,:) - 1, 1); min(interv_ind(2,:) + 1, N)]; 
  
  interv = matrix(x(interv_ind) + (x(interv_ind_m) - x(interv_ind)) * margin, 2, -1); 

elseif (margin == 1) 
  // margin = 1 => enlarge intervals by 1 step 
  // remove overlapping intervals (=> union of intervals)
  // If only one interval => keep it.  
  interv_ind_m = [max(interv_ind(1,:) - 1, 1); min(interv_ind(2,:) + 1, N)]; 
  if (size(interv_ind_m, 2) > 1)
    interv_ind_m = CL_intervUnion(interv_ind_m); 
  end 
  
  interv = matrix(x(interv_ind_m), 2, -1); 

else 
  // margin < 0 => reduce intervals 
  // only keep "valid intervals" (not shrinking to 0 length)   
  interv_ind_m = [min(interv_ind(1,:) + 1, N); max(interv_ind(2,:) - 1, 1)]; 
  
  interv_tmp = interv_ind - (interv_ind_m - interv_ind) * margin; 
  I = find(interv_tmp(1,:) < interv_tmp(2,:));
  interv_ind = interv_ind(:,I); 
  interv_ind_m = interv_ind_m(:,I); 
 
  interv = matrix(x(interv_ind) - (x(interv_ind_m) - x(interv_ind)) * margin, 2, -1); 

end 
    
endfunction
