//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.


function [indices] = CL_intervSearch(interv, vals, crit)
// Intervals containing values
//
// Calling Sequence
// [indices] = CL_intervSearch(interv, vals [[, crit]])
//
// Description
// <itemizedlist><listitem>
// <p>Returns the intervals (actually the interval indices) where values are found to be. </p>
// <p>If a value does not belong to any interval, 0 is returned. </p>
// <p></p>
// <p>Two selection criteria are available:  </p>
// <p> - "o" (interval order): the returned index is the index of the first found interval that contains
// the input value. </p>
// <p> - "d" (distance to center): the returned index is the index of the interval whose center is the closest 
// to the input value.  </p>
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// interv: Set of intervals [start; end] with start <= end (2xN)
// vals: Set of values (any size)
// crit: (string, optional) Selection criterion: "o" or "d". Default is "o"
// indices: Interval indices (same size as vals)
//
// See also
// CL_intervSelectIn
//
// Authors
// CNES - DCT/SB
//
// Examples
// interv = [[1;20], [30;40], [15; 25]];  
// vals = 0:4:45; 
// 
// // Criterion: first found
// CL_intervSearch(interv, vals, crit="o") 
// 
// // Criterion: min distance to center
// CL_intervSearch(interv, vals, crit="d") 


// ---------------------------------------------------------
// returns intervals indices or 0
// priority = interval order
// interv: 2xN
// vals: 1xP
// ---------------------------------------------------------
function [index] = intervSearch_order(interv, vals)

  // check arguments
  // interv : 2xP, vals: 1xN ou matrice quelconque

  Ninterv = size(interv, 2); 

  // selected interval indices
  index = zeros(1, size(vals, "*"));

  for k = 1 : Ninterv
    I = find(index == 0 & vals >= interv(1,k) & vals <= interv(2,k)); 
    index(I) = k; 
  end

endfunction


// ---------------------------------------------------------
// returns intervals indices or 0
// priority = interval order
// vectorized version (NOT USED YET)
// interv: 2xN
// vals: 1xP
// ---------------------------------------------------------
function [index] = intervSearch_order_vect(interv, vals)

  // check arguments
  // interv : 2xP, vals: 1xN ou matrice quelconque

  Ninterv = size(interv, 2); 
  Nv = size(vals, 2);

  Vals = repmat(vals, Ninterv, 1); 
  T1 = repmat(interv(1,:)', 1, Nv); 
  T2 = repmat(interv(2,:)', 1, Nv); 

  Index = %nan * ones(Ninterv, Nv); 

  I = find(Vals >= T1 & Vals <= T2);
  Index(I) = 1
  Index = Index .* repmat((1:Ninterv)', 1, Nv);   

  index = min(Index, "r"); 
  index(isnan(index)) = 0; 

endfunction


// ---------------------------------------------------------
// returns intervals indices or 0 
// priority = min distance to interval center
// interv: 2xN
// vals: 1xP
// ---------------------------------------------------------
function [index] = intervSearch_distc(interv, vals)

  Ninterv = size(interv, 2); 

  // selected interval indices
  index = zeros(1, size(vals, "*"));
  
  // min distance to interval center
  distc = %inf * ones(vals);

  for k = 1 : Ninterv
    I = find(vals >= interv(1,k) & vals <= interv(2,k)); 
    if (I <> [])
      distI = abs(vals(I) - (interv(1,k) + interv(2,k))/2); 
      K = find(distI < distc(I)); 
      index(I(K)) = k; 
      distc(I(K)) = distI(K); 
    end
  end

endfunction


// ---------------------------------------------------------
// returns intervals indices or 0
// priority = min distance to interval center
// vectorized version (NOT USED YET)
// interv: 2xN
// vals: 1xP
// ---------------------------------------------------------
function [index] = intervSearch_distc_vect(interv, vals)

  Ninterv = size(interv, 2); 
  Nv = size(vals, 2);

  Vals = repmat(vals, Ninterv, 1); 
  T1 = repmat(interv(1,:)', 1, Nv); 
  T2 = repmat(interv(2,:)', 1, Nv); 

  Dist = %nan * ones(Ninterv, Nv); 

  I = find(Vals >= T1 & Vals <= T2);
  Dist(I) = abs(Vals(I) - (T1(I) + T2(I))/2); 

  [vmin, index] = min(Dist, "r"); 
  index(isnan(vmin)) = 0; 

endfunction


// -------------------------------------------------
// Argument Checking
// -------------------------------------------------
if (~exists("crit", "local")); crit = "o"; end

if (size(interv,1) <> 0 & size(interv,1) <> 2)
  CL__error("Invalid size for argument interv"); 
end

// NB : "find" not called if interv is empty
if (interv <> [] & find(interv(1,:) > interv(2,:)) <> [])
  CL__error("Invalid intervals (lower bound > upper bound)"); 
end

if (crit <> "o" & crit <> "d") 
  CL__error("Invalid value for argument crit"); 
end

indices = []; 

if (crit == "o")
  indices = intervSearch_order(interv, matrix(vals, 1, -1)); 
  
else // if (crit == "d")
  indices = intervSearch_distc(interv, matrix(vals, 1, -1)); 
  
end

// indices => same size as vals
indices = matrix(indices, size(vals)); 

endfunction



