//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [y] = CL_quantile(x, p, orient)
// Quantiles of a data set (inverse of cumulative distribution function) 
//
// Calling Sequence
//   y = CL_quantile(x, p [[, orient]])
//
// Description
//   <itemizedlist>
//     <listitem>
//       <p>Evaluates quantiles of a data set.</p>
//       <p></p>
//       <p>
//         Let F be the empirical cumulative distribution function 
//         associated with the data set <b>x</b>. 
//         The quantity (y) that is computed is such that:
//       </p>
//       <p>F(y) = p (p: probability in [0,1]). </p>
//       <p></p>
//       <p>
//         The method used to calculate sample quantiles is the same as in the
//         R-project (method number 5):
//       </p>
//       <p>For a data set containing N elements, quantiles are computed as follows:</p>
//       <p>
//         1) The sorted values in x are considered as the (0.5/N), (1.5/N),
//         ..., ((N-0.5)/N) quantiles.
//       </p>
//       <p>
//         2) Linear interpolation is used to compute quantiles for
//         probabilities between (0.5/N) and ((N-0.5)/N)
//       </p>
//       <p>
//         3) The minimum or maximum values in x are assigned to quantiles
//         for probabilities outside that range.
//       </p>
//       <p></p> 
//       <p>Note: This method is used in Matlab's quantile function as well.</p>      
//       <p></p>
//       <p>
//         Matrices are handled depending on the <b>orient</b> argument:
//         the quantiles can be computed globally ("m"), 
//         column-wise ("c") or row-wise ("r").
//       </p>
//       <p></p>
//     </listitem>
//   </itemizedlist>
//
// Parameters
//   x: Data set (MxN)
//   p: Probabilities (1xP)
//   orient: (string, optional) Quantile direction: "m", "c", or "r". Default is "m". (1x1)
//   y: Quantiles: (1xP) if orient = "m", (MxP) if orient = "c", (PxN) if orient = "r".
//
// Authors
//   CNES - DCT/SB
//
// Bibliography
//   1) R: A Language and Environment for Statistical Computing; http://cran.r-project.org/doc/manuals/fullrefman.pdf. 
//
// Examples
//   // Random gaussian sample
//   N = 1000;
//   m = 0;  // mean
//   sd = 1; // standard deviation
//   x = grand(1, N, "nor", m, sd);
//
//   // Probability values
//   p = [0.6827, 0.9545, 0.9973];
//   y = CL_quantile(abs(x), p) // close to [1, 2, 3]
// 

// -------------------------------------------------------
// Utility functions
// -------------------------------------------------------
function [y] = globalQuantile(x, p)
  // Computes quantiles of all elements in an array.
  // See description above: it is the same as in the R-project (method number 5)
  // x: values (1xN)
  // p: probabilities (1xP)
  // y: quantiles (1xP)
  X = gsort(x, "g", "i");
  N = length(X);

  // (ptab,x) such that F is a piecewise linear function where 
  // the knots are the values midway through the steps of the empirical cdf
  ptab = [0, (0.5 + (0 : N-1)) / N, 1];
  X = [X(1), X(:)', X($)];

  // Linear interpolation (on row vectors) 
  y = CL_interpLin(ptab, X, p); 
  
endfunction


// -------------------------------------------------------
// INPUTS CHECKS
// -------------------------------------------------------
nbargsin = argn(2); 
if (nbargsin < 2 | nbargsin > 3)
  CL_error("Invalid number of input arguments"); 
end 

if (~exists("orient", "local"))
  orient = "m";
end

if (size(p, 1) <> 1)
  CL__error("Invalid size for argument p"); 
end

if (or(p < 0 | p > 1))
  CL__error("Invalid probability values (must be in [0,1])"); 
end

// orient must be "m", "r" or "c".
if (orient <> "m" & orient <> "r" & orient <> "c")
  CL__error("Invalid value for argument orient.");
end

// special case 
if (x == [])
  y = x;
  return
end

// -------------------------------------------------------
// MAIN
// -------------------------------------------------------
select (orient)
  case ("m")
    y = globalQuantile(x(:)', p);

  case ("c")
    y = zeros(size(x, 1), length(p));
    for (i = 1 : size(x, 1))
      y(i, :) = globalQuantile(x(i, :), p);
    end

  else // "r" 
    y = zeros(length(p), size(x, 2));
    for (j = 1 : size(x, 2)) 
      y(:, j) = globalQuantile(x(:, j)', p)';
    end
	
end

endfunction
