//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [meanv,covm] = CL_stat(samples)
// Statistics on samples
//
// Calling Sequence
// [meanv, covm] = CL_stat(samples)
//
// Description
// <itemizedlist><listitem>
// <p>Computes the mean value and the covariance matrix of samples.</p> 
// </listitem>
// </itemizedlist>
//
// Parameters
// samples: Set of drawn vectors. Each vector (= column vector) is made of P indidual variables. The number of samples (N) is the number of columns. (PxN)
// meanv: Estimated mean value (Px1)
// covm: Estimated covariance matrix (PxP)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_cor2cov
//
// Examples
// // 2 independent random variables 
// // (Gaussian law, mean = 0, stdev = 1)
// samples = grand(2,1000,"nor", 0, 1); 
// [meanv,covm] = CL_stat(samples)
// [cor,sd] = CL_cov2cor(covm)

// Declarations:

// Version of "cov" for Scilab 2023.0.0 (work-around) 
// Based of version 6.0.2 code, but using matrices 
// x : Nxp, N: number of samples
// opt = 1: normalized by number of samples, 0 : number - 1
function [C] = CL__cov(x, opt)  
  nobs = size(x, "r");
  r = %nan; 
  if (opt == 1 & nobs > 0) 
    r = 1 / nobs;
  elseif (opt == 0 & nobs > 1) 
    r = 1 / (nobs-1);
  end
  A = x - ones(nobs, 1) * mean(x, "r"); 
  C = (A' * A) * r; 
endfunction 

// Code:
sciver = CL__scilabVersion(); 

meanv = mean(samples,2);

if (sciver >= 202300)
  covm = CL__cov(samples', 1); // normalised by number of samples

elseif (sciver >= 550 )
  covm = cov(samples', 1); // normalised by number of samples
  
else 
  // previous Scilab versions
  covm = mvvacov(samples');
  covm = (covm + covm') / 2; // forces symmetric result
  
end
  
endfunction
