//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// Generate compatible "HDF5" binary data file
// => ".dat" (binary) files kept in place 
// => ".sod" files (HDF5 format) are  added 
// NB: should be executed in Scilab version >= 5.4 and < 6
// (see help load) 
// opt = 1 => convert to HDF5 (add ".sod" file)
// opt = -1 => restore (remove ".sod" files) 

function CL__genHDF5DataFiles(opt)

// load binary file and save in HDF5 (in new file name)
function genHDF5_convertFile(fname, fnewname)
  // file contents
  [names, typs, dims, vols] = listvarinfile(fname);  

  // create "save" command compatible with HDF5
  str = strcat("''" + names + "''", ","); 
  cmd = "save(" + "''" + fnewname + "''" + ','  + str + ");";  
  
  load(fname); 
  execstr(cmd); 
endfunction

// Convert all files
// fnames: absolute paths (1xN)
// root_dir: root of CelestLab (for message only)
function genHDF5_convert(fnames, root_dir)
  mprintf("\n-- Processing directory: %s\n", root_dir); 
  
  for fname = fnames
    mprintf("-> %s\n", getrelativefilename(root_dir, fname)); 
    try
      fnewname = fullfile(fullfile(fileparts(fname)), basename(fname)) + ".sod"; 
      if (~isfile(fnewname))
        genHDF5_convertFile(fname, fnewname); 
      else
        mprintf("=> HDF5 file already created\n"); 
      end
    catch
      CL__error("Anomaly converting binary file: " + lasterror()); 
    end
  end
  
  mprintf("-- End of processing. \n\n"); 

endfunction

// Restore all files
// fnames: absolute paths (1xN)
// root_dir: root of CelestLab (for message only)
function genHDF5_restore(fnames, root_dir)
  mprintf("\n-- Processing directory: %s\n", root_dir); 

  for fname = fnames
    mprintf("-> %s\n", getrelativefilename(root_dir, fname)); 
    try
      fnewname = fullfile(fullfile(fileparts(fname)), basename(fname)) + ".sod"; 
      if (isfile(fnewname))
        deletefile(fnewname); 
      else
        mprintf("=> No HDF5 file found\n"); 
      end
    catch
      CL__error("Anomaly restoring binary file: " + lasterror()); 
    end
  end
  
  mprintf("-- End of processing. \n\n"); 

endfunction
  
// check argument
if (argn(2) <> 1 | (opt <> 1 & opt <> -1))
  CL__error("Invalid value for argument opt"); 
end

// NB : exécuter en Scilab5
scilab_version = CL__scilabVersion(); 

if ~(scilab_version >= 540 & scilab_version < 600)
  CL__error("Invalid Scilab version: should be >= 5.4 and < 6.0"); 
end

// locate binary files in CelestLab (all "*.dat" files in CL_home()) 
root_dir = uigetdir(CL_home(), "Select a valid CelestLab root directory"); 
if (isempty(root_dir))
  return;
end

if (~isfile(fullfile(root_dir, "macros", "Utils", "CL__error.sci")))
   messagebox("This is not a CelestLab root directory!", "Error", "error")
   return; 
end

// limit search to "data" sub-directory as binary data files are there
root_dir = fullfile(root_dir, "data"); 

fnames = CL_path("*.dat", root_dir, "all"); // 1xN

// remove potentially created ".bak" files
// (previous procedures) 
for (fname = fnames)
  fname_bak = fname + ".bak"; 
  if (isfile(fname_bak))
    mprintf("Deleting: %s\n", fname_bak); 
    deletefile(fname_bak); 
  end
end

if (opt == 1)
  genHDF5_convert(fnames, root_dir); 
else
  genHDF5_restore(fnames, root_dir); 
end

endfunction

