//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [X, Xdot, Xdot2] = CL__ephDE_getValues(param_name, cjd, ephem); 
// Extract values from DExxx JPL ephemerides : 

// -> pos, vel, acc in ICRF frame (m, m/s, m/s^2) for a body relative to another body
// Available bodies : 
// "Mercury", "Venus", "Earth-Moon-bary" (or "EMB"), "Mars", "Jupiter", "Saturn", 
// "Uranus", "Neptune", "Pluto", "Moon", "Sun", "Earth", "solar-sys-bary" (or "SSB")</p>
// 
// -> other data:
// TT-TDB + 1st, 2nd derivatives  (s, s/s, s/s^2) 
// 
// param_name : ["body", "orig"] for position, velocity, acceleration of celestial bodies 
// param_name : "TDB" for TT - TDB 
//
// cjd: in TDB time scale (in principle) 
// 
// ephem: (complex) ephem structure


  // Check sizes
  N = size(cjd,2);

  // Replace "SSB" and "EMB" by "solar-sys-bary" and "Earth-Moon-bary"
  param_name(param_name == "SSB") = "solar-sys-bary"; 
  param_name(param_name == "EMB") = "Earth-Moon-bary"; 

  // Flag to compute velocity, acceleration only if requested
  cder1 = (argn(1) >= 2);
  cder2 = (argn(1) >= 3);

  // nmber of rows of outputs 
  dim = ephDE_getDim(param_name); 

  // Default initialization
  X = %nan * ones(dim, N); 
  Xdot = []; 
  Xdot2 = []; 

  // initialize Xdot and Xdot2 with correct size 
  if (cder1); Xdot = X; end 
  if (cder2); Xdot2 = X; end 

  // stops if cjd is empty (=> returns [])
  if (N == 0)
    return; // <= RETURN
  end


  if (ephem.ephem_type == 1)
    // simple ephemeris structure is provided (type 1)
    [X, Xdot, Xdot2] = ephDE_getValues_eph(param_name, cjd, ephem, cder1, cder2);

  elseif (ephem.ephem_type == 2)
    // multiple ephemeris structure is provided (type 2)

    // date intervals in descriptor
    interv = [ephem.desc.cjd_begin, ephem.desc.cjd_end($)]; 

    // Search dates in intervals
    ind = dsearch(cjd, interv); 

    I = find(ind > 0); 
    index_desc = []; 
    if (I <> []); index_desc = min(ind(I)) : max(ind(I)); end
  
    for k = index_desc
      I = find(ind == k); 
      if (I == []); continue; end
      [Xtmp, Xtmpdot, Xtmpdot2] = ephDE_getValues_eph(param_name, cjd(I), ephem.eph(k), cder1, cder2);
      X(:,I) = Xtmp; 
      Xdot(:,I) = Xtmpdot; 
      Xdot2(:,I) = Xtmpdot2; 
    end
  
  elseif (ephem.ephem_type == 3)
    // reference to ephemeris is provided (type 3)

    interv = [ephem.desc.cjd_begin, ephem.desc.cjd_end($)]; 

    // Search dates in intervals
    ind = dsearch(cjd, interv); 

    I = find(ind > 0); 
    indmin = min(ind(I)); 
    indmax = max(ind(I)); 

    for k = indmin : indmax

      I = find(ind == k); 
      if (I == []); continue; end

      // Load ephem from binary file
      fpath = ephem.desc.fname(k);
      eph = CL_ephDE_load(fpath); 
    
      [Xtmp, Xtmpdot, Xtmpdot2] = ephDE_getValues_eph(param_name, cjd(I), eph, cder1, cder2);
      X(:,I) = Xtmp; 
      Xdot(:,I) = Xtmpdot; 
      Xdot2(:,I) = Xtmpdot2; 
    end

  else
    CL__error("Invalid ephemeris structure"); 
  
  end 

endfunction


function [X, Xdot, Xdot2] = ephDE_getValues_eph(param_name, t, ephem, cder1, cder2); 

// "Earth" and  "solar-sys-bary" : added (don't exist in JPL files)
// "Earth": obtained from "Earth-Moon-bary" + "Moon")
// "Moon" : is in fact Earth->Moon in JPL file (origin = Earth)
// Other bodies: from solar system barycenter (same order as in file)
// Frame = ICRS
// 
// Other parameters available: 
// TDB (returns TT - TDB) 

  BODY_NAMES = [ "Mercury", "Venus", "Earth-Moon-bary", "Mars", "Jupiter", "Saturn", ..
            "Uranus", "Neptune", "Pluto", "Moon", "Sun", ..
            "Earth", "solar-sys-bary"];

  OTHER_NAMES = ["TDB"]; 			

  if (size(param_name, "*") == 2)
    // body and origin (another body) 
    numpar = find(BODY_NAMES == param_name(1)); 
    if (numpar == []); CL__error("Unknown body name: " + param_name(1)); end

    numorg = find(BODY_NAMES == param_name(2)); 
    if (numorg == []); CL__error("Unknown body name (origin): " + param_name(2)); end
  
  elseif (size(param_name, "*") == 1)
    // other parameters (TDB) 
    numpar = find(OTHER_NAMES == param_name(1)); 
    if (numpar == []); CL__error("Unknown parameter name: " + param_name(1)); end
    
	numorg = ""; // unused 
  
  else
    CL__error("Invalid size for argument param_name"); 
  end   
  
  // dimension and unit of quantities (dim = 3 except if TDB) 
  [dim, unit_coef] = ephDE_getDim(param_name); 

  // Earth / solar system barycenter
  function [pos, vel, acc] = get_earth(t, cder1, cder2)
    [pos, vel, acc] = ephDE_interp(ephem, 3, t, cder1, cder2, dim, unit_coef); // Earth-Moon barycenter
    [posm, velm, accm] = ephDE_interp(ephem, 10, t, cder1, cder2, dim, unit_coef); // Earth->Moon
    emrat = ephem.emrat; 
    pos = pos - posm/(1+emrat); 
    vel = vel - velm/(1+emrat); 
    acc = acc - accm/(1+emrat); 
  endfunction 

  // Moon / solar system barycenter
  function [pos, vel, acc] = get_moon(t, cder1, cder2)
    [pos, vel, acc] = ephDE_interp(ephem, 3, t, cder1, cder2, dim, unit_coef); // Earth-Moon barycenter
    [posm, velm, accm] = ephDE_interp(ephem, 10, t, cder1, cder2, dim, unit_coef); // Earth->Moon
    emrat = ephem.emrat; 
    pos = pos + posm * (emrat/(1+emrat)); 
    vel = vel + velm * (emrat/(1+emrat)); 
    acc = acc + accm * (emrat/(1+emrat)); 
  endfunction 

  // solar system barycenter (=> [0;0;0])
  function [pos, vel, acc] = get_ssbary(t, cder1, cder2)
    // NB: vel, acc set to [] if cder1 or cder2 = %f
    pos = zeros(3, size(t,2)); 
    vel = zeros(3, size(t,2) * cder1); 
    acc = zeros(3, size(t,2) * cder2); 
  endfunction 

  // Special case : Moon and Earth
  if (param_name(1) == "Moon" & param_name(2) == "Earth")
    [X, Xdot, Xdot2] = ephDE_interp(ephem, 10, t, cder1, cder2, dim, unit_coef); 

  elseif (param_name(1) == "Earth" & param_name(2) == "Moon")
    [X, Xdot, Xdot2] = ephDE_interp(ephem, 10, t, cder1, cder2, dim, unit_coef); 
    X = -X;
    Xdot = -Xdot; 
    Xdot2 = -Xdot2; 
   
  elseif (param_name(1) == "TDB")
    [X, Xdot, Xdot2] = ephDE_interp(ephem, 15, t, cder1, cder2, dim, unit_coef); 

  else
    // process "special" bodies first

    if (param_name(1) == "Earth")
      [X, Xdot, Xdot2] = get_earth(t, cder1, cder2); 
    elseif (param_name(1) == "Moon");  
      [X, Xdot, Xdot2] = get_moon(t, cder1, cder2); 
    elseif (param_name(1) == "solar-sys-bary");  
      [X, Xdot, Xdot2] = get_ssbary(t, cder1, cder2); 
    else
      [X, Xdot, Xdot2] = ephDE_interp(ephem, numpar, t, cder1, cder2, dim, unit_coef); 
    end
   
    if (param_name(2) == "Earth")
      [Xorg, Xorgdot, Xorgdot2] = get_earth(t, cder1, cder2); 
    elseif (param_name(2) == "Moon");  
      [Xorg, Xorgdot, Xorgdot2] = get_moon(t, cder1, cder2); 
    elseif (param_name(2) == "solar-sys-bary");  
      [Xorg, Xorgdot, Xorgdot2] = get_ssbary(t, cder1, cder2); 
    else
      [Xorg, Xorgdot, Xorgdot2] = ephDE_interp(ephem, numorg, t, cder1, cder2, dim, unit_coef); 
    end

    X = X - Xorg; 
    Xdot = Xdot - Xorgdot; 
    Xdot2 = Xdot2 - Xorgdot2; 
  end

endfunction


// ephem: JPL ephem structure (elementary structure) 
// numparam: index between 1 and 15 (depends on DE versions)   
// t: cjd (TDB time scale = time scale of ephemerides )
function [X, Xdot, Xdot2] = ephDE_interp(ephem, numparam, t, cder1, cder2, dim, unit_coef)

  // check that param exists (particularly for TDB) 
  if (numparam > size(ephem.ptr, 2))
    CL__error("Requested parameter not in ephemeris file"); 
  end 

  N = size(t, 2); 

  // NB: xdot, xdot2 initialized to [] if cder1 or cder2 = %f
  X = %nan * ones(dim,N); 
  Xdot = []; 
  Xdot2 = []; 
  if (cder1); Xdot = %nan * ones(dim,N); end
  if (cder2); Xdot2 = %nan * ones(dim,N); end

  tbeg = ephem.data(1,1); 
  tend = ephem.data(2,$); 

  // number and length (days) of intervals - based on data only
  nbint = size(ephem.data,2); 
  Tint = (tend-tbeg)/nbint; 

  // number of sub-intervals per interval
  nbsub = ephem.ptr(3, numparam); 

  // total number and length(days) of sub-intervals
  T = Tint / nbsub; 
  nb = nbint * nbsub; 

  // number of chebyshev coefs per sub-interval (x, y or z)
  nbcoef = ephem.ptr(2, numparam);  

  // check dates are in correct range
  Iok = find(t >= tbeg & t <= tend); 
  t = t(Iok); 

  // all Chebyshev coef for body (x,y,z,x,y,z...)
  // 1 column <=> 1 sub-interval 
  i1 = ephem.ptr(1, numparam); // 1st index in each interval
  i2 = i1 + dim * nbsub * nbcoef - 1;  // last index in each interval
  allcoefs = matrix(ephem.data(i1:i2, :), nbcoef, -1); 

  // sub-interval indices for each date
  It = max(1, min(nb, floor((t - tbeg) / T) + 1));

  // normalized date: in (-1, 1) in each sub-interval
  tn = 2 * (t - tbeg - (It-1) * T) / T - 1; 

  // values of Chebyshev polynomials for each normalized date
  // p = column vector of polynomials
  p = CL_chebyshevPoly(0:nbcoef-1)'; 
  y0 = horner(p, tn); 
  y1 = zeros(tn); 
  y2 = zeros(tn); 
  if (cder1)
    p = derivat(p); 
    y1 = horner(p, tn);  
    if (cder2)
      p = derivat(p); 
      y2 = horner(p, tn);  
    end
  end

  // Chebyshev interpolation pour x,y,z (<=> j = 1,2,3)
  for (j = 1:dim)

     // indices of coefficients for each date
     J = dim * (It-1) + j;  
   
     // multiply coeffs by polynomial values
     X(j,Iok) = sum(y0 .* allcoefs(:,J), "r");
     if (cder1); Xdot(j,Iok) = sum(y1 .* allcoefs(:,J), "r"); end;
     if (cder2); Xdot2(j,Iok) = sum(y2 .* allcoefs(:,J), "r"); end;
  end

  // change units => meters, seconds
  // derivatives / time (and not normalized time)
  // s = 2 (t-t0)/T - 1 => d/dt = d/ds * ds/dt = d/ds * (2/T)
  X = X * unit_coef; 
  Xdot = Xdot * unit_coef * 2 / (T*86400); 
  Xdot2 = Xdot2 * unit_coef * (2 / (T*86400))^2; 

endfunction


// get dimension (number of rows) and coefficient
// => 3 (position, velocity, acceleration)
// except if param_name(1) == "TDB" => 1
function [dim, coef_unit] = ephDE_getDim(param_name) 
  
  dim = 3; 
  coef_unit = 1000; // km => m 
  
  if (param_name(1) == "TDB")
    dim = 1; 
	coef_unit = 1; // seconds => seconds
  end 
  
endfunction
 



