//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [pos, vel, acc] = CL_ephDE_getPVA(body,cjd, orig, ephem, tt_tref, opts); 
// Position, velocity and acceleration of a body using JPL's DExxx ephemerides
//
// Calling Sequence
// [pos, vel, acc] = CL_ephDE_getPVA(body, cjd [[, orig, ephem, tt_tref, opts]])
//
// Description
// <itemizedlist><listitem>
// <p>Computes the position, velocity and acceleration of a body using JPL's DExxx ephemerides.</p>
// <p>The results are given in the ICRF frame.</p>
// <p>Available bodies are: "Mercury", "Venus", "Earth-Moon-bary" (or "EMB"), "Mars", "Jupiter", "Saturn", 
//      "Uranus", "Neptune", "Pluto", "Moon", "Sun", "Earth", "solar-sys-bary" (or "SSB")</p>
// <p></p>
// <p>The computed data are:</p>
// <p>pos: vector from <b>orig</b> to <b>body</b></p>
// <p>vel: first time derivative of pos</p>
// <p>acc: second time derivative of pos</p>
// <p></p>
// <p>By default, the function looks for the appropriate JPL's DE405 ephemeris files located 
// in CL_home()/data/ephem/de405.</p>
// <p>One can also specify the ephemerides to be used with the optional argument <b>ephem</b>. 
// See <link linkend="CL_ephDE_load">CL_ephDE_load</link>, 
// <link linkend="CL_ephDE_loadT">CL_ephDE_loadT</link>, or 
// <link linkend="CL_ephDE_setPath">CL_ephDE_setPath</link>. </p>
// <p></p>
// <p>The optional structure <b>opts</b> contains the optional fields: </p>
// <p>- <b>conv_to_tdb</b> (boolean): if %f, the time scale used internally 
// to retrieve the data is TT. Otherwise, TDB is used, which is more precise. 
// By default: %t. </p>
// <p></p></listitem>
// <listitem>
// <p>Notes:</p>
// <p>- For dates that are not covered by the ephemeris file(s), %nan values are returned.</p>
// <p>- The default value for the origin is the solar system barycenter (even for the Moon!)</p>
// <p>- The results returned for all planets except Earth actually are the position velocity and acceleration of the 
// barycenter of the planet system (i.e barycenter of planet + satellites).</p>
// <p>- Body names are case sensitive.</p>
// <p>- (advanced) Setting opts.conv_to_tdb to %f and tt_tref to 0 comes down to considering 
// that cjd is actually given in the TDB time scale.</p>
// <p></p></listitem>
// <listitem>
// <p>See <link linkend="Reference_frames">Reference frames</link> for more details on the ICRF frame,
// and <link linkend="Ephemerides">Ephemerides</link> for more details on ephemerides of celestial bodies.</p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// body: (string) Name of the body. (1x1)
// cjd: Modified (1950.0) julian day (Time scale: TREF) (1xN)
// orig: (optional, string) Name of the origin. Default is "solar-sys-bary" (1x1)
// ephem: Scilab structure describing  the ephemerides (see "See Also").
// tt_tref: (optional) TT-TREF [s]. Default is %CL_TT_TREF. (1xN or 1x1)
// opts: (structure, optional). Additional options. See above for details. 
// pos: Position of body, relative to orig, in ICRF frame [m] (3xN)
// vel: Velocity of body, relative to orig, in ICRF frame [m/s] (3xN)
// acc: Acceleration of body, relative to orig, in ICRF frame [m/s^2] (3xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_ephDE_load
// CL_ephDE_loadT
// CL_ephDE_setPath
//
// Examples
// // Position of Mars in ICRF (Origin = Earth)
// // Using DE405 ephemeris files (default) 
// cjd = 21915:21920;
// pos = CL_ephDE_getPVA("Mars", cjd, "Earth")
//
// // Position, velocity, acceleration of Jupiter in ICRF (Origin = Solar system barycenter)
// // Using DE405 ephemeris files (default) 
// cjd = 21915:21920;
// [pos, vel, acc] = CL_ephDE_getPVA("Jupiter", cjd)
//

// Code
if (argn(2) < 2 | argn(2) > 6)
  CL__error("Invalid number of input arguments");
end

// Default origin
if (~exists("orig", "local")); orig = ""; end
if (orig == "" | orig == [])
  orig = "solar-sys-bary"; 
end

if (~exists("ephem", "local")); ephem = []; end
if (~exists("tt_tref", "local")); tt_tref = CL__dataGetEnv("TT_TREF"); end
if (~exists("opts", "local")); opts = struct(); end

if (~isfield(opts, "conv_to_tdb"))
  opts.conv_to_tdb = %t; 
end 


if (typeof(body) <> "string" | typeof(orig) <> "string")
  CL__error("Invalid input arguments");
end

// check type of tt_tref in case 
if (typeof(tt_tref) <> "constant")
  CL__error("Invalid type for argument tt_tref");
end

// Check sizes
[cjd, tt_tref] = CL__checkInputs(cjd,1, tt_tref,1);
N = size(cjd,2);

// Flag to compute velocity, acceleration only if requested
cvel = (argn(1) >= 2);
cacc = (argn(1) >= 3);

// Default initialization
pos = []; 
vel = []; 
acc = []; 

// stops if cjd is empty (=> returns [])
if (N == 0)
  return; // <= RETURN
end

// TS: time scale used to retrieve the data.  
if (opts.conv_to_tdb)
  TS = "TDB"; 
else 
  TS = "TT"; 
end 

cjd_ts = CL_dat_scaleConvert("TREF", TS, cjd, tt_tref = tt_tref);

// use DE405 by default 
if (ephem == [])
  ephem = CL_ephDE_setPath(""); 
end 

[pos, vel, acc] = CL__ephDE_getValues([body, orig], cjd_ts, ephem); 

endfunction



