//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [tdb_tref] = CL_ephDE_getTDB(cjd, ephem, tt_tref, opts); 
// Time scale difference TDB minus TREF using JPL's DExxx ephemerides
//
// Calling Sequence
// [tdb_tref] = CL_ephDE_getTDB(cjd, ephem [[, tt_tref, opts]])
//
// Description
// <itemizedlist><listitem>
// <p>Returns the value TDB - TREF using JPL's DExxx ephemerides.</p>
// <p></p>
// <p>The optional structure <b>opts</b> contains the optional fields: </p>
// <p>- <b>conv_to_tdb</b> (boolean): if %f, the time scale used internally 
// to retrieve the data is TT. Otherwise, a more precise method is used,
// equivalent to using TDB as defined in the ephemeris file. 
// By default: %t. </p>
// <p></p>
// </listitem>
// <listitem>
// <p>Notes:</p>
// <p>- An error is raised if the ephemeris file does not contain TDB.</p>
// <p>- (advanced) Setting opts.conv_to_tdb to %f and tt_tref to 0 comes down to considering 
// that cjd is actually given in the TDB time scale. </p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// cjd: Modified (1950.0) julian day (Time scale: TREF) (1xN)
// ephem: Scilab structure describing  the ephemerides (see "See Also").
// tt_tref: (optional) TT-TREF [s]. Default is %CL_TT_TREF. (1xN or 1x1)
// opts: (structure, optional). Additional options. See above for details. 
// tdb_tref: Value of TDB minus TREF [s] (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_ephDE_load
// CL_ephDE_loadT
// CL_ephDE_setPath
//
// Examples
// cjd = 21915:21920;
// ephem = CL_ephDE_setPath(""); 
// // Would generate an error as DE405 does not contain TDB
// // tdb_tref = CL_ephDE_getTDB(cjd, ephem);
//

// Code
if (argn(2) < 2 | argn(2) > 4)
  CL__error("Invalid number of input arguments");
end

// Default origin
if (~exists("tt_tref", "local")); tt_tref = CL__dataGetEnv("TT_TREF"); end
if (~exists("opts", "local")); opts = struct(); end

if (~isfield(opts, "conv_to_tdb"))
  opts.conv_to_tdb = %t; 
end 

// Check sizes
[cjd, tt_tref] = CL__checkInputs(cjd,1, tt_tref,1);
N = size(cjd,2);

// Default initialization
tdb_tref = []; 

// stops if cjd is empty (=> returns [])
if (N == 0)
  return; // <= RETURN
end

cjd_tt = cjd + tt_tref / 86400; 

if (opts.conv_to_tdb)
  // Precise method :   
  // 
  // The function "CL__ephDE_getValues" computes TT minus TDB at a date in TDB time scale: 
  // tt - tdb = f(tdb)
  // 
  // What we want: TDB - TT at cjd_tt (TT time scale) 
  // 
  // tdb - tt =  -f(tdb) = -f(tt + (tdb - tt)) 
  //          ~= -[ f(tt) + f'(tt) * (tdb - tt) ] 
  // Hence:
  // tdb - tt = -f(tt) / (1 + f'(tt)) 

  // returns f(tt), f'(tt) 			
  [tt_tdb, tt_tdbdot] = CL__ephDE_getValues("TDB", cjd_tt, ephem);
  
  // (TDB - TT) + (TT - TREF)
  tdb_tref = -tt_tdb ./ (1 + tt_tdbdot) + tt_tref; 

else
  // Simplified method (TDB approximated to TT): 
  // relative error due to the approximation is ~4.e-10 
  // absolute error due to the approximation is ~3.e-13 seconds 

  // returns TT minus TDB 
  tt_tdb = CL__ephDE_getValues("TDB", cjd_tt, ephem);

  // (TDB - TT) + (TT - TREF)
  tdb_tref = -tt_tdb + tt_tref; 
end 


endfunction



