//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [ephem] = CL_ephDE_load(fpath)
// Loads a DExxx ephemeris file
//
// Calling Sequence
// ephem = CL_ephDE_load(fpath)
//
// Description
// <itemizedlist><listitem>
// <p>Loads a DExxx ephemeris file.</p>
// <p>The result is returned in a Scilab structure <b>ephem</b> that can be used by the function
// <link linkend="CL_ephDE_getPVA">CL_ephDE_getPVA</link>.</p>
// <p></p></listitem>
// <listitem>
// <p>Notes: </p>
// <p> - The format of the ephemeris file is specific to CelestLab.</p>
// <p> - The file name suffix should be ".dat" (Scilab version < 6) or ".sod" (Scilab version >= 6). 
// If the suffix is ".dat" and Scilab version is 6 or more, the suffix is changed to ".sod". 
// If the suffix is not ".dat" nor ".sod", an appropriate suffix is added to the file name. </p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// fpath: (string) Path of ephemeris file (1x1)
// ephem: Scilab structure containing the data (1x1)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_ephDE_loadT
// CL_ephDE_setPath
// CL_ephDE_getPVA
//
// Examples
// // Load a DE405 (binary) ephemeris file
// // NB: appropriate file name suffix added
// fpath = CL_path("1960.*", fullfile(CL_home(), "data"));
// ephem = CL_ephDE_load(fpath);
//
// // Use it! 
// cjd = CL_dat_convert("cal", "cjd", [1962;1;1]);
// [pos, vel, acc] = CL_ephDE_getPVA("Mars",cjd, "Earth", ephem=ephem)

// Arguments checking
if (argn(2) <> 1)
  CL__error("Invalid number of input arguments");   
end 

if (typeof(fpath) <> "string" | size(fpath,"*") <> 1)
  CL__error("Invalid size or type for argument fname");
end

// adjust file suffix 
fpath = CL__binDataFileName(fpath); 

if (~isfile(fpath))
  CL__error("File " + fpath + " not found");
end

load(fpath, "ephem");

// default : simple ephemeris file 
if (~isfield(ephem, "ephem_type"))
  ephem.ephem_type = 1; 
end

endfunction
