//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [ephem] = CL_ephDE_loadT(cjd, dpath, tt_tref)
// Loads DExxx ephemeris files (for a time range)
//
// Calling Sequence
// ephem = CL_ephDE_loadT(cjd [[, dpath, tt_tref]])
//
// Description
// <itemizedlist><listitem>
// <p>Loads DExxx ephemeris files for a given time span.</p>
// <p>The time span is defined by the interval containing all the dates given as input.</p>
// <p><b>dpath</b> of the path of the directory where DExxx ephemeris files are to be found. 
// By default, or if the path is an empty string (""), "de405" in CelestLab data directory is used. </p>
// <p>The result is returned in a Scilab stucture <b>ephem</b> that can be used by the function
// <link linkend="CL_ephDE_getPVA">CL_ephDE_getPVA</link>.</p>
// <p></p></listitem>
// <listitem>
// <p><b>Note</b>: </p>
// <p>The format of the ephemeris file is specific to CelestLab. </p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// cjd: Modified (1950.0) julian day (Time scale: TREF) (1xN)
// dpath: (string, optional) Path of ephemeris directory. Default value: "" (see above) (1x1)
// tt_tref: (optional) TT-TREF [seconds]. Default is %CL_TT_TREF. (1xN or 1x1)
// ephem: Scilab structure containing the ephemeris data
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_ephDE_load
// CL_ephDE_setPath
// CL_ephDE_getPVA
//
// Examples
// cjd_min = CL_dat_cal2cjd(1980, 1, 1); // TREF
// cjd_max = CL_dat_cal2cjd(2020, 1, 1); // TREF
// ephem = CL_ephDE_loadT([cjd_min, cjd_max]);
// 
// // Use it! 
// cjd = linspace(cjd_min, cjd_max, 5);
// [pos, vel, acc] = CL_ephDE_getPVA("Mars", cjd, "Earth", ephem=ephem) 
// [pos, vel, acc] = CL_ephDE_getPVA("Mars", cjd, "Earth") // => same, but can be slower


// Arguments checking
if (argn(2) < 1 | argn(2) > 3)
  CL__error("Invalid number of input arguments");   
end 
if (~exists("dpath", "local")); dpath = ""; end
if (~exists("tt_tref", "local")); tt_tref = CL__dataGetEnv("TT_TREF"); end

ephem_desc = CL_ephDE_setPath(dpath);
desc = ephem_desc.desc; 

// Convert from TREF to TDB
// (Actually to TT: small approximation)
[cjd, tt_tref] = CL__checkInputs(cjd,1, tt_tref,1);
cjd = cjd + tt_tref/86400;
cjd_min = min(cjd); 
cjd_max = max(cjd); 


// k1:k2 = indices of files that contain data for the interval: [cjd_min; cjd_max]
k1 = find(cjd_min >= desc.cjd_begin & cjd_min < desc.cjd_end); 
if (k1 == []); k1 = 1; end

k2 = find(cjd_max > desc.cjd_begin & cjd_max <= desc.cjd_end); 
if (k2 == []); k2 = length(desc.cjd_begin); end

if (k1 == k2)
  // simple ephemeris structure (type 1)
  ephem = CL_ephDE_load(desc.fname(k1));
  ephem.ephem_type = 1; 
  
else
  // multiple ephemeris structure (type 2)
  ephem = struct(); 
  
  ephem.desc = desc;
  // limit to necessary files 
  for name = matrix(fieldnames(desc), 1, -1)
    ephem.desc(name) = ephem.desc(name)(k1:k2); 
  end

  // include ephemeris structures: 
  ephem.eph = list(); 
  for k = 1 : k2 - k1 + 1
    ephem.eph(k) = []; // because of Scilab bug in previous versions
    ephem.eph(k) = CL_ephDE_load(ephem.desc.fname(k));
  end
  
  ephem.ephem_type = 2; 

end

endfunction
