//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [ephem] = CL_ephDE_setPath(dpath)
// Records the path of DExxx ephemeris files
//
// Calling Sequence
// ephem = CL_ephDE_setPath(dpath)
//
// Description
// <itemizedlist><listitem>
// <p>Records the path of the directory where DExxx ephemeris files are to be found. 
// If the path is an empty string (""), "de405" in CelestLab data directory is used. </p>
// <p>The result is returned in a Scilab stucture <b>ephem</b> that can be used by the function
// <link linkend="CL_ephDE_getPVA">CL_ephDE_getPVA</link>.</p>
// <p></p></listitem>
// <listitem>
// <p>Notes: </p>
// <p> - The format of the ephemeris file is specific to CelestLab.</p>
// <p> - The difference with <link linkend="CL_ephDE_load">CL_ephDE_load</link> or 
// <link linkend="CL_ephDE_loadT">CL_ephDE_loadT</link> is that the ephemeris files are not loaded into memory. 
// Only a file descriptor is. The ephemeris files will be loaded in memory as needed, at least once per 
// call to <link linkend="CL_ephDE_getPVA">CL_ephDE_getPVA</link> for instance. 
// If the function <link linkend="CL_ephDE_getPVA">CL_ephDE_getPVA</link> has to be called multiple times, it is
// more efficient to load the ephemeris files into memory by using 
// <link linkend="CL_ephDE_load">CL_ephDE_load</link> or <link linkend="CL_ephDE_loadT">CL_ephDE_loadT</link>. </p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// dpath: (string) Path of ephemeris directory. Default value: "" (see above). (1x1)
// ephem: Scilab structure containing the path and necessary data (1x1)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_ephDE_load
// CL_ephDE_loadT
// CL_ephDE_getPVA
//
// Examples
// // Records the path for DE405 ephemeris files
// dpath = CL_path("de405", CL_home());  
// ephem = CL_ephDE_setPath(dpath);
//
// // Use it! 
// cjd = CL_dat_convert("cal", "cjd", [1962;1;1]);
// [pos, vel, acc] = CL_ephDE_getPVA("Mars", cjd, "Earth", ephem=ephem) 


// Internal function to load the descriptor of ephemeris files (located in data folder)
function [desc] = ephDE_loadDesc(fpath)
    if (~isfile(fpath))
      CL__error("File " + fpath + " not found");
    end
    load(fpath, "desc");
endfunction

// Arguments checking
if (argn(2) <> 1)
  CL__error("Invalid number of input arguments");   
end 

if (dpath == "")
  dpath = fullfile(CL_home(), "data", "ephem", "de405"); 
end 

if (typeof(dpath) <> "string" | size(dpath,"*") <> 1)
  CL__error("Invalid size or type for argument dpath");
end

// load descriptor
fpath = fullfile(dpath, "desc" + CL__binDataFileExt());
desc = ephDE_loadDesc(fpath); 

// update file names (absolute paths) 
for k = 1 : size(desc.fname, "*")
  desc.fname(k) = fullfile(dpath, CL__binDataFileName(desc.fname(k))); 
end 

ephem = struct(); 
ephem.desc = desc; 
ephem.ephem_type = 3; 

endfunction
