//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [acc] = CL_fo_aeroAcc(v, rho, coef_aero)
// Acceleration due to aerodynamic forces
//
// Calling Sequence
// [acc] = CL_fo_aeroAcc(v, rho, coef_aero)
//
// Description
// <itemizedlist>
// <listitem>
// <p>Computes the acceleration due to aerodynamic forces.</p>
// <p></p>
// <p>The acceleration is obtained by:</p> 
// <p><b>acc</b> = 1/2 * rho * v^2 * <b>coef_aero</b></p>
// <p>with:</p>
// <p><b>coef_aero</b> = <b>aero_vec</b> * area / mass</p>
// <p></p>
// <p><b>aero_vec</b> is the aerodynamic coefficients vector with coordinates in the 
// frame in which <b>acc</b> components are to be computed. It is parallel to the
// aerodynamic force and points to the same direction. </p>
// <p><b>aero_vec</b> can be computed for a flat plate by 
// <link linkend="CL_mod_aeroPanelCoefV">CL_mod_aeroPanelCoefV</link>.</p>
// <p></p></listitem>
// <listitem>
// <p>Notes:</p>
// <p>- The velocity (v) is the module of the velocity vector (velocity relative to the atmosphere).</p> 
// <p>- The coordinates frame can be any frame. The origin of the frame does not matter. </p> 
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// v: Norm of velocity vector (velocity relative to the atmosphere) [m/s]. (1xN or 1x1)
// rho: Density of the atmosphere [kg/m^3]. (1xN or 1x1)
// coef_aero: Aerodynamic coefficients vector * area / mass [m^2/kg]. (3xN or 3x1)
// acc: Acceleration vector [m/s^2]. (3xN) 
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_mod_aeroPanelCoefV
//
// Examples
// // --- Example 1: acceleration for a sphere 
// vel = [7000; 0; 0]; // Spacecraft velocity vector / atmosphere
// rho = 4.E-12; // Atmospheric density (kg/m^3)
// 
// // Aero coeff vector (aero_vec): in the direction of the force => -vel
// cx = 2.2;
// aero_vec = cx * CL_unitVector(-vel); 
// area = 1; // m^2  (area = pi * radius^2)
// mass = 100; // kg
// acc = CL_fo_aeroAcc(CL_norm(vel), rho, aero_vec .* (area / mass))
// 
// // --- Example 2: Acceleration for a flat plate
// // Only one side considered (defined by "normal")
// vel = [7000; 0; 0]; // Spacecraft velocity vector / atmosphere
// rho = 4.E-12; // Atmospheric density (kg/m^3)
// normal = [[1; 0; 0], [1; 1; 0]]; // Normal vector to the plate 
// temp = 1100; // K
// temp_p = 300; // K
// m_mol_mass = 0.015; // kg
// ratio_spec = 0.1;
// alpha = 1;
// 
// // NB: function CL_mod_aeroPanelCoefV expects velocity of wind / plate => -vel
// aero_vec = CL_mod_aeroPanelCoefV(-vel,  normal, temp, temp_p, m_mol_mass, ratio_spec, alpha, 1, "acv");
// area = 1; // m^2  
// mass = 100; // kg
// acc = CL_fo_aeroAcc(CL_norm(vel), rho, aero_vec .* (area / mass))


// Code: 

// Check / Resize
[v, rho, coef_aero] = CL__checkInputs(v, 1, rho, 1, coef_aero, 3);

// Checking positive rho
I = find(rho < 0);
if (I <> [])
    CL__error("Invalid value for rho (must be positive).");
end

// Acceleration
K = 0.5 .* rho .* v.^2;
acc = CL_dMult(K, coef_aero);

endfunction
