//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [angles, angles_der] = CL_mod_IAUBodyAngles(cjd, body, tt_tref)
// Rotation angles defining the IAU body-fixed rotating frame
//
// Calling Sequence
// [angles, angles_der] = CL_mod_IAUBodyAngles(cjd, body [[, tt_tref]])
//
// Description
// <itemizedlist><listitem>
// <p>This function computes the three rotation angles that define the IAU body-fixed rotating frame.  </p>
// <p>The body fixed frame axes are obtained from the ICRS axes by applying 3 successive rotations of respective angles 
// [alpha0+pi/2, pi/2-delta0, W] around the respectives axes [3,1,3], where:  </p>
// <p>- alpha0 and delta0: right ascension and declination of the North pole of the body in the ICRS frame, </p>
// <p>- W: angle defining the prime meridian of the body. </p>
// <p></p> 
// <p>The function also computes the time derivatives of these angles: angles_der = [alpha0dot; delta0dot; Wdot]. </p>
// <p></p>
// <p>Available bodies are: "Mercury", "Venus", "Earth", "Mars", "Jupiter", "Saturn", "Uranus", "Neptune", 
// "Sun" and "Moon". More bodies may be available if data are defined. </p>
// <p></p></listitem>
// <listitem>
// <p>Notes:</p>
// <p>- The date should be given in TDB time scale. In practice using TT time scale leads to a negligible 
// error (about 2 ms).</p>
// <p>- Body names are case sensitive</p>
// <p>- This implementation of the function is compliant with the 2009 IAU report. (See bibliography)</p>
// </listitem>
// </itemizedlist>
//
// Parameters
// cjd: Modified (1950.0) Julian day (Time scale: TREF) (1xN)
// body: (string) Body name. (1x1)
// tt_tref: (optional) TT-TREF [seconds]. Default is %CL_TT_TREF. (1xN or 1x1)
// angles: Angles [alpha0; delta0; W] defining the orientation of the body [rad]. (1xN)
// angles_der: Time derivatives of angles = [alpha0dot; delta0dot; Wdot] [rad/s]. (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// Report of the IAU/IAG working group on cartographic coordinates and rotational elements: 2009
//
// See also
// CL_fr_bodyConvertMat
// CL_fr_bodyConvert
//
// Examples
// cjd = CL_dat_cal2cjd(2010,02,03,05,35,25); // TREF time scale
//
// [angles, angles_der] = CL_mod_IAUBodyAngles(cjd, "Mars")


// Declarations:

// -------------------------------------------------
// Evaluates polynomials + derivatives 
// coef: Polynomial coefficients (PxQ), Q: degree of polynomials +1
// t: 1xN 
// val, vald: value and derivative (PxN)
// -------------------------------------------------
function [val, vald] = iaubody_evalPoly(coef, t)
  nc = size(coef, 2); 
  
  val = zeros(size(coef,1), length(t)); 
  vald = val; // zero
  
  if (nc >= 1)
    val = coef(:,1) * ones(t); 
  end 

  tkm1 = ones(t); 
  for k = 2 : nc
    val = val + coef(:,k) * (tkm1 .* t); 
    vald = vald + (k-1) * coef(:,k) * tkm1; 
	tkm1 = tkm1 .* t; 
  end
endfunction 

// -------------------------------------------------
// Evaluates "coefPol + coefCos * cosang + coefSin * sinang" + derivative
// data.coefPol: polynomial coefficients (1xQ)
// cosang, sinang, angdot: PxN 
// data.coefCos, data.coefSin: 1xP
// t: 1xN 
// val, vald: value and derivative (1xN)
// -------------------------------------------------
function [val, vald] = iaubody_evalAngle(data, cosang, sinang, angdot, t)
  val = zeros(t); 
  vald = zeros(t); 
  
  if (isfield(data, "coefPol") & data.coefPol <> [])
    [val, vald] = iaubody_evalPoly(data.coefPol, t); 
  end
  
  if (isfield(data, "coefCos") & data.coefCos <> [])
    coef = (data.coefCos') * ones(t); 
    val = val + sum(coef .* cosang, "r"); 
    vald = vald + sum(- coef .* angdot .* sinang, "r");
  end

  if (isfield(data, "coefSin") & data.coefSin <> [])
    coef = (data.coefSin') * ones(t); 
    val = val + sum(coef .* sinang, "r"); 
    vald = vald + sum(coef .* angdot .* cosang, "r");
  end

endfunction 

// Code:
if (argn(2) <> 2 & argn(2) <> 3) 
  CL__error("Invalid number of input arguments"); 
end

if (~exists("tt_tref", "local")); tt_tref = CL__dataGetEnv("TT_TREF"); end
if (typeof(body) <> "string"); CL__error("Invalid input argument ''body''"); end
if (size(body,"*") <> 1); CL__error("Invalid input argument ''body''"); end

if (cjd == [])
  angles = [];
  angles_der = [];
  return; // <-- RETURN
end

// data containing orientation fields 
// The data structure contains the following fields : 
// coefPol : PxQ (Q : degree of polynomial + 1). 
// X.coefPol : 1xK (K: degree of polynomial + 1)
// X.coefSin : 1xP
// X.coefCos : 1xP
// X = "alpha", "delta", "wrot" 

try
  data = CL__dataGetEnv(["body", body, "orient"]); 
catch
  CL__error("Invalid body name: " + body + ". " + "Orientation data not found"); 
end

// t = Number of days since reference epoch (J2000.0 = 1 January 2000 12:00:00 TDB)
// NB : To avoid losing numerical precision: avoid conversion to julian days
t = (cjd - 18262.5) + tt_tref/86400 ;


// Computes characteristic angles
// NB: coefPol is [], all derived quantities are also [] 
if (isfield(data, "coefPol"))
  coefPol = data.coefPol; 
else
  coefPol = []; 
end 

[ang, angd] = iaubody_evalPoly(coefPol, t); 
cosang = cos(ang); 
sinang = sin(ang); 

// Computes polar axis and rotation angle
[ra, radot] = iaubody_evalAngle(data.alpha, cosang, sinang, angd, t); 
[dec, decdot] = iaubody_evalAngle(data.delta, cosang, sinang, angd, t); 
[w, wdot] = iaubody_evalAngle(data.wrot, cosang, sinang, angd, t); 

// Output angles are in radians
w = CL_rMod(w, 0, 2*%pi);
angles = [ra; dec; w];

// Output time derivatives of angles are in rad/s
angles_der = [radot; decdot; wdot] / 86400;

endfunction
