//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [alpha] = CL_mod_aeroPanelAlpha(alpha_k, m_mol_mass, m_mol_mass_p)
// Accommodation coefficient (Cook theory)
//
// Calling Sequence
// [alpha] = CL_mod_aeroPanelAlpha(alpha_k, m_mol_mass, m_mol_mass_p)
//
// Description
// <itemizedlist>
// <listitem>
// <p>Computes the accommodation coefficient.</p>
// <p></p>
// <p>According to the COOK theory, the accommodation coefficient is given by:</p>
// <p><b>alpha</b> = <b>alpha_k</b> * mu / (1 + mu)^2  </p>
// <p>with: </p>
// <p>mu = <b>m_mol_mass</b> / <b>m_mol_mass_p</b> </p>
// <p><b>m_mol_mass</b>: Mean molar mass of incident flow molecules </p>
// <p><b>m_mol_mass_p</b>: Mean molar mass of surface molecules. 
// The recommended value is 16.e-3 (kilograms). 
// It must be greater than m_mol_mass, otherwise the theory is not valid (the
// accommodation coefficient is then set to %nan).</p>
// <p></p>
// <p><b>alpha_k</b> is between 2 and 4. Recommended values are: 3.6 (according to COOK)
// or 3.8 (according to JACCHIA).  </p>
// <p></p></listitem>
// <listitem>
// <p>Hypothesis:</p>
// <p>- Free molecular flow: Kn = lambda / L >> 1 with lambda: mean free 
//      path in the free stream and L: typical linear dimension of the body.
//      That occurs for altitudes greater than 130km.</p>
// <p>- Maxwellian distribution of molecular velocities in the incident 
//      flow.</p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// alpha_k: Constant for the computation of the accommodation coefficient. (1xN or 1x1)
// m_mol_mass: Mean molar mass of incident flow molecules [kg/mol]. (1xN or 1x1)
// m_mol_mass_p: Mean molar mass of surface molecules [kg/mol]. (1xN or 1x1)
// alpha: Accommodation coefficient (between 0 and 1). (1xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1) Cook, G.E., 1965. Satellite drag coefficients. Planet. Space Sci. 13, 929-946.
//
// See also
// CL_mod_aeroPanelCoef
// CL_mod_aeroPanelCoefV
// CL_fo_aeroAcc
// 
// Examples
// alpha_k = 4;
// m_mol_mass = 0.015; // kg/mol
// m_mol_mass_p = 0.016; // kg/mol
// alpha = CL_mod_aeroPanelAlpha(alpha_k, m_mol_mass, m_mol_mass_p);

// Code: 

// Check / Resize
[alpha_k, m_mol_mass, m_mol_mass_p] = CL__checkInputs(alpha_k, 1, m_mol_mass, 1, m_mol_mass_p, 1);

// Checking alpha_k is between 2 and 4
if (find(alpha_k < 2 | alpha_k > 4) <> [])
  CL__error("alpha_k must be between 2 and 4.");
end
// Checking m_mol_mass is positive
if (find(m_mol_mass <= 0) <> [])
  CL__error("m_mol_mass must be positive.");
end
// Checking m_mol_mass_p m_mol_mass is positive
if (find(m_mol_mass_p <= 0) <> [])
  CL__error("m_mol_mass_p must be positive.");
end

// Mean molar masses ratio
mu = m_mol_mass ./ m_mol_mass_p;

// Accommodation coefficient
alpha = %nan * ones(mu);
I = find(mu <= 1);
if (I <> [])
  alpha(I) = alpha_k(I) .* mu(I) ./ (1 + mu(I)) .^ 2;
end

endfunction