//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [varargout] = CL_mod_aeroPanelCoef(v_wind, theta, temp, temp_p, m_mol_mass, ratio_spec, alpha, nsides, res)
// Aerodynamic coefficients for a flat plate (free molecular flow)
//
// Calling Sequence
// [res1, res2, ...] = CL_mod_aeroPanelCoef(v_wind, theta, temp, temp_p, m_mol_mass, ratio_spec, alpha, nsides, res)
//
// Description
// <itemizedlist>
// <listitem>
// <p>Computes various aerodynamic coefficients for a flat plate. </p>
// <p></p>
// <p>Only one side or the two sides of the flat plate can be taken into account.
// In either case, one of the two sides is considered as the reference one, 
// and the results depend on which one it is. </p>
// <p>The reference side is implicitly defined by a normal vector (pointing outwards). 
// The normal vector is  defined with respect to the wind direction vector through 
// the "theta" angle: </p> 
// <p><b>theta = pi/2 - angle(normal vector, -wind velocity vector) </b></p>
// <p>where theta is in radians and angle() is between 0 and pi. </p>
// <p>This means that |theta| is the (acute) angle between the wind velocity vector and the plate. 
// If theta is positive, the reference side faces the wind. If theta is negative, 
// the side at the back (of the reference side) faces the wind.  And if theta is 0, the reference side 
// is undefined, and some of the results are ambiguous. </p>
// <p></p>
// <p>The aerodynamic coefficients <b>ct</b> (tangential) and <b>cn</b> (normal) are defined such that: </p>
// <p>F = 1/2 * rho * area * v_wind^2 * (ct * <b>ut</b> + cn * <b>un</b>)</p>
// <p>where: </p>
// <p>- F is the force vector exerted by the wind on the flat plate </p> 
// <p>- rho is the atmospheric density </p> 
// <p>- area is the area of the plate </p> 
// <p>- v_wind is the wind speed (> 0) </p> 
// <p>- <b>ut</b> and <b>un</b> are 2 unit vectors as shown in the following figure. </p>
// <p></p>
// <p><inlinemediaobject><imageobject><imagedata fileref="aero_coef_panel.png"/></imageobject></inlinemediaobject></p>
// <p></p>
// <p><b>ut</b> is parallel to the plate and <b>un</b> is perpendicular to the plate such that 
// the wind velocity vector can be written: </p>
// <p>vel_wind = ||vel_wind|| * (cos(theta) * <b>ut</b> + sin(theta) * <b>un</b>). </p>
// <p>Thus, <b>un</b> and the normal vector point to opposite directions. </p>
// <p></p>
// <p>The force can also be given in a "ux, uz" frame, where <b>ux</b> and <b>uz</b> are the respective 
// images of <b>ut</b> and <b>un</b> by the rotation of angle theta around the axis -vel_wind ^ normal:  </p>
// <p><b>ux</b> =  <b>ut</b> * cos(theta) + <b>un</b> * sin(theta) </p>
// <p><b>uz</b> = -<b>ut</b> * sin(theta) + <b>un</b> * cos(theta) </p>
// <p>The force vector F is then : </p>
// <p>F = 1/2 * rho * area * v_wind^2 * (cx * <b>ux</b> + cz * <b>uz</b>)</p>
// <p></p>
// <p>The computed quantities (described by the <b>res</b> argument) are: </p>
// <p>- <b>cx</b>: Drag coefficient.</p>
// <p>- <b>cx_abs</b>: Drag coefficient due to absortion.</p>
// <p>- <b>cx_spec</b>: Drag coefficient due to specular re-emission.</p>
// <p>- <b>cx_diff</b>: Drag coefficient due to diffuse re-emission.</p>
// <p>- <b>cz</b>: Lift coefficient.</p>
// <p>- <b>cz_abs</b>: Lift coefficient due to absortion.</p>
// <p>- <b>cz_spec</b>: Lift coefficient due to specular re-emission.</p>
// <p>- <b>cz_diff</b>: Lift coefficient due to diffuse re-emission.</p>
// <p>- <b>ct</b>: Aerodynamic coefficient in the tangential direction.</p>
// <p>- <b>ct_abs</b>: Aerodynamic coefficient in the tangential direction due to absortion.</p>
// <p>- <b>ct_spec</b>: Aerodynamic coefficient in the tangential direction due to specular re-emision.</p>
// <p>- <b>ct_diff</b>: Aerodynamic coefficient in the tangential direction due to diffuse re-emision.</p>
// <p>- <b>cn</b>: Aerodynamic coefficient in the normal direction.</p>
// <p>- <b>cn_abs</b>: Aerodynamic coefficient in the normal direction due to absortion.</p>
// <p>- <b>cn_spec</b>: Aerodynamic coefficient in the normal direction due to specular re-emision.</p>
// <p>- <b>cn_diff</b>: Aerodynamic coefficient in the normal direction due to diffuse re-emision.</p>
// <p></p>
// <p>Other possible values for <b>res</b> are: </p>
// <p>- <b>all</b>: a structure containing all the coefficients is returned. </p>
// <p>- <b>all-tot</b>: a structure containing all the "total" coefficients (cx, cz, ct, cn) 
// is returned. </p>
// <p></p>
// <p><b>nsides</b> specifies if only one side of the plate is considered or if both
// sides are. The frame relative to which the results are given is the same in the 2 cases: it is the one indirectly defined by the normal vector. </p> 
// <p></p>
// <p><b>ratio_spec</b> specifies the ratio of the number of molecules in the 
// incident flow that are re-emitted by specular reflexion (as opposed to diffuse). 
// It must be between 0 and 1: 0 => 0% specular re-emission (or 100% diffuse
// re-emission), 1 => 100% specular re-emission (or 0% diffuse re-emission). 
// If ratio_spec is 0, the computation is consistent with COOK theory. Otherwise
// it also includes BIRD theory aspects. </p> 
// <p></p></listitem>
// <listitem>
// <p>Hypotheses:</p>
// <p>- Free molecular flow: Kn = lambda / L >> 1 with lambda: mean free 
//      path in the free stream and L: typical linear dimension of the flat plate.
//      That occurs for altitudes greater than 130km.</p>
// <p>- Maxwellian distribution of molecular velocities in the incident 
//      flow relative to the plat plate.</p>
// <p>- The incident molecules that are absorbed by the plate remain inside it
//      for a long time compared to the time for the mean free path in the free
//      stream before they are re-emitted. This thin layer of trapped molecules
//      called wall gas is considered as a perfect gas in Maxwellian equilibrium.</p>
// <p>- COOK theory for the accommodation coefficient: see 
//      <link linkend="CL_mod_aeroPanelAlpha">CL_mod_aeroPanelAlpha</link>. </p>
// <p></p></listitem>
// <listitem>
// <p>Note:</p>
// <p>- There is a slight discontinuity in cn_diff for theta = 0 (around 1.E-3). 
//      Same for cn, cz_diff and cz.</p> 
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// v_wind: Speed of the air flow relative to the plate (positive) [m/s]. (1x1 or 1xN)
// theta: Signed acute angle (in [-pi/2, pi/2]) between the wind velocity vector and the plate [rad]. (1x1 or 1xN)
// temp: Atmospheric temperature [K]. (1x1 or 1xN)
// temp_p: Temperature of the flat plate (wall) [K]. (1x1 or 1xN)
// m_mol_mass: Mean molar mass of incident flow molecules [kg/mol]. (1x1 or 1xN)
// ratio_spec: Ratio of number of molecules re-emitted specularly (between 0 and 1). (1x1 or 1xN)
// alpha: Accommodation coefficient (between 0 and 1). (1x1 or 1xN)
// nsides: (integer) Number of sides to be considered: 1 or 2. (1x1 or 1xN)
// res (string): Names of computed quantities or "all" or "all-tot". (1xP)
// res1, res2, ...: Computed quantities. (1xN) 
// 
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1) Satellite drag coefficients, Cook G.E., 1965, Planet Space Sci. 13, 929-946.
// 2) Molecular gas dynamics and the direct simulation of gas flows, G.A. Bird, 1994, Oxford Science Publications 
//
// See also
// CL_mod_aeroPanelAlpha
// CL_mod_aeroPanelCoefV
// CL_fo_aeroAcc
//
// Examples
// // --- Parameters
// v_wind = 7000; // m/s
// temp = 1100; // K
// temp_p = 300; // K 
// m_mol_mass = 0.015; // kg
// ratio_spec = 0.1;
// alpha_k = 4;
// m_mol_mass_p = 0.016; // kg
// alpha = CL_mod_aeroPanelAlpha(alpha_k, m_mol_mass, m_mol_mass_p);
// 
// // --- Example 1: flat plate (one side)
// theta = linspace(-%pi/2, %pi/2, 361);
// res = CL_mod_aeroPanelCoef(v_wind, theta, temp, temp_p, m_mol_mass, ratio_spec, alpha, 1, "all");
// 
// scf();
// plot(CL_rad2deg(theta), res.cx_abs, "r");
// plot(CL_rad2deg(theta), res.cx_spec, "g");
// plot(CL_rad2deg(theta), res.cx_diff, "b");
// plot(CL_rad2deg(theta), res.cx, "k");
// xtitle("Drag Coefficient (1 side)", "theta = oriented angle between wind velocity vector and flat plate (deg)");
// CL_g_legend(gca(), ["Absorption", "Specular reemiss.", "Diffuse reemiss.", "Total"]);
// CL_g_stdaxes(); 
// 
// scf();
// plot(CL_rad2deg(theta), res.cz_abs, "r");
// plot(CL_rad2deg(theta), res.cz_spec, "g");
// plot(CL_rad2deg(theta), res.cz_diff, "b");
// plot(CL_rad2deg(theta), res.cz, "k");
// xtitle("Lift Coefficient (1 side)", "theta = oriented angle between wind velocity vector and flat plate (deg)");
// CL_g_legend(gca(), ["Absorption", "Specular reemiss.", "Diffuse reemiss.", "Total"]);
// CL_g_stdaxes(); 
// 
// // --- Example 2: computation for a sphere 
// // Integration over rings with velocity vector as axis of revolution
// // => cx_sphere = integral from 0 -> pi of: 
// // [2 * pi * R * sin(phi)] * [R * d(phi)] * cx_plate(phi) / (pi * R^2) 
// // phi = angle between normal and -vel_wind = pi/2 - theta
// // NB: Only cx matters (symmetry around velocity vector)
// theta = linspace(-%pi/2, %pi/2, 51); 
// cx = CL_mod_aeroPanelCoef(v_wind, theta, temp, temp_p, m_mol_mass, ratio_spec, alpha, 1, "cx");
// cx_sphere = intsplin(theta, 2 * cx .* cos(theta))


// Code: 

// ==================================
// UTILITY FUNCTIONS
// ==================================

function [ct_abs, ct_spec, ct_diff, cn_abs, cn_spec, cn_diff] = aeroPanel_ctcn_1side(s, theta, sqrt_T_Tp, ratio_spec)
// Computes aerodynamic coefficients, in the normal and tangential directions
// to the plate, due to the molecules absorved, reemitted specularly and reemitted
// diffusely of one face of a flat plate in free molecular flow regime.
//
// Calling Sequence
// [ct_abs, ct_spec, ct_diff, cn_abs, cn_spec, cn_diff] = aeroPanel_ctcn_1side(s, theta, sqrt_T_Tp, ratio_spec)
//
// Parameters
// s: Dimensionless speed factor between the air flow and the plate. (1xN)
// theta: Incidence angle = pi/2 - angle(normal, -vel_wind) [rad]. (1xN)
// sqrt_T_Tp: Square root of the ratio between the plate face temperature
// and the atmosphere temperature. (1xN)
// ratio_spec: Ratio of molecules of the incident flow that are re-emitted by 
//      specular reflexion (against diffuse reflexion). It must be between in [0,1]: 
//      0 (0% specular reflection) and 
//      1 (100% diffuse reflection). (1xN)
// ct_abs: Aerodynamic coefficient in the tangential direction due to absortion
//         of the front face of the plate. (1xN)
// ct_spec: Aerodynamic coefficient in the tangential direction due to specular
//          re-emission of the face of the plate. (1xN)
// ct_diff: Aerodynamic coefficient in the tangential direction due to diffuse
//          re-emission of the face of the plate. (1xN)
// cn_abs: Aerodynamic coefficient in the normal direction due to absortion
//         of the face of the plate. (1xN)
// ct_spec: Aerodynamic coefficient in the normal direction due to specular
//          re-emission of the face of the plate. (1xN)
// cn_diff: Aerodynamic coefficient in the normal direction due to diffuse
//          re-emission of the face of the plate. (1xN)

// Code: 

  // Variables shortcuts
  s2 = s .^ 2;
  sqrt_pi = sqrt(%pi);
  u = s .* sin(theta);
  u2 = u .^ 2;
  exp_mu2 = exp(-u2);
  erf_u = erf(u);

  // For the tangential direction to the plate
  // Absortion
  ct_abs = (1 ./ (s .* sqrt_pi)) .* cos(theta) .* (exp_mu2 + sqrt_pi .* u .* (1 + erf_u));
  // Re-emitted - specular
  ct_spec = -ratio_spec .* ct_abs;
  // Re-emitted - diffuse
  ct_diff = zeros(ct_abs);
  
  // For the normal direction to the plate
  // Absortion
  cn_abs = (1 ./ (s2 .* sqrt_pi)) .* (u .* exp_mu2 + sqrt_pi .* (1/2 + u2) .* (1 + erf_u));
  // Re-emitted - specular
  cn_spec =  ratio_spec .* cn_abs;
  // Re-emitted - diffuse
  cn_diff = ((1 - ratio_spec) ./ (2 .* s2)) .* sqrt_T_Tp .* (exp_mu2 + sqrt_pi .* u .* (1 + erf_u));

endfunction

function [cx, cz] = aeroPanel_cxcz(ct, cn, theta)
// Computes drag and lift coefficients from tangential and normal coefficients.
//
// Calling Sequence
// [cx, cz] = aeroPanel_cxcz(ct, cn, theta)
//
// Parameters
// ct: Aerodynamic coefficient in the tangential direction. (1xN)
// ct: Aerodynamic coefficient in the tangential direction. (1xN)
// theta: Incidence angle = pi/2 - angle(normal, -vel_wind) [rad]. (1xN)
// cx: Drag coefficient. (1xN)
// cz: Lift coefficient. (1xN)

// Code: 

  cos_i = cos(theta);
  sin_i = sin(theta);
  
  // Drag coefficient
  cx  = cn .* sin_i + ct .* cos_i;

  // Lift coefficient
  cz  = cn .* cos_i - ct .* sin_i;
endfunction

function [results] = aeroPanel_compute(s, theta, ratio_spec, sqrt_T_Tp_fr, sqrt_T_Tp_re, nsides, opt)
// Compute all the results in "results" structure
//
// Parameters
// s: Dimensionless speed factor between the air flow and the plate. (1xN)
// theta: Incidence angle = pi/2 - angle(normal, -vel_wind) [rad]. (1xN)
// ratio_spec: Ratio of molecules in the incident flow that are re-emitted by 
//      specular reflexion (rather than diffuse). It must be between in [0,1]: 
//      0 (0% specular reflection) and 
//      1 (100% diffuse reflection). (1xN)
// sqrt_T_Tp_fr: Square root of the ratio between the plate face temperature
// and the atmosphere temperature (front side). (1xN)
// sqrt_T_Tp_re: Square root of the ratio between the plate face temperature
// and the atmosphere temperature (rear side). (1xN)
// nsides: (integer) Number of sides to be considered: 1 or 2 (1xN)
// opt: 1 => structure with only (cx, cz, ct, cn), otherwise: all the results 
// results: structure containing all the results. Each field: 1xN
//

  // nominal_side_front/rear: true if reference side is the front/rear side
  ref_side_front = (theta >= 0 & theta <= %pi/2 + 4 * %eps); 
  ref_side_rear = (theta < 0 & theta >= -%pi/2 - 4 * %eps); 

  // --- step 1: computation of ct (abs, spec, diff) and cn (abs, spec, diff)
  // for the reference side
  nan_array = %nan * ones(theta);
  ct_abs = nan_array; 
  ct_spec = nan_array;
  ct_diff = nan_array;
  cn_abs = nan_array;
  cn_spec = nan_array;
  cn_diff = nan_array;
  
  // side 1 (= reference side) = front side (facing the wind) 
  I = find(ref_side_front);
  if (I <> [])
    [ct_abs(I), ct_spec(I), ct_diff(I), cn_abs(I), cn_spec(I), cn_diff(I)] = ...
        aeroPanel_ctcn_1side(s(I), theta(I), sqrt_T_Tp_fr(I), ratio_spec(I));
  end

  // side 1 (= reference side) = rear side  
  I = find(ref_side_rear);
  if (I <> [])
    [ct_abs(I), ct_spec(I), ct_diff(I), cn_abs(I), cn_spec(I), cn_diff(I)] = ...
        aeroPanel_ctcn_1side(s(I), theta(I), sqrt_T_Tp_re(I), ratio_spec(I));
  end

  // --- step 2: contribution of the second side (if nsides == 2)
  // To obtain the result for the second side: 
  // - The normal vector becomes -normal => un becomes -un, ut is unchanged
  // - But the wind direction must not change => theta becomes -theta 
  // - compute ct and cn by the model (integration on side given by "normal")
  // - change sign of cn so that cn corresponds to the initial un 
  //   (i.e. of reference side)
  // NB: use sqrt_T_Tp_fr if the wind faces the side given by the new normal
  //     and sqrt_T_Tp_re otherwise. 

  // side 1 = front side => side 2 = rear side 
  I = find(nsides == 2 & ref_side_front);
  if (I <> [])
    [ct_abs_2, ct_spec_2, ct_diff_2, cn_abs_2, cn_spec_2, cn_diff_2] = ...
        aeroPanel_ctcn_1side(s(I), -theta(I), sqrt_T_Tp_re(I), ratio_spec(I));

    ct_abs(I)  = ct_abs(I)  + ct_abs_2;
    ct_spec(I) = ct_spec(I) + ct_spec_2;
    ct_diff(I) = ct_diff(I) + ct_diff_2;
    cn_abs(I)  = cn_abs(I)  - cn_abs_2;
    cn_spec(I) = cn_spec(I) - cn_spec_2;
    cn_diff(I) = cn_diff(I) - cn_diff_2;
  end

  // side 1 = rear side => side 2 = front side 
  I = find(nsides == 2 & ref_side_rear);
  if (I <> [])
    [ct_abs_2, ct_spec_2, ct_diff_2, cn_abs_2, cn_spec_2, cn_diff_2] = ...
        aeroPanel_ctcn_1side(s(I), -theta(I), sqrt_T_Tp_fr(I), ratio_spec(I));
    
    ct_abs(I)  = ct_abs(I)  + ct_abs_2;
    ct_spec(I) = ct_spec(I) + ct_spec_2;
    ct_diff(I) = ct_diff(I) + ct_diff_2;
    cn_abs(I)  = cn_abs(I)  - cn_abs_2;
    cn_spec(I) = cn_spec(I) - cn_spec_2;
    cn_diff(I) = cn_diff(I) - cn_diff_2;  
  end

  // --- step 3: all the other results  
  // Drag (cx) and lift (cz) coefficients
  [cx_abs,  cz_abs]  = aeroPanel_cxcz(ct_abs,  cn_abs,  theta);
  [cx_spec, cz_spec] = aeroPanel_cxcz(ct_spec, cn_spec, theta);
  [cx_diff, cz_diff] = aeroPanel_cxcz(ct_diff, cn_diff, theta);

  // Total cn and ct coefficients
  ct = ct_abs + ct_spec + ct_diff;
  cn = cn_abs + cn_spec + cn_diff;

  // Total cx and cz coefficients
  cx = cx_abs + cx_spec + cx_diff;
  cz = cz_abs + cz_spec + cz_diff;

  // --- step 4: define "results" structure 
  if (opt == 1)
    results = struct("cx", cx, "cz", cz, "ct", ct, "cn", cn); 
  else  
    results = struct("cx", cx, "cz", cz, "ct", ct, "cn", cn,  ... 
                     "cx_abs", cx_abs, "cx_spec", cx_spec, "cx_diff", cx_diff, ...
                     "cz_abs", cz_abs, "cz_spec", cz_spec, "cz_diff", cz_diff, ...
                     "ct_abs", ct_abs, "ct_spec", ct_spec, "ct_diff", ct_diff, ...
                     "cn_abs", cn_abs, "cn_spec", cn_spec, "cn_diff", cn_diff); 
  end
  
endfunction

// ==================================
// MAIN
// ==================================

// Check number of input arguments
nbargs = argn(2); 
if (nbargs <> 9)
  CL__error("Invalid number of input arguments (9 expected)"); 
end

// Check size / Resize
[v_wind, theta, temp, temp_p, m_mol_mass, ratio_spec, alpha, nsides] = ...
   CL__checkInputs(v_wind, 1, theta, 1, temp, 1, temp_p, 1, m_mol_mass, 1, ...
   ratio_spec, 1, alpha, 1, nsides, 1);

// Checking v_wind is positive 
I = find(v_wind <= 0);
if (I <> [])
  CL__error("Invalid value for v_wind (must be > 0).");
end

// Checking temp is positive 
I = find(temp <= 0);
if (I <> [])
  CL__error("Invalid value for temp (must be > 0).");
end

// Checking temp_p is positive 
I = find(temp_p <= 0);
if (I <> [])
  CL__error("Invalid value for temp_p (must be > 0).");
end

// Checking m_mol_mass is positive 
I = find(m_mol_mass <= 0);
if (I <> [])
  CL__error("Invalid value for m_mol_mass (must be > 0).");
end

// Checking ratio_spec between 0 and 1
I = find(ratio_spec < 0 | ratio_spec > 1);
if (I <> [])
  CL__error("Invalid value for ratio_spec (must be between 0 and 1).");
end

// Checking alpha between 0 and 1
I = find(alpha < 0 | alpha > 1);
if (I <> [])
  CL__error("Invalid value for alpha (must be between 0 and 1).");
end

// Checking nsides
if (find(nsides <> 1 & nsides <> 2) <> [])
  CL__error("Invalid value for nsides (1 or 2 expected)"); 
end

// Checking res
// Slight optimization (when used by other CL functions)
// res_code = 0: nothing special => analyse res for details
// res_code = 1: structure containing all the "total" coefficients (cx, cz, ct, cn)
// res_code = 2: structure containing all the coefficients
nbres = size(res, "*"); 
res_code = 0; 

if (nbres == 1 & res == "all")
  res_code = 2; 
elseif (nbres == 1 & res == "all-tot")
  res_code = 1; 
else 
  if (size(res, 1) <> 1 | typeof(res) <> "string")
    CL__error("Invalid size or type for argument res"); 
  end

  // Possible values for res ("all" not included) 
  RES_NAMES = ["cx", "cx_abs", "cx_spec", "cx_diff", ...
               "cz", "cz_abs", "cz_spec", "cz_diff", ...
               "ct", "ct_abs", "ct_spec", "ct_diff", ...
               "cn", "cn_abs", "cn_spec", "cn_diff"]; 
               
  if (CL_setdiff(res, RES_NAMES) <> []) 
    CL__error("Invalid value for argument res"); 
  end
  if (size(res, "*") <> size(unique(res), "*"))
    CL__error("Invalid value for argument res"); 
  end
end

// Check number of output arguments
if (nbres < argn(1))
  CL__error("Invalid number of output arguments (too many)"); 
end

// Computation

// Perfect gas constant
// reference: CODATA - https://physics.nist.gov/cgi-bin/cuu/Value?r|search_for=R
R = 8.3144598;            // J/(K·mol)

// Most probable speed of atmospheric molecules (Vm)
r = R ./ m_mol_mass;      // m2/(s2·K)
Vm = sqrt(2 * r .* temp); // m/s

// Speed factor
s = v_wind ./ Vm;

// Square root of the ratio of plate front (fr) and rear (re) faces
// temperatures (T) to plate temperature (Tp)
sqrt_T_Tp_fr = (sqrt(temp_p) + sqrt(1 - alpha) .* ...
               (s .* sqrt(temp) - sqrt(temp_p))) ./ sqrt(temp);
                  
sqrt_T_Tp_re = (sqrt(temp_p) + sqrt(1 - alpha) .* ...
               (sqrt(temp) - sqrt(temp_p))) ./ sqrt(temp);

               
// All the results in "results" structure (or only sub-part if res_code == 1)
results = aeroPanel_compute(s, theta, ratio_spec, sqrt_T_Tp_fr, sqrt_T_Tp_re, nsides, res_code); 

// Set output arguments                                        
if (res_code <> 0) 
  // res_code == 1 or 2 => structure
  varargout(1) = results;
else
  for k = 1 : size(res,2)
    varargout(k) = results(res(k));
  end
end

endfunction
