//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [varargout] = CL_mod_aeroPanelCoefV(vel_wind, normal, temp, temp_p, m_mol_mass, ratio_spec, alpha, nsides, res)
// Aerodynamic coefficients for a flat plate (free molecular flow) 
//
// Calling Sequence
// [res1, res2, ...] = CL_mod_aeroPanelCoefV(vel_wind, normal, temp, temp_p, m_mol_mass, ratio_spec, alpha, nsides, res)
//
// Description
// <itemizedlist>
// <listitem>
// <p>Computes aerodynamic coefficients for a flat plate in free
// molecular flow regime. </p>
// <p></p>
// <p>The computed quantities (described by the <b>res</b> argument) are: </p>
// <p>- <b>cx</b>: drag coefficient (1xN)</p>
// <p>- <b>cz</b>: lift coefficient (1xN)</p>
// <p>- <b>acv</b>: aerodynamic coefficients vector - drag + lift (3xN)</p>
// <p>- <b>acvx</b>: aerodynamic coefficients vector - drag only (3xN)</p>
// <p>- <b>acvz</b>: aerodynamic coefficients vector - lift only (3xN)</p>
// <p></p>
// <p><b>acv</b> (resp. <b>acvx</b>, <b>acvz</b>) is a vector in the direction of the 
// aerodynamic force <b>F</b> exerted by the wind on the plate: </p>
// <p><b>F</b> = 1/2 * rho * area * ||vel_wind||^2 * <b>acv</b></p>
// <p>where: </p>
// <p>- rho is the atmospheric density </p> 
// <p>- area is the area of the plate </p> 
// <p>- vel_wind is the wind velocity vector relative to the plate</p> 
// <p></p></listitem>
// <listitem>
// <p><b>nsides</b> specifies if only one side of the plate is considered or if both
// sides are. If nsides is equal to 1, the computed coefficients only take into account the 
// effect of the wind on the side defined by <b>normal</b>. If nsides is equal to 2, the 
// effects of the wind on both sides are considered.</p>
// <p></p>
// <p>For the meaning of other arguments, see 
// <link linkend="CL_mod_aeroPanelCoef">CL_mod_aeroPanelCoef</link>. </p>
// <p></p></listitem>
// <listitem>
// <p>Notes:</p>
// <p>- The velocity vector (<b>vel_wind</b>) is the velocity vector of the wind relative to the plate.</p> 
// <p>- The coordinates frame can be any frame. The origin of the frame does not matter. </p> 
// <p>- The normal vector may not be a unit vector. </p> 
// <p>- The aerodynamic coefficients vectors (<b>acv</b>, <b>acvx</b>, <b>acvz</b>) have
// components in the same coordinates frame as <b>vel_wind</b> and <b>normal</b>. </p> 
// <p>- By definition, <b>acv</b> is the same as <b>acvx</b> + <b>acvz</b>. </p> 
// <p>- If the effect of lift is neglected, the aerodynamic coefficients vector
// can be computed by: 
// <b>acv</b> = <b>acvx</b> = <b>cx</b> * <b>vel_wind</b> / ||<b>vel_wind</b>||. </p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// vel_wind: Wind velocity vector (velocity relative to the plate) [m/s]. (3xN or 3x1)
// normal: Normal vector to the flat plate defining the reference side. (3xN or 3x1)
// temp: Atmospheric temperature [K]. (1x1 or 1xN)
// temp_p: Temperature of the flat plate (wall) [K]. (1x1 or 1xN)
// m_mol_mass: Mean molar mass of incident flow molecules [kg/mol]. (1x1 or 1xN)
// ratio_spec: Ratio of number of molecules re-emitted specularly (between 0 and 1). (1x1 or 1xN)
// alpha: Accommodation coefficient. It must be between 0 and 1. (1x1 or 1xN)
// nsides: (integer) Number of sides to be considered: 1 or 2. (1x1 or 1xN)
// res (string): Names of computed quantities. (1xP)
// res1, res2, ...: Computed quantities. (1xN or 3xN) 
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_mod_aeroPanelCoef
// CL_mod_aeroPanelAlpha
// CL_fo_aeroAcc
//
// Examples
// // vel_wind = wind velocity vector relative to the plate
// vel_wind = [-7000; 0; 0];
// // ang = angle between normal and -vel_wind
// ang = CL_deg2rad(0 : 180);
// normal = [cos(ang); zeros(ang); sin(ang)];
// temp = 1100; // K
// temp_p = 300; // K
// m_mol_mass = 0.015; // kg
// ratio_spec = 0.1;
// alpha = 1;
// 
// // Drag/lift coefficients - one side 
// [cx, cz] = CL_mod_aeroPanelCoefV(vel_wind,  normal, temp, temp_p, m_mol_mass, ratio_spec, alpha, 1, ["cx", "cz"]);
// 
// scf();
// plot(CL_rad2deg(ang), cx, "r");
// plot(CL_rad2deg(ang), cz, "b");
// CL_g_legend(gca(), ["cx", "cz"]); 
// 
// // Aerodynamic coefficients vector - two sides - drag + lift 
// aero_vec = CL_mod_aeroPanelCoefV(vel_wind,  normal, temp, temp_p, m_mol_mass, ratio_spec, alpha, 2, "acv");
// 
// scf();
// plot(CL_rad2deg(ang), aero_vec(1,:), "r");
// plot(CL_rad2deg(ang), aero_vec(2,:), "g");
// plot(CL_rad2deg(ang), aero_vec(3,:), "b");
// CL_g_legend(gca(), ["x", "y", "z"]); 


// Code:
 
// Check number of input arguments
nbargs = argn(2); 
if (nbargs <> 9)
  CL__error("Invalid number of input arguments (9 expected)"); 
end

if (size(res, 1) <> 1 | typeof(res) <> "string")
  CL__error("Invalid size or type for argument res"); 
end

// Possible values for res 
RES_NAMES = ["cx", "cz", "acv", "acvx", "acvz"]; 
             
if (CL_setdiff(res, RES_NAMES) <> []) 
  CL__error("Invalid value for argument res"); 
end

// Check number of output arguments
if (size(res, "*") < argn(1))
  CL__error("Invalid number of output arguments (too many)"); 
end

// Check size / Resize
[vel_wind, normal] = CL__checkInputs(vel_wind, 3, normal, 3);

// Wind speed
v_wind = CL_norm(vel_wind);

// Oriented angle between the plate and the wind velocity vector
theta = %pi/2 - CL_vectAngle(-vel_wind, normal);

// Drag and Lift coefficients
coeffs = CL_mod_aeroPanelCoef(v_wind, theta, temp, temp_p, m_mol_mass, ratio_spec, alpha, nsides, "all-tot"); 
cx = coeffs.cx; 
cz = coeffs.cz; 

// Frame transformation
// M = transformation matrix from reference frame (where coordinates are given) 
// to "ux,uy,uz" frame (ref. frame for drag and lift coefficients)
// ux (corresponding to cx): parallel to and same sense as vel_wind
// uz (corresponding to cz):  perpendicular to vel_wind, in (vel_wind, normal) plane, 
//   such that dot product of normal and uz is negative 

M = CL_rot_defFrameVec(vel_wind, -normal, 1, 3);

// Coefficients in reference frame
acv  = M' * [cx; zeros(cx); cz];
acvx = M' * [cx; zeros(cx); zeros(cx)];
acvz = M' * [zeros(cx); zeros(cx); cz];

// Set output arguments  
results = struct("cx", cx, "cz", cz, "acv", acv, "acvx", acvx, "acvz", acvz); 
                                      
for k = 1 : size(res,2)
  varargout(k) = results(res(k));
end

endfunction
