//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [varargout] = CL_mod_atmMSIS00(cjd, lon, lat, alt, f107, f107a, ap, res, opts, tt_tref)
// MSIS2000 atmospheric model
//
// Calling Sequence
// [res1, res2, ...] = CL_mod_atmMSIS00(cjd, lon, lat, alt, f107, f107a, ap [[, res, opts, tt_tref]])
//
// Description
// <itemizedlist><listitem>
// <p>Computes various quantities using NRLMSISE-00 atmospheric model.</p>
// <p></p>
// <p>The quantities that can be computed are: </p>
// <p>- dens: total density [kg/m^3] </p>
// <p>- temp: temperature at current position [K] </p>
// <p>- temp_exo: exospheric temperature [K] </p>
// <p>- dens_part: partial densities of various species [kg/m3] 
// in the order: He, O, N2, O2, Ar, H, N, O1 (O1=anomalous oxygen). </p>
// <p></p>
// </listitem>
// <listitem>
// <p>The necessary solar and geomagnetic data are: </p>
// <p>- f107: daily F10.7 flux for previous day (sfu) </p>
// <p>- f107a: 81 day centered average of F10.7 flux (sfu) </p>
// <p>- ap: either daily Ap index, or an array with 7 rows containing: </p>
// <p>> 1 : daily Ap </p>
// <p>> 2 : 3 hr Ap index for current time </p>
// <p>> 3 : 3 hr Ap index for 3 hrs before current time </p>
// <p>> 4 : 3 hr Ap index for 6 hrs before current time </p>
// <p>> 5 : 3 hr Ap index for 9 hrs before current time </p>
// <p>> 6 : average of eight 3 hr Ap indices from 12 to 33 hrs prior to current time </p>
// <p>> 7 : average of eight 3 hr Ap indices from 36 to 57 hrs prior to current time </p>
// <p></p>
// <p>By default, the computation is in "daily Ap mode" if only one value
// (one row) is passed. To force the computation to only use 7 identical
// Ap values, use the "opts" argument. </p>
// <p></p>
// <p> The "opts" argument is a structure that may contain the optional fields: </p>
// <p> - <b>nap</b>: Number of ap values to be used: 1 (only the daily ap) or 7. Default is the 
// <b>number of ap values</b> passed (= number of rows in ap). </p>
// <p> - <b>lst</b>: Type of local time passed to the internal function: 
// "mhl": mean local time or "tlh": true local time. Default is "<b>tlh</b>" 
// (<b>true local time</b>). </p>
// <p> - <b>cv_utc</b>: %t if the input date is internally converted to UTC, 
// %f otherwise. Default is <b>%f</b>. </p> 
// <p></p>
// </listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p>- By default, the TREF time scale is supposed to be identical to UTC (or close enough). 
// If necessary, the input date can be explicitly converted to UTC by setting the "cv_utc" field 
// in the "opts" structure to true. The argument tt_tref is used only if cv_utc is true. </p>
// <p>- The geodetic coordinates are given in a terrestrial frame (tied to the Earth). </p>
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// cjd: Modified (1950.0) julian day (Time scale: TREF) (1x1 or 1xN)
// lon : Geodetic longitude [rad] (1x1 or 1xN)
// lat : Geodetic latitude [rad] (1x1 or 1xN)
// alt : Geodetic altitude [m] (1x1 or 1xN)
// f107 : Solar flux at 10.7 cm for previous day [sfu] (1x1 or 1xN)
// f107a : solar flux at 10.7 cm - average value [sfu] (1x1 or 1xN)
// ap : Geomagnetic indices [-] (1x1 or 7xN) - see above for details
// res : (optional) Name(s) of the quantities to be computed. Possible names are: 'dens','dens_part','temp','temp_exo' or "all" (1 x Noutput). Default is "all"
// opts : (optional) Computation options - see above
// tt_tref: (optional) TT-TREF [seconds]. Default is %CL_TT_TREF (1x1 or 1xN)
// res1, res2, ... : Computed quantities (Noutput variables)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1) NRLMSISE-00 model, M. Picone, A.E. Hedin, D. Drob, Naval Research Laboratory 
// 2) NRLMSISE-00, C language implementation, Dominik Brodowski, http://www.brodo.de/english/pub/nrlmsise/index.html
// 
// Examples
// cjd = CL_dat_cal2cjd(2000,6,21); // TREF
// lon = 0; 
// lat = 0; 
// alt = 700.e3;
// f107 = 150;
// f107a = 150;
// ap = 15;
// // 
// // Compute all quantities in a structure
// res = CL_mod_atmMSIS00(cjd, lon, lat, alt, f107, f107a, ap)
// // 
// // Compute total density and temperature only  
// [dens, temp] = CL_mod_atmMSIS00(cjd, lon, lat, alt, f107, f107a, ap, res=["dens", "temp"])
// // 
// // Compute total density for 2 longitudes 
// dens = CL_mod_atmMSIS00(cjd, [lon, lon], lat, alt, f107, f107a, ap, res="dens")
// // 
// // Compute total density using 7 ap values
// dens = CL_mod_atmMSIS00(cjd, lon, lat, alt, f107, f107a, ap, res="dens", opts=struct("nap", 7))
//

// Check presence of extension module
CL__checkExtensionModule(); 

// Check number of input arguments
if (argn(2) < 7)
  CL__error("Invalid number of input arguments"); 
end

// Arguments default values 
if (~exists("res", "local")); res = "all"; end; 
if (~exists("opts", "local")); opts = struct(); end; 
if (~exists("tt_tref", "local")); tt_tref = CL__dataGetEnv("TT_TREF"); end

// Output quantities ("res" argument)
if (typeof(res) <> "string")
   CL__error("Invalid type for argument res (string expected)"); 
end

Names = ["dens", "dens_part", "temp", "temp_exo", "all"];
if (CL_setdiff(res, Names) <> [])
  CL__error("Invalid names in res argument"); 
end

if ( (argn(1) <> 1 & res == "all") | size(res,"*") <> argn(1))
   CL__error("Wrong number of output arguments"); 
end


// check sizes / resize
nap = size(ap,1); 

if (nap <> 1 & nap <> 7)
   CL__error("Invalid size for argument ap"); 
end

[cjd, lon, lat, alt, f107, f107a, ap] = CL__checkInputs(cjd, 1, lon, 1, lat, 1, alt, 1, f107, 1, f107a, 1, ap, nap); 

// check flux and ap values
if (find(f107 <= 0 | f107a <= 0) <> [])
  CL__error("Invalid values for f107 or f107a (negative values found)"); 
end

if (find(ap < 0)  <> [])
  CL__error("Invalid values for ap (negative values found)"); 
end

if (find(alt < 0)  <> [])
  CL__error("Invalid values for alt (negative values found)"); 
end

// Options ("opts" argument)
Names = ["nap", "lst", "cv_utc"]; 
optsfields = fieldnames(opts); 
if (CL_setdiff(optsfields, Names) <> [])
  CL__error("Invalid fields in opts structure"); 
end

if (~isfield(opts, "nap")); opts.nap = nap; end  
if (~isfield(opts, "lst")); opts.lst = "tlh"; end  
if (opts.lst <> "mlh" & opts.lst <> "tlh")
  CL__error("Invalid value for lst field in opts structure"); 
end
if (~isfield(opts, "cv_utc")); opts.cv_utc = %f; end  


// ------------------------------
// MAIN
// ------------------------------

// convert to UTC (unless TREF considered as identical to UTC) 
if (opts.cv_utc)
  cjd_utc = CL_dat_scaleConvert("TREF", "UTC", cjd, tt_tref=tt_tref); 
else
  cjd_utc = cjd; 
end  
  
jj = floor(cjd_utc);
secs = (cjd_utc - jj) * 86400; 

// Note: on January 1st: doy = 1
[year, month, day] = CL_dat_cjd2cal(jj);
jj1 = CL_dat_cal2cjd(year, 1, 1);
doy = jj - jj1 + 1;

// opts.lst = "mlh" or "tlh"
// Note: here, UTC = UT1. The frame is assumed to be ECF. 
lst = CL_op_locTime(cjd_utc, "lon", lon, opts.lst, ut1_tref=0); 

result = struct(); 

[result.dens, result.temp, result.temp_exo, nb_part] = CLx_msis00(year, doy, secs, ..
        lon, lat, alt, lst, ..
        f107, f107a, ap, opts.nap); 

// partial densities
// ratios : prop_i = n_i / sum(n_i)
// dens_i = dens_tot * prop_i
prop = CL_dMult(nb_part, 1 ./ sum(nb_part, 'r')); 
// order for partial densities       
result.elem = ["He", "O", "N2", "O2", "Ar", "H", "N", "O1"];     
result.dens_part = CL_dMult(result.dens, prop);; 

// Generate outputs
if (res == "all")
  varargout(1) = result; 

else 
  for k = 1:size(res, "*")
     varargout(k) = result(res(k));
  end
end
        

endfunction
