//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [oe, nums] = CL_csl_defWalker(type_oe, oe_ref, T, P, F, opts)
// Definition of a Walker constellation
//
// Calling Sequence
// [oe, nums] = CL_csl_defWalker(type_oe, oe_ref, T, P, F [[, opts]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>Computes orbital elements for a "Walker" constellation.</p>
// <p></p>
// <p>A Walker constellation is defined by: </p>
// <p>- <b>T</b> (integer) : Total number of satellites, </p>
// <p>- <b>P</b> (integer) : Total number of planes, </p>
// <p>- <b>F</b> (integer) : Phasing parameter that defines the gap in mean argument of latitude between 
// 2 satellites in two adjacent planes. F must be between 0 and P-1. The gap is then: F * (2*pi/T). </p>
// <p></p>
// <p>In addition: </p>
// <p>- All orbits have the same orbital elements except RAAN and argument of latitude. 
// All constant orbital elements come from <b>oe_ref</b> that defines the orbital elements of 
// the reference satellite (number 1 in plane 1). </p>
// <p>- Each plane contains T/P satellites with evenly distributed mean argument of latitudes 
// between 0 and 2*pi.</p>
// <p>- The orbits' RAAN are evenly distributed in an interval whose length is defined in <b>opts</b> 
// (raan_span). It is 2*pi by default. </p>
// <p></p>
// <p>The type of orbital elements can be "kep" or "cir". For "cir" type, only the argument of latitude
// is affected. For "kep" type, either the argument of periapsis or the mean anomaly 
// is affected. See below for more information (pso_param option). </p>
// <p></p> 
// <p>Some options can be defined in <b>opts</b>, which is a structure (optionally) containing the following fields: </p>
// <p>- <b>raan_span</b>: angular span (spread) for the right ascensions of the ascending nodes. Default is 2*pi.</p>
// <p>- <b>pso_param</b>: orbit parameter to be modified. It can be "M": mean anomaly, "pom": argument of periapsis, 
// "pso": mean argument of latitude, or "" (default). </p>
// <p>The default value ("") means "M" for orbits of type "kep" and "pso" for orbits of type "cir". "M" and "pom" 
// are only available for orbits of type "kep". "pso" is only available for orbits of type "cir". </p>
// <p>- <b>numbering</b>: "p" (plane by plane) or "sp" (sat in plane by sat in plane) defined as follows: </p>
// <p>Let i be the plane number (between 1 and P) and j the satellite number in each plane (between 1 and T/P). Then: </p>
// <p>"p" corresponds to the order: (i=1, j=1), (i=1, j=2), ... (i=1, j=T/P), (i=2, j=1), ...  </p>
// <p>"sp" corresponds to the order: (i=1, j=1), (i=2, j=1), ... (i=P, j=1), (i=1, j=2), ...  </p>
// <p>The default ordering is "p" (plane by plane). </p>
// <p></p>
// </listitem>
// <listitem>
// <p><b>Notes</b>:</p>
// <p>- Orbits making up a Walker constellation are usually circular. </p>
// <p>- The default value for raan_span (2*pi) corresponds to a pattern called "Walker delta pattern". 
// If raan_span's value is equal to pi, the pattern is called "Walker star pattern". </p>
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// type_oe: (string) Type of orbital elements: "kep" or "cir"
// oe_ref: Reference orbital elements = orbital elements for satellite 1 in plane 1 (6x1)
// T: (integer) Total number of satellites (1x1)
// P: (integer) Total number of planes (1x1)
// F: (integer) Phasing parameter, between 0 and P-1 (1x1)
// opts: (struct, optional) Computation options, see above for details 
// oe: Resulting orbital elements (6xN)
// nums: (struct) Structure containing the fields: n = sat. number, p = plane number, sp = sat. number in plane (1xN)
//
// Authors
// CNES - DCT/SB
//
// Examples
// // Reference orbital elements ("cir" type)
// cir0 = [7078.e3; 0; 0; 1.71; 0; 0];
// T = 12; // 12 satellites in all
// P = 3; // 3 planes 
// F = 1; // Gap between 2 sat in 2 adjacent planes = 1 * (2 * pi / 12)
//
// // Orbital elements for all satellites
// [cir, nums] = CL_csl_defWalker("cir", cir0, T, P, F); 
// 
// // Plot
// function Plot()
//   scf(); 
//   plot(CL_rMod(cir(5,:) * 180 / %pi, 0, 360), CL_rMod(cir(6,:) * 180 / %pi, 0, 360), "o"); 
//   e = CL_g_select(gce(), "Polyline"); 
//   e.mark_background = color("blue"); 
//   e.mark_size = 8; 
//   a = gca(); 
//   a.data_bounds = [-10, -10; 370, 370]; 
//   a.tight_limits = "on"; 
//   a.x_ticks = CL_g_ticks(0 : 120 : 360); 
//   a.y_ticks = CL_g_ticks(0 : 30 : 360); 
//   a.sub_ticks = [0,0]; 
//   xtitle(msprintf("T=%d, P=%d, F=%d", T, P, F), "RAAN (deg)", "Argument of latitude (deg)"); 
//   CL_g_stdaxes(a); 
// endfunction
// 
// Plot(); 

// --- Check number of input arguments
nbargs_in = argn(2); 
if (nbargs_in < 5 | nbargs_in > 6)
  CL__error("Invalid number of input arguments"); 
end

// --- Check opts
if (~exists("opts", "local")); opts = struct(); end

if (~isstruct(opts)) 
  CL__error("Invalid type for argument opts: structure expected"); 
end

OPTS = ["raan_span", "pso_param", "numbering"]; 
if (CL_setdiff(matrix(fieldnames(opts), 1, -1), OPTS) <> [])
  CL__error("Invalid field names in opts");
end

// Default values 
// RAAN on 2*pi interval by default (Walker delta pattern) 
if ~isfield(opts, "raan_span")
  opts.raan_span = 2 * %pi; 
end; 

if ~isfield(opts, "pso_param")
  opts.pso_param = ""; 
end; 

if ~isfield(opts, "numbering")
  opts.numbering = "p"; 
end; 


if (type_oe <> "kep" & type_oe <> "cir") 
  CL__error("Invalid value for argument type_oe: kep or cir expected"); 
end

if (opts.pso_param == "")
  if (type_oe == "kep"); opts.pso_param = "M"; end
  if (type_oe == "cir"); opts.pso_param = "pso"; end
end

if (size(oe_ref,1) <> 6 | size(oe_ref, 2) <> 1)
  CL__error("Invalid size for argument oe_ref"); 
end

if ((type_oe == "kep" & (opts.pso_param <> "M" & opts.pso_param <> "pom")) | ...
    (type_oe == "cir" & (opts.pso_param <> "pso")))
  CL__error("Invalid value for option pso_param");
end

if (typeof(opts.numbering) <> "string" | (opts.numbering <> "p" & opts.numbering <> "sp"))
  CL__error("Invalid value or type for option numbering"); 
end

// T: integer > 0
// P: integer > 0 and T / P: integer
// F: integer >= 0 and < P  
if (T <= 0 | T <> round(T) | P <= 0 | P <> round(P) | ...
          modulo(T, P) <> 0 | F < 0 | F <> round(F) | F >= P)
  CL__error("Invalid value for argument T, P or F"); 
end


// --- define constellation. No errors at this point

// Number of satellites in each plane
Np = round(T / P);

// Angular gap between 2 sat in the same plane
delta_psop = 2 * %pi / Np;

// Angular gap between 2 sat in 2 adjacent planes
delta_pso2p = 2 * %pi * F / T;

// RAAN gap between 2 adjacent planes 
delta_gom = opts.raan_span / P;

// Initialization
oe = repmat(oe_ref, 1, T);
 
if (opts.numbering == "p")
  // [plane, sat/plane] = (1,1), (1,2), (1,3), ... (2,1), (2,2), (2,3), ... 
  [Id_satp, Id_plane] = ndgrid(1 : Np, 1 : P); 

else // if (numbering == "sp")
  // [plane, sat/plane] = (1,1), (2,1), (3,1), ... (1,2), (2,2), (3,2), ... 
  [Id_plane, Id_satp] = ndgrid(1 : P, 1 : Np); 
end

// Sat number
id_sat = 1 : T; 

// Plane number (row vector)
id_plane = matrix(Id_plane, 1, -1); 

// Sat number in plane (row vector)
id_satp = matrix(Id_satp, 1, -1);   

// RAAN ("kep" and "cir") 
oe(5,:) = oe_ref(5) + (id_plane - 1) * delta_gom;

// Arg of latitude ("kep" or "cir")  
// numpar: affected orbital element number: 4 or 6 (4 only if opts.pso_param = "pom")
delta_pso = (id_satp - 1) * delta_psop + (id_plane - 1) * delta_pso2p;  
numpar = 6; 
if (opts.pso_param == "pom")
  numpar = 4; 
end  
oe(numpar,:) = oe_ref(numpar) + delta_pso;

// Numbering info
nums = struct(); 
nums.n = id_sat; 
nums.p = id_plane; 
nums.sp = id_satp; 

endfunction
