//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [varargout] = CL_kp_apsConvert(type1, val1, type2, val2, output, mu)
// Conversion between quantities at periapsis or apoapsis (or at infinity)
//
// Calling Sequence
// [res1, res2...] = CL_kp_apsConvert(type1, val1, type2, val2, output [[, mu]])
//
// Description
// <itemizedlist><listitem>
// <p>Computation of various quantities at periapsis or apoapsis (or at infinity) 
// from other quantities.</p>
// <p></p>
// <p>The quantities that can be computed are given in <b>output</b> and can be:</p> 
// <p>- <b>rp</b>: Radius at periapsis.</p>
// <p>- <b>ra</b>: Radius at apoapsis (elliptic orbits).</p>
// <p>- <b>vp</b>: Velocity at periapsis.</p>
// <p>- <b>va</b>: Velocity at apoapsis (elliptic orbits).</p>
// <p>- <b>sma</b>: Semi major axis.</p>
// <p>- <b>ecc</b>: Eccentricity.</p>
// <p>- <b>vinf</b>: Hyperbolic excess velocity (velocity at infinity). It is strictly positive for hyperbolic orbits, %nan otherwise.</p>
// <p></p>
// <p><b>type1</b> and <b>type2</b> take the same values as given above, but
// not all combinations are possible. The allowed possibilities are:</p> 
// <p>- If type1 = "ra" => type2 can be "rp", "vp", "va".</p>
// <p>- If type1 = "va" => type2 can be "rp", "ra", "vp".</p>
// <p>- If type1 = "rp" => type2 can be "ra", "vp", "va", "vinf".</p>
// <p>- If type1 = "vp" => type2 can be "rp", "ra", "va", "vinf".</p>
// <p>- If type1 = "sma" => type2 can be "ecc".</p>
// <p></p></listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p>- If output is "all", all the results are gathered in a structure containing the
// fields: "sma", "ecc", "ra", "rp", "va", "vp", "vinf".</p>
// <p></p>
// <p>- No error is raised if rp > ra or va > vp, but ra, rp, va, vp are guaranteed consistent.</p>
// <p>For instance the call:</p>
// <p>> CL_kp_apsConvert("ra", 7000.e3, "rp", 8000.e3, "va");</p>
// <p>will effectively compute the velocity where the radius is ra, although ra actually is the 
// radius at the periapsis.</p>
// <p>But also note that this possible inversion of apoapsis and periapsis is 
// not reflected in sma and ecc so that after the two successive calls:</p>
// <p>> [sma, ecc] = CL_kp_apsConvert("ra", 7000.e3, "rp", 8000.e3, ["sma", "ecc"]);</p>
// <p>> [ra, rp] = CL_kp_apsConvert("sma", sma, "ecc", ecc, ["ra", "rp"]);</p>
// <p>ra is 8000.e3 and rp is 7000.e3.</p>
// <p></p>
// <p>- Undefined output quantities are set to Nan.</p>
// <p></p>
// <p>- If the excentricity is close to 1 (to within 1.e-8) results are undefined, 
// that is, set to Nan.</p>
// </listitem>
// </itemizedlist>
//
// Parameters
// type1: (string) Type of 1st quantity - see above for details. (1x1)
// val1: Value of 1st quantity. [m or m/s or -] (1xN or 1x1)
// type2: (string) Type of 2nd quantity - see above for details. (1x1)
// val2: Value of 2nd quantity. [m or m/s or -] (1xN or 1x1)
// output: (string) Names of wanted results - see above for details. (1xP)
// mu: (optional) Gravitational constant. Default: %CL_mu. [m^3/s^2] (1x1)
// res1, res2...: Results. [m or m/s or -] (1xN)
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_kp_characteristics
//
// Examples
// // Example 1
// sma = 7.e6;
// ecc = 1.e-2;
// // Computes ra and rp from sma and ecc
// [ra, rp] = CL_kp_apsConvert("sma", sma, "ecc", ecc, ["ra", "rp"])
// // Computes everything from ra and rp
// res = CL_kp_apsConvert("ra", ra, "rp", rp, "all")
// 
// // Example 2: Hohmann transfer  
// r1 = 7.e6;
// r2 = 8.e6;
// v1 = CL_kp_apsConvert("rp", r1, "ra", r1, ["vp"]); 
// v2 = CL_kp_apsConvert("rp", r2, "ra", r2, ["vp"]); 
// [vp, va] = CL_kp_apsConvert("rp", r1, "ra", r2, ["vp", "va"]); 
// dv_hohmann = (vp - v1) + (v2 - va); 
// dv_hohmann - CL_man_dvHohmann(r1, r2) // => 0
// 
// // Example 3
// // Note that rp can be greater or smaller than ra 
// ra = 8.e6;
// rp = ra + (-1 : 0.1 : 1) * 1.e6;
// vp = CL_kp_apsConvert("rp", rp, "ra", ra, ["vp"]); 
// scf(); 
// plot(rp / 1000, vp); 

// Declarations:

// -----------------------------------------------------------------
// Utility functions 
// -----------------------------------------------------------------

// Formula used for all cases : 
// ra = sma * (1 + ecc)
// rp = sma * (1 - ecc)
// va^2 = (mu/sma) * ((1 - ecc) / (1 + ecc))
// vp^2 = (mu/sma) * ((1 + ecc) / (1 - ecc))
// vinf^2 = -mu / sma 
// 
// This implies the following relations : 
// ecc = (ra - rp) / (ra + rp)
// sma = (ra + rp) / 2
// rp * vp^2 / mu = 1 + ecc 
// rp * va^2 / mu = (1 - ecc)^2 / (1 + ecc)
// vp / va = (1 + ecc) / (1 - ecc) 
//
// Notes for the internal computations (above formulas): 
// - sma for hyperbolic orbits is negative
// - apoapsis and periapsis play a symmetrical role:  
//  "a" <-> "p" <=> "ecc" <-> "-ecc" 

// ------------------------------------------
// rp/ra => sma/ecc (elliptic orbits)
// Also works for ra/rp (but ecc is then negative) 
// Note : mu is unused 
// ------------------------------------------
function [sma, ecc] = apsides_rp_ra(rp, ra, mu)
  if (find(rp <= 0 | ra <= 0) <> [])
    CL__error("Invalid Arguments (should be > 0)"); 
  end 
  sma = (ra + rp) / 2; 
  ecc = (ra - rp) ./ (ra + rp); 
endfunction 

// ------------------------------------------
// rp/vp => sma/ecc (elliptic or hyperbolic orbits)
// Also works for ra/va (but ecc is then negative) 
// ------------------------------------------
function [sma, ecc] = apsides_rp_vp(rp, vp, mu)
  if (find(rp <= 0 | vp <= 0) <> [])
    CL__error("Invalid Arguments (should be > 0)"); 
  end   
  A = (rp .* vp.^2) / mu; 
  ecc = A - 1; 
  I = find(abs(1 - ecc) <= 0); 
  ecc(I) = %nan; 
  sma = rp ./ (1 - ecc);  
endfunction 

// ------------------------------------------
// rp/va => sma/ecc (elliptic orbits)
// Also works for ra/vp (but ecc is then negative) 
// ------------------------------------------
function [sma, ecc] = apsides_rp_va(rp, va, mu)
  if (find(rp <= 0 | va <= 0) <> [])
    CL__error("Invalid Arguments (should be > 0)"); 
  end   
  A = (rp .* va.^2) / mu; 
  x = (A - sqrt(A .* (A + 8))) / 2; 
  // note: x <> 0 as x = 0 => A = 0, but A <> 0  
  ecc = 1 + x; 
  sma = -rp ./ x;  
endfunction 

// ------------------------------------------
// vp/va => sma/ecc (elliptic orbits)
// Also works for va/vp (but ecc is then negative) 
// ------------------------------------------
function [sma, ecc] = apsides_vp_va(vp, va, mu)
  if (find(vp <= 0 | va <= 0) <> [])
    CL__error("Invalid Arguments (should be > 0)"); 
  end   
  sma = mu ./ (va .* vp); 
  A = vp ./ va;  
  ecc = (A - 1) ./ (A + 1); 
endfunction 

// ------------------------------------------
// rp/vinf => sma/ecc (hyperbolic orbits)
// ------------------------------------------
function [sma, ecc] = apsides_rp_vinf(rp, vinf, mu)
  if (find(rp <= 0 | vinf < 0) <> [])
    CL__error("Invalid Arguments (rp > 0 and vinf >= 0 expected)"); 
  end   
  vinf(vinf <= 0) = %nan; 
  sma = -mu ./ vinf.^2; 
  ecc = 1 - rp ./ sma; 
endfunction 

// ------------------------------------------
// vp/vinf => sma/ecc (hyperbolic orbits)
// ------------------------------------------
function [sma, ecc] = apsides_vp_vinf(vp, vinf, mu)
  if (find(vp <= 0 | vinf < 0) <> [])
    CL__error("Invalid Arguments (vp > 0 and vinf >= 0 expected)"); 
  end   
  vinf(vinf <= 0) = %nan; 
  sma = -mu ./ vinf.^2; 
  x = 1 - vp.^2 ./ vinf.^2; 
  x(abs(x) <= 0) = %nan; 
  ecc = (x - 2) ./ x;   
endfunction 

// ------------------------------------------
// sma/ecc => sma/ecc (elliptic or hyperbolic orbits)
// sma is made negative for hyperbolic orbits. 
// Note : mu is unused 
// ------------------------------------------
function [sma, ecc] = apsides_sma_ecc(sma, ecc, mu)
  if (find(sma <= 0 | ecc < 0) <> [])
    CL__error("Invalid Arguments (sma > 0 and ecc >= 0 expected)"); 
  end   
  I = find(ecc > 1); 
  sma(I) = -sma(I); 
endfunction 

// ------------------------------------------
// Compute sma/ecc from any input
// NB: sma/ecc(output): internal conventions 
// swap == %t => swap ra<->rp and va<->vp 
// ------------------------------------------
function [sma, ecc, swap] = apsides_compute_sma_ecc(type1, val1, type2, val2, mu)
  // Conversions: 
  // row 1-3: type1, type2, convertion_code (<-> function),
  // row 4: "swap" if arguments have to be swapped, "" otherwise
  // row 5: "swap" if results have to be swapped (ra<->rp, va<->vp), "" otherwise
  Conversions = [... 
    ["rp";  "ra";    "rp_ra";    "";       ""], ...
    ["rp";  "vp";    "rp_vp";    "";       ""], ...
    ["rp";  "va";    "rp_va";    "";       ""], ...
    ["rp";  "vinf";  "rp_vinf";  "";       ""], ...
    ["ra";  "rp";    "rp_ra";    "";       "swap"], ...
    ["ra";  "va";    "rp_vp";    "";       "swap"], ...
    ["ra";  "vp";    "rp_va";    "";       "swap"], ...
    ["vp";  "rp";    "rp_vp";    "swap";   ""], ...
    ["vp";  "ra";    "rp_va";    "swap";   "swap"], ...
    ["vp";  "va";    "vp_va";    "";       ""], ...
    ["vp";  "vinf";  "vp_vinf";  "";       ""], ...
    ["va";  "rp";    "rp_va";    "swap";   ""], ...
    ["va";  "ra";    "rp_vp";    "swap";   "swap"], ...
    ["va";  "vp";    "vp_va";    "";       "swap"], ...
    ["sma"; "ecc";   "sma_ecc";  "";       ""]];  
  
  Func_names = ["rp_ra", "rp_vp", "rp_va", "vp_va", "rp_vinf", "vp_vinf", "sma_ecc"]; 
  Funcs = list(apsides_rp_ra, apsides_rp_vp, apsides_rp_va, apsides_vp_va, ... 
               apsides_rp_vinf, apsides_vp_vinf, apsides_sma_ecc); 
  
  I = find(type1 == Conversions(1,:) & type2 == Conversions(2,:)); 
  if (I == [])
    CL__error("Invalid value for type1 and/or type2: " + strcat([type1, type2], ", ")); 
  end 

  k = find(Func_names == Conversions(3, I)); 
  func = Funcs(k); 
  if (Conversions(4,I) == "swap")
    [sma, ecc] = func(val2, val1, mu); 
  else
    [sma, ecc] = func(val1, val2, mu); 
  end
  
  // need to swap apoapsis and periapsis in results 
  swap = (Conversions(5,I) == "swap"); 
endfunction

// ------------------------------------------
// Compute all quantities form sma/ecc
// NB: sma/ecc(input): internal conventions 
// swap == %t => swap ra<->rp and va<->vp 
// ------------------------------------------
function [result] = apsides_compute_all(sma, ecc, swap, mu)
  result = struct(); 
  I = find(abs(abs(ecc) - 1) < 1.e-8); 
  ecc(I) = %nan; 
  sma(I) = %nan; 
  result.sma = abs(sma); 
  result.ecc = abs(ecc); 
  result.ra = sma .* (1 + ecc); 
  result.rp = sma .* (1 - ecc); 
  result.va = real(sqrt((mu ./ sma) .* ((1 - ecc)./ (1 + ecc))));  
  result.vp = real(sqrt((mu ./ sma) .* ((1 + ecc)./ (1 - ecc)))); 
  result.vinf = real(sqrt(-mu ./ sma));   
  if (swap)
    [result.ra, result.rp] = (result.rp, result.ra); 
    [result.va, result.vp] = (result.vp, result.va); 
  end 
  I = find(abs(ecc) >= 1); 
  result.ra(I) = %nan; 
  result.va(I) = %nan; 
  I = find(abs(ecc) < 1); 
  result.vinf(I) = %nan; 
endfunction 

// Code:
// -----------------------------------------------------------------
// Main 
// -----------------------------------------------------------------
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end

nbargin = argn(2); 
if (nbargin <> 5 & nbargin <> 6)
  CL__error("Invalid number of input arguments"); 
end

nbargout = argn(1); 
if (nbargout <> size(output,2))
  CL__error("Invalid number of output arguments"); 
end

if (typeof(type1) <> "string" | size(type1, "*") <> 1)
  CL__error("Invalid size or type for type1"); 
end 
if (typeof(type2) <> "string" | size(type2, "*") <> 1)
  CL__error("Invalid size or type for type2"); 
end 
if (typeof(output) <> "string" | size(output, 1) <> 1)
  CL__error("Invalid size or type for output"); 
end 

Output_names = ["sma", "ecc", "ra", "rp", "va", "vp", "vinf", "all"]; 
invalid_names = CL_setdiff(output, Output_names); 
if (invalid_names <> [])
  CL__error("Invalid output names: " + strcat(invalid_names, ", ")); 
end

[val1, val2] = CL__checkInputs(val1, 1, val2, 1); 

if ((val1 == [] & val2 <> []) | (val2 == [] & val1 <> []))
  CL__error("Inconsistent sizes for val1 and val2"); 
end 

if (val1 <> [] | val2 <> []) 
  // -- Step 1 : computes sma and ecc 
  [sma, ecc, swap] = apsides_compute_sma_ecc(type1, val1, type2, val2, mu); 

  // -- Step 2: computes everything from sma and ecc 
  result = apsides_compute_all(sma, ecc, swap, mu); 
  
  // -- Step 3: input values are copied in "result" structure
  result(type1) = val1; 
  result(type2) = val2;   
else 
  result = struct("sma", [], "ecc", [], "ra", [], "rp", [], "va", [], "vp", [], "vinf", []); 
end 

for (k = 1:size(output,2))
  name = output(k); 
  if (name == "all")
    varargout(k) = result;
  else
    varargout(k) = result(name);
  end
end

endfunction
