//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [eccv] = CL_op_eccVector3d(pos, vel, mu)
// Eccentricity vector (3D)
//
// Calling Sequence
// [eccv] = CL_op_eccVector3d(pos, vel [, mu])
//
// Description
// <itemizedlist>
// <listitem>
// <p>Computes the eccentricity vector from the inertial position and velocity vectors.</p>
// <p>The eccentricity vector points to the periapsis of the orbit, and its norm is equal to 
// the orbit's eccentricity. </p>
// <p>If pos or vel is [], the result is []. </p> 
// <p></p>
// </listitem>
// <listitem>
// <p>Notes:</p>
// <p>- The origin of the implicit frame is the center of the central body (coordinales = [0;0;0]). </p>
// <p>- This function can be used for elliptical or hyperbolic orbits. </p>
// <p></p>
// </listitem>
// </itemizedlist>
// Parameters
// pos: Satellite's position vector  [m] (3xN or 3x1)
// vel: Satellite's velocity vector [m/s] (3xN or 3x1)
// mu: (optional) Gravitational constant. Default: %CL_mu. [m^3/s^2] (1x1). (1x1)
// eccv : Eccentricity vector (3xN)
//
// Authors
// CNES - DCT/SB
//
// Examples
// // 1) Computation of eccentricity vector from Keplerian orbital elements 
// kep = [7.e6; 0.1; 1; 2; 3; 4];
// eccv = kep(2) * CL_rot_rotVect(CL_rot_angles2quat([3, 1, 3], [kep(5); kep(3); kep(4)]), [1; 0; 0]); 
// 
// // 2) Computation of eccentricity vector from position and velocity vectors 
// [pos, vel] = CL_oe_kep2car(kep); 
// CL_op_eccVector3d(pos, vel) - eccv // => 0 

// Declarations:


// Code:

// Check number of input arguments
[lhs, rhs] = argn(); 
if (rhs < 2 | rhs > 3) 
  CL__error("Invalid number of input arguments"); 
end

// Default values
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end

// Check size / resize
[pos, vel] = CL__checkInputs(pos, 3, vel, 3); 

if (pos == [] | vel == [])
  eccv = []; 
  return; // <== RETURN 
end

// eccentricity vector
eccv = CL_cross(vel, CL_cross(pos, vel)) / mu - CL_unitVector(pos); 

endfunction
