//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [exdot, eydot] = CL_op_frozenOrbitDerCir(sma, ex, ey, inc, er, mu, j1jn)
// Derivatives of eccentricity vector (ex, ey) with respect to time
//
// Calling Sequence
// [exdot, eydot] = CL_op_frozenOrbitDerCir(sma, ex, ey, inc [[, er, mu, j1jn]])
//
// Description
// <itemizedlist><listitem>
// <p>Computes the time derivatives of the eccentricity vector components  
// resulting from the gravitational effects due to J2 and J3.</p>
// <p>The eccentricity vector components are defined by: ex = e * cos(pom) and ey = sin(pom), 
// where ecc is the eccentricity, and pom the argument of the periapsis. </p>
// <p>The time derivatives are undefined (%nan) if the orbit is equatorial (inc = 0 or pi).</p>
// <p>The formulas are equivalent to the ones presented below 
// (see <link linkend="CL_op_frozenOrbitDer">CL_op_frozenOrbitDer</link>), 
// except that they are valid even if eccentricity is 0: </p>
// <p><inlinemediaobject><imageobject><imagedata fileref="driftJ2J3.gif"/></imageobject></inlinemediaobject></p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// sma: Semi-major axis. [m] (1x1 or 1xN)
// ex: Eccentricity vector "x" component. [-] (1x1 or 1xN)
// ey: Eccentricity vector "y" component. [-] (1x1 or 1xN)
// inc: Inclination. [rad] (1x1 or 1xN)
// er: (optional) Equatorial radius. Default: %CL_eqRad. [m] (1x1)
// mu: (optional) Gravitational constant. Default: %CL_mu. [m^3/s^2] (1x1)
// j1jn: (optional) Vector of zonal coefficients J1 to Jn, troncated to J3. Default: %CL_j1jn(1:3)). [-] (1xNz)
// exdot: Time derivative of ex. [-/s] (1xN)
// eydot: Time derivative of ey. [-/s] (1xN)
//
// Authors
// CNES - DCT/SB
// 
// Bibliography
// 1) "Frozen orbits in the J2+J3 problem", Krystyna Kiedron and Richard Cook, AAS 91-426. 
// 
// See also
// CL_op_frozenOrbitDer
//
// Examples
// sma = 7000.e3;
// ex = 1.e-4; 
// ey = 1.2e-3; 
// inc = CL_deg2rad(98.0);
// [exdot, eydot] = CL_op_frozenOrbitDerCir(sma, ex, ey, inc); 
// 
// // Check using CL_op_frozenOrbitDer
// ecc = sqrt(ex^2 + ey^2); 
// pom = atan(ey, ex);   
// // Derivatives of ecc and pom: 
// [deccdt, dpomdt] = CL_op_frozenOrbitDer(sma, ecc, inc, pom);
// exdot2 = deccdt * cos(pom) - ecc * sin(pom) * dpomdt; 
// eydot2 = deccdt * sin(pom) + ecc * cos(pom) * dpomdt; 
// exdot - exdot2 // => 0
// eydot - eydot2 // => 0


// Declarations:

// Code:
if (~exists("er", "local")); er = CL__dataGetEnv("eqRad"); end
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end
if (~exists("j1jn", "local")); j1jn = CL__dataGetEnv("j1jn", 1:3); end

[sma, ex, ey, inc] = CL__checkInputs(sma, 1, ex, 1, ey, 1, inc, 1); 

if (find(ex.^2 + ey.^2 >= 1 | sma <= 0) <> []); 
  CL__error("Invalid arguments"); 
end

if (length(j1jn) < 3)
  CL__error("j1jn must be a vector of size 3 (or more)");
end

j2 = j1jn(2);
j3 = j1jn(3);

sini = sin(inc);

// n = mean motion
n = CL_kp_params("mm", sma, mu); 

// check possible division by 0 (result is then %nan)
I = find(abs(sini) < %eps);
sini(I) = %nan;  

// eccentricity
ecc = sqrt(ex.^2 + ey.^2); 

// K1, K2, K3 defined by (see CL_op_frozenOrbitDer) 
// (K1, K2, K3 depend on sma, ecc, inc) 
// 
// decc/dt = K1 * cos(pom)
// dpom/dt = K2 + (-K1 * sin(pom) / ecc + K3 * ecc * sin(pom)) / (1 - ecc^2) 
// 
// Which can also be written: 
// decc/dt  = K1 * ex / ecc
// dpom/dt = K2 - K1 * ey/ecc^2 + K3 * ey + ey * (-K1 + K3 * ecc^2) / (1 - ecc^2) 
// 
// Then :
// exdot = deccdt * ex / ecc - ey * dpomdt;
// eydot = deccdt * ey / ecc + ex * dpomdt;
// 
// => after calculation : see below. 
//
// Note: equation developped to order 1 in eccentricity give: 
// exdot = K1 - ey * K2 
// eydot = K2 * ex 
// (K1 and K2: constant) 
// 
// The frozen eccentricity is given by : exf = 0, eyf = K1 / K2 
// 
// Computing the 2nd ordre derivatives: 
// d2ex/dt2 + K2^2 * ex = 0
// d2dy/dt2 + K2^2 * ey = K1*K2 
// 
// Which can also be written, calling delta_ex = ex - exf, delta_ex = ey - eyf: 
// d2(delta_ex)/dt2 + K2^2 * delta_ex = 0; 
// d2(delta_ey)/dt2 + K2^2 * delta_ey = 0; 
//
// => circle described with angular velocity = |K2| 
// (K2 = d(pom)/dt due to J2) 

K1 = -0.375 * (er ./ sma).^3 .* n * j3 .* sini .* (4 - 5 * sini .^ 2) ./ (1 - ecc .^ 2).^2 ; 

K2 = 0.75 * (er ./ sma) .^ 2 .* n * j2 .*(4 - 5 * sini .^ 2) ./ (1 - ecc .^ 2).^2 ; 

K3 = -0.375 * (er ./ sma) .^ 3 .* n * j3 .* ...
        ((4 - 35 * sini .^ 2 .* (1 - sini .^ 2))) ./ ((1 - ecc .^ 2) .^ 2 .* sini) 

exdot = K1 - ey .* K2 - ey.^2 .* (K3 - K1) ./ (1 - ecc.^2); 

eydot = K2 .* ex + ex .* ey .* (K3 - K1) ./ (1 - ecc.^2); 

endfunction
