//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// -------------------------------
// Computation of "mean" elements (generic, internal function)
// => Determines "moy" such that: fct_extrap(moy) = osc
//
// Interface: fct_extrap(moy, I, args), 
// I: indices being analysed, args: any (constant) variable
//
// typ: "kep", "cir" or "equin" only
// moy and osc: type = typ
//
// Algorithm: 
// Iterative process: 
// Loop: 
//   p = fct(moy);
//   Apply corrections to moy: moy = moy - (p - osc)
//   after conversion to equinoxial elements. 
// Stop when (p - osc) small enough (relative difference)
// Returns %nan if no convergence. 
//
// Author: AL
// -------------------------------

function [moy] = CL__ex_inverse(typ, osc, fct_extrap, args)

  // Conversion: equin -> kep or cir or equin (if cir: equin->kep->cir)
  // NB: mu not used (not included in function call)
  function [oe] = exinv_equin2oe(equin, typ)
     if (typ == "equin") 
       oe = equin;
     elseif (typ == "kep")
       oe = CL__oe_equin2kep(equin); 
     else // if (typ == "cir")
       oe = CL__oe_equin2cir(equin); 
     end
  endfunction

  // kep or cir or equin -> equin (if cir: cir->kep->equin)
  // NB: mu not used (not included in function call)
  function [equin] = exinv_oe2equin(oe, typ)
    if (typ == "equin") 
      equin = oe; 
    elseif (typ == "kep")
      equin = CL__oe_kep2equin(oe); 
    else // if (typ == "cir")
      equin = CL__oe_cir2equin(oe); 
    end
  endfunction

  // prints a warning
  function exinv_printWarning(errel_final, msg)
    [m, k] = max(errel_final); 
    CL__warning(msprintf("CL__ex_inverse: %s: max(err/tol) = %.1f (index = %d)", msg, m, k));  
  endfunction
  
  
  // Check type to avoid wrong computations
  if (typ <> "cir" & typ <> "kep" & typ <> "equin")
    CL_error("Invalid argument: ''typ''"); 
  end
  
  // moy: current value in iterations
  // moy_final: best result (will be the output) 
  moy = osc; 
  osc_equin = exinv_oe2equin(osc, typ); 
  moy_equin = osc_equin; 
  moy_final = moy; 
  
  // size of input arg
  N = size(osc, 2); 
  
  // (relative) tolerance ("equin" type)  
  tol = 10 * %eps * max(abs(osc_equin), 1); 

  // K = indices to examine (initially: all)
  K = 1 : N; 

  // relative error (ratio to tol) for convergence checking
  // indic_err: indices for which the relative error does not decrease 
  errel = %inf * ones(1 : N); 
  errel_final = errel; 
  indic_err = zeros(errel); 
  
  // max number of iterations and current number
  maxiter = 20; 
  iter = 1; 
  
  while (iter <= maxiter & K <> [])
    // note: "K" in names means "vector has K columns" 

    oeK = fct_extrap(moy(:,K), K, args); 
    oeK_equin = exinv_oe2equin(oeK, typ);  

    deltaK = oeK_equin - osc_equin(:,K); 
    deltaK(6,:) = CL_rMod(deltaK(6,:), -%pi, %pi); 

    errel(K) = max(abs(deltaK) ./ tol(:,K), "r"); 

    moy_equin(:,K) = moy_equin(:,K) - deltaK; 
    moy(:,K) = exinv_equin2oe(moy_equin(:,K), typ); 

	// Kdecr: indices for which errel decreases => store best result obtained so far (in moy_final) 
	// and store associated error (errel_final)
	// Kincr: indices for which errel does not decrease => memorize
	// for check on maximum number of iterations  
	err_decreasesK = (errel(K) < errel_final(K)); 
    Kdecr = K(find(err_decreasesK)); 
    Kincr = K(find(~err_decreasesK)); 
	
    moy_final(:,Kdecr) = moy(:,Kdecr); 
	errel_final(Kdecr) = errel(Kdecr); 
	indic_err(Kdecr) = 0; 
    indic_err(Kincr) = indic_err(Kincr) + 1 * ones(Kincr); 
	
	// indices for next iteration
	// give up if error does not decrease several times in a row
    K = find(errel_final > 1 & indic_err <= 2); 
	
    iter = iter+1; 
  end
    
  // output = best result
  moy = moy_final; 
  
  // if relative error > 1000 : no results to avoid erroneous computations 
  // warning generated to inform user 
  // errel_final set to %nan to avoid 2nd check 
  K = find(errel_final > 1000); 
  if (K <> [])
    exinv_printWarning(errel_final, "no convergence"); 
    moy(:,K) = %nan;
    errel_final(K) = %nan; 
  end
  
  // if relative error > 1 : best result obtained is returned (supposed valid enough)
  // warning generated to inform user 
  K = find(errel_final > 1); 
  if (K <> [])
    exinv_printWarning(errel_final, "conversion may be inaccurate"); 
  end

  // adjusts "modulo" 
  // value of angles made close to values in "osc"
  if (typ == "kep")
    moy(4,:) = CL_rMod(moy(4,:), osc(4,:)-%pi, osc(4,:)+%pi);
  end
  if (typ == "kep" | typ == "cir")
    moy(5,:) = CL_rMod(moy(5,:), osc(5,:)-%pi, osc(5,:)+%pi);
  end
  moy(6,:) = CL_rMod(moy(6,:), osc(6,:)-%pi, osc(6,:)+%pi);

endfunction


