//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [varargout] = CL_man_dvHohmannRotP(ai, af, rotang, c_mode, alpha, mu, res)
// Hohmann transfer with orbit plane rotation
//
// Calling Sequence
// [deltav, dv1, dv2, alpha] = CL_man_dvHohmannRotP(ai, af, rotang [[, c_mode, alpha, mu, res="d"]])
// man = CL_man_dvHohmannRotP(ai, af, rotang [[, c_mode, alpha, mu, res="s"]])
//
// Description
// <itemizedlist><listitem>
// <p>Computes the impulsive maneuvers of a transfer from an initial circular orbit with semi major axis 
// <b>ai</b> to a final circular orbit with semi major axis <b>af</b> where the plane rotates about the axis 
// joining the 2 maneuver locations. </p>
// <p></p> 
// <p>The transfer is derived from a Hohmann transfer. Out-of-plane (non radial) components are added
// to the two DVs in order to achieve the desired plane rotation. The function can be used in two ways 
// depending on the argument <b>c_mode</b> (computation mode). If <b>c_mode</b> = "o" (optimize, which is the default), 
// the proportion of rotation angle between the 2 maneuvers is determined automatically in order to obtain 
// the minimal total DV. If <b>c_mode</b> = "c" (compute), the proportion given by <b>alpha</b> is used. 
// <b>alpha</b> (between 0 and 1) represents the proportion of rotation angle performed at the fist impulse. 
// This means that if <b>alpha</b> is 0, the plane rotation is entirely obtained by the 2nd impulse. </p>
// <p></p>
// <p>The total plane rotation angle is <b>rotang</b>. The direct sense for the rotation is deduced 
// from the directon of the rotation axis: from maneuver location 2 to maneuver location 1. </p>
// <p></p>
// <p>The output argument <b>deltav</b> is the sum of the norms 
// of the velocity increments (|<b>dv1</b>| + |<b>dv2</b>|).</p>
// <p>Velocity increments are expressed in the "qsw" frame. </p>
// <p></p>
// <p>If the argument <b>res</b> is present and is equal to "s", all the output data are returned 
// in a structure. </p>
// <p></p></listitem>
// <listitem>
// <p>The typical use is for changing the semi-major axis and the inclination 
// of a circular orbit simultaneously. <b>rotang</b> is then the inclination change 
// if the first maneuver is at the ascending node. </p>
// <p></p></listitem>
// </itemizedlist>
//
// Parameters
// ai : Semi-major axis of initial circular orbit. [m] (1xN or 1x1)
// af : Semi-major axis of final circular orbit. [m] (1xN or 1x1)
// rotang : Plane rotation angle in [-%pi, %pi]. [rad] (1xN or 1x1)
// c_mode : (string, optional) "o": optimize DV, "c": use alpha values. Default is "o" (1x1)
// alpha : (optional) Proportion of rotation angle for the first velocity increment. Only used if c_mode = "c". Default value is 0.5 (1xN or 1x1)
// mu : (optional) Gravitational constant. Default is %CL_mu. [m^3/s^2] (1x1)
// res : (string, optional) Type of output: "d" for multiple outputs or "s" for a single structure output. Default is "d".
// deltav : Som of norms of velocity increments. [m/s] (1xN)
// dv1: First velocity increment in cartesian coordinates in the "qsw" local orbital frame. [m/s]. (3xN)
// dv2: Second velocity increment at opposite location in cartesian coordinates in the "qsw" local orbital frame. [m/s] (3xN)
// alpha (output) : Proportion of rotation angle actually used for the first velocity increment. 
// man: Structure containing all the output data. 
//
// Authors
// CNES - DCT/SB
//
// See also
// CL_man_dvHohmann
// CL_man_dvInc
//
// Examples
// // Change of semi-major axis and inclination by 2 maneuvers 
// ai = 7000.e3;
// af = 7100.e3;
// rotang = 0.01; // rad 
// [deltav_opt, dv1, dv2, alpha_opt] = CL_man_dvHohmannRotP(ai, af, rotang);
//
// // Check results:
// // First maneuver at the ascending node 
// kep0 = [ai; 0; 1; 0; 0; 0]; 
// kep1 = CL_man_applyDvKep(kep0, dv1, dv_frame = "qsw");
// // 2nd maneuver at the descending node (also the apogee)
// kep1(6) = CL_kp_v2M(kep1(2), %pi - kep1(4));
// kep2 = CL_man_applyDvKep(kep1, dv2, dv_frame = "qsw")
// 
// // Effect of inclination change ratio between the 2 maneuvers 
// alpha = linspace(0, 1, 100); // proportion at 1st impulse  
// deltav = CL_man_dvHohmannRotP(ai, af, rotang, c_mode="c", alpha=alpha);
// scf(); 
// plot(alpha, deltav); 
// plot(alpha_opt, deltav_opt, "o"); 


// Declarations: (internal function)
// method: 
// optimise dv = dv1 + dv2  = f(alpha). alpha = proportion of rotation angle for 
// first maneuver 
// dv1^2 = vi^2 + v1^2 - 2*vi*v1*cos(alpha*angrot); 
// dv2^2 = vf^2 + vf^2 - 2*vf*v2*cos((1-alpha)*angrot); 
// vi, vf: initial and final velocities 
// v1, v2: intermediate hohmann velocities
// Derivative of dv: angrot * (vi*v1*sin(alpha*angrot)/dv1 - vf*v2*sin((1-alpha)*angrot)/dv2
//
// It can be shown that: 
// - if smai < smaf (vi > vf) and angrot <> 0: 
// The minimum of dv(alpha) belongs to the interval ]0, 0.5[. The derivative dv'(alpha) 
// has only one zero in this interval. The function dv'(alpha) is not necessarily monotonous
// in this interval.    
// - if smai > smaf and angrot <> 0: 
// The minimum of dv(alpha)  belongs to the interval ]0.5, 1[. The derivative dv'(alpha) 
// has only one zero in this interval. The function dv'(alpha) is not necessarily monotonous
// in this interval.   
// 
// It can be also shown that dv could be written: dv = vi * f(smaf/smai, alpha) 
// (initial velocity times some function of smaf/smai and alpha). 
// This could be used to normalize the computations. It is finally not implemented in this code.   

// Important: rotang in [-%pi, %pi], otherwise alpha would not be in [0, 1]
function [dv1, dv2, alpha] = dvHohmannRotP(ai, af, rotang, c_mode, alpha_user, mu)
  
  // initial and final velocities 
  vi = sqrt(mu ./ ai); //  = 1
  vf = sqrt(mu ./ af); 
  
  // DV Hohmann (dans qsw). Only component: tangential. 
  [deltav_h, dv1_h, dv2_h, anv1, anv2] = CL_man_dvHohmann(ai, af, mu=mu); 
  
  // intermediate Hohmann velocities 
  v1 = vi + dv1_h(2,:); 
  v2 = vf - dv2_h(2,:); 

  function [dv] = Fdvtot(alpha)
	theta1 = alpha .* rotang; 
	theta2 = (1 - alpha) .* rotang;
	// note: formulas for better accuracy 
	dv1 = sqrt((v1 .* cos(theta1) - vi).^2 + (v1 .* sin(theta1)).^2); 
	dv2 = sqrt((v2 .* cos(theta2) - vf).^2 + (v2 .* sin(theta2)).^2); 
    dv = dv1 + dv2; 
  endfunction 

  // F: similar to derivative of dv(alpha), but multiplied by factors. 
  // interface for fsolveb
  // Note: same solution for +rotang and -rotang. 
  // NB: use variables defined in upper level. 
  function [y] = F(alpha, I, args)
	theta1 = alpha .* rotang(I); 
	theta2 = (1 - alpha) .* rotang(I);
		
	// note: formulas for better accuracy 
	dv1 = sqrt((v1(I) .* cos(theta1) - vi(I)).^2 + (v1(I) .* sin(theta1)).^2); 
	dv2 = sqrt((v2(I) .* cos(theta2) - vf(I)).^2 + (v2(I) .* sin(theta2)).^2); 
  
    y = rotang(I) .* ...
       (dv2 .* (v1(I) ./ vf(I)) .* sin(theta1) - ...
        dv1 .* (v2(I) ./ vi(I)) .* sin(theta2)); 
  endfunction 

  // initialize bounds: should be [0, 1] 
  // but slightly enlarged so that zeros are computed in all cases, 
  // in particular for rotang = %pi 
  binf = 0 * ones(ai); 
  bsup = 1 * ones(ai); 
  bsup(ai <= af) = 0.5; 
  binf(ai > af) = 0.5; 
  
  if (c_mode == "o")
    // finds alpha / F(alpha) = 0 (optimal dv) 
    // Note the algorithm is important; Not all may work as the function is not monotonous. 
    alpha = CL_fsolveb(F, binf, bsup, meth = "ds", dxtol = 1.e-12); 

    // prevents wrong results in particular cases. 
	// in particular if ai = af => may converges towards 0.5. 
	alpha(isnan(alpha)) = 0.5; 
	alpha(Fdvtot(0) < Fdvtot(alpha)) = 0; 
	alpha(Fdvtot(1) < Fdvtot(alpha)) = 1; 
		
  else
    alpha = alpha_user; 
  end 
  
  // vectors in qsw frame 
  Z = zeros(ai); 
  
  // First maneuver 
  vi_vect = [Z; vi; Z]; 
  v1_vect = [Z; v1 .* cos(alpha .* rotang); v1 .* sin(alpha .* rotang)]; 

  // 2nd maneuver - opposite location 
  v2_vect = [Z; v2; Z]; 
  vf_vect = [Z; vf .* cos((1 - alpha) .* rotang); -vf .* sin((1 - alpha) .* rotang)]; 

  dv1 = v1_vect - vi_vect; 
  dv2 = vf_vect - v2_vect; 

endfunction 


// Code:
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end
if (~exists("c_mode", "local")); c_mode = "o"; end
if (~exists("alpha", "local")); alpha = 0.5; end
if (~exists("res", "local")); res = "d"; end

// check "res" argument
if (res <> "d" & res <> "s"); 
  CL__error("Invalid value for argument ''res''"); 
end
if (argn(2) < 3)
  CL__error("Invalid number of input arguments"); 
end 
if (argn(1) > 1 & res == "s"); 
  CL__error("Invalid number of output arguments"); 
end
if (c_mode <> "o" & c_mode <> "c")
  CL__error("Invalid value for parameter c_mode"); 
end 

// checks arguments sizes are OK / resizes
[ai, af, rotang, alpha] = CL__checkInputs(ai, 1, af, 1, rotang, 1, alpha, 1); 

if (find(ai <= 0 | af <= 0) <> [])
  CL__error("Invalid input arguments"); 
end

// check rotang is in [-pi, pi]
if (find(abs(rotang) > %pi * (1 + 1.e-12)) <> [])
  CL__error("Invalid value for argument rotang. Should be in [-%pi, %pi)"); 
end

// computation
[dv1, dv2, alpha_used] = dvHohmannRotP(ai, af, rotang, c_mode, alpha, mu); 

deltav = CL_norm(dv1) + CL_norm(dv2); 

// output
if (res == "d")
  varargout = list(deltav, dv1, dv2, alpha_used); 
else
  // maneuver sructure also includes alpha_used
  varargout(1) = struct("deltav", deltav, "dv1", dv1, "dv2", dv2, "alpha", alpha_used); 
end

endfunction
