//  Copyright (c) CNES 2020
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [varargout] = CL_man_dvIncSmaCirc(ai, inci, af, incf, posman, icheck, vectorize, mu, res)
// Inclination + semi-major axis combined maneuvers (circular orbits)
//
// Calling Sequence
// [deltav, dv1, dv2, anv1, anv2] = CL_man_dvIncSmaCirc(ai, inci, af, incf [[, posman, icheck, vectorize, mu, res="d"]])
// man = CL_man_dvIncSmaCirc(ai, inci, af, incf, [[, posman, icheck, vectorize, mu, res="s"]])
//
// Description
// <itemizedlist><listitem>
// <p>Computes the velocity increment needed to change both the 
// inclination and the semi major axis of a circular orbit using two combined maneuvers.</p>
// <p></p>
// <p>Two maneuvers are performed at the ascending and the descending node
// in order to change both the semi major axis and the inclination while keeping the initial 
// and final orbits circular. </p>
// <p>The ratio of inclination change between the 1st and the 2nd maneuver is determined 
// so that the total DV is minimum. </p> 
// <p></p> 
// <p>The output argument <b>deltav</b> is the sum of the norms 
// of the velocity increments: (<b>deltav</b> = |<b>dv1</b>| + |<b>dv2</b>|).</p>
// <p>Velocity increments are expressed in the "qsw" frame.</p>
// <p><b>anv1</b> and <b>anv2</b> are the true anomalies where the DV are 
// performed. As the initial orbit is circular, anv1 is arbitrarily set to 0 or %pi, 
// which is consistent with an implicit value of the argument of periapsis of 0.</p>
// <p></p> 
// <p>The optional flag <b>posman</b> can be used to define the position
// of the 1st maneuver (maneuver at ascending node or descending node).</p>
// <p>The optional flag <b>icheck</b> is used to enforce no checking 
// on the final inclination value. If the targetted inclination is less 
// than 0 or more than pi, the ascending node will rotate 180 degrees. 
// If icheck is true, the right ascension of the ascending node is not 
// changed by the maneuver.</p>
// <p>The optional flag <b>vectorize</b> is used to manage vectorization.  
// Setting it to true makes the computation run faster, but because of the method used, 
// the results slightly depend on the vectorization index. </p>
// <p>If the argument <b>res</b> is present and is equal to "s", all the
// output data are returned in a structure. </p>
// <p></p></listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p> - This function is not a particular case of 
// <link linkend="CL_man_dvIncSma">CL_man_dvIncSma</link>, as the latter does not 
// preserve the eccentricity of the initial orbit. </p>
// <p> - Because of the method use to efficiently compute the solutions, the accuracy
// of the results slightly depends on the index (in case of "vectorized" inputs and if 
// the "vectorize" option is true).  </p> 
// <p> - The implicit value of the argument of the periapsis is 0.  </p> 
// </listitem>
// </itemizedlist>
//
// Parameters
// ai : Initial semi major-axis [m] (1xN or 1x1)
// inci : Initial inclination [rad] (1xN or 1x1)
// af : Final semi major-axis [m] (1xN or 1x1)
// incf : Final inclination [rad] (1xN or 1x1)
// posman : (optional) Flag specifying the position of the maneuver: 1 or "an" for ascending node, -1 or "dn" for descending node. Default is at the ascending node. (1xN or 1x1)
// icheck: (optional, boolean) Flag specifying if incf must be checked in the standard range for inclination values ([0, pi]). Default is %t. (1x1)
// vectorize: (optional, boolean) Vectorization option. Default is %t (1x1) 
// mu : (optional) Gravitational constant. Default is %CL_mu. [m^3/s^2]
// res : (string, optional) Type of output: "d" or "s". Default is "d".
// deltav : Sum of norms of velocity increments. [m/s] (1xN)
// dv1: First velocity increment in cartesian coordnates in the "qsw" local orbital frame. [m/s] (3xN)
// dv2: Second velocity increment in cartesian coordnates in the "qsw" local orbital frame. [m/s] (3xN)
// anv1: True anomaly at the location of the first velocity increment (relative to the initial orbit). The implicit value of the argument of perigee is 0. [rad] (1xN)
// anv2: True anomaly at the location of the second velocity increment (relative to the intermediate orbit). [rad] (1xN)
// man: Structure containing all the output data. (1x1)
//
// Authors
// CNES - DSO/DV/IF
//
// See also
// CL_man_dvInc
// CL_man_dvSma
// CL_man_dvIncSma
//
// Examples
// // Transfer from initial to final orbit 
// ai = 7.e6; 
// inci = CL_deg2rad(0);
// af = 8.e6; 
// incf = CL_deg2rad(1); 
// [deltav, dv1, dv2, anv1, anv2] = CL_man_dvIncSmaCirc(ai, inci, af, incf) 
//
// // Check results:
// // Note that true and mean anomalies are equal here 
// // (circular orbit or maneuver at apoapsis or periapsis) 
// kep0 = [ai ; 0 ; inci ; 0 ; 0 ; anv1];  
// kep1 = CL_man_applyDvKep(kep0, dv1); 
// kep1(6) = anv2; 
// kep2 = CL_man_applyDvKep(kep1, dv2); 
// kep2(1) - af // => 0
// kep2(2) // => 0
// kep2(3) - incf // => 0

// Declarations:
// --------------------------------------------------
// Utility functions
// --------------------------------------------------

// Compute the 2 maneuvers for a given ratio of inclination change: 
// 1st maneuver (PSO = 0): delta_inclinasion = s * di 
// 2nd maneuver (PSO = pi): delta_inclination = (1-s) * di 
function [dv, der, dv1_qsw, dv2_qsw, anv1, anv2] = calcul_dv(ai, af, di, s, mu)
  vi = sqrt(mu ./ ai); 
  vf = sqrt(mu ./ af); 

  // va and v2 are the norms of the velocities after 1st maneuver and before 2nd maneuver
  // (same as for a hohmann transfer) 
  [v1, v2] = CL_kp_apsConvert("rp", ai, "ra", af, ["vp", "va"], mu=mu); 

  // Vi, V1 : in qsw frame before maneuvre
  Vi = CL_dMult(vi, [0; 1; 0]); 
  V1 = CL_dMult(v1, [zeros(s); cos(s .* di); sin(s .* di)]); 

  // V2, Vf : in qsw frame before maneuver 
  V2 = CL_dMult(v2, [0; 1; 0]); 
  Vf = CL_dMult(vf, [zeros(s); cos(-(1-s) .* di); sin(-(1 - s) .* di)]); 

  dv = CL_norm(V1 - Vi) + CL_norm(Vf - V2); 
  
  // der = d(dv)/ds normalized
  // d(dv)/ ds = (vi .* v1 * sin(eps * di) ./ CL_norm(V1 - Vi) - ...
  //              v2 .* vf .* sin((1 - eps) * di) ./ CL_norm(Vf - V2)) * (di)  
  // => der = (d(dv)/ ds) * CL_norm(V1 - Vi) / (vi * v1 * di)
  der =  sin(s .* di) - sin((1 - s) .* di) .* ... 
         ((v2 .* vf  .* CL_norm(V1 - Vi)) ./ (vi .* v1 .* CL_norm(Vf - V2))); 

  // compute additional results if asked for   
  if (argn(1) > 2)
    dv1_qsw = CL_fr_inertial2qsw([1; 0; 0], Vi, V1 - Vi); 
    dv2_qsw = CL_fr_inertial2qsw([1; 0; 0], V2, Vf - V2);
  
    anv1 = zeros(ai); 
    anv2 = zeros(af); 
    I = find(af > ai); 
    anv2(I) = %pi; 
  end 
  
endfunction

// Compute maneuvers by optimization (optimum value of s = ratio of delta_inclination) 
// Method: computes zeros of dv/ds and select best solutions if there are more than 1. 
// ai, af, inci, incf : 1xN 
function [deltav, dv1, dv2, anv1, anv2] = optim_dv(ai, af, inci, incf, mu)
  // To make the computation efficient: build a single function dv of x: 
  // index 1 => x between 1 and 2, index 2 => x between 2 and 3, etc... 
  // For each index, s is function of x: s = SMIN + ((x - a) ./ (b - a)) * (SMAX - SMIN);
  // a = floor(x), b = floor(x)+1 
  SMIN = -0.1; 
  SMAX = 1.1; 

  // compute s as a function of x
  function [eps] = compute_s(x)
    a = floor(x); 
    b = floor(x) + 1; 
    eps = SMIN + ((x - a) ./ (b - a)) * (SMAX - SMIN); 
  endfunction 

  // interface for the computation of the zeros of dv/ds = 0 
  // y = dv/ds (normalized) 
  // args: structure that contains the input parameters (ai, af ...)
  function [y] = fct(x, args)
    eps = compute_s(x);  
    I = floor(x); 
    [dv, der] = calcul_dv(args.ai(I), args.af(I), args.di(I), eps, args.mu); 
    y = der; 
  endfunction
  
  // retains the best solutions (maximizing dv) given all the solution of dv/ds = 0 
  // only one solution is kept for each index. 
  // x: all the solutions of dv/ds = 0
  // args: structure that contains the input parameters (ai, af ...) 
  function [s_opt] = get_s_opt(x, args)
  
    N = size(args.ai, "*"); 
	  // optimum solutions of dv and s 
    dv_opt = %inf * ones(1 : N); 
    s_opt = %nan * ones(1 : N); 

    // compute dv and s using all initial solutions 
    s = compute_s(x); 
    K = floor(x); 
    dv = calcul_dv(args.ai(K), args.af(K), args.di(K), s, args.mu);

    // extract solutions for each index using "unique" 
	  // compare with solutions already found for each index
	  // remove processed solutions 
	  while (K <> []) 
      [K1, k] = unique(K); 
      [dv_opt(K1), num] = min(dv_opt(K1), dv(k)); 
      J = find(num == 2);
      s_opt(K1(J)) = s(k(J)); 
      K(k) = []; 
      dv(k) = []; 
      s(k) = []; 
	  end 
  endfunction 
  
  // compute final (optimized) results from all solutions of dv/ds = 0 
  function [deltav, dv1, dv2, anv1, anv2] = make_res(x, args)
    s_opt = get_s_opt(x, args); 
    [deltav, der, dv1, dv2, anv1, anv2] = calcul_dv(args.ai, args.af, args.di, s_opt, args.mu); 
  endfunction 

  N = size(ai, "*"); 
  I = 1 : N; 

  interv = [I; I+1]; 
  x1 = CL_intervLinspace(interv, 40); 
  x = matrix(x1(2:$-1, :), 1, -1); 

  // xbmin and xbmax: bounds for the computation of dv/ds = 0
  // correspond to: s=0 and s=1 
  xbmin = interv(1,:) + (-SMIN / (SMAX - SMIN)) * (interv(2,:) - interv(1,:)); 
  xbmax = interv(1,:) + ((1 - SMIN) / (SMAX - SMIN)) * (interv(2,:) - interv(1,:)); 

  // structure of parameters for detectZero
  args = struct(...
    "ai", ai, ...
    "af", af, ...
    "di", incf - inci, ...
    "mu", mu);

  // find all the zeros 
  xz = CL_detectZero(x, list(fct, args), xbounds = [xbmin; xbmax], ytol = 1.e-10); 
  
  // select the best solutions for each index and generate the final results 
  [deltav, dv1, dv2, anv1, anv2] = make_res(xz, args); 

endfunction 

// Same as optim_dv, except that a maximum number of elements (700) are processed 
// at the same time. 
// 2 reasons: avoid memory overflow and avoid numerical errors due to the intermediate variable x
// (results for all indices may not have the same accuracy)
function [deltav, dv1, dv2, anv1, anv2] = optim_dv_loop(ai, af, inci, incf, mu, vectorize)
  N = size(ai, "*"); 
  nmax = 700; 
  if (~vectorize)
    nmax = 0; 
  end 
  
  deltav = zeros(1, N); 
  dv1 = zeros(3, N);
  dv2 = zeros(3, N);
  anv1 = zeros(1, N);
  anv2 = zeros(1, N);
  
  // call optim_dv for successive sets of input parameters (maximum nmax+1 each time)
  // Note : utilisation de variables intermédiaires (dv1_tmp, dv2_tmp) pour compatibilité avec Scilab 5.5.2
  k = 1; 
  while (k <= N)
    I = k : min(N, k+nmax); 
    [deltav(I), dv1_tmp, dv2_tmp, anv1(I), anv2(I)] = optim_dv(ai(I), af(I), inci(I), incf(I), mu); 
    dv1(:,I) = dv1_tmp; 
    dv2(:,I) = dv2_tmp;  
    k = I($) + 1; 
  end
endfunction 
  

// --------------------------------------------------
// MAIN
// --------------------------------------------------
// Code:

// 1) check / update arguments 
if (~exists("posman", "local")); posman = 1; end   // default: AN 
if (~exists("icheck", "local")); icheck = %t; end 
if (~exists("vectorize", "local")); vectorize = %t; end 
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end
if (~exists("res", "local")); res = "d"; end

// check "res" argument
if (res <> "d" & res <> "s"); 
  CL__error("Invalid value for argument ''res''"); 
end
if (argn(1) > 1 & res == "s"); 
  CL__error("Invalid number of output arguments"); 
end

// convert posman type to "real"
if (typeof(posman) == "string")
  str = posman; 
  posman = %nan * ones(str); 
  posman(find(str == "an")) = 1; 
  posman(find(str == "dn")) = -1; 
end

if (typeof(vectorize) <> "boolean" | size(vectorize, "*") <> 1)
  CL__error("Wrong type or size for argument ''vectorize''"); 
end

// checks arguments sizes are OK / resizes
[ai, inci, af, incf, posman, N] = CL__checkInputs(ai, 1, inci, 1, af, 1, incf, 1, posman, 1); 

if (find(ai <= 0 | af <= 0 | inci < 0 | inci > %pi) <> [])
  CL__error("Invalid input arguments (orbital elements)"); 
end

if (icheck & find(incf < 0 | incf > %pi ) <> [])
  CL__error("Invalid input arguments (final inclination)"); 
end

if (find(posman <> 1 & posman <> -1) <> [])
  CL__error("Invalid value for ''posman''"); 
end


// 2) compute maneuvers 
// First step: the computation assumes : 1st maneuver at ascending node 
// 2nd step : If 1st maneuver at ddescending node: 
// anv => anv + %pi for 1st maneuver and sign of W components changed 
deltav = zeros(1, N); 
dv1 = zeros(3, N);
dv2 = zeros(3, N);
anv1 = zeros(1, N);
anv2 = zeros(1, N);

// Note : utilisation de variables intermédiaires (dv1_tmp, dv2_tmp) pour compatibilité avec Scilab 5.5.2
I = find(abs(inci - incf) <= 0 & abs(ai - af) > 0); 
if (I <> [])
  [deltav(I), dv1_tmp, dv2_tmp, anv1(I), anv2(I)] = CL_man_dvHohmann(ai(I), af(I), mu=mu); 
  dv1(:,I) = dv1_tmp; 
  dv2(:,I) = dv2_tmp;  
end

I = find(abs(inci - incf) > 0 & abs(ai - af) <= 0);  
if (I <> [])
  eiI = zeros(I); 
  pomiI = zeros(I); 
  dincI = incf(I) - inci(I);  
  [deltav1I, dv1_tmp, anv1(I)] = CL_man_dvInc(ai(I), eiI, inci(I), pomiI, inci(I) + dincI/2, mu=mu, posman="an"); 
  [deltav2I, dv2_tmp, anv2(I)] = CL_man_dvInc(ai(I), eiI, inci(I) + dincI/2, pomiI, incf(I), mu=mu, posman="dn"); 
  dv1(:,I) = dv1_tmp; 
  dv2(:,I) = dv2_tmp;   
  deltav(I) = deltav1I + deltav2I; 
end

I = find(abs(inci - incf) > 0 & abs(ai - af) > 0);  
if (I <> [])
  [deltav(I), dv1_tmp, dv2_tmp, anv1(I), anv2(I)] = optim_dv_loop(ai(I), af(I), inci(I), incf(I), mu, vectorize); 
  dv1(:,I) = dv1_tmp; 
  dv2(:,I) = dv2_tmp; 
end

// adjust for cases where 1st maneuver at the descending node
I = find(posman == -1); 
if (I <> [])
  dv1(3,I) = -dv1(3,I); 
  dv2(3,I) = -dv2(3,I); 
  anv1 = anv1 + %pi; 
end 

// output
if (res == "d")
  varargout = list(deltav, dv1, dv2, anv1, anv2); 
else
  varargout(1) = struct("deltav", deltav, "dv1", dv1, "dv2", dv2, "anv1", anv1, "anv2", anv2); 
end

endfunction
