//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [result] = CL_man_lowThrustApsKep(kep, acc, acc_frame, alpha, apsis, res, opts, mu)
// Effect of continuous low thrust on the Keplerian orbital elements (elliptical orbits)
//
// Calling Sequence
// [result] = CL_man_lowThrustApsKep(kep, acc, acc_frame, alpha, apsis, res [[, opts, mu]])
//
// Description
// <itemizedlist><listitem>
// <p>Computes the effect of a continuous low thrust maneuver on the Keplerian 
// orbital elements. </p>
// <p></p>
// <p> The thrust acceleration is supposed to be zero everywhere except in an orbit
// portion centered on the periapsis or the apoapsis. The interval in <b>true anomaly</b>
// where the acceleration is not zero is: </p>
// <p>- if apsis = "<b>per</b>" (periapsis): [ <b>-alpha</b>, <b>+alpha</b> ], </p>
// <p>- if apsis = "<b>apo</b>" (apoapsis): [ <b>pi - alpha</b>, <b>pi + alpha</b> ]. </p>
// <inlinemediaobject><imageobject><imagedata fileref="lowThrustApsKep.png"/></imageobject></inlinemediaobject>
// <p></p>
// <p> The components of the thrust acceleration are given in a frame that can be: "qsw", "tnw" or "aps". 
// "aps" is defined by: x axis from center of central body to periapsis and z axis towards the orbit's 
// angular momentum. </p>
// <p></p>
// <p>The result of the computation is indicated by <b>res</b>: </p>
// <p>- "<b>dkepdt</b>" : Orbital elements average time derivatives </p>
// <p>- "<b>dkep</b>" : Increments on the orbital elements </p>
// <p> The average time derivatives are by definition the increments on the orbital elements 
// divided by one orbital period. </p>
// <p></p>
// <p> The argument opts is a structure that defines computation options: </p> 
// <p>- integr_meth: "int" (default) or "sq". "sq" means that some internal 
// calculations are approximated (integrals computed by squaring), but are also
// faster. </p> 
// <p></p>
// </listitem>
// <listitem>
// <p><b>Notes</b>:</p>
// <p>- Most of the formulas used come from Pollard's paper (see reference). Some results have 
// been added: derivatives for the mean anomaly, or for the case the acceleration is parallel to the 
// radius vector. </p>
// <p>- The results are representative only if the thust acceleration is small enough. </p>
// <p>- %nan is returned if the result is undefined or not yet coded. That's in particular the case if
// acc_frame is "tnw" and the "n" component is not 0.  </p>
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// kep : Keplerian orbital elements [m, rad]. (6xN or 6x1)
// acc : Thust acceleration (in cartesian coordinates) in the specified frame [m/s^2]. (3xN or 3x1)
// acc_frame : (string) Frame in which the acceleration is defined : "qsw", "tnw" or "aps". (1x1)
// alpha : Angle (true anomaly) defining the thrust arc [rad]. (1xN or 1x1)
// apsis : (string) Center of thrust arc : "per" or "apo". (1xN or 1x1)
// res : (string) Computed result: "dkepdt" or "dkep". (1x1) 
// opts : (structure, optional) Computation options (see above). Default: empty structure. 
// mu : (optional) Gravitational constant. [m^3/s^2]. Default value is %CL_mu
// result : Orbital elements time derivatives [m/s, rad/s] or increments [m, rad]. (6xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// Simplified Analysis of Low-Thrust Orbital Maneuvers, J. E. Pollard, The Aerospace Corporation, 15 August 2000.  
// 
// Examples
// kep = [10000.e3; 0.2; 1; 0; 0; 0]; // m, rad
// acc = [0; 1; 0] * 1.e-4; // m/s^2
// acc_frame = "aps";
// apsis = "per"; // around periapsis
// alpha = %pi/4; // => between -alpha and alpha (true anomaly)
// dkepdt = CL_man_lowThrustApsKep(kep, acc, acc_frame, alpha, apsis, "dkepdt")
//

// -----------------------------------------------------
// Declarations:
// -----------------------------------------------------

// Internal function (Pollard equations + extensions) 
// kep: Keplerian orbital elements (6xN)
// f: acceleration component (1xN)
// alphaE: thrusting interval = [-alphaE, alphaE] (eccentric anomaly). alphaE >= 0 (1xN)
// fcase: case number (frame + component) 
// sigma: -1 => periapsis, 1 => apoapsis (1xN)   
// integr_opt: option for calculation of integrals: 1 => "exact" 0 => "approx"
function [dkepdt] = pollard(kep, f, alphaE, fcase, sigma, integr_opt, mu)

  // integrals needed for case 2
  function [val1, val2] = pollard_calcIntegrals(ecc, alphaE, sigma, integr_opt)
  
    // functions to be integrated 
    // note: eccentricity = global variable (= ECC)            
    function [y] = pollard_fct1(E)
      y = sqrt(1 - ECC^2 * (cos(E))^2); 
    endfunction
    
    function [y] = pollard_fct2(E)
      y = cos(E) * (1 - ECC * cos(E)) / sqrt(1 - ECC^2 * (cos(E))^2); 
    endfunction
      
    if (integr_opt == 0)
      // integrals using squaring (CL_mean)
      
      // integration interval: [0, alphaE] or [%pi, %pi + alphaE]
      interv = [zeros(alphaE); alphaE] + repmat(%pi * (sigma+1)/2, 2, 1);  
      
      // Number of points for the squaring (~proportional to interval length) 
      n = 4 * 12 + 1; 

      mat_E = CL_intervLinspace(interv, n);
      mat_ecc = repmat(ecc, n, 1);
      
      // Integration method for CL_mean function
      meth = "boole";

      Y = sqrt(1 - mat_ecc.^2 .* (cos(mat_E)).^2);
      val1 = CL_mean(Y, meth, "r") .* alphaE;

      // H function
      Y = cos(mat_E) .* (1 - mat_ecc .* cos(mat_E)) ./ sqrt(1 - mat_ecc.^2 .* (cos(mat_E)).^2);
      val2 = CL_mean(Y, meth, "r") .* alphaE;
      
    else
      // integrals using "intg"

      val1 = %nan * ones(alphaE); 
      val2 = %nan * ones(alphaE); 
      
      E1 = %pi * (sigma + 1) / 2;
      E2 = %pi * (sigma + 1) / 2 + alphaE; 
      K = find(~isnan(ecc)); 
      
      for (k = K)
        ECC = ecc(k); 
        val1(k) = intg(E1(k), E2(k), pollard_fct1); 
        val2(k) = intg(E1(k), E2(k), pollard_fct2);
      end
    end
    
  endfunction 

  // check hyperbolic/parabolic orbit
  I = find(kep(2,:) >= 1); 
  kep(:,I) = %nan; 
  
  sma = kep(1,:);
  ecc = kep(2,:);
  inc = kep(3,:);
  pom = kep(4,:); // Arg of periapsis
  gom = kep(5,:);
  anm = kep(6,:); // Mean anomaly

  G = (2 * sigma .* sin(alphaE) .* (1 + ecc.^2) + 3 * ecc .* alphaE + ...
       ecc .* cos(alphaE) .* sin(alphaE) ) ./ sqrt(1 - ecc.^2);
 
  // eccx = ecc except if ecc = 0 (%nan)
  eccx = ecc; 
  eccx(eccx == 0) = %nan; 
  
  // sini = sin(inc) or %nan
  sini = sin(inc);
  sini(sini == 0) = %nan;

  // other constants
  C = sqrt(sma / mu);
  eta = sqrt(1 - ecc.^2); 

  dkepdt = zeros(kep); 
  
  select fcase
  
    // frame = "qsw" <=> Pollard case 1
    
    case 11 then
      // extension to pollard: q axis 
      dkepdt(4,:) = (1 / %pi) * f .* C .* (eta ./ eccx) .* (alphaE .* ecc + sigma .* sin(alphaE));
      dkepdt(6,:) = (1 / %pi) * f .* C .* (1 ./ eccx) .* (-3 * ecc .* alphaE - sigma .* ...
                      (1 + 3 * ecc.^2) .* sin(alphaE) - ecc.^3 .* sin(2 * alphaE) / 2); 
      
    case 12 then
      dkepdt(1,:) = (2 / %pi ) .* f .* alphaE .* sma .* C .* eta;
      dkepdt(2,:) = (1 / (2 * %pi)) .* (-f) .* ...
                C .* eta .* ( 4 * sigma .* sin(alphaE) + ...
                3 * ecc .* alphaE + ecc .* cos(alphaE) .* sin(alphaE) );
                
    case 13 then
      dkepdt(3,:) = (1 / (2 * %pi)) * (-f) .* C .* cos(pom) .* G;
      dkepdt(4,:) = (1 / (2 * %pi)) * f .* C .* (cos(inc) ./ sini) .* sin(pom) .* G;
      dkepdt(5,:) = (1 / (2 * %pi)) * (-f) .* C .* sin(pom) .* G ./ sini;

    // frame = "tnw" <=> Pollard case 2
    
    case 21 then
      [Hsma, H] = pollard_calcIntegrals(ecc, alphaE, sigma, integr_opt); 
      dkepdt(1,:) = (2 / %pi) * f .* sma .* C .* Hsma;
      dkepdt(2,:) = (2 / %pi) * f .* C .* (1 - ecc.^2) .* H;
      
    case 22 then
      // NOT CODED
      dkepdt(4,:) = %nan;
      dkepdt(6,:) = %nan;

    case 23 then
      dkepdt(3,:) = (1 / (2 * %pi)) * (-f) .* C .* cos(pom) .* G;
      dkepdt(4,:) = (1 / (2 * %pi)) * f .* C .* (cos(inc) ./ sini) .* sin(pom) .* G;
      dkepdt(5,:) = (1 / (2 * %pi)) * (-f) .* C .* sin(pom) .* G ./ sini;
      
    // frame = "aps" <=> Pollard case 3 and 4
    
    case 31 then
      dkepdt(4,:) = (1 / (2 * %pi)) * C .* (f .* (eta ./ eccx) .* ...
                (-2 * sigma .* ecc .* sin(alphaE) - 3 * alphaE + ...
                 cos(alphaE) .* sin(alphaE)));
      // NOTE: danmdt: not in Pollard's paper
      dkepdt(6,:) = (1 / (2 * %pi)) * C .* (f ./ eccx) .* ...
                (3 * (ecc.^2 + 1) .* alphaE + 2 * sigma .* ecc .* (3 + ecc.^2) .* ...
                sin(alphaE) + (3 * ecc.^2 - 1) .* sin(2 * alphaE) / 2);
    
    case 32 then
      dkepdt(1,:) = (-2 * sigma / %pi ) .* f .* sin(alphaE) .* ...
                sma .* C .* eta;
      dkepdt(2,:) = (1 / (2 * %pi)) * f .* C .* eta .* ...
                (4 * sigma .* ecc .* sin(alphaE) + 3 * alphaE + ...
                 cos(alphaE) .* sin(alphaE));

    case 33 then
      dkepdt(3,:) = (1 / (2 * %pi)) * (-f) .* C .* cos(pom) .* G;
      dkepdt(4,:) = (1 / (2 * %pi)) * f .* C .* (cos(inc) ./ sini) .* sin(pom) .* G;
      dkepdt(5,:) = (1 / (2 * %pi)) * (-f) .* C .* sin(pom) .* G ./ sini;
      
  end
    
endfunction

// -----------------------------------------------------
// Code:
// -----------------------------------------------------

// Check number of input arguments (at least 5 expected)
if (argn(2) < 6 | argn(2) > 8)
  CL__error("Invalid number of input arguments"); 
end

// Check optional arguments
if (~exists("opts", "local")); opts = struct(); end
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end


// Check size of input arguments / resize
[kep, acc, alpha, apsis] = CL__checkInputs(kep, 6, acc, 3, alpha, 1, apsis, 1);

// Check kep argument
// Sma > 0
if (find(kep(1,:) <= 0) <> []) 
  CL__error("Invalid orbital elements. Semi-major axis must be > 0."); 
end
// Ecc >= 0
if (find(kep(2,:) < 0) <> [])
  CL__error("Invalid orbital elements. Eccentricity must not be negative."); 
end
// Inc > 0 and < pi
if (find(kep(3,:) < 0) <> [] | find(kep(3,:) > %pi + 10.0*%eps) <> []) 
  CL__error("Invalid orbital elements. Inclination must be within [0, pi]."); 
end

// Check acc_frame argument
if (acc_frame == [] | typeof(acc_frame) <> "string" | size(acc_frame,"*") <> 1)
  CL__error("Invalid type or size for argument acc_frame"); 
end
FRAME_NAMES = ["qsw", "tnw", "aps"]; 
if (CL_setdiff(acc_frame, FRAME_NAMES) <> [])
  CL__error("Invalid value for argument acc_frame"); 
end  

// Check alpha argument
if (alpha == [])
  CL__error("Invalid value for argument alpha: should not be empty."); 
end
if (find(alpha < 0.0 | alpha > %pi + 10 * %eps) <> []) then
  CL__error("Invalid value for argument alpha: must be in [0, pi]."); 
end

//  Check apsis argument
if (apsis == [] | typeof(apsis) <> "string" | size(apsis,1) <> 1)
  CL__error("Invalid type or size for argument apsis"); 
end
APSIS_NAMES = ["per", "apo"]; 
if (find(apsis <> APSIS_NAMES(1) & apsis <> APSIS_NAMES(2)) <> [])
  CL__error("Invalid value for argument apsis"); 
end  

// Check res
if (res <> "dkepdt" & res <> "dkep")
  CL__error("Invalid value for argument res"); 
end

// Check opts
if (typeof(opts) <> "st")
  CL__error("Invalid type for argument opts."); 
end
OPTS_FIELDS = ["integr_meth"]; 
if (CL_setdiff(fieldnames(opts), OPTS_FIELDS) <> [])
  CL__error("Invalid field names in opts structure."); 
end
if (~isfield(opts, "integr_meth")); opts.integr_meth = "int"; end
if (opts.integr_meth <> "int" & opts.integr_meth <> "sq")
  CL__error("Invalid value for option opts.integr_meth"); 
end

if (acc == [])
  CL__error("Invalid value for argument acc: should not be empty."); 
end

if (kep == [])
  result = []; 
  return; // <== RETURN 
end

// Initializing  apsis_num (-1 = periapsis, 1 = apoapsis) 
// and converting alpha angle from true to eccentric anomaly
apsis_num = zeros(apsis); 
apsis_num(:) = -1;
alphaE = CL_kp_v2E(kep(2,:), alpha);

I = find(apsis == "apo"); 
if (I <> [])
  apsis_num(I) = +1;
  alphaE(I) = %pi - CL_kp_v2E(kep(2,I), %pi - alpha(I));
end

// calculation option 
integr_opt = 1; // integrals with intg
if (opts.integr_meth == "sq"); integr_opt = 0; end

// computation of derivatives (pollard + extensions)
// NB: derivatives are 0 (and not %nan) if acceleration is exactly 0 (even in cases: ecc=0 or inc=0) 
iframe = find(acc_frame == FRAME_NAMES); 

dkepdt = zeros(kep); 

for k = 1 : 3
  f = acc(k,:); 
  fcase = iframe * 10 + k; 
  I = find(abs(f) > 0 | isnan(f)); 
  if (I <> [])
    dkepdt(:,I) = dkepdt(:,I) + pollard(kep(:,I), f(I), alphaE(I), fcase, apsis_num(I), integr_opt, mu);
  end
end


// final results
if (res == "dkepdt")
  result = dkepdt;
else
  // T = orbit period (s)
  T = CL_kp_params("per", kep(1,:), mu=mu); 
  result = CL_dMult(dkepdt, T); 
end

endfunction
