//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [vel1, vel2] = CL_man_vLambert(pos1, pos2, delta_t, meth, direction, nrev, branch, mu)
// Lambert's problem
//
// Calling Sequence
// [vel1, vel2] = CL_man_vLambert(pos1, pos2, delta_t [[, meth, direction, nrev, branch, mu]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>This function solves Lambert's problem. </p>
// <p>It Computes the velocity vectors <b>vel1</b> and <b>vel2</b>, 
// given the position vectors <b>pos1</b> and <b>pos2</b>, 
// and the time of flight <b>delta_t</b>.</p>
// <p>Two methods are availalable and are
// specified using the argument <b>meth</b>: </p> 
// <p>- meth = 1 => less than one revolution only. </p>
// <p>- meth = 2 => any number of revolutions.
// Method based on Dario Izzo's algorithm (see reference). </p>
// <p>Set <b>direction</b> to 'pro' for a prograde orbit. 
// That is a counter-clockwise rotation around the Z axis
// (this is the prograde direction for the solar system). </p>
// <p>Set <b>direction</b> to 'retro' for a retrograde orbit. 
// That is a clockwise rotation around the Z axis
// (this is the retrograde direction for the solar system).</p>
// <p>If the selected method is 2, the number of revolutions <b>nrev</b>
// can be set to a value greater than 0, and in that case there are two
// solutions which can be selected using the argument <b>branch</b>.</p>
// <inlinemediaobject><imageobject><imagedata fileref="lambert_m0.gif"/></imageobject></inlinemediaobject>
// <inlinemediaobject><imageobject><imagedata fileref="lambert_m1.gif"/></imageobject></inlinemediaobject>
// </listitem>
// </itemizedlist>
//
// Parameters
// pos1 : Initial position vector [m] (3xN or 3x1)
// pos2 : Final position vector [m] (3xN or 3x1)
// delta_t : Time of flight from pos1 to pos2 [s] (1xN or 1x1)
// meth : (integer, optional) Selected method: 1 or 2. Default is 1. (1x1) 
// direction : (optional) 'pro' for prograde, 'retro' for retrograde. Default is 'pro' (1x1)
// nrev : (optional, only used if meth = 2) Number of full revolutions. Default is 0 (1x1)
// branch : (optional, only used if meth = 2) Selected branch: "left" or "right". Default is "right" (1x1)
// mu : (optional) Gravitational constant. Default is %CL_mu [m^3/s^2]
// vel1 : Initial velocity vector [m/s] (3xN)
// vel2 : Final velocity vector [m/s] (3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1) Orbital Mechanics for Engineering Students, H.D. Curtis, Section 5.3 and Appendix D.11 (algorithm 5.2)
// 2) Modern astrodynamics Fundamentals and perturbation methods, V Bond and M Allman, Chapter 6.2
// 3) Izzo, D. ESA Advanced Concepts team. Code used available in mga.m, on https://www.esa.int/gsp/ACT/projects/gtop. Last retrieved Oct, 2022.
//
// Examples
// dt = 1000; // seconds
// kep1 = [7000.e3; 0.1; 1; 0; 0; 0];
// // Same orbit, dt seconds later
// kep2 = CL_ex_kepler(0, kep1, dt/86400);
// [pos1, vel1] = CL_oe_kep2car(kep1);
// [pos2, vel2] = CL_oe_kep2car(kep2);
// 
// // CL_man_vLambert will find the keplerian arc from pos1 to pos2 in dt
// [vel1b, vel2b] = CL_man_vLambert(pos1, pos2, dt);
// // Velocities should be the same
// CL_norm(vel1b - vel1) + CL_norm(vel2 - vel2b)
// 
// // Find the required impulses to reach pos2 faster
// dt2 = 950; // seconds
// [vel1c, vel2c] = CL_man_vLambert(pos1, pos2, dt2);
// DV1 = vel1c - vel1;
// DV2 = vel2 - vel2c;
// // Total consumption
// CL_norm(DV1) + CL_norm(DV2)


// Default values 
if (~exists("meth","local")); meth = 1; end;
if (~exists("direction", "local")); direction = "pro"; end;
if (~exists("nrev", "local")); nrev = 0; end;
if (~exists("branch", "local")); branch = "right"; end;
if (~exists("mu", "local")); mu = CL__dataGetEnv("mu"); end

// Check values  
if (find(delta_t < 0) <> [])
  CL__error("Invalid value for ''delta_t''."); 
end

if (meth <> 1 & meth <> 2)
  CL__error("Invalid value for argument ''meth'' (1 or 2 expected)."); 
end

if (direction <> "pro" & direction <> "retro")
  CL__error("Invalid value for argument ''direction'' (""pro"" or ""retro"" expected)."); 
end

if (size(nrev, "*") <> 1) 
  CL__error("Invalid size for argument ''nrev'' (1x1 expected)."); 
end

if (nrev < 0 | nrev <> round(nrev)) 
  CL__error("Invalid value for argument ''nrev'' (integer non negative value expected)."); 
end

if (branch <> "left" & branch <> "right")
  CL__error("Invalid value for argument ''direction'' (""pro"" or ""retro"" expected)."); 
end


if (branch == "right")
  leftbranch = %f;
else
  leftbranch = %t;
end

if (meth == 1) 
  [vel1, vel2] = CL__man_lambert1(pos1, pos2, delta_t, direction, mu); 
else
  [vel1, vel2] = CL__man_lambert2(pos1, pos2, delta_t, direction, nrev, leftbranch, mu); 
end 

endfunction



