//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [params] = CL_stela_params(enabled_forces)
// Model parameters for STELA
//
// Calling Sequence
// [params] = CL_stela_params([[enabled_forces]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>Generates a structure of model parameters for STELA. </p>
// <p>Optional parameters are not included. </p>
// <p></p>
// <p>The argument <b>enabled_forces</b> can take the following values: </p>
// <p>- "all": all forces are enabled. (default) </p>
// <p>- "none": all forces are disabled. </p>
// <p></p>
// </listitem>
// <listitem>
// <p>See the <link linkend="STELA">STELA</link> page for more details.</p>
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// enabled_forces: (String, optional) "all" or "none". Default is "all" (1x1)
//
// Authors
// CNES - DCT/SB/MS
// 
// Examples
// // -- Example 1
// // Get default values
// params = CL_stela_params();
// // then change some of them
// params.appAcc_refFrame = "ECI";
// // or add new ones
// params.reentryAltitude = 100.e3;
// params
//  
// // -- Example 2
// // Only central force + drag are enabled
// params = CL_stela_params("none");
// params.central_enabled = %t;
// params.drag_enabled = %t;
// params


// Defines default structure
function [params] = stela_params_init()

  params = struct();

  params.mass = 1000.0;

    // Gravitation - Earth 
  params.central_enabled = %t;

  params.zonal_enabled = %t;
  params.zonal_maxDeg = 7;

  params.tesseral_enabled = %t;
  params.tesseral_maxDeg = 7;
  params.tesseral_minPeriod = 5 * 86400;

  // Gravitation - Third body
  params.thirdBody_enabled = %t;
  params.thirdBody_bodies = ["Sun", "Moon"];

  // Atmospheric drag
  params.drag_enabled = %t;
  params.drag_coefType = "constant";
  params.drag_area = 10;
  params.drag_coef = 2.2;

  // Solar activity
  params.drag_solarActivityType = "constant";
  params.drag_solarActivityFlux = 140.0;
  params.drag_solarActivityAp = 15.0;

  // SRP
  params.srp_enabled = %t;
  params.srp_area = 10;
  params.srp_coef = 1.5;

  // Solid Tides
  // NB: solidTides_enabled is optional (false by default)
  params.solidTides_enabled = %t; 
  
  // Apparent acceleration / reference frame
  // NB: appAcc_enabled is optional (by default, true, but depends of the reference frame)
  params.appAcc_enabled = %t; 
  params.appAcc_refFrame = "ICRS";  

  // integrator
  params.integrator_step = 86400;

endfunction

// disable all forces 
function [params] = stela_params_disableAll(params)
  ALL_NAMES = ["central", "zonal", "tesseral", "thirdBody", "drag", "srp", "solidTides", "appAcc"]; 

  for name = ALL_NAMES 
    params(name + "_enabled") = %f; 
  end
  
endfunction


// check number of input arguments: must be 0
nbargs = argn(2); 
if (nbargs > 1)
  CL__error("Invalid number of input arguments (0 expected)"); 
end

if (~exists("enabled_forces", "local")); enabled_forces = "all", end; 

if (enabled_forces <> "all" & enabled_forces <> "none")
  CL__error("Invalid value for argument enabled_forces"); 
end

// default initialization 
params = stela_params_init(); 

if (enabled_forces == "none")
  params = stela_params_disableAll(params); 
end

endfunction


