//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.


function [str, desc] = CL_tle_format(tle, form, opts)
// Two-line string generation from TLE structure
//
// Calling Sequence
// [str, desc] = CL_tle_format(tle [[, form, opts]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>This function creates string from a TLE structure: </p>
// <p>- Classical two-line string if the format is "TLE", </p>
// <p>- JSON string if the format is "JSON".  </p>
// <p>- CSCSDS OMM XML string if the format is "OMM-XML".  </p>
// <p></p> 
// <p>The "desc" field in the "tle" structure is provided: </p>
// <p>- in the <b>desc</b> argument for "TLE" format, </p>
// <p>- in the "OBJECT_NAME" field for "JSON" and "OMM-XML" format. </p> 
// <p></p>
// <p>The <b>opts</b> structure enables passing additional parameters if the format 
// is OMM-XML (parameters that are not definéd in the input TLE structure). </p>
// <p>The possible fields are: </p>
// <p>- comment: (string) Comment in the metadata section (1x1). 
// Default: "Generated via CelestLab" (</p>
// <p>- originator: (string) Originator in the metadata section (1x1). Default: "CNES"</p>
// <p>- center_name: (string) Frame center (1x1). Default: "EARTH"</p>
// <p>- ref_frame: (string) Reference frame (1x1). Default: "TEME"</p>
// <p>- time_system: (string) Time system (1x1). Default: "UTC"</p>
// <p>- mean_element_theory: (string) Mean element theory (1x1). Default: "SGP4"</p>
// <p></p>  
// <p>The output string is generated even if the "status" value is not 0. </p>
// <p>If a value does not fit with the format, the slot in the string is filled with "#" characters (for TLE format only). </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// tle: TLE structure (size N) 
// form: (string, optional) Output format: "TLE", "JSON" or "OMM-XML". Default is "TLE" 
// opts: (struct) Additional parameters (see above) 
// str: (string) Standard two-line element (2xN), JSON string (1x1), or XML string (Px1)
// desc: (string) Description or empty string (1xN)
//
// Authors
// CNES - DCT/SB/MS
//
// See also
// CL_tle_parse
//
// Examples
// str = [..
// "1 00005U 58002B   00179.78495062  .00000023  00000-0  28098-4 0  4753"; ..
// "2 00005  34.2682 348.7242 1859667 331.7664  19.3264 10.82419157413667" ];
// tle = CL_tle_parse(str)
// str2 = CL_tle_format(tle)


// transforms cjd into epochyr, epoch_days
// NB: epochdays: from 0 jan 1950 0h. 
// Manages %nan
function [epochyr, epochdays] = tle_cjd2yd(cjd)
  epochyr = %nan * ones(cjd); 
  epochdays = %nan * ones(cjd); 

  I = find(~isnan(cjd)); 
  if (I <> [])
    // everything here has size I
    cjd = cjd(I); 
    cal = CL_dat_cjd2cal(cjd); 
    year = cal(1,:); 
    cjd1 = CL_dat_cal2cjd(year, 1, 1); 
    ep_yr = year - 2000; 
    J = find(ep_yr < 0); 
    if (J <> []); ep_yr(J) = ep_yr(J) + 100; end
    ep_days = cjd - cjd1 + 1; 
    epochyr(I) = ep_yr; 
    epochdays(I) = ep_days
  end
endfunction


function [str] = tle_toString(form, lg, varargin)
  N = size(varargin); 
  if (N == 1); str = sprintf(form + "\n", varargin(1)); 
  elseif (N == 2); str = sprintf(form + "\n", varargin(1), varargin(2)); 
  elseif (N == 3); str = sprintf(form + "\n", varargin(1), varargin(2), varargin(3)); 
  elseif (N == 4); str = sprintf(form + "\n", varargin(1), varargin(2), varargin(3), varargin(4)); 
  else CL__error("Too many arguments"); end

  // looks for %nan (real)
  nan = zeros(varargin(1)); 
  for i = 1 : length(varargin)
    I = find(typeof(varargin(i)) == "constant" & isnan(varargin(i))); 
    nan(I) = 1;  
  end
  I = find(length(str) <> lg | nan); 
  if (I <> [])
    s = strcat(repmat("#", 1, lg), ""); 
    s = repmat(s, length(I), 1); 
    str(I) = s; 
  end   
endfunction

// Get all values as strings (used for JSON or OMM formats)
// The keywords are OMM keywords (capitalized) 
function [s] = tle_getTleValues(tle)
  s = struct(); 
  s.OBJECT_NAME = format_val("%s", tle.desc); 
  s.OBJECT_ID = format_val("%s", tle.intldesg); 
  s.EPOCH = CL_dat_cal2str(CL_dat_cjd2cal(tle.epoch_cjd), form = "ISO", nd = 6); 
  s.MEAN_MOTION = format_val("%.15g", tle.n * 86400 / (2 * %pi)); 
  s.ECCENTRICITY = format_val("%.15g", tle.ecc);
  s.INCLINATION = format_val("%.15g", tle.inc * 180 / %pi);
  s.RA_OF_ASC_NODE = format_val("%.15g", tle.raan * 180 / %pi);
  s.ARG_OF_PERICENTER = format_val("%.15g", tle.argp * 180 / %pi);
  s.MEAN_ANOMALY = format_val("%.15g", tle.ma * 180 / %pi);
  s.EPHEMERIS_TYPE = format_val("%d", tle.ephtype);
  s.CLASSIFICATION_TYPE = format_val("%s", tle.classif); 
  s.NORAD_CAT_ID = format_val("%d", tle.satnum); 
  s.ELEMENT_SET_NO = format_val("%d", tle.elnum); 
  s.REV_AT_EPOCH = format_val("%d", tle.revnum); 
  s.BSTAR = format_val("%.15g", tle.bstar);
  s.MEAN_MOTION_DOT = format_val("%.15g", tle.ndot * 86400^2 / (%pi / 180));
  s.MEAN_MOTION_DDOT = format_val("%.15g", tle.nddot * 86400^3 / (%pi / 180));
endfunction 


// Case TLE format 
function [str, desc] = format_TLE(tle) 
  // Notes: 
  // Format: 
  // 1 NNNNNC NNNNNAAA NNNNN.NNNNNNNN +.NNNNNNNN +NNNNN-N +NNNNN-N N NNNNN
  // 2 NNNNN NNN.NNNN NNN.NNNN NNNNNNN NNN.NNNN NNN.NNNN NN.NNNNNNNNNNNNNN
  // 
  // To ensure that the total lenght is correct (69 characters, including checksum), 
  // each field is formatted individually (with the a guaranteed length), and the whole string is obtained by catenating all the fields. 


  // ----------
  // line 1
  // ----------
 
  // num1: 1-1
  s_num1 = tle_toString("%d", 1, ones(N,1)); 

  // satnum: 3-7
  s_satnum = tle_toString("%05d", 5, tle.satnum'); 

  // classif: 8-8
  s_classif = tle_toString("%s", 1, tle.classif'); 

  // international designator: 10-17
  s_intldesg = tle_toString("%-8s", 8, tle.intldesg'); 

  // epoch: 19-32 -> YYDDD.DDDDDDDD
  [epochyr, epochdays] = tle_cjd2yd(tle.epoch_cjd); 
  epochdays_i = floor(epochdays); 
  epochdays_f = epochdays-floor(epochdays); 

  s_epoch = tle_toString("%02d%03d.%08d", 14, epochyr', epochdays_i', round(epochdays_f'*1.e8)); 

  // ndot/2 (rev/day^2): 34-43 -> S.NNNNNNNN (S = sign)
  ndot = tle.ndot * (86400^2) * (180/%pi); 
  s_sgn = repmat(" ", 1, N); 
  I = find(ndot <= 0); 
  s_sgn(I) = "-"; 
  s_ndot = tle_toString("%1s.%08d", 10, s_sgn', round(abs(ndot)'*1.e8)); 

  // nddot/6 (rev/day^3): 45-52 -> S[.]NNNNNSN ([] = implicit)
  nddot = tle.nddot * (86400^3) * (180/%pi); 
  nexp = zeros(nddot); 
  I = find(abs(nddot) > 0); 
  nexp(I) = int(log10(abs(nexp(I)))); // rounds towards 0
  nddot = nddot ./ (10 .^ nexp); 
  s_sgn = repmat(" ", 1, N); 
  I = find(nddot < 0); 
  s_sgn(I) = "-"; 
  s_sgnexp = repmat(" ", 1, N); 
  I = find(nexp <= 0); 
  s_sgnexp(I) = "-"; 

  s_nddot = tle_toString("%1s%05d%1s%1d", 8, s_sgn', round(abs(nddot'))*1.e5, s_sgnexp', abs(nexp)'); 

  // bstar (unit=1/Earth radii): 54-61 -> S[.]NNNNNSN
  bstar = tle.bstar; 
  ibexp = zeros(bstar); 
  I = find(abs(bstar) > 0); 
  ibexp(I) = int(log10(abs(bstar(I)))); // rounds towards 0
  bstar = bstar ./ (10 .^ ibexp); 
  s_sgn = repmat(" ", 1, N); 
  I = find(bstar < 0); 
  s_sgn(I) = "-"; 
  s_sgnexp = repmat(" ", 1, N); 
  I = find(ibexp <= 0); 
  s_sgnexp(I) = "-"; 
  s_bstar = tle_toString("%1s%05d%1s%1d", 8, s_sgn', round(abs(bstar')*1.e5), s_sgnexp', abs(ibexp)'); 

  // ephemeris type: 63-63
  s_ephtype = tle_toString("%1d", 1, tle.ephtype');

  // element number: 65-68
  s_elnum = tle_toString("%4d", 4, tle.elnum');

  // line1 (68 characters)
  str1 = s_num1 + " " + s_satnum + s_classif + " " + ..
       s_intldesg + " " + s_epoch + " " + ..
       s_ndot + " " + s_nddot + " " + s_bstar + " " + ..
       s_ephtype + " " + s_elnum; 

  // Compute checksum and add it to str1
  chks = CL__tle_checksum(str1); 
  str1 = str1 + tle_toString("%1d", 1, chks); 


  // ----------
  // line 2
  // ----------

  // num1: 1-1
  s_num2 = tle_toString("%1d", 1, 2*ones(N,1)); 

  // satnum: 3-7
  s_satnum = tle_toString("%05d", 5, tle.satnum'); 

  // inclination (rad): 9-16 -> NNN.NNNN
  inc = tle.inc * 180/%pi;
  s_inc = tle_toString("%8.4f", 8, inc');

  // RAAN (rad): 18-25 -> NNN.NNNN
  raan = CL_rMod(tle.raan * 180/%pi, 0, 360); // deg
  s_raan = tle_toString("%8.4f", 8, raan');

  // eccentricity: 27-33 ->  [.]NNNNNNN
  s_ecc = tle_toString("%07d", 7, round(tle.ecc' * 1.e7));

  // arg of perigee (deg): 35-42 -> NNN.NNNN
  argp = CL_rMod(tle.argp * 180/%pi, 0, 360); 
  s_argp = tle_toString("%8.4f", 8, argp');

  // mean anom (deg): 44-51 -> NNN.NNNN
  anm = CL_rMod(tle.ma * 180/%pi, 0, 360); 
  s_anm = tle_toString("%8.4f", 8, anm');

  // mean motion (rev/day): 53-63 -> NN.NNNNNNNN
  n = tle.n * 86400/(2*%pi); 
  s_n = tle_toString("%11.8f", 11, n');

  // revolution number: 64-68  
  s_revnum = tle_toString("%05d", 5, tle.revnum');

  // line2 (68 characters)
  str2 = s_num2 + " " + s_satnum + " " + s_inc + " " + ..
       s_raan + " " + s_ecc + " " + s_argp + " " + ..
       s_anm + " " + s_n + s_revnum; 

  // Compute checksum and add it to str2
  chks = CL__tle_checksum(str2); 
  str2 = str2 + tle_toString("%1d", 1, chks); 


  str = [str1, str2]'; 

  // ----------
  // desc
  // ----------

  desc = tle.desc; 

endfunction 


// Case JSON format 
function [str, desc] = format_JSON(tle)
  function [str] = format_val(form, val); 
    str = msprintf(form + "\n", val')';
  endfunction 

  // check Scilab version (existence of "fromJSON")  
  if (CL__scilabVersion() < 610)
    CL__error("Feature not available for this version of Scilab (6.1 or more expected)");   
  end 

  // format all values at once (spacetrack order) 
  tle_values = tle_getTleValues(tle); 

  // format each element individually 
  N = size(tle); 
  str = emptystr(N); 

  for k = 1 : N
    s = struct(); 
    for name = fieldnames(tle_values)'
      values = tle_values(name); 
      s(name) = values(k); 
    end  
    str(k) = toJSON(s); 
  end 

  // patch everything together 
  str = "[" + strcat(str, ",") + "]"; 
  desc = []; 

endfunction

// Case OMM-XML format 
function [str, desc] = format_OMM_XML(tle, opts)
  function [str] = format_val(form, val); 
    str = msprintf(form + "\n", val')';
  endfunction 

  // Get all values as string  
  tle_values = tle_getTleValues(tle); 

  // format each element individually 
  N = size(tle); 
 
  // 1st line 
  str = "<ndm xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xsi:noNamespaceSchemaLocation=""https://sanaregistry.org/r/ndmxml_unqualified/ndmxml-3.0.0-master-3.0.xsd"">"; 
  
  // creation date (UTC) 
  creation_date = CL_dat_cal2str(CL_dat_now("cal"), form = "ISO", nd = 0); 

  for k = 1 : N 
    // omm block start
    omm_start = "<omm id=""CCSDS_OMM_VERS"" version=""2.0"">"; 
  
    // header
    header = msprintf("<header>\n" ...
      + "<COMMENT>%s</COMMENT>\n" ...
      + "<CREATION_DATE>%s</CREATION_DATE>\n" ...
      + "<ORIGINATOR>%s</ORIGINATOR>\n" ... 
      + "</header>", opts.comment, creation_date, opts.originator); 

    // metadata block 
    metadata = msprintf("<metadata>\n" ...
      + "<OBJECT_NAME>%s</OBJECT_NAME>\n" ... 
      + "<OBJECT_ID>%s</OBJECT_ID>\n" ... 
      + "<CENTER_NAME>%s</CENTER_NAME>\n" ...
      + "<REF_FRAME>%s</REF_FRAME>\n" ...
      + "<TIME_SYSTEM>%s</TIME_SYSTEM>\n" ... 
      + "<MEAN_ELEMENT_THEORY>%s</MEAN_ELEMENT_THEORY>\n" ...
      + "</metadata>", ...  
        tle_values.OBJECT_NAME(k), tle_values.OBJECT_ID(k), ... 
        opts.center_name, opts.ref_frame, opts.time_system, ...
        opts.mean_element_theory); 

    // mean elements block
    mean_elem = msprintf("<meanElements>\n" ...
      + "<EPOCH>%s</EPOCH>\n" ... 
      + "<MEAN_MOTION>%s</MEAN_MOTION>\n" ... 
      + "<ECCENTRICITY>%s</ECCENTRICITY>\n" ... 
      + "<INCLINATION>%s</INCLINATION>\n" ...
      + "<RA_OF_ASC_NODE>%s</RA_OF_ASC_NODE>\n" ...
      + "<ARG_OF_PERICENTER>%s</ARG_OF_PERICENTER>\n" ... 
      + "<MEAN_ANOMALY>%s</MEAN_ANOMALY>\n" ...
      + "</meanElements>", ...  
        tle_values.EPOCH(k), tle_values.MEAN_MOTION(k), ... 
        tle_values.ECCENTRICITY(k), tle_values.INCLINATION(k), ... 
        tle_values.RA_OF_ASC_NODE(k), tle_values.ARG_OF_PERICENTER(k), ... 
        tle_values.MEAN_ANOMALY(k)); 

    // tle parameters block 
    tle_param = msprintf("<tleParameters>\n" ...
      + "<EPHEMERIS_TYPE>%s</EPHEMERIS_TYPE>\n" ... 
      + "<CLASSIFICATION_TYPE>%s</CLASSIFICATION_TYPE>\n" ... 
      + "<NORAD_CAT_ID>%s</NORAD_CAT_ID>\n" ... 
      + "<ELEMENT_SET_NO>%s</ELEMENT_SET_NO>\n" ...
      + "<REV_AT_EPOCH>%s</REV_AT_EPOCH>\n" ...
      + "<BSTAR>%s</BSTAR>\n" ... 
      + "<MEAN_MOTION_DOT>%s</MEAN_MOTION_DOT>\n" ...
      + "<MEAN_MOTION_DDOT>%s</MEAN_MOTION_DDOT>\n" ...
      + "</tleParameters>", ...  
        tle_values.EPHEMERIS_TYPE(k), tle_values.CLASSIFICATION_TYPE(k), ... 
        tle_values.NORAD_CAT_ID(k), tle_values.ELEMENT_SET_NO(k), ... 
        tle_values.REV_AT_EPOCH(k), tle_values.BSTAR(k), ... 
        tle_values.MEAN_MOTION_DOT(k), tle_values.MEAN_MOTION_DDOT(k)); 
        

    omm_block = [omm_start; header; "<body>"; "<segment>"; metadata; "<data>"; ...    
         mean_elem; tle_param; "</data>"; "</segment>"; "</body>"; "</omm>"]; 
         
    str = [str; omm_block]; 
      
  end 
  
  str = [str; "</ndm>"]; 
    
  desc = []; 
  
endfunction 
  

if (~exists("form", "local"))
  form = "TLE"; 
end 

// check fields in "opts" structure
if (~exists("opts", "local"))
  opts = struct(); 
end 
if (~isfield(opts, "comment")); opts.comment = "Generated via CelestLab"; end 
if (~isfield(opts, "originator")); opts.originator = "CNES"; end 
if (~isfield(opts, "center_name")); opts.center_name = "EARTH"; end 
if (~isfield(opts, "ref_frame")); opts.ref_frame = "TEME"; end 
if (~isfield(opts, "time_system")); opts.time_system = "UTC"; end 
if (~isfield(opts, "mean_element_theory")); opts.mean_element_theory = "SGP4"; end 


if (form <> "TLE" & form <> "JSON" & form <> "OMM-XML")
  CL__error("Invalid format: TLE, JSON or OMM-XML expected"); 
end 

// check structure is valid
CL__tle_checkValid(tle); 

str = []; 
desc = []; 

// N = number of TLEs in struct
N = size(tle); 
if (N == 0)
  return; 
end

if (form == "JSON")
  [str, desc] = format_JSON(tle); 

elseif (form == "OMM-XML")
  [str, desc] = format_OMM_XML(tle, opts); 

else 
  [str, desc] = format_TLE(tle); 

end 
  

endfunction

