//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [tle] = CL_tle_load(fnames)
// Loads TLEs from files 
//
// Calling Sequence
// tle = CL_tle_load(fnames)
//
// Description
// <itemizedlist>
// <listitem>
// <p>Loads two-line elements from files. </p>
// <p>The files may contain: </p>
// <p>- Classical TLEs on 2 lines: line1 and line2: TLE1, line3 and line4: TLE2, etc... </p>
// <p>- Classical TLEs on 3 lines (called "3LEs"): line1, line2, line3: TLE1, line4, line5, line6: TLE2, 
// etc... Line1, line4, etc... contain the respective satellite names.  </p>
// <p>- TLEs given as JSON format. </p>
// <p>- TLEs given as XML format (CCSDS OMM). </p>
// <p></p>
// <p>The file names are considered in sequence. </p>
// <p>The status field is set to a non-zero value if parsing errors occur (see <link linkend="CL_tle_parse">CL_tle_parse</link>). </p>
// <p></p> 
// </listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p>The spaces at the beginning or end of a line are ignored. </p> 
// <p>Empty lines are removed. </p> 
// <p>For classical "TLEs" or "3LEs", for the files to be read correctly, one of the following conditions should be true: </p>
// <p>-No description line is present in the file </p>
// <p>-The description line does not begin with "1" or "2" or its length is not 69 </p>
// <p></p> 
// </listitem>
// </itemizedlist>
//
// Parameters
// fnames: (string) File names or paths (1xP)
// tle: Structure containing all the TLEs present in the files (size N)
//
// Authors
// CNES - DCT/SB/MS
//
// See also
// CL_tle_parse
// CL_tle_new
//
// Examples
// fnames = CL_path("tle_examples.txt", CL_home(), "all"); 
// tle = CL_tle_load(fnames); 
// // show 2nd element of TLE structure
// tle(2)
// // show all eccentricities 
// tle.ecc
// // select TLE indices such that eccentricity is more than 0.01
// I = find(tle.ecc > 0.01); 
// tle2 = tle(I)


// internal function: process a TLE/3LE string  
// M: string (Nx1)
function [tle] = process_TLE(M)
  LEN = 69; 

  // remove empty lines and unnecessary spaces 
  str = stripblanks(M, %t);
  I = find(M == ""); 
  if (I <> [])
    M(I) = []; 
  end 

  if (M == [])
    tle = CL__tle_create(); 
    return; 
  end

  cond1 = (part(M,1) == "1" & length(M) == LEN); 
  cond2 = (part(M,1) == "2" & length(M) == LEN); 

  I1 = find(cond1); // line 1
  I2 = find(cond2); // line 2

  if (length(I1) <> length(I2))
    CL__error("Invalid TLE file"); 
  end

  if (find(I2-I1 <> 1) <> [])
    CL__error("Invalid TLE file"); 
  end

  str = [M(I1), M(I2)]'; 

  // Line with description: above TLE line 1
  // except if it is a data line (that begins with 1 or 2 and is  composed of 69 characters)
  Idesc = max(I1-1,1); 
  desc = M(Idesc)'; 
  I = find((part(desc,1) == "1" | part(desc,1) == "2") & length(desc) == LEN); 
  if (I <> [])
    desc(I) = ""; 
  end

  tle = CL_tle_parse(str, desc); 

endfunction


// internal function: process a JSON string  
// M: string (Nx1)
function [tle] = process_JSON(M)

  tle = CL_tle_parse(M, []); 

endfunction


// internal function: process a CSSDS OMM XML string  
// M: string (Nx1)
function [tle] = process_OMM_XML(M)

  tle = CL_tle_parse(M, []); 

endfunction


// empty by default
tle = CL__tle_create(); 

if (fnames == [])
  return;  
end

if (typeof(fnames) <> "string")
  CL__error("Wrong type for input argument ''fnames''"); 
end

for k = 1 : size(fnames,2)
  fname = fnames(k); 

  if (~isfile(fname))
    CL__error("File not found: " + fname); 
  end
  
  // load file into string 
  str = mgetl(fname); 

  // guess format
  form = CL__tle_guessFormat(str); 
  
  if (form == "JSON"); 
    // assumes JSON format 
    tle1 = process_JSON(str); 
  
  elseif (form == "OMM-XML"); 
    // assumes OMM XML format 
    tle1 = process_OMM_XML(str); 
  
  else 
    // assumes TLE/3LE format 
    tle1 = process_TLE(str); 
  end 
  
  // add to previously read TLEs
  tle = [tle, tle1]; 

end

endfunction
