//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.


function [tle] = CL_tle_parse(str, desc)
// TLE structure from string
//
// Calling Sequence
// tle = CL_tle_parse(str [[, desc]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>This function creates a TLE structure from two-line elements given as a two-line string, or 
// a string containing JSON or XML (CCSDS OMM) format.</p>
// <p></p>
// <p>The optional argument <b>desc</b> may be used to pass additional information (name of object...). 
// if the format is JSON or XML, the "OBJECT_NAME" field is used by default.  </p>
// <p>The created structure contains the same information as initially present in the string: there is no change of reference frame or time scale. </p>
// <p></p>
// <p>In case of parsing errors (unexpected characters encountered, bad checksum value(s)...), the status field in the TLE structure is set to a non-zero value:  </p>
// <p>-1: Error while trying to get the value of some field. </p>
// <p>-2: Invalid checksum for the first or second line (for "classical" TLEs only). </p>
// <p></p>
// <p><b>Note</b>:</p>
// <p>- The checksum checking can be by-passed by removing the checksum characters in the two-line strings or by replacing them by an non-numeric characters ("classical" TLEs only). </p>
// <p>- XML and JSON formats are supposed to be valid. The validity of the format is not checked.</p>
// </listitem>
// </itemizedlist>
//
// Parameters
// str: (string) String containing two-line elements (variable format) (2xN for 2-line elements, 1x1 or Px1 for JSON format, 1x1 or Px1 for XML format)
// desc: (string, optional) TLE Descriptions (i.e. objects names). May be empty (1xN)
// tle: TLE structure containing all the (decoded) two-line elements (size N)
//
// Authors
// CNES - DCT/SB/MS
//
// See also
// CL_tle_load
// CL_tle_new
//
// Examples
// str = [..
// "1 00005U 58002B   00179.78495062  .00000023  00000-0  28098-4 0  4753"; ..
// "2 00005  34.2682 348.7242 1859667 331.7664  19.3264 10.82419157413667" ];
// tle = CL_tle_parse(str)


// ----------------------------
// string or double to double 
// returns %nan if invalid string is given
// val (input): Nx1 or 1xN: string or double 
// val (output): Nx1 or 1xN: double (may be %nan)
// ----------------------------
function [val] = tle_convtod(val)
  if (typeof(val) == "string") 
    str = stripblanks(val, %t);
    // c: character that should not be present in the string
    c = ascii(1); 
    [val, endc] = strtod(str + c); 
    I = find(endc <> c); 
    val(I) = %nan; 
  // else do nothing: val assumed double 
  end 
endfunction

// remove spaces at beginning/end (if string) 
function [str] = tle_strip(str)
  if (typeof(str) == "string") 
    str = stripblanks(str, %t);
    // else do nothing: val assumed double 
  end 
endfunction


// ----------------------------
// x: NxP
// ----------------------------
// not valid if one element is %nan
function [valid] = tle_validp(x)
  valid = ~isnan(sum(x, "c")); 
endfunction


// Case original TLE (2 lines) 
// str: string 2xN (not empty) 
// desc: string 1xN (description)
function [tle] = parse_TLE(str, desc)  

  if (size(str,1) <> 2)
    CL__error("Invalid TLE string"); 
  end

  if (desc == [])
    desc = emptystr(str(1,:)); 
  end 

  if (size(desc, 2) <> size(str,2))
    CL__error("Invalid arguments sizes"); 
  end

  if (size(desc,1) <> 1)
    CL__error("Invalid description string"); 
  end

  // column vectors
  line1 = str(1,:)'; 
  line2 = str(2,:)'; // column
  N = size(line1, 1); 
  desc = desc'; 

  // ----------
  // line 1
  // ----------
  num1 = tle_convtod(part(line1, 1));

  satnum1 = tle_convtod(part(line1, 3:7));

  classif = part(line1, 8); 

  intldesg = stripblanks(part(line1, 10:17)); 

  // epoch (year + decimal day of year)
  epochyr = tle_convtod(part(line1, 19:20)) + 2000;
  I = find(epochyr >= 2057); 
  epochyr(I) = epochyr(I) - 100 * ones(I)';
  epochdays = tle_convtod(part(line1, 21:32));

  // ndot/2 (rev/day^2 -> rad/s^2)
  ndot = tle_convtod(part(line1, 34:43)) * (%pi/180)/86400^2;

  // nddot/6 (rev/day^3 -> rad/s^3)
  nddot = tle_convtod(part(line1, 45:50));
  nexp = tle_convtod(part(line1, 51:52));
  nddot = nddot .* 10.^(nexp-5) * (%pi/180)/86400^3;

  // bstar (unit=1/Earth radii)
  bstar = tle_convtod(part(line1, 54:59));
  ibexp = tle_convtod(part(line1, 60:61));
  bstar = bstar .* 10.^(ibexp-5);

  // ephemeris type (0-7)
  ephtype = tle_convtod(part(line1, 63));

  // element number
  elnum = tle_convtod(part(line1, 65:68));

  // checksum: - <=> 1, letter, . <=> 0, numbers <=> value
  checksum1 = tle_convtod(part(line1, 69)); 
  valid1 = tle_validp([num1,satnum1,epochyr,epochdays,ndot,..
                    nddot,bstar,ephtype,elnum]); 

  // ----------
  // line 2
  // ----------
  num2 = tle_convtod(part(line2, 1));

  satnum2 = tle_convtod(part(line2, 3:7));

  // inclination (rad)
  inc = tle_convtod(part(line2, 9:16)) * (%pi/180);

  // right ascension of ascending node (rad)
  raan = tle_convtod(part(line2, 18:25)) * (%pi/180);

  // eccentricity
  ecc = tle_convtod(part(line2, 27:33)) * 1.e-7;

  // arg of perigee (rad) 
  argp = tle_convtod(part(line2, 35:42)) * (%pi/180);

  // anm (rad) 
  ma = tle_convtod(part(line2, 44:51)) * (%pi/180);

  // mean motion (rad/sec) 
  n = tle_convtod(part(line2, 53:63)) * 2*%pi / 86400;

  // revolution number  
  revnum = tle_convtod(part(line2, 64:68));

  checksum2 = tle_convtod(part(line2, 69)); 
  valid2 = tle_validp([num2,satnum2,inc,raan,ecc,..
                    argp,ma,n,revnum]); 

  // ----------
  // check
  // ----------
  status = zeros(N,1); 
  I = find(num1 <> 1 | num2 <> 2 | satnum1 <> satnum2 | ~valid1 | ~valid2);
  status(I) = -1; 

  // check checksum line 1
  I = find(status == 0 & CL__tle_checksum(part(line1, 1:68)) <> checksum1 & ~isnan(checksum1)); 
  status(I) = -2; 

  // check checksum line 2
  I = find(status == 0 & CL__tle_checksum(part(line2, 1:68)) <> checksum2 & ~isnan(checksum2)); 
  status(I) = -2; 


  // create structure

  // epoch from 1/1/1950 0h (CJD)
  epoch_cjd = %nan * ones(N,1); 
  I = find(~isnan(epochyr) & ~isnan(epochdays)); 
  if (I <> [])
    epoch_cjd(I) = (CL_dat_cal2cjd(epochyr(I)', 1, 1) + epochdays(I)' - 1)'; 
  end

  tle = CL__tle_create(satnum1', classif', intldesg', ephtype', ..
         elnum', epoch_cjd', revnum', ecc', inc', argp', ..
         raan', ma', n', ndot', nddot', bstar', desc', status');

endfunction  


// Case JSON format 
// str: string 1x1 or Px1
// desc: string 1xN or []  (N must match the number of elements) 
function  [tle] = parse_JSON(str, desc)  
    
  // check Scilab version (existence of "fromJSON")  
  if (CL__scilabVersion() < 610)
    CL__error("Feature not available for this version of Scilab (6.1 or more expected)");   
  end 
  
  json_struct = fromJSON(str); 
   
  N = size(json_struct, "*"); 
  
  if (desc <> [] & size(desc, "*") <> N)
    CL__error("Invalid size for desc argument"); 
  end 
    
  tle = CL_tle_new(N); 
    
  for (k = 1 : N)
    s = json_struct(k); 
        
    // get fields of interest and change units  
    // NOTE: some fields may be of type string or double 
    // depending on data source (see tle_convtod)     
    satnum = tle_convtod(s.NORAD_CAT_ID);
    classif = tle_strip(s.CLASSIFICATION_TYPE); 
    sat_name = tle_strip(s.OBJECT_NAME);  
    intldesg = tle_strip(s.OBJECT_ID);
    ephtype = tle_convtod(s.EPHEMERIS_TYPE);
    elnum = tle_convtod(s.ELEMENT_SET_NO);
    revnum = tle_convtod(s.REV_AT_EPOCH);
    ecc = tle_convtod(s.ECCENTRICITY);
    inc = tle_convtod(s.INCLINATION) * %pi / 180;
    argp = tle_convtod(s.ARG_OF_PERICENTER) * %pi / 180;
    raan = tle_convtod(s.RA_OF_ASC_NODE) * %pi / 180;
    ma = tle_convtod(s.MEAN_ANOMALY) * %pi / 180;
    bstar = tle_convtod(s.BSTAR);
    n = tle_convtod(s.MEAN_MOTION) * (2 * %pi) / 86400;
    ndot = tle_convtod(s.MEAN_MOTION_DOT) * (%pi/180) / 86400^2;
    nddot = tle_convtod(s.MEAN_MOTION_DDOT) * (%pi/180) / 86400^3;

    cal = CL_dat_str2cal(s.EPOCH);
    epoch_cjd = CL_dat_convert("cal", "cjd", cal);

    // Check nan
    valid = tle_validp([satnum, ephtype, elnum, ...
      epoch_cjd, revnum, ecc, inc, argp, raan, ...
      ma, n, ndot, nddot, bstar]);   
    
    status = 0; 
    if (~valid); 
      status = -1;
    end 

    // description: identical to satellite name if desc is []
    if (desc <> [])    
      descript = desc(k); 
    else 
      descript = sat_name; 
    end 
      
    tle(k) = CL__tle_create(satnum, classif, intldesg, ephtype, ..
       elnum, epoch_cjd, revnum, ecc, inc, argp, ..
       raan, ma, n, ndot, nddot, bstar, descript, status);
    
  end
  
endfunction


// Case CCSDS OMM XML format 
// str: string 1x1 or Px1
// desc: string 1xN or []  (N must match the number of elements) 
function  [tle] = parse_OMM_XML(str, desc)  
  
  // Get value from keyword in XML structure
  // name : string (1x1)
  // val: string (1xP)
  // NB: xml_struct: global variable 
  function [val] = getKW(name)
    x = xmlXPath(xml_struct, "//" + name); 
    val = matrix(x.content, 1, -1); 
  endfunction 
  
  // check Scilab version (existence of "xmlReadStr")
  if (CL__scilabVersion() < 540)
    CL__error("Feature not available for this version of Scilab (5.4 or more expected)");   
  end 
  
  xml_struct = xmlReadStr(str); 
  
  // use OBJECT_ID to determine the size 
  val = getKW("OBJECT_ID"); 
  N = size(val, "*");  
  
  if (desc <> [] & size(desc, "*") <> N)
    CL__error("Invalid size for desc argument"); 
  end 
    
  // each keyword in turn. Values are row vectors. 
  satnum = tle_convtod(getKW("NORAD_CAT_ID"));
  classif = tle_strip(getKW("CLASSIFICATION_TYPE")); 
  sat_name = tle_strip(getKW("OBJECT_NAME")); 
  intldesg = tle_strip(getKW("OBJECT_ID")); 
  ephtype = tle_convtod(getKW("EPHEMERIS_TYPE"));
  elnum = tle_convtod(getKW("ELEMENT_SET_NO"));
  revnum = tle_convtod(getKW("REV_AT_EPOCH"));
  ecc = tle_convtod(getKW("ECCENTRICITY"));
  inc = tle_convtod(getKW("INCLINATION")) * %pi / 180;
  argp = tle_convtod(getKW("ARG_OF_PERICENTER")) * %pi / 180;
  raan = tle_convtod(getKW("RA_OF_ASC_NODE")) * %pi / 180;
  ma = tle_convtod(getKW("MEAN_ANOMALY")) * %pi / 180;
  bstar = tle_convtod(getKW("BSTAR"));
  n = tle_convtod(getKW("MEAN_MOTION")) * 2 * %pi / 86400;
  ndot = tle_convtod(getKW("MEAN_MOTION_DOT")) * (%pi/180) / 86400^2;
  nddot = tle_convtod(getKW("MEAN_MOTION_DDOT")) * (%pi/180) / 86400^3;

  cal = CL_dat_str2cal(getKW("EPOCH"));
  epoch_cjd = CL_dat_convert("cal", "cjd", cal);
  
  // Check nan
  // each element (="TLE") corresponds to a row 
  valid = tle_validp([satnum; ephtype; elnum; ...
      epoch_cjd; revnum; ecc; inc; argp; raan; ...
      ma; n; ndot; nddot; bstar]');   

  status = zeros(1, N); 
  I = find(~valid); 
  status(I) = -1;
  
  if (desc <> [])    
    descript = desc(k); 
  else 
    descript = sat_name; 
  end 
  
  tle = CL__tle_create(satnum, classif, intldesg, ephtype, ..
     elnum, epoch_cjd, revnum, ecc, inc, argp, ..
     raan, ma, n, ndot, nddot, bstar, descript, status);
    
endfunction


if (~exists("desc", "local"))
  desc = []; 
end


// str == empty => returns empty TLE
if (str == [])
  tle = CL__tle_create(); 
  return; 
end


// tries to guess TLE format (2-lines or JSON)
form = CL__tle_guessFormat(str); 

if (form == "JSON"); 
  // assumes JSON format 
  tle = parse_JSON(str, tle_strip(desc)); 

elseif (form == "OMM-XML"); 
    // assumes CCSDS OMM XML format 
  tle = parse_OMM_XML(str, tle_strip(desc)); 
  
else 
  // supposes TLE format 
  tle = parse_TLE(str, tle_strip(desc)); 
end 

endfunction
