//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.


function CL_tle_write(tle, fpath, form, opts)
// TLE structure to file 
//
// Calling Sequence
// CL_tle_write(tle, fpath [[, form, opts]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>This function writes a TLE structure to a file: </p>
// <p>- Each TLE on two lines if the format is "TLE", </p>
// <p>- Each TLE on three lines if the format is "3LE". 
// First line is the description (= object name), </p>
// <p>- Using JSON syntax if the format is "JSON".  </p>
// <p>- Using CCSDS OMM XML syntax if the format is "OMM-XML".  </p>
// <p></p> 
// <p>The "desc" field in the "tle" structure is provided: </p>
// <p>- In the first (out of 3) line for "3LE" format, </p>
// <p>- in the "OBJECT_NAME" field for "JSON" and "OMM-XML" format. </p> 
// <p></p>
// <p>The <b>opts</b> structure enables passing additional parameters if the format 
// is OMM-XML (parameters that are not definéd in the input TLE structure). </p>
// <p>The possible fields are: </p>
// <p>- comment: (string) Comment in the metadata section (1x1). 
// Default: "Generated via CelestLab" </p>
// <p>- originator: (string) Originator in the metadata section (1x1). Default: "CNES"</p>
// <p>- center_name: (string) Frame center (1x1). Default: "EARTH"</p>
// <p>- ref_frame: (string) Reference frame (1x1). Default: "TEME"</p>
// <p>- time_system: (string) Time system (1x1). Default: "UTC"</p>
// <p>- mean_element_theory: (string) Mean element theory (1x1). Default: "SGP4"</p>
// <p></p>  
// <p>The output is generated even if the "status" value is not 0. </p>
// <p>If a value does not fit with the format, the slot in the string is filled with "#" characters (for TLE or 3LE formats only). </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// tle: TLE structure (size N) 
// fpath: (string) File name or path 
// form: (string, optional) Output format: "TLE", "3LE", "JSON" or "OMM-XML". Default is "3LE" 
// opts: (struct) Additional parameters (see above) 
//
// Authors
// CNES
//
// See also
// CL_tle_format
//
// Examples
// str = [..
// "1 00005U 58002B   00179.78495062  .00000023  00000-0  28098-4 0  4753"; ..
// "2 00005  34.2682 348.7242 1859667 331.7664  19.3264 10.82419157413667" ];
// tle = CL_tle_parse(str)
// CL_tle_write(tle, fullfile(TMPDIR, "example_tle.txt"), "TLE"); 

// check arguments 
if (argn(2) < 2)
  CL__error("Not enough arguments"); 
end 

if (~exists("form", "local"))
  form = "3LE"; 
end 

// check fields in "opts" structure
if (~exists("opts", "local"))
  opts = struct(); 
end 
if (~isfield(opts, "comment")); opts.comment = "Generated via CelestLab"; end 
if (~isfield(opts, "originator")); opts.originator = "CNES"; end 
if (~isfield(opts, "center_name")); opts.center_name = "EARTH"; end 
if (~isfield(opts, "ref_frame")); opts.ref_frame = "TEME"; end 
if (~isfield(opts, "time_system")); opts.time_system = "UTC"; end 
if (~isfield(opts, "mean_element_theory")); opts.mean_element_theory = "SGP4"; end 

if (form <> "TLE" & form <> "3LE" & form <> "JSON" & form <> "OMM-XML")
  CL__error("Invalid format: TLE, 3LE, JSON or OMM-XML expected"); 
end 

// format TLE 
form1 = form; 
if (form == "3LE"); form1 = "TLE"; end 

[str, desc] = CL_tle_format(tle, form1, opts); 

// for TLE, 3LE: get one column string 
if (form == "3LE")
  // insert "desc" line as first line
  str = matrix([desc; matrix(str, 2, -1)], -1, 1); 
else 
  str = matrix(str, -1, 1); 
end 

mputl(str, fpath);   

endfunction

