//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [C] = %s_m_s(A,B)
// Multiplication of 2 hypermatrices or hypermatrix x matrix 


function [C] = hm_m_hm(A,B)
// Multiplication of 2 hypermatrices. 
// A is considered as a set of matrices : A = [A1, A2, ...Ak] 
// size(A) = (N,P,K) 
// B is considered as a set of matrices : B = [B1, A2, ...Bk]
// size(B) =(P,Q,K) 
// Then C is defined as : 
// C = [A1*B1, A2*B2, ... Ak*Bk]
// size(C) =(N,Q,K) 


// Declarations:

// Code:

[N,P,K] = size(A);
[P2,Q,K2] = size(B);

if (K <> K2 | P <> P2)
  CL__error("Hypermatrices have incompatible sizes");
end


C = CL__hypermat([N, Q, K]); 

// Note : 
// we note Aik = A(i,:,k)(:), and Bjk = B(:,j,k)(:)
// (Aik and Bjk = column vectors of P elements)
// => A(i,:,:) contains IN THIS ORDER: Ai1; Ai2;... Aik
// which can be reformatted COLUMN BY COLUMN: 
// matrix(A(i,:,:),P,K) = [Ai1, Ai2... ]  -> size = PxK
// => B(:,j,:) contains IN THIS ORDER: Bj1; Bj2;... Bjk
// which can be reformatted COLUMN BY COLUMN:
// matrix(B(:,j,:),P,K) = [Bj1, Bj2,... ] -> size = PxK
// => easy to compute sum(Ai1 .* Bj1) = C(i,j,1), etc... 

for (i=1:N)
  for (j=1:Q)
    C(i,j,:) = sum(matrix(A(i,:,:),P,K) .* matrix(B(:,j,:),P,K), 'r');
  end
end


endfunction

function [C] = hm_m_s(A,B)

// A is considered as a set of matrices: 
// A = [A1, A2, ...Ak], k=1:K, with size of A = NxPxK
// B is considered as a set of (column) vectors : 
// B = [B1, B2, ...Bk], k=1:K with size of B = PxK
// Then C is defined by: 
// C = [A1*B1, A2*B2, ... Ak*Bk]; size(C) = (N,K) 
// special cases : 
// - B scalar => C = A * B(1,1)
// - B is a column vector => 
// B considered as [B,B,..., B] (K times)
//

// Declarations:

// Code:

[N,P,K] = size(A);
[P2,K2] = size(B);


// --- special case : B = [] 

if (P2 == 0 & K2 == 0)  
  // A = hypermatrix , B = []
  C = [];
  return;  // <== RETURN
end 

// --- special case : B = scalar 

if (P2 == 1 & K2 == 1)  
  // A = hypermatrix , B = scalar (1x1)
  C = zeros(A);
  C(:) = A(:) * B(1,1);
  return;  // <== RETURN
end 


// --- A = hypermatrix , B = matrix (or vector)

if (K2 == 1)
  B = B * ones(1, K); // B now has K columns
  K2 = K;
end

if (P2 <> P | K2 <> K)
  CL__error("Hypermatrix and matrix have incompatible sizes");
end

C = zeros(N,K);

// Note : 
// we note Aik = A(i,:,k)(:), and Bk = B(:,k)(:)
// (Aik and Bjk = column vectors of P elements)
// => A(i,:,:) contains IN THIS ORDER: Ai1; Ai2;... Aik
// which can be reformatted COLUMN BY COLUMN: 
// matrix(A(i,:,:),P,K) = [Ai1, Ai2... ]  -> size = PxK
// => B(:,:) contains IN THIS ORDER: B1; B2;... Bk
// which can be reformatted COLUMN BY COLUMN:
// matrix(B(:,:),P,K) = B = [B1, B2,... ] -> size = PxK
// => easy to compute sum(Ai1 .* B1) = C(i,1), etc... 

for (i = 1:N)
  C(i,:) = sum( matrix(A(i,:,:),P,K) .* B, 'r' );
end


endfunction
 

if (ndims(A) == 3)
  ndimsB = ndims(B); 
  
  if (ndimsB == 3)
    C = hm_m_hm(A,B); 
   
  elseif (ndimsB < 3)
    C = hm_m_s(A,B); 
    
  else 
    CL__error("Invalid hypermatrix sizes"); 
  end

else 
  CL__error("Invalid hypermatrix sizes"); 
  
end

endfunction








