//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [cmap] = CL_colormap(cmap_name, n)
// Colormap creation (interface).
//
// Calling Sequence
// cmap = CL_colormap([[cmap_name, n]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>Creation of colormap (array of RGB values).</p> 
// <p></p>
// <p>This function shall be used as follows: </p>
// <p>f = scf(); </p>
// <p>f.color_map = CL_colormap(...);  </p>
// <p></p>
// </listitem>
// <listitem>
// <b>Notes</b>: 
// <p>- This function has been introduced for compatibility reasons, as the 
// the names of colormap functions have changed as of Scilab 2024.1.0. </p>
// <p>- Valid colormap names are derived from the names of the corresponding scilab 
// functions without the "colormap" ending. For instance "jetcolormap" shall be refered 
// to as "jet", "rainbowcolormap" as "rainbow" and so on. </p>
// <p>- The allowed predefined names are Scilab version dependent. </p>
// <p>- A colormap name can also the name be a user-defined function with the same
// interface as Scilab colormap functions, that is: cmap = f(n), with cmap 
// a nx3 matrix containing values between 0 and 1. </p>
// </listitem></itemizedlist>
//
// Parameters
// cmap_name: (string, optional) name of the colormap function to be used. See Scilab help for the list of possible names. Default value is "jet". 
// n: (integer, optional) colormap size (number of colors), it should to be >= 3. Default value is 32.
// cmap: generated colormap (nx3).
//
// Authors
// CNES
//
// Examples
// cmap = CL_colormap()
// cmap = CL_colormap("jet", 32)
// cmap = CL_colormap("rainbow")

// default number of values
if (~exists("cmap_name", "local"))
  cmap_name = "jet";
end

// default number of values
if (~exists("n", "local"))
  n = 32;
end

// Check arguments type 
if (typeof(cmap_name) <> "string" | size(cmap_name, "*") <> 1)
  CL__error("Invalid type or size for argument cmap_name. ");
end

if (typeof(n) <> "constant" | size(n, "*") <> 1 | n <> round(n) | n < 0)
  CL__error("Invalid value for argument n. ");
end


// Manage backwards compatibility 
// For Scilab versions < 2024.1.0, the predefined function names 
// must end with "colormap"

if (CL_scilabVersion() >= 202410)
  str = "f =" + cmap_name + "; cmap = f(n)"; 
else 
  str = "f =" + cmap_name + "colormap" + "; cmap = f(n)"; 
end

err = execstr(str, "errcatch");

if (err <> 0) 
  CL__error("Invalid colormap function call."); 
end 

endfunction


