//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [ticks] = CL_g_ticks(locations, labels, form)
// Creation of a Scilab "ticks" structure
//
// Calling Sequence
// [ticks] = CL_g_ticks(locations [[, labels, form]])
//
// Description
// <itemizedlist>
// <listitem>
// <p>Returns a structure to be used for ticks initialization in an "axes" entity. </p> 
// <p><b>locations</b> gives the ticks locations (where the ticks are written). It may be empty. </p> 
// <p><b>labels</b> gives the corresponding ticks labels. By default the location values are printed using the 
// format given by <b>form</b>. </p>
// <p>The default format is "%.12g" except if all values are integers, in which case it is "%d". </p> 
// <p></p>
// </listitem>
// <listitem>
// <p><b>Remark</b>:</p>
// <p>The main objective for this function is to hide the underlying definition of a "ticks" structure. </p> 
// <p></p>
// </listitem>
// </itemizedlist>
//
// Parameters
// locations: Ticks locations. (1xN)
// labels: (string, optional) Ticks labels. (1xN)
// form: (string, optional) Format to be used if labels are not given. (1x1)
//
// Authors
// CNES - DCT/SB (AL)
//
// Examples
// scf();
// x = 0 : 10;
// plot(x, x);
// a = gca(); 
// a.x_ticks = CL_g_ticks(1 : 2 : 9); 
// a.y_ticks = CL_g_ticks([1, 3, 7], ["One", "Three", "Seven"]); 
// a.sub_ticks = [0,0]; 
// CL_g_stdaxes(a);


// Code:
nbargs_in = argn(2); 
if (nbargs_in < 1 | nbargs_in > 3)
  CL__error("Invalid number of input arguments"); 
end

if ((locations <> [] & typeof(locations) <> "constant") | size(locations, 1) > 1 | ...
    (typeof(locations) == "constant" & ~isreal(locations)))
  CL__error("Invalid size or type for argument locations"); 
end

if ~exists("form", "local")
  form = "%.12g";  
  if (find(locations <> round(locations)) == [])
    form = "%d"; 
  end
end

if (typeof(form) <> "string" | size(form, "*") <> 1)
  CL__error("Invalid size or type for argument form"); 
end

if ~exists("labels", "local")
  labels = []; 
  if (locations <> [])
    labels = msprintf(form + "\n", locations')'; 
  end
end

if ((labels <> [] & typeof(labels) <> "string") | size(labels, 1) > 1 | size(labels, 2) <> size(locations, 2))
  CL__error("Invalid size or type for argument labels"); 
end

ticks = tlist(["ticks", "locations", "labels"], locations, labels); 

endfunction
