//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [paths] = CL_path(names, directories, opt, err, hid, maxdepth, smode, ftype)
// File paths (search in selected directories) 
//
// Calling Sequence
// paths = CL_path(names [[, directories, opt, err, hid, maxdepth, smode, ftype]])
//
// Description
// <itemizedlist><listitem>
// <p>Looks for files in directories and returns their paths.</p>
// <p><b>names</b> can be a file (or directory) name or a string containing wildcards ("*"). </p>
// <p>If <b>directories</b> is omitted or is a white space (""): %CL_PATH is searched if it is defined, otherwise, 
// an error is raised. </p> 
// <p></p>
// <p>The argument <b>opt</b> specifies the possible search options: </p>
// <p>opt = "first" (or "std"): Returns the path of the first file found.  </p>
// <p>opt = "all": Returns the paths of all the files found. </p>
// <p>Default is "first".</p>
// <p></p>
// <p>The argument <b>smode</b> specifies the type of search: </p>
// <p>smode = "df": depth-first: look for names in directory1, directory1/subdir1, directory1/subdir1/subdir1 ..., 
// directory1/subdir1/subdir2, ...  directory1/subdir2, ..., directory2, ... </p>
// <p>smode = "bf": breadth-first: look for names in directory1, directory2 ..., then in all subdirectories of 
// directory1, directory2 ...,  then in all subdirectories of subdirectories of directory1, directory2..., etc... </p> 
// <p>Default is "bf".</p>
// <p></p>
// <p>The option <b>ftype</b> specifies the type of file to find: </p>
// <p>ftype = "f" : Files only.  </p>
// <p>ftype = "d" : Directories only. </p>
// <p>ftype = "any" : Any type. </p>
// <p>Default is "any".</p>
// <p></p>
// <p>If no file is found, the returned value is []. An error is raised if <b>err</b> == %t and opt == "first"). </p>
// <p>If <b>hid</b> is %t, hidden sub-directories (whose name begin with ".") are searched, otherwise 
// they're not. Only (sub) directories not explicitly mentioned in the calling sequence are concerned. </p>
// <p>The maximum depth for the search can be specified by <b>maxdepth</b>. It is 50 by default. 0 means that files 
// are looked-for in the specified directories only. </p>
// <p>If last part of a directory name (in "directories") is "*", the name of the directory  is replaced by the list 
// of all its sub-directories. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// names: (string) Name(s) of file(s) to be looked for (may contain wildcards). (1xN)
// directories: (string, optional) Directories to be searched. Default is "", meaning %CL_PATH if it exists (1xP) 
// opt: (string, optional) Search option: "first" (or "std") or "all". Default is "first" (1x1)
// err: (boolean, optional) %t is an error is raised if no file is found. Default is %t except if opt == "all" (1x1)
// hid: (boolean, optional) %t if hidden sub-directories are included in the search. Default is %f (1x1)
// maxdepth: (integer >=0, optional) Maximum depth for the search. Default is 50 (1x1)
// smode: (string, optional) Search smode: "df" or "bf". Default is "bf" (1x1)
// ftype: (string, optional) Type of files: "d" or "f" or "any". Default is "any" (1x1)
// paths: (string) File path(s) or [] if no file is found (1xQ)
//
// Authors
// CNES - DCT/SB
//
// Examples
// // Looks for "readme.txt" in CL_home() - returns the first file found 
// CL_path("readme.txt", CL_home())
//
// // Returns all "*.scd" file in "data" (deep search)
// CL_path("*.scd", fullfile(CL_home(), "data"), "all") 
//

// =============================================
// internal functions
// =============================================

// ---------------------------------------------
// file parts of a name: 
// fname = file part
// dname = directory part ("" if there is no directory part)
// ---------------------------------------------
function [fname, dname] = Fname(name)
  if (name == [])
    fname = []; 
	dname = []; 
  else
    [dname, f1, f2] = fileparts(name); 
    fname = f1 + f2;
    // remove separator at the end of the directory part if exists
    dname = pathconvert(dname, %f); 
  end
endfunction


// ---------------------------------------------
// Returns the list (1xN vector) of sub-directories
// d: name of 1 directory (supposed to exist)
// hid: %t if "hidden" directories are included. 
// ---------------------------------------------
function [subd] = Listsubdirs(d, hid)
  subd = []; 
  f = matrix(findfiles(d), 1, -1); 
  if (f <> [])
    subd = fullfile(d, f); 
	// select "not hidden" 
    if (~hid)
      subd = subd(part(f,1) <> "."); 
    end       
	// select directories only
    if (subd <> [])
	  subd = subd(isdir(subd)); 
	end
  end
endfunction


// ---------------------------------------------
// Looks for files in one directory and returns the paths
// name: (string) 1x1
// d: (string) directory (supposed to exist) 1x1 
// nres == 1 if only one result returned, 0 otherwise
// ---------------------------------------------
function [paths] = Filepath0(name, d, nres, ftype)
  paths = []; 
  f = findfiles(d, name); 
  if (f <> [])
    paths = fullfile(d, matrix(f, 1, -1)); 
  end
  if (paths <> [] & ftype <> "any")
    if (ftype == "d")
	  paths = paths(isdir(paths)); 
	else
	  paths = paths(isfile(paths)); 
	end 
  end
  if (paths <> [] & nres == 1)
    paths = paths(1); 
  end
endfunction


// ---------------------------------------------
// find a file in list of directories and returns the paths
// mode = depth-first
// fnames, dnames: (string) files to look-for 1xP 
// dirs: (string) 1xN
// nres == 1 if only one result returned, 0 otherwise
// maxdepth = 0 => only search dirs(1), dirs(2), ... 
// ---------------------------------------------
function [paths] = Filepath1_depthfirst(fnames, dnames, dirs, maxdepth, nres, hid, ftype, paths)
  if (dirs == [])
    // nothing to do
    return; 
  end 
  
  for (d = dirs)
    if (nres == 1 & paths <> [])
	  break; 
	end
	
    for (k = 1 : size(fnames, 2))
	  p = Filepath0(fnames(k), fullfile(d, dnames(k)), nres, ftype);
      paths = [paths, p];
      if (nres == 1 & paths <> [])
	    break; 
	  end 
	end 
	  
    // continue search if: 
    // - max depth not reached and
    // - nres == 0 | (nres == 1 & file not yet found)
    if (maxdepth > 0 & (nres == 0 | paths == []))
      subdirs = Listsubdirs(d, hid); 
      paths = Filepath1_depthfirst(fnames, dnames, subdirs, maxdepth-1, nres, hid, ftype, paths); 
	end
  end
  
endfunction


// ---------------------------------------------
// find a file in list of directories and returns the paths
// mode = breadth-first
// fnames, dnames: (string) files to look-for 1xP 
// dirs: (string) 1xN
// nres == 1 if only one result returned, 0 otherwise
// maxdepth = 0 => only search dirs(1), dirs(2), ... 
// ---------------------------------------------
function [paths] = Filepath1_breadthfirst(fnames, dnames, dirs, maxdepth, nres, hid, ftype, paths)
  if (dirs == [])
    // nothing to do
    return; 
  end 
  
  // search at depth 0
  for (d = dirs)
    for (k = 1 : size(fnames, 2))
	  p = Filepath0(fnames(k), fullfile(d, dnames(k)), nres, ftype);
      paths = [paths, p];
      if (nres == 1 & paths <> []); break; end 
	end 
    if (nres == 1 & paths <> []); break; end 
  end
  
  // continue search if: 
  // - max depth not reached and
  // - nres == 0 | (nres == 1 & file not yet found)
  if (maxdepth > 0 & (nres == 0 | paths == []))
    subdirs = []; 
    for (d = dirs)
      subdirs = [subdirs, Listsubdirs(d, hid)]; 
    end
  
    paths = Filepath1_breadthfirst(fnames, dnames, subdirs, maxdepth-1, nres, hid, ftype, paths); 
  end
  
endfunction


// =============================================
// MAIN
// =============================================
if (~exists("directories", "local")); directories = ""; end
if (~exists("opt", "local")); opt = "first"; end
if (~exists("err", "local")); err=%t; end
if (exists("hidden", "local"))
  CL__warning("Deprecated use of argument ''hidden''"); 
  hid = evstr("hidden"); 
end
if (~exists("hid", "local")); hid=%f; end
if (~exists("maxdepth", "local")); maxdepth=50; end
if (~exists("smode", "local")); smode="bf"; end
if (~exists("ftype", "local")); ftype="any"; end

// note: "std" checked for compatibility with previous versions 
if (opt <> "std" & opt <> "all" & opt <> "first")
  CL__error("Invalid value for argument opt"); 
end
if (opt == "std")
  opt = "first"; 
end

if (typeof(err) <> "boolean" | typeof(hid) <> "boolean")
  CL__error("Invalid type for arguments ''err'' or ''hid''"); 
end

if (size(maxdepth, "*") <> 1 | typeof(maxdepth) <> "constant" | round(maxdepth) <> maxdepth | maxdepth < 0)
  CL__error("Invalid argument type or value for argument maxdepth"); 
end

if (smode <> "bf" & smode <> "df")
  CL__error("Invalid value for argument ''smode''"); 
end

if (ftype <> "d" & ftype <> "f" & ftype <> "any")
  CL__error("Invalid value for argument ''ftype''"); 
end

if (typeof(names) <> "string" & names <> [])
  CL__error("Invalid type for argument ''names''"); 
end

if (typeof(directories) <> "string" & directories <> [])
  CL__error("Invalid type for argument ''directories''"); 
end

// error if more than one row
if (size(names,1) > 1 | size(directories,1) > 1)
  CL__error("Invalid arguments sizes (names or directories)"); 
end

// error names empty
if (names == [])
  CL__error("Invalid value for argument names: cannot be empty"); 
end

// directory = white space => %CL_PATH
if (directories == "")  
  if (exists("%CL_PATH")) 
    directories = matrix(%CL_PATH, 1, -1);
  else
    CL__error("%CL_PATH is not defined"); 
  end 
end


// nres: number of results to be returned. 
// 0 means any number 
nres = 1; // one result expected 

if (opt == "all")
  nres = 0; // means: any number of results
end

// default result
paths = []; 


// replaces "/*" in directory names. 
// removes hidden directories from list (if not wanted)
dirlist = [];  
for (d = directories)
  [dname, ddname] = Fname(d); // name after last "/"
  if (dname == "*" & ddname <> "")
    dirlist = [dirlist, Listsubdirs(ddname, hid)]; 
  else
    dirlist = [dirlist, d]; 
  end   
end

// keep existing directories only
if (dirlist <> [])
  dirlist = dirlist(isdir(dirlist)); 
end 

// split names in fnames and dnames (file part and directory part)
[fnames, dnames] = Fname(names);
if (find(fnames == "." | fnames == "..") <> [])
  CL__error("Invalid names: cannot be . or .."); 
end


// search ... 
if (smode == "df")
  paths = Filepath1_depthfirst(fnames, dnames, dirlist, maxdepth, nres, hid, ftype, []); 
else
  paths = Filepath1_breadthfirst(fnames, dnames, dirlist, maxdepth, nres, hid, ftype, []); 
end

// error if no file found (and err is %t) 
if (err & paths == [] & opt <> "all")
  CL__error("File ''" + strcat(names," ") + "'' not found"); 
end

endfunction
