//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function CL_plot_rects(rects, icolors, clip_area, rectype)
// Plots filled rectangles
//
// Calling Sequence
// CL_plot_rects(rects, icolors [[, clip_area, rectype]])
//
// Description
// <itemizedlist><listitem>
// <p>Plots filled rectangles. </p>
// <p></p>
// <p>The kth rectangle is filled with the color icolors(k) which should be an integer value. </p>
// <p>If the color index is 0 or if the rectangle is outside the clip area, the rectangle is not drawn. </p>
// <p></p>
// <p>The argument "rectype" defines the type of rectangle description: "r" or "xy" (default is "r"). </p>
// <p>If rectype == "r", the rectangles are defined by: [x_upper_left_corner = xmin; y_upper_left_corner = ymax; 
// width = xmax - xmin; height = ymax - ymin] </p>
// <p>If rectype == "xy", the rectangles are defined by: [xmin; xmax; ymin; ymax] </p>
// <p></p> 
// </listitem>
// <listitem>
// <p>The function is optimized for the case where the rectangles are arranged in lines: 
// contiguous rectangles on the same line with the same color are packed in order to reduce the number of 
// rectangles. </p>
// </listitem>
// </itemizedlist>
//
// Parameters
// rects: Rectangles (4xN)
// icolors: Color indices (1xN)
// clip_area: (optional) Clip region: [xmin; xmax; ymin; ymax]. Default is [-%inf; %inf; -%inf; %inf] (4x1)
// rectype: (string, optional) Type of rectangle description: "r" or "xy". Default is "r" (1x1)
//
// Authors
// CNES - DCT/SB
//
// See also
// xrects
//
// Examples
// // Example 1 
// rects = [[0; 1; 1; 2], [1; 2; 1; 2], [2; 3; 1; 2], [1; 2; 2; 3], [2; 6; 2; 3]]; 
// icolors = [2, 2, 3, 5, 5]; 
// 
// scf(); 
// clip_area = [0; 5; 0; 4]; 
// CL_plot_rects(rects, icolors, clip_area=clip_area, rectype="xy"); 
// 
// // Example 2
// n = 100; 
// vals = linspace(-1, 1, n+1); 
// step = (vals($) - vals(1)) / n; 
// x = (vals(1:$-1) + vals(2:$))/2;  
// y = x; 
// [X, Y] = ndgrid(x, y); 
// Z = abs(X .* Y); 
// 
// f = scf(); 
// nbcols = 64; 
// f.color_map = CL_colormap("jet", nbcols); 
// N = size(Z, "*"); 
// rects = [X(:)' - step/2; Y(:)' + step/2; step * ones(1:N);  step * ones(1:N)]; 
// icolors = 1 + round(Z(:)' * (nbcols - 1)); 
// CL_plot_rects(rects, icolors); 



// Declarations:

// ---------------------------------------------------
// indices start/end of segments such that bool == %t
// bool: array of booleans (1xN) 
// interv: intervals  = [istart; iend] (2xN) 
// ---------------------------------------------------
function [interv] = rects_intervTrue(bool)
  interv = []; 
  I = find(bool); 
  if (I <> [])
    // note: OK if length(I) < 2 
    J = [0, find(I(2:$)-I(1:$-1) > 1), length(I)]; 
    N = length(J); 
    interv = [I(J(1:N-1)+1); I(J(2:N))]; 
  end
endfunction


// ---------------------------------------------------
// sort rectangles 
// ymin increasing, then xmin  
// ---------------------------------------------------
function [rects, icolors] = rects_sort(rects, icolors)
  if (rects == [])
    return; 
  end
  
  x = rects(1,:); 
  y = rects(3,:); 
  xmin = min(x);  
  xmax = max(x);  
  ymin = min(y); 
  ymax = max(y); 
  
  if (xmax > xmin)
    X = (x - xmin) / (xmax - xmin); 
  else 
    X = zeros(x); 
  end
  
  if (ymax > ymin)
    Y = (y - ymin) / (ymax - ymin); 
  else
    Y = zeros(y); 
  end
  
  Z = Y * 1.e7 + X; 
  
  [rects, I] = CL_sortMat(rects, Z); 
  icolors = icolors(I); 

endfunction 	

// ---------------------------------------------------
// compacts rectangles (horizontally)
// rects: 4 x N  (xmin, xmax, ymin, ymax) 
// icolors: 1 x N
// ---------------------------------------------------
function [rects, icolors] = rects_pack(rects, icolors)
  
  if (size(rects, 2) <= 1)
    return; 
  end
  
  // Sorts rectangles
  [rects, icolors] = rects_sort(rects, icolors); 
  
  xtol = (max(rects(2,:)) - min(rects(1,:))) / 1.e7; 
  ytol = (max(rects(4,:)) - min(rects(3,:))) / 1.e7; 

  // indices des mailles telles que : 
  // - valeur identique  la valeur suivante
  // - mme latitude que latitude suivante
  // - ecart de longitude = 1 pas (donc < 1.5 pas)
  // NB: fonctionne cas pas en longitude = constant sur une mme latitude
  I = find(icolors(2:$) == icolors(1:$-1) & ..
	   abs(rects(3,2:$) - rects(3,1:$-1)) <= ytol & ..
	   abs(rects(4,2:$) - rects(4,1:$-1)) <= ytol & ..
	   abs(rects(1,2:$) - rects(2,1:$-1)) <= xtol); 

  if (I <> [])
    bool = (zeros(icolors) == 1); // init  %f
    bool(I) = %t; 

    interv = rects_intervTrue(bool); 

    // change dimension en longitude (multiple par nb de mailles)
    rects(2, interv(1,:)) = rects(2, interv(2,:)+1); 

    // Initial algorithm: icolors(I+1) = []; rects2(:,I+1) = [];
    // But very slow with Scilab 6 => affect values instead of removing elements
    J = CL_setdiff(1 : size(icolors,"*"), I+1); 
    icolors = icolors(J);
    rects = rects(:,J);  
  end

endfunction


// convert [xmin; xmax; ymin; ymax] => [xmin; ymax; w=xmax-xmin, h=ymax-ymin] 
function [rects, icolors] = rects_filter(rects, icolors, clip_area)
  if (rects == [])
    return; 
  end

  rects(1,:) = max(rects(1,:), clip_area(1)); 
  rects(2,:) = min(rects(2,:), clip_area(2)); 
  rects(3,:) = max(rects(3,:), clip_area(3)); 
  rects(4,:) = min(rects(4,:), clip_area(4)); 
  
  I = find(rects(1,:) < rects(2,:) & rects(3,:) < rects(4,:) & icolors > 0.5); 
  
  rects = rects(:,I); 
  icolors = icolors(I); 
  
endfunction 	

// Plot the rectangles 
// (using the function "fec")
// nbcols: number of colors in current colormap
function rects_plot(rects, icolors, nbcols)
  if (rects == [])
    return; 
  end
    
  // build arguments for "fec" 
  N = length(icolors); 
  x = matrix([rects(1,:); rects(1,:); rects(2,:); rects(2,:)], 1, -1); 
  y = matrix([rects(3,:); rects(4,:); rects(4,:); rects(3,:)], 1, -1); 
  T = [(1:N)', matrix(1:4*N, 4, -1)', ones(1:N)']; 
  z = matrix(repmat(icolors, 4, 1), 1, -1); 
  fec(x, y, T, z, zminmax = [1, nbcols], colminmax = [1, nbcols], mesh=%f); 
endfunction 	


// Code:

// Check number of arguments
nbargsin = argn(2); 
if (nbargsin < 2 | nbargsin > 4)
  CL__error("Invalid number of input arguments"); 
end

// Defaut values
if ~exists("clip_area", "local"); clip_area = []; end
if ~exists("rectype", "local"); rectype = "r"; end

if (clip_area == [])
  f = gcf(); 
  clip_area = [-%inf; %inf; -%inf; %inf]; 
end

if (rectype <> "r" & rectype <> "xy")
  CL__error("Invalid value for argument rectype"); 
end

[rects, icolors] = CL__checkInputs(rects, 4, icolors, 1); 

if (size(clip_area, 1) <> 4 | size(clip_area, 2) <> 1)
  CL__error("Invalid size for argument clip_area"); 
end

if (rects == [] | icolors == [])
  return; 
end

if (rectype == "r" & find(rects(3,:) < 0 | rects(4,:) < 0) <> [])
  CL__error("Invalid value for argument rects: width or height < 0"); 
end
if (rectype == "xy" & find(rects(1,:) > rects(2,:) | rects(3,:) > rects(4,:)) <> [])
  CL__error("Invalid value for argument rects: xmin > xmax or ymin > ymax < 0"); 
end

// number of colors in current colormap
f = gcf(); 
nbcols = size(f.color_map, 1); 

if (find(icolors <> round(icolors) | icolors < 0 | icolors > nbcols) <> []) 
  CL__error("Invalid value for argument icolors: not positive integer values"); 
end

// rects transformed into [xmin; xmax; ymin; ymax] 
if (rectype == "r")
  rects = [rects(1,:); rects(1,:) + rects(3,:); rects(2,:) - rects(4,:); rects(2,:)]; 
end

[rects, icolors] = rects_filter(rects, icolors, clip_area); 

[rects, icolors] = rects_pack(rects, icolors); 

rects_plot(rects, icolors, nbcols); 

endfunction

