//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [a, ka] = CL_setdiff(a, b, orien)
// CelestLab version of setdiff function.
//
// Calling Sequence
// [a, ka] = CL_setdiff(a, b [, orien])
//
// Description
// <itemizedlist>
// <listitem>
// <p>This function has been developped to fix bug #15967 in Scilab 6.0.2. 
// and bug #16340 in Scilab 6.1.0.  </p>
// <p>See <link linkend="setdiff">setdiff</link> for more details. </p>
// </listitem>
// <listitem>
// <b>Notes</b>: 
// <p>- If b is empty, the outputs are the same as unique(a [, orien]). </p>
// </listitem></itemizedlist>
//
// Parameters
// a, b: vectors, matrices or hypermatrices of real numbers or encoded integers or strings. Both must have the same data types (and integer types). If the option orien="r" is used, a and b must have the same number of columns. If the orien="c" is used, they must have the same number of rows.
// orien: oriented processing: "r": rows of a are searched among b ones. "c": columns of a are searched among b ones. no orien: elements of a are searched among b ones.
// a: sorted vector of a's components that are not in b. orien="r": matrix of rows of a that are not in b, sorted in lexicographic order. orien="c": matrix of columns of a that are not in b, sorted in lexicographic order.
// ka: vector of linear indices of selected a's components, rows, or columns, such that: v = a(ka), or v = a(ka,:) (orien="r"), or v = a(:,ka) (orien="c")
//
// Authors
// CNES - DCT/SB
//
// Examples
// CL_setdiff(1:10, [2 5 7])

// number of input arguments
nbargs = argn(2); 

// Deal with empty b case
if (b == [])
  // work-around for bug #15967 in Scilab 6.0.2
  if (nbargs == 3) 
    [a, ka] = unique(a, orien); 
  else
    [a, ka] = unique(a); 
  end
  
elseif (typeof(a) == "string" | typeof(b) == "string")
  // work-around for bug #16340 in Scilab 6.1.0
  // "" replaced by another string (REPL). 
  // REPL supposed not to be used in practical applications ...  
  REPL = ascii(1) + ascii(3) + ascii(5); 
  
  if (find(a == REPL) <> [] | find(b == REPL) <> []) 
    CL__error("Function is not able to process the inputs ("); 
  end 
  
  if (find(a == "") <> [])
    a = strsubst(a, "", REPL); 
  end 
  if (find(b == "") <> [])
    b = strsubst(b, "", REPL);
  end
         
  if (nbargs == 3) 
    [a, ka] = setdiff(a, b, orien);
  else
    [a, ka] = setdiff(a, b);
  end
  if (find(a == REPL) <> [])
    a = strsubst(a, REPL, ""); 
  end 
  
else
  // Call original setdiff
  if (nbargs == 3) 
    [a, ka] = setdiff(a, b, orien);
  else
    [a, ka] = setdiff(a, b);
  end
  
end

endfunction


